#include "qcalc.h"
#include "qdutil.h"
#include "qwui.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;
extern void QexBuildExportQIEObj (QdObjectSeq_var,vector<KVDataBase_var>&);
extern bool QexWriteFile (QWidget*, bool);

void QcalcWindow::defaultAll()
 {

 if(QuSlotDebug())
 {
  printf("QcalcWindow::defaultAll %ld\n", this);
  fflush(stdout);
  }

	QcalcListVirtualItem *next = ( QcalcListVirtualItem * ) _join_listview->firstChild( );
	while ( next )
	{
		if ( next->getEntryType() == ListTypeEntry )
		{
			next->m_pcalcTab->_width_field->setText(      _width_field_Def );
			next->m_pcalcTab->_gap->setCurrentItem(       _gap_Def         );
			next->m_pcalcTab->_gap_field->setText(        _gap_field_Def   );
			next->m_pcalcTab->_algorithm->setCurrentItem( _algorithm_Def   );
		}
		next = ( QcalcListVirtualItem * ) next->QguGetNext( );
	}
  
  _TimeIvlSelect->setCurrentItem(  _TimeIvlSelect_Def );
  _IvlSource->setText(             _IvlSource_Def     );
  _TimeTagSelect->setCurrentItem(  _TimeTagSelect_Def );
  _TagSource->setText(             _TagSource_Def     );

}

void QcalcWindow::setDefaults()
{

  _width_field_Def   = QString("60");
  _width_field_cpy   = QString("60");

  _gap_Def           = 0;
  _gap_cpy           = _gap_Def;

  _gap_field_Def     = QString("6");
  _gap_field_cpy     = QString("6");

  _algorithm_Def     = 0;
  _algorithm_cpy     = _algorithm_Def;

  _IvlSource_Def     = QString(" ");
  _IvlSource_cpy     = QString(" ");

  _TimeIvlSelect_Def = 0;
  _TimeIvlSelect_cpy = _TimeIvlSelect_Def;

  _TagSource_Def     = QString(" ");
  _TagSource_cpy     = QString(" ");

  _TimeTagSelect_Def = 1;
  _TimeTagSelect_cpy = _TimeTagSelect_Def;


}

 void QcalcWindow::keepChoices()
 {

 if ( QuSlotDebug() )
 {
	printf("QcalcWindow::keepChoices %ld\n", this);
	fflush(stdout);
 }
 
   	QcalcListVirtualItem *next = ( QcalcListVirtualItem * ) _join_listview->firstChild( );
	while ( next )
	{
		if( next->getEntryType() == ListTypeEntry && next->m_pcalcTab->isVisible())
  	 	{
			// keep the choices of the visible tab
 		 	_width_field_cpy   = QString(next->m_pcalcTab->_width_field->text().latin1());
 		 	_gap_cpy           = next->m_pcalcTab->_gap->currentItem();
 		 	_gap_field_cpy     = QString(next->m_pcalcTab->_gap_field->text().latin1());
 		 	_algorithm_cpy     = next->m_pcalcTab->_algorithm->currentItem();
			 break;
  		}
		next = ( QcalcListVirtualItem * ) next->QguGetNext( );
	}
  
  _IvlSource_cpy     = QString(_IvlSource->text().latin1());
  _TimeIvlSelect_cpy = _TimeIvlSelect->currentItem();
  _TagSource_cpy     = QString(_TagSource->text().latin1());
  _TimeTagSelect_cpy = _TimeTagSelect->currentItem();


}

 void QcalcWindow::applyChoices()
 {

 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::applyChoices %ld\n", this );
	fflush(stdout);
 }

  	for(int i=0; i<_join_listview->localList.size(); i++)
	{
    		QcalcListVirtualItem * item = (QcalcListVirtualItem*) _join_listview->localList[i];
		if( item->getEntryType() != ListTypeEntry ) continue;
		
		item->m_pcalcTab->_width_field->setText(          _width_field_cpy   );
  		item->m_pcalcTab->_gap->setCurrentItem(           _gap_cpy           );
  		item->m_pcalcTab->_gap_field->setText(            _gap_field_cpy     );
  		item->m_pcalcTab->_algorithm->setCurrentItem(     _algorithm_cpy     );

  	}
	
  _TimeIvlSelect->setCurrentItem( _TimeIvlSelect_cpy );
  _IvlSource->setText(            _IvlSource_cpy     );
  _TimeTagSelect->setCurrentItem( _TimeTagSelect_cpy );
  _TagSource->setText(            _TagSource_cpy     );

}

 void QcalcWindow::setTTivl( int item )
 {
 static QString lastIvlObj = QString("");
 
 if(QuSlotDebug())
 {
	printf("QcalcWindow::setTTivl %ld, %d\n", this, item);
	fflush(stdout);
 }

  if( item == 0 )
  {
    // from data
    if( !(_IvlSource->text().isEmpty()) ) lastIvlObj = QString(_IvlSource->text().latin1());
    
    IvlSource_label->setText( " " );
    QToolTip::remove( _IvlSource );
    QToolTip::add( _IvlSource, "not active" );
    _IvlSource->setText( " " ); 
    _IvlSource->active = false;
  }
  else if ( item == 1 )
  {
    IvlSource_label->setText( "Interval Object" );
    QToolTip::remove( _IvlSource );
    QToolTip::add( _IvlSource, "drag and drop object here" );
    _IvlSource->active       = true;
    _IvlSource->acceptNumber = false;
    _IvlSource->acceptTIvlObject = true;
    _IvlSource->acceptTimeSubset = true;
    _IvlSource->setText( lastIvlObj );
    _IvlSource->lineBox->setFocus();
  }

    _IvlSource->validateInput( _IvlSource->text() );
}


void QcalcWindow::setTTsource( int item )
{
 static QString lastObj = "";
 static QString lastNumber = "";
 
 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::setTTsource %ld, %d\n", this, item );
	fflush(stdout);
 }

  if( item == 0 )
  {
    if ( !(_TagSource->text().isEmpty()) ) lastObj = QString(_TagSource->text().latin1());
    // create regular
    TagSource_label->setText("Spacing (sec)");
    QToolTip::remove(_TagSource);
    QToolTip::add( _TagSource, "enter spacing in seconds here");
    _TagSource->acceptNumber = true;
    _TagSource->acceptTTagObject = false;
    _TagSource->setText(lastNumber);
  }
  else if ( item == 1 )
  {
    if ( !(_TagSource->text().isEmpty()) ) lastNumber = QString(_TagSource->text().latin1());
    TagSource_label->setText("Time Tag Object");
    QToolTip::remove(_TagSource);
    QToolTip::add( _TagSource, "drag and drop object here");
    _TagSource->acceptNumber = false;
    _TagSource->acceptTTagObject = true;
    _TagSource->setText(lastObj);
  }

  _TagSource->reValidate( ) ; // updates T display if valid
  _TagSource->lineBox->setFocus();

}

 void QcalcWindow::setTimeFeedback()
 {

 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::setTimeFeedback %ld\n", this );
	fflush(stdout);
 }

  QdTimeInterval_var ivl;
  int                Ntags;
  double             Spacing;
  double             Tinterval;

  // set interval

  ivl = QcalcGetInterval( );

  if ( ivl.is_nil() )
  {
    Ntags_field->setText(  "??");
    spacing_field->setText("??");
    Start_field->setText(  "0000-00-00 00:00:00.000");
    End_field->setText(    "0000-00-00 00:00:00.000");
    Tinterval = -1.0;
  }
  else
  {
    Start_field->setText( QString( ivl->start().iso().c_str()) );
    End_field->setText(   QString( ivl->end().iso().c_str()  ) );
    Tinterval = Seconds(ivl->duration_abs());
  }

  // set tags

  if( _TimeTagSelect->currentItem() == 0 )
  {
    // create regular tags
    bool OK;
    Spacing = _TagSource->text().toDouble(&OK);
    if( !OK || Spacing < 1.e-20 )
    {
      Spacing = -1.0;
      Ntags   = -1;
    }
    else
    {
      Ntags = (int) (Tinterval/Spacing + 1.0);
    }

  }
  else
  {
    // tags from object, get_timetags() is safe against nil object
    QdTimeSeq_var ts     = get_timetags(_TagSource->getSlotObject());
    Spacing              = get_time_spacing(ts);
    Ntags                = (int) (Tinterval/Spacing + 1.0);
  }

  QString txt;
  if(Ntags > 0)
  {
     Ntags_field->setText(txt.setNum(Ntags));
  }
  else
  {
      Ntags_field->setText("??");
  }

  if(Spacing > -1)
  {
      spacing_field->setText(txt.setNum(Spacing));
  }
  else
  {
      spacing_field->setText("??");
  }

}


void QcalcWindow::writeFile()
 {

 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::writeFile %ld\n", this );
	fflush(stdout);
 }

   // keep settings for default behaviour
   keepChoices();

   QdObjectSeq_var        JoinedList = new QdObjectSeq();
   vector<KVDataBase_var> OptionList;
   bool OK;

   // do multi-join first
  bool isTimeSeries = joinObjects( JoinedList, OptionList, TRUE ); // TRUE skips virtual objects

  if ( !isTimeSeries )
  {

    	// time tags not set: copy over inputs to outputs
    	// for each item on Join List, set choices
		JoinedList->clear();
		QcalcListVirtualItem *next = ( QcalcListVirtualItem * ) _join_listview->firstChild( );
		while ( next )
		{
			if ( next->getEntryType() == ListTypeEntry )
			{
				QdObject_var obj = next->getDataObject();
				if ( obj.is_nil() )
				{
					messageHandler( "object no longer exists", true );
					return;
				}

				QString ctxt      = next->getWLcontext();
				QguListItem *item = qsasWindow->WorkingList->getItemByName( ctxt );
				if( item == NULL )
				{
					messageHandler( "Cannot find items name", true );
				}

				KVDataBase_var Join_options = new KVDataBase();

				QString name              = item->getWLcontext();
				QdString_var new_obj_name = new QdString( name.latin1() );
				Join_options->set( NEW_NAME, new_obj_name );


				JoinedList->push_back( obj );
				OptionList.push_back( Join_options );
			}

			next = ( QcalcListVirtualItem * ) next->QguGetNext( );
		}
  
  }

  if ( OptionList.size()  == 0 )
  {
      return;
  }

  if( JoinedList->size() == 0 )
  {
      return;
  }

   //  Build QIE Export file object from Qcalc join object and Write file

  QexBuildExportQIEObj ( JoinedList, OptionList );

  OK = QexWriteFile ( this, isTimeSeries );


 return;
}




void QcalcWindow::dragEnterEvent(QDragEnterEvent *event)
{
 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::dragEnterEvent %ld\n", this );
	fflush(stdout);
 }

  this->setActiveWindow();
  this->raise();
  this->setAcceptDrops(TRUE);
}

