#include "qcalc.h"
#include "qwui.h"
#include "qdutil.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;

void QcalcWindow::save(Vostream& v_os)
{
	string guiID;
	guiID=QSAS_VERSION_ID;

	v_os << guiID; 

	QDateTime  qdatetime = QDateTime::currentDateTime();
#ifdef QT_VERSION
#if QT_VERSION <0x030000
	SAVE_ALIAS_EXT =  qdatetime.toString(  );
#else
	SAVE_ALIAS_EXT =  qdatetime.toString( "dd.MM.yyyy-hh:mm:ss.zzz" );
#endif // end of #if
#endif // end of #ifdef

	v_os <<  SAVE_ALIAS_EXT.latin1();
	
     int with = _join_listview->with;

#if QT_VERSION < 0x040000
	QList< QcalcListVirtualItem > lst;
#else
        QList< QcalcListVirtualItem *> lst;
#endif

	QcalcListVirtualItem *item = dynamic_cast< QcalcListVirtualItem* >( _join_listview->firstChild() );
	int cnt = 0;

	// count items to write and add extension to alias to help sort out restore names
	while ( item )
	{
		lst.append( item );
		item->with = with;
		cnt++;
		if(item->getEntryType() == ListTypeEntry) {
			if (item->m_pcalcTab){
				item->m_alias_cpy = QString(item->text(Alias_).latin1());
				item->reNameAlias(item->m_alias_cpy + SAVE_ALIAS_EXT);
			}
		}
		item = dynamic_cast< QcalcListVirtualItem* >( item->nextSibling() );
	}

	v_os << cnt;
	bool IvlObjFound = 0;
	bool TagObjFound = 0;
	
	for ( int i = 0; i < cnt; i++ )
	{
		QcalcListVirtualItem *item = lst.at( i );
                v_os << (int) item->getEntryType();
                v_os << (int) item->m_eObjType;
                
                if( ((Q_LIST_VIEW_ITEM *)item)->text(Obj_Name_) == _IvlSource->text()) {
                    item->isIvlObj = 1;
                    IvlObjFound = 1;
                }
                else  item->isIvlObj = 0;

                if( ((Q_LIST_VIEW_ITEM *)item)->text(Obj_Name_) == _TagSource->text()) {
                    item->isTagObj = 1;
                    TagObjFound = 1;
                }
                else  
                    item->isTagObj = 0;

                v_os << *item;
                v_os << item->isIvlObj;
                v_os << item->isTagObj;
				
		}
	
   
   v_os << _TimeIvlSelect->currentItem();

   if(IvlObjFound)
     _IvlSource->with = 0; // don't save if already being saved by item on calc list
   else
     _IvlSource->with = with;

   v_os << *_IvlSource;

   v_os << _TimeTagSelect->currentItem();

   if(TagObjFound)
     _TagSource->with = 0; // don't save if already being saved by item on calc list
   else
     _TagSource->with = with;
	
    v_os << *_TagSource;


   bool bJoinFirst = multiJn_check1->isChecked();
   bool bJoinLast = multiJn_check2->isChecked();
   v_os << bJoinFirst;
   v_os << bJoinLast;

   	// now reset alias
	for ( int i = 0; i < cnt; i++ )
	{
		QcalcListVirtualItem *item = lst.at( i );
		item->reNameAlias(item->m_alias_cpy);
	}	

}


void QcalcWindow::restore( Vistream& v_is, bool replace )
{

	QcalcListVirtualItem *tagItem = NULL;   
	QcalcListVirtualItem *ivlItem = NULL;
	string guiID; 
	v_is >> guiID;
	char *ptr;
   
	if( QuSlotDebug() ) printf("QcalcWindow::restore set ID =  %s\n-----------------------------\n", guiID.c_str());
  
	if( (ptr = (char*)strstr(guiID.c_str(), "QSASQcalcV")) == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
		qcalc_version = 2.1; // from here on use QSAS_VERSION_ID
	}
	else
	{
		qcalc_version = strtod( &(ptr[10]), NULL);
		// vs 1.1 change in isArray to virtualArrDim in calc at 2.0
		
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}
   
	qsasWindow->WorkingList->blockSignals( true );
	_join_listview->blockSignals( true );
   
	// clear list for true restore
	if(replace)  _join_listview->deleteAllItems();
   
	Q_LIST_VIEW_ITEM *dummy = _join_listview->findItem("a", Alias_) ;
	if(dummy) dummy->setText(Alias_, "empty");
   
	
	string aliasExt;
	v_is >>  aliasExt;
	SAVE_ALIAS_EXT = QString(aliasExt.c_str());

   	int cnt;
	v_is >> cnt;
        QcalcListVirtualItem* lastItem=NULL;
	for ( int i = 0; i < cnt; i++ )
	{

		int entryType;
		v_is >>  entryType; // this ensures correct item type constructed
		int eObjType;
		v_is >> eObjType; // this ensures correct calc tab type constructed
		QString WL;
		if (eObjType == WORKING_LIST_OBJECT) WL = QString(" from WL");
		else WL = QString("");
		
		QcalcListVirtualItem* item = NULL;
		item = new QcalcListVirtualItem( _join_listview,  "Restored", WL, (QguListType) entryType, lastItem );
		lastItem = item;
		if ( item )
		{
			v_is >> *item;
			v_is >> item->isIvlObj;
			v_is >> item->isTagObj;
			if(item->isIvlObj == 1) ivlItem = item;
			if(item->isTagObj == 1) tagItem = item;
		}
		
	}
		
   // set copy to pre-restore values for recovery option

   _TimeIvlSelect_cpy = _TimeIvlSelect->currentItem();
   int tIvl;
   v_is >> tIvl;
   _TimeIvlSelect->setCurrentItem( tIvl );
   setTTivl(tIvl);

   _IvlSource_cpy = QString(_IvlSource->text().latin1());
   v_is >> *_IvlSource;

   _TimeTagSelect_cpy = _TimeTagSelect->currentItem();
   int tTag;
   v_is >> tTag;
   _TimeTagSelect->setCurrentItem( tTag );
   setTTsource( tTag );

   _TagSource_cpy = QString(_TagSource->text().latin1());

   v_is >> *_TagSource;
   
   // Joining options:
   bool bJoinFirst;
   bool bJoinLast;
   v_is >> bJoinFirst;
   v_is >> bJoinLast;

   multiJn_check1->setChecked( bJoinFirst );
   multiJn_check2->setChecked( bJoinLast );
   
   	// now all items have been restored to list reset aliases and references to them
	Q_LIST_VIEW_ITEM_ITERATOR it(_join_listview->firstChild());
  	while(it.current())
  	{
  		QcalcListVirtualItem *item = dynamic_cast<QcalcListVirtualItem *> (it.current());
		if ( item && item->m_pcalcTab ){
			QString newName = QString(item->text(Alias_).latin1());
			int posn = newName.find(SAVE_ALIAS_EXT);
 	 		if(posn != -1) {
				newName.truncate(posn);
				while ( _join_listview->findItem(newName, Alias_) ){
					newName+="_A";
				}
				item->reNameAlias(newName.latin1());
			}
			for(int i=0; i<item->m_pcalcTab->nSlots; i++){
				item->m_pcalcTab->operandStack[i]->slot->validateInput( item->m_pcalcTab->operandStack[i]->slot->text() );
			}
		}
 		++it;
  	}

	// finally we can reset the tag and ivl slots to the new name if needed
	if( tagItem != NULL ) _TagSource->setText( tagItem->WLcontext.latin1());
	if( ivlItem != NULL ) _IvlSource->setText( ivlItem->WLcontext.latin1());

	// now we have some objects it is safe to delete place holder "? + ?" object
	_join_listview->clearSelection();
	_join_listview->clearSelectedList();
     bool safeToDel = false;
	Q_LIST_VIEW_ITEM_ITERATOR itDel((Q_LIST_VIEW *)_join_listview);
	while(itDel.current()){
		QcalcListVirtualItem * dummyItem = (QcalcListVirtualItem*) itDel.current();
		// ensure it isn't the only item
		if(dummyItem->text(Alias_) == QString("empty") ){
			_join_listview->AddToSelectedList(dummyItem);
		}
		else safeToDel = true; // real item exists, so safe to delete place holder
		itDel++;
	}
	if(safeToDel) _join_listview->deleteListItems();
	qsasWindow->WorkingList->blockSignals( false );
	_join_listview->blockSignals( false );

	if(!replace) _join_listview->validateWLitems();
	
	logArea->setFocus(); // allows selected item to be deleted as it makes window active
}


 void QcalcWindow::saveAs()
 {

 if ( QuSlotDebug() )
 {
   printf( "QcalcWindow::saveAs %ld\n", this );
   fflush(stdout);
 }

   QString _filter = QString( "save (*" );
   _filter        += SAVE_EXTENSION;
   _filter        += QString( ")" );
   QString fn = QFileDialog::getSaveFileName( _filename, _filter, this, "save", "Select file - Save Join Settings" ) ;

   if ( !fn.isEmpty() )
   {
     int pos = fn.findRev( SAVE_EXTENSION );
     if ( pos < 0 )
     {
       pos = fn.findRev( "." );
       if ( pos > 0 )
       {
           fn.truncate( pos );
       }

       fn += SAVE_EXTENSION;
     }

     _filename = fn;
     save();
   }
   else
   {
     messageHandler( "saving aborted ", true );
   }

 }
      

 void QcalcWindow::save()
 {

 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::save %ld\n", this ) ;
	fflush(stdout);
 }


  _join_listview->with = 0;
  if ( _filename.isEmpty() )
  {
       saveAs();
       return;
  }

  try
  {
      VXDRostream xdr_os( _filename.latin1() );
      save(xdr_os);
  }
  catch ( Exception &e )
  {
       messageHandler( e.msg().c_str(), true );
       return;

  }

  messageHandler( _filename+" saved", false );
  return;

 }


void QcalcWindow::saveSession( QString folder )
{

 if ( QuSlotDebug() )
 {
	printf("QcalcWindow::saveSession %ld\n", this);
	fflush(stdout);
 }

  if ( folder.isEmpty() )
  {
     return;
  }

  // keep original name
  QString fileN = QString( _filename.latin1() );

  // set session folder and save
  this->_filename  = QString( folder.latin1() );
  this->_filename += QString( "/saveSet" );
  this->_filename += this->SAVE_EXTENSION;
  save();

  // put file name back
  _filename = QString( fileN.latin1() );
}

