//  qjui.cc
//
// Join Widget
//
// Mar 2002: Tony Allen
//

#include "qjui.h"
#include "qexuiW.h"
#include "Interp.h"
#include "qdutil.h"
#include "qwui.h"
#include "qdutil.h"

#include <qpalette.h>
#include <qgroupbox.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <stdio.h>

#include <stdlib.h>

using namespace QSAS;

extern qwuiWindow* qsasWindow;

// QjuiListItem class
// ------------------

QjuiListItem::QjuiListItem(QguListView *parent,  QString txt,  QString WL,
   QguListType type, QguListItem *after)
   :  QguListItem(parent, txt, type, after)
{
  
  if(type == ListTypeEntry){
    WLcontext = QString(WL);
    
    setChoices(-1);
  }
} 

QjuiListItem::QjuiListItem(QguListItem *parent,  QString txt,  QString WL,
   QguListType type, QguListItem *after)
   :  QguListItem(parent, txt, type, after)
{
  
  if(type == ListTypeEntry){
    WLcontext = QString(WL);


    setChoices(-1);
    
  }  
} 

// QjuiListItem::~QjuiListItem()
// {  
// }


void QjuiListItem::setChoices(int w){
   QjuiListView *view = (QjuiListView *)this->listView();
   if(view == NULL) return;
   
   // if w = -1 update all selections
   // otherwise only update the one changed to avoid interlaced signals

   if(w == -1 || w == 0) {
     if(view->owner->_algorithm->currentItem() == 2)
        this->I_algorithm =  view->owner->_algorithm_cpy;
     else
        this->I_algorithm =   view->owner->_algorithm->currentItem();
  }

   if(w == -1 ) {
     if(view->owner->_width_field->text().isEmpty()) 
        this->I_width_field = view->owner->_width_field_cpy;       
     else
        this->I_width_field = view->owner->_width_field->text();
   }
   if(w == 1 && view->owner->_width_field->edited()) {
      this->I_width_field = view->owner->_width_field->text();
   }

   if(w == -1 || w == 2){ // Gap Handling
   
     if(view->owner == QJUI_widget){ // watch for stripping fills on WL
     
       if( view->owner->_gap->currentItem() == 2){
         // issue warning as FILLVAL gap fill is selected
         view->owner->status("Uncheck Strip-Fill-Values toggle on main window",10000);
       }
       else if( view->owner->_gap->currentItem() == 1){
         // issue warning if fillval is also zero
	 QdObject_var dobj = this->QguGetObject();
	 if(!dobj.is_nil() && dobj->xref_exists("FILLVAL")) {
	   QdRScalar_var xref = QdRScalar_var::narrow(dobj->get_xref("FILLVAL"));
	   if( !xref.is_nil() && *xref == 0.)
             view->owner->status("WARNING Fill Value is also zero for this obj, uncheck Strip-Fill-Values on WL",10000);
	 }       
       }
       else{
         view->owner->status(" ",10000);
       }
     }
     
     if(view->owner->_gap->currentItem() == 5)
       this->I_gap = view->owner->_gap_cpy;
     else
       this->I_gap = view->owner->_gap->currentItem();
       
   }
   
   if(w == -1 ) {
     if(view->owner->_gap_field->text().isEmpty())
       this->I_gap_field = view->owner->_gap_field_cpy;
     else
       this->I_gap_field = view->owner->_gap_field->text();
   }
   
   if(w == 3 && view->owner->_gap_field->edited()) {
      this->I_gap_field =   view->owner->_gap_field->text();
   }
   
}




// QjuiListView class
// ------------------


QjuiListView::QjuiListView(QWidget *parent, QjuiWindow* QjWindow, const char *name) 
   :  QguListView(parent,name)
{
   this->owner = QjWindow;

   RButtonMenu = new Q_POPUP_MENU( this );
   
   RButtonMenu->insertItem("Copy Settings <-", QjWindow, SLOT( keepChoices() ), Qt::CTRL+Qt::Key_Left  );
   RButtonMenu->insertItem("Apply Settings ->", QjWindow, SLOT( applyChoices() ), Qt::CTRL+Qt::Key_Right  );
   RButtonMenu->insertSeparator(); 
   RButtonMenu->insertItem("&New Folder", this, SLOT( makeGroup() ), Qt::CTRL+Qt::Key_N  );
   RButtonMenu->insertItem("&Re-Name", this, SLOT( renameSelectedItem() ), Qt::CTRL+Qt::Key_R );
   RButtonMenu->insertItem("&Edit WL Links", this, SLOT( editWLLink() ), Qt::CTRL+Qt::Key_E );
   RButtonMenu->insertSeparator(); 
   RButtonMenu->insertItem("&Copy", this, SLOT( copy() ), Qt::CTRL+Qt::Key_C );
   RButtonMenu->insertItem("Paste", this, SLOT( paste() ) , Qt::CTRL+Qt::Key_V);
   RButtonMenu->insertItem("select &All", this, SLOT( selectAll() ), Qt::CTRL+Qt::Key_A );
   RButtonMenu->insertSeparator(); 
   RButtonMenu->insertItem("Move Down", this, SLOT( moveItemDown() ) , Qt::CTRL+Qt::Key_Down);
   RButtonMenu->insertItem("Move Up", this, SLOT( moveItemUp() ) , Qt::CTRL+Qt::Key_Up);
   RButtonMenu->insertSeparator();
   RButtonMenu->insertItem("Delete", this, SLOT( deleteListItems() ),
      Qt::CTRL+Qt::Key_X );

   setAcceptDrops(true);
   connect(qsasWindow->WorkingList, SIGNAL(listModified()), this, SLOT(validateWLitems()));
   connect(this, SIGNAL(selectionChanged()), this, SLOT(updateOnSelection()));
   connect(this, SIGNAL(selectionChanged()), this, SLOT(update1OnSelection()));
   connect(this, SIGNAL(selectionChanged()), this, SLOT(update2OnSelection()));
   connect(this, SIGNAL(selectionChanged()), this, SLOT(update3OnSelection()));
   connect(this, SIGNAL(selectionChanged()), this, SLOT(update4OnSelection()));
   connect(this, SIGNAL(listModified()), owner, SLOT(setTimeFeedback()));

}

QjuiListView::~QjuiListView()
{  
}
 
void QjuiListView::keyPressEvent(QKeyEvent * e )
{  
  if(e->key() == Qt::Key_Up || e->key() == Qt::Key_Down) {
    return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}
 


void QjuiListView::insertEntries(QguListItem *dropAt, QguListItem *item){

  // call for non-WL widgets to insert WL context as item
  // when creating entry from drag or copy off WL
  // if item is a folder, it gets ignored

  QjuiListItem * newItem;
  
  if(item->getEntryType() == ListTypeEntry){
    
    if(dropAt == 0 ) {
       newItem = new QjuiListItem(this, item->nameContext(), 
	                   item->getWLcontext(), ListTypeEntry);    
    }
    else if(dropAt->getEntryType() == ListTypeEntry){
      if( dropAt->parent() ){
        newItem = new QjuiListItem((QguListItem *)dropAt->parent(),item->nameContext(), 
	                   item->getWLcontext(), ListTypeEntry);
         if(newItem != NULL) newItem->moveItem(dropAt);
      }
      else{
        newItem = new QjuiListItem(this, item->nameContext(), 
	                   item->getWLcontext(), ListTypeEntry);          
      }
    } 
    else{
       newItem = new QjuiListItem(dropAt, item->nameContext(), 
	                   item->getWLcontext(), ListTypeEntry);
    }
    newItem->setChoices(-1);
  }

}

void QjuiListView::paste(){     

 if(QuSlotDebug())
 {
  printf("QjuiListView::paste %ld\n", this);
  fflush(stdout);
  }

  QuMsgBar(" ", false);

  QString text;
  
    // do you mean me?
//   if(!(this->hasFocus())){
// 	  std::cout << "emit pasteDetected" << std::endl;
//     emit pasteDetected();
//     return;
//   }

   QguListItem *dropAtItem = (QguListItem *)selectedItem();
   
#if QT_VERSION < 0x040000
    if ( QTextDrag::decode(QApplication::clipboard()->data(), text) )
#else
   if ( QApplication::clipboard()->mimeData()->hasText() )
#endif
    {
#if QT_VERSION >= 0x040000
	text=QApplication::clipboard()->mimeData()->text();
#endif
      	QguListItem * item = qsasWindow->WorkingList->getItemByName(text);
        if (item) insertEntries(dropAtItem, item);
	else QuMsgBar("failed to paste item", text.latin1(), false);
    }
    emit listModified();
    validateWLitems();
}


void QjuiListView::makeGroup(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::makeGroup %ld\n", this);
	fflush(stdout);
  }
 
  // this function is overloaded 
    
  QjuiListItem *newFolder;
  bool ok;

  // make new group item in folder or parent of selected item
    // if nothing is selected put at root (if allowed)
    if(localList.size() == 0){  
      if( this->rootIsLocked() )
        // root is locked, ie Working List, so there must be a first child
        newFolder = new QjuiListItem((QguListItem*)this->firstChild(), QString("Group"), QString(""), ListTypeGroup, 0);
      else{ 
        newFolder = new QjuiListItem(this, QString("Group"), QString(""), ListTypeGroup, 0);
      } 
            
      clearSelection();
      newFolder->setSelected(true);
      ok=reNameItem(newFolder);  // launch re-name text box  
      if(!ok)  delete newFolder;
      
      return;
    }
    
    
    // else something is selected, find where to put folder
    QguListItem *commonRoot = findCommonFolder();   

    if(commonRoot != NULL)
    {     
      newFolder = new QjuiListItem(commonRoot, QString("Group"), QString(""), ListTypeGroup);    
    }
    else if( !this->rootIsLocked() ){
      newFolder = new QjuiListItem(this, QString("Group"), QString(""), ListTypeGroup);    
    }
    else return;  // cannot find where to put folder that is allowed
        
    ok = reNameItem(newFolder);  // launch re-name text box
    if(!ok) {
      delete newFolder;
      return;
    }
    // move selected items to this new folder

    for (int i=0; i<localList.size(); i++){
       // move item from selection list
      QguListItem * movingItem = (QguListItem *)localList[i];
      QguListItem * parentItem = (QguListItem *)movingItem->parent();
	
	// move from old folder or listView
      if (parentItem == 0) this->takeItem(movingItem);
      else                 parentItem->takeItem(movingItem);
	
      newFolder->insertItem(movingItem);
	
    }
    clearSelectedList();
}

void QjuiListView::editWLLink(){
//   QguListItem *atItem = (QguListItem *)currentItem() ;  
//   if(atItem == NULL || atItem->getEntryType() == ListTypeGroup){
//     QuMsgBar("Folder cannot be linked to data object", true);
//     return;
//   }
  

 // This will launch editor with wild card for multiple selections
 
   reLinkItem();

}


void QjuiListView::setAlgorithm(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::setAlgorithm %ld\n", this);
	fflush(stdout);
 }
 
 
  setItemChoices(0);
}

void QjuiListView::setWidthField(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::setWidthField %ld\n", this);
	fflush(stdout);
 }
 
 
  if(  this->owner->_width_field->edited()) setItemChoices(1);
}

void QjuiListView::setGap(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::setGap %ld\n", this);
	fflush(stdout);
 }
 
 
  setItemChoices(2);
}

void QjuiListView::setGapField(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::setGapField %ld\n", this);
	fflush(stdout);
 }
 
 
  if(  this->owner->_gap_field->edited()) setItemChoices(3);
}

void QjuiListView::setItemChoices(int w)
{
    
    if ( !(this->firstChild()) ) {
      owner->status("Drag items to join list", 10000);
      return; 
    }
    

    static bool dontShow = false;
    QuMsgBar(" ", false);
    
    // if nothing is selected in this list  
    if (localList.size() == 0 ||
       ((Q_LIST_VIEW_ITEM *)localList[0])->listView() != this) {
       if(  this->firstChild() != NULL) {
         if( dontShow ) {
	   QuMsgBar("No items selected", true);
           return;
	 }
         else{
	   switch( QMessageBox::information( this, "Settings",
                               "No items selected",
                               "Apply to All", "Don't Show Again", "Cancel") ) 
           {
           case 0:
	     owner->keepChoices();
	     selectAll();
	     owner->applyChoices();
	     return;   
           case 1:
             dontShow = true;
	     return;
	   case 2:
	     return;
           }
	 }
       }
    }
    
    // something selected so just apply new choice
    for (int i=0; i<localList.size(); i++){
      if(localList[i]->getEntryType() == ListTypeEntry){
        ((QjuiListItem *)localList[i])->setChoices(w);
      }
    }
    
}

void QjuiListView::update1OnSelection(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::update1OnSelection %ld\n", this);
	fflush(stdout);
 }
 
 QjuiListItem * item;
 
 if(this->owner->_algorithm->count() == 2)
       this->owner->_algorithm->insertItem(" ",2);
 this->owner->_algorithm->setCurrentItem(2);
 if (localList.size() == 0 ) return;
 
 int I_alg_0 = -1;
 
 for (int i=0; i<localList.size(); i++){
   item = dynamic_cast<QjuiListItem*> (localList[i]);
   if (item == 0 || (QjuiListView *)item->listView() != this) return;
   if (item->getEntryType() != ListTypeEntry) continue;
   if (I_alg_0 == -1) I_alg_0 = item->I_algorithm;
   if (item->I_algorithm != I_alg_0) return;
 }
 
 this->owner->_algorithm->setCurrentItem(I_alg_0);
 
 if(this->owner->_algorithm->count() > 2)
       this->owner->_algorithm->removeItem(2);
       
}

void QjuiListView::update2OnSelection(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::update2OnSelection %ld\n", this);
	fflush(stdout);
 }
 
 QjuiListItem * item;

 if(this->owner->_gap->count() == 5)
       this->owner->_gap->insertItem(" ",5);
 this->owner->_gap->setCurrentItem(5);
 if (localList.size() == 0 ) return;
 
 int I_gap_0 = -1;
 for (int i=0; i<localList.size(); i++){
   item = dynamic_cast<QjuiListItem*> (localList[i]);
   if (item == 0 || (QjuiListView *)item->listView() != this) return;
   if (item->getEntryType() != ListTypeEntry) continue;
   if (I_gap_0 == -1) I_gap_0 = item->I_gap;
   if (item->I_gap != I_gap_0) return;
 }
 this->owner->_gap->setCurrentItem(I_gap_0);
 if(this->owner->_gap->count() > 5)
       this->owner->_gap->removeItem(5);
 
}
void QjuiListView::update3OnSelection(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::update3OnSelection %ld\n", this);
	fflush(stdout);
 }
 
 QjuiListItem * item;
 
 this->owner->_width_field->setText("");
 if (localList.size() == 0 ) return;
 
 QString I_width_field_0 = QString("");
 
 for (int i=0; i<localList.size(); i++){
   item = dynamic_cast<QjuiListItem*> (localList[i]);
   if (item == 0 || (QjuiListView *)item->listView() != this) return;
   if (item->getEntryType() != ListTypeEntry) continue;
   if (I_width_field_0 == "") I_width_field_0 = QString(item->I_width_field);
   if (item->I_width_field != I_width_field_0) return;
 }
 this->owner->_width_field->setText(I_width_field_0);

}
 
void QjuiListView::update4OnSelection(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::update4OnSelection %ld\n", this);
	fflush(stdout);
 }
 
 QjuiListItem * item;
 
 this->owner->_gap_field->setText("");
 if (localList.size() == 0 ) return;
 
 QString I_gap_field_0 = QString("");
 
 for (int i=0; i<localList.size(); i++){
   item = dynamic_cast<QjuiListItem*> (localList[i]);
   if (item == 0 || (QjuiListView *)item->listView() != this) return;
   if (item->getEntryType() != ListTypeEntry) continue;
   if (I_gap_field_0 == "") I_gap_field_0 = QString(item->I_gap_field);
   if (item->I_gap_field != I_gap_field_0) return;
 }
 this->owner->_gap_field->setText(I_gap_field_0);

}

void QjuiListView::updateOnSelection(){

 if(QuSlotDebug())
 {
	printf("QjuiListView::updateOnSelection %ld\n", this);
	fflush(stdout);
 }
 
 QguListItem * item;
 
 if (localList.size() == 0 ) return;
 
 QString WLcontxt_0 = QString("");
 
 for (int i=0; i<localList.size(); i++){
   item = localList[i];
   if (item == 0 || (QjuiListView *)item->listView() != this) return;
   if (item->getEntryType() != ListTypeEntry) continue;
   if (WLcontxt_0 == "") WLcontxt_0 = QString(item->WLcontext);
   if (item->WLcontext != WLcontxt_0) return;
 }
 
}


// JOIN Window
//---------------
#if QT_VERSION < 0x040000
QjuiWindow::QjuiWindow( QWidget *parent, const char *name ) 
  : QguWindow(parent,name)
{
  this->setPalette(qApp->palette());

   setAcceptDrops(true);
   setFocusPolicy(WHEEL_FOCUS);

  verbose = false;
  
   _top_widget = new QWidget(this,"top_widget");
   setCentralWidget( _top_widget );
   _top_widget->setPalette(qsasWindow->palette());

   _top_gl = new QGridLayout( _top_widget, 3, 1 );

   _top_gl->setSpacing(2);
   _top_gl->setMargin(2);

  // splitter to manage database list view and data selection widget

   _top_splitter = new QSplitter( _top_widget );
   _top_gl->addWidget( _top_splitter, 0,0 );
  
//  list view of objects to export

  _join_listview = new QjuiListView( _top_splitter, this, "Join List");
  _join_listview->setRootIsDecorated(true);
  _join_listview->addColumn("Object List", 350);
  _join_listview->setMinimumWidth(350);

  connect(_join_listview, SIGNAL(viewWL_ObjRequest()), qsasWindow->WorkingList, SIGNAL(viewObjRequest()));
  
//   Q_SCROLL_VIEW * scrollW = new Q_SCROLL_VIEW(_top_splitter);

  QGroupBox* opts_groupbox =  
    new QGroupBox( 1, Qt::Horizontal, _top_splitter/*scrollW*/, "opts_groupbox");

  opts_groupbox->setPalette(qsasWindow->palette());
  
  QGroupBox* join_groupbox =
    new QGroupBox( 1, Qt::Vertical, opts_groupbox, "join_groupbox");

  join_groupbox->setTitle("Join Settings for Selected Items");

  QWidget* join_widget =
     new QWidget(join_groupbox,"join_widget");

  QGridLayout* join_layout = new QGridLayout( join_widget, 5, 2);
  join_layout->setSpacing(2);
  join_layout->setColStretch(1,10);

  QLabel* algorithm_label = new QLabel("Join Algorithm ",join_widget,"algorithm_label");
  join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );

  _algorithm = new QComboBox(false,join_widget,"algorithm");
  _algorithm->insertItem(LINEAR, -1 );
  _algorithm->insertItem(BOXCAR, -1 );
  join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
  connect(_algorithm, SIGNAL(activated(int)), _join_listview, SLOT(setAlgorithm()));
	 
  QLabel* width_label = new QLabel("Ave width (sec) ",join_widget,"width_label");
  join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );

  _width_field = new QguWLinput( join_widget, "width_field" );  
  _width_field->setMinimumWidth(150);
  _width_field->setMaxLength(250);
  _width_field->acceptNumber = TRUE;
   connect(_width_field, SIGNAL(textChanged(const QString &)), _join_listview, SLOT(setWidthField()));
 QToolTip::add( _width_field,   "width in seconds of the boxcar");
  join_layout->addWidget( _width_field, 1, 1 );

  QLabel* blank_label = new QLabel(" ",join_widget,"blank_label");
  join_layout->addWidget( blank_label, 2, 0, Qt::AlignRight );

  QLabel* blank2_label = new QLabel(" ",join_widget,"blank2_label");
  join_layout->addWidget( blank2_label, 2, 1, Qt::AlignRight );

  QLabel* gap_label = new QLabel("Gap Handling ",join_widget,"gap_label");
  join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );

  _gap = new QComboBox(false,join_widget,"gap");
  _gap->insertItem(LINEAR_INTERP_GAP, -1 );
  _gap->insertItem(ZERO_FILL_GAP, -1 );
  _gap->insertItem(FILL_GAP, -1 );
  _gap->insertItem(NEAREST_NEIGHBOUR_GAP, -1 );
  _gap->insertItem(REMOVE_GAP, -1 );
  connect(_gap, SIGNAL(activated(int)), _join_listview, SLOT(setGap()));
  join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );

  QLabel* gapSize_label = new QLabel("Gap threshold (sec) ",join_widget,"gapSize_label");
  join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight );

  _gap_field = new QguWLinput( join_widget, "gap_field" );  
  _gap_field->setMinimumWidth(150);
  _gap_field->setMaxLength(100);
  _gap_field->acceptNumber = TRUE;
   connect(_gap_field, SIGNAL(textChanged(const QString &)), _join_listview, SLOT(setGapField()));
  QToolTip::add( _gap_field, \
    "minimum time interval in seconds to be treated as a gap if empty");
  join_layout->addWidget( _gap_field, 4, 1 );

  QGroupBox* time_groupbox =
    new QGroupBox( 1, Qt::Vertical, opts_groupbox, "time_groupbox");

  time_groupbox->setTitle("Time Source");

  QWidget* time_widget =
     new QWidget(time_groupbox,"time_widget");

  QGridLayout* time_layout = new QGridLayout( time_widget, 5, 2);
  time_layout->setSpacing(4);
  time_layout->setColStretch(1,10);

  QLabel* TimeInterval_label = new QLabel("Time Interval ",time_widget,"TimeInterval_label");
  time_layout->addWidget( TimeInterval_label, 0, 0, Qt::AlignRight );

  _TimeIvlSelect = new QComboBox(false,time_widget,"TimeIvlSelect");
  _TimeIvlSelect->insertItem("from Data", 0 );
  _TimeIvlSelect->insertItem("from Object", 1 );
  time_layout->addWidget( _TimeIvlSelect, 0, 1, Qt::AlignLeft );
  connect(_TimeIvlSelect, SIGNAL(activated(int)), this, SLOT(setTTivl(int)));

  IvlSource_label = new QLabel("  ",time_widget,"IvlSource_label");
  time_layout->addWidget( IvlSource_label, 1, 0, Qt::AlignRight );

  _IvlSource = new QguWLinput( time_widget, "IvlSource" );  
  _IvlSource->setMinimumWidth(2050);
  _IvlSource->setMaxLength(100);
  _IvlSource->acceptTIvlObject = true;
  _IvlSource->active = false;

  QToolTip::add( _IvlSource,  "not active");
  time_layout->addWidget( _IvlSource, 1, 1, Qt::AlignLeft );

  connect(_join_listview, SIGNAL(pasteDetected()), _IvlSource, SLOT(paste()));
  connect(_join_listview, SIGNAL(raiseWindow()), this, SLOT(bringToFront()));
  connect(_IvlSource, SIGNAL(validated_OK()), this, SLOT(setTimeFeedback()));

   // space
  QLabel* Tblank1_label = new QLabel(" ",time_widget,"Tblank1_label");
  time_layout->addWidget( Tblank1_label, 2, 0, Qt::AlignRight );

  QLabel* Tblank2_label = new QLabel(" ",time_widget,"Tblank2_label");
  time_layout->addWidget( Tblank2_label, 2, 1, Qt::AlignRight );
  

  QLabel* TimeTags_label = new QLabel("Time Tags ",time_widget,"TimeTags_label");
  time_layout->addWidget( TimeTags_label, 3, 0, Qt::AlignRight );

  _TimeTagSelect = new QComboBox(false,time_widget,"TimeTagSelect");
  _TimeTagSelect->insertItem("Create Regular", 0 );
  _TimeTagSelect->insertItem("from Object", 1 );
  time_layout->addWidget( _TimeTagSelect, 3, 1, Qt::AlignLeft );
  connect(_TimeTagSelect, SIGNAL(activated(int)), this, SLOT(setTTsource(int)));

  TagSource_label = new QLabel("Time Tag Object ",time_widget,"TagSource_label");
  time_layout->addWidget( TagSource_label, 4, 0, Qt::AlignRight );

  _TagSource = new QguWLinput( time_widget, "TagSource" );  
  _TagSource->setMinimumWidth(250);
  _TagSource->setMaxLength(100);
  _TagSource->acceptTTagObject = true;
  connect(_join_listview, SIGNAL(pasteDetected()), _TagSource, SLOT(paste()));
  connect(_TagSource, SIGNAL(validated_OK()), this, SLOT(setTimeFeedback()));
  QToolTip::add( _TagSource, "drag and drop object here");
  time_layout->addWidget( _TagSource, 4, 1, Qt::AlignLeft );
  

 QGroupBox* feedback_groupbox =
    new QGroupBox( 1, Qt::Vertical, _top_widget, "feedback_groupbox");

  feedback_groupbox->setTitle("Time Tag Details");
   _top_gl->addWidget( feedback_groupbox, 1,0 );

  feedback_groupbox->addSpace(100);
  
  QWidget* feedback_widget =
     new QWidget(feedback_groupbox,"feedback_widget");

  QGridLayout* feedback_layout = new QGridLayout( feedback_widget, 2, 4);
  feedback_layout->setSpacing(4);
  feedback_layout->setColStretch(1,10);
  feedback_groupbox->addSpace(100);

  QLabel* Start_label = new QLabel("Start Time: ",feedback_widget,"Start_label");
  feedback_layout->addWidget( Start_label, 0, 0, Qt::AlignRight );
 
  Start_field = new QLabel("<Not yet set>",feedback_widget,"Start_field");
  feedback_layout->addWidget( Start_field, 0, 1, Qt::AlignLeft );
 
  QLabel* End_label = new QLabel("End Time: ",feedback_widget,"End_label");
  feedback_layout->addWidget( End_label, 1, 0, Qt::AlignRight );
  
  End_field = new QLabel("<Not yet set>",feedback_widget,"End_field");
  feedback_layout->addWidget( End_field, 1, 1, Qt::AlignLeft );
 
  QLabel* Ntags_label = new QLabel("     Number of Time Tags in Interval: ",feedback_widget,"Ntags_label");
  feedback_layout->addWidget( Ntags_label, 0, 2, Qt::AlignRight );
 
  Ntags_field = new QLabel("<Not yet set>",feedback_widget,"Ntags_field");
  feedback_layout->addWidget( Ntags_field, 0, 3, Qt::AlignLeft );
 
  QLabel* spacing_label = new QLabel("Time Tag Spacing (sec): ",feedback_widget,"spacing_label");
  feedback_layout->addWidget( spacing_label, 1, 2, Qt::AlignRight );
  
  spacing_field = new QLabel("<Not yet set>",feedback_widget,"spacing_field");
  feedback_layout->addWidget( spacing_field, 1, 3, Qt::AlignLeft );
 
  buttons_layout=new QHGroupBox(_top_widget, "buttons");
   _top_gl->addWidget( buttons_layout, 2,0 );
  QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);
  
  //  buttons added in JOIN or EXPORT create windows  
  
  _menu_bar->clear();

  QPopupMenu* file_menu = new QPopupMenu(this,"file_menu");
  QPopupMenu* edit_menu = new QPopupMenu(this,"edit_menu");


  file_menu->clear();
  file_menu->insertItem("&Save", this, SLOT(save()), Qt::CTRL+Qt::Key_S);
  file_menu->insertItem("Save &As...", this, SLOT(saveAs()), Qt::CTRL+Qt::Key_A);
  file_menu->insertSeparator();
  file_menu->insertItem("Include...", this, SLOT(include()));
  file_menu->insertItem("&Restore", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
  file_menu->insertSeparator();
  file_menu->insertItem("Use Default Settings", this, SLOT(defaultAll()));
  _menu_bar->insertItem("File", file_menu );
  
  edit_menu->clear();
  edit_menu->insertItem("&New Folder", _join_listview, SLOT( makeGroup() ), Qt::CTRL+Qt::Key_N );
  edit_menu->insertItem("&Re-Name Item", _join_listview, SLOT( renameSelectedItem()), Qt::CTRL+Qt::Key_R );
  edit_menu->insertItem("&Edit WL Links", _join_listview, SLOT( editWLLink()), Qt::CTRL+Qt::Key_E );
  edit_menu->insertSeparator();
  edit_menu->insertItem("&Copy", _join_listview, SLOT( copy() ), Qt::CTRL+Qt::Key_C );
  edit_menu->insertItem("Paste", _join_listview, SLOT(paste()), Qt::CTRL+Qt::Key_V);
  edit_menu->insertItem("select &All", _join_listview, SLOT( selectAll() ), Qt::CTRL+Qt::Key_A );
  edit_menu->insertSeparator();
  edit_menu->insertItem("Copy Settings <-",this,SLOT(keepChoices()), Qt::CTRL+Qt::Key_Left);
  edit_menu->insertItem("Paste Settings ->",this,SLOT(applyChoices()), Qt::CTRL+Qt::Key_Right);
  edit_menu->insertSeparator();
  edit_menu->insertItem("Delete Item", _join_listview, SLOT(deleteListItems()), Qt::CTRL+Qt::Key_X);
  edit_menu->insertItem("Clear List", this, SLOT(deleteAllItems()));
  _menu_bar->insertItem("Edit", edit_menu );

  options_menu = new QPopupMenu(this,"options_menu");

  options_menu->setCheckable(true);
  options_menu->clear();
  toggleID = options_menu->insertItem("Verbose",this,SLOT(verboseToggle()));
  _menu_bar->insertItem("Options", options_menu );

  _menu_bar->insertSeparator();

  QPopupMenu * help = new QPopupMenu( this, "help_menu" );

  _menu_bar->insertItem( " Help", help );
  help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
  help->insertSeparator();
  help->insertItem("...Data Export", this, SLOT( launchManExport() ));
  help->insertItem("...Join", this, SLOT( launchManJoin() ));
  help->insertSeparator();
  help->insertItem("Module Version", this, SLOT( launchVersion() ));

  status(" ");
  this->setMinimumSize(this->sizeHint());

  this->fromSession = false;

}
#else
QjuiWindow::QjuiWindow(QWidget* parent, const char * name)
	:QguWindow(parent, name)
{
	this->setPalette(qApp->palette());
	setAcceptDrops(true);
	setFocusPolicy(WHEEL_FOCUS);
	
	verbose=false;

	//// Body of the window
	
	_top_widget=new QWidget(this);
	
	QBoxLayout * main_layout=new QBoxLayout(QBoxLayout::TopToBottom, _top_widget);
	main_layout->setSpacing(2);
	main_layout->setMargin(2);
	
	//// SPLITTER
	
	_top_splitter=new QSplitter(_top_widget);
	
	//// LIST VIEW
	
	_join_listview=new QjuiListView(_top_splitter, this, "Join List");
	_join_listview->setRootIsDecorated(true);
	_join_listview->addColumn("Object List", 350);
	
	connect(_join_listview, SIGNAL(viewWL_ObjRequest()), qsasWindow->WorkingList, SIGNAL(viewObjRequest()));
	

	_top_splitter->addWidget(_join_listview);

	
	//// Options panel
	
	QWidget * opts_widget=new QWidget(_top_splitter);

	QBoxLayout * opts_layout=new QBoxLayout(QBoxLayout::TopToBottom, opts_widget);

	//// Join option
	
	QGroupBox * join_widget=new QGroupBox("Join Settings for Selected Items", opts_widget);
	
	QGridLayout * join_layout=new QGridLayout(join_widget);
	join_layout->setSpacing(2);
	join_layout->setColStretch(1, 10);
	
	QLabel* algorithm_label = new QLabel("Join Algorithm ",join_widget,"algorithm_label");
	join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
		
	_algorithm = new QComboBox(false,join_widget,"algorithm");
	_algorithm->insertItem(LINEAR, -1 );
	_algorithm->insertItem(BOXCAR, -1 );
	join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
	connect(_algorithm, SIGNAL(activated(int)), _join_listview, SLOT(setAlgorithm()));
			
	QLabel* width_label = new QLabel("Ave width (sec) ",join_widget,"width_label");
	join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );
		
	_width_field = new QguWLinput( join_widget, "width_field" );  
	_width_field->acceptNumber = TRUE;
	connect(_width_field, SIGNAL(textChanged(const QString &)), _join_listview, SLOT(setWidthField()));
	QToolTip::add( _width_field, \
	"width in seconds of the boxcar");
	join_layout->addWidget( _width_field, 1, 1 );
		
	QLabel* blank_label = new QLabel(" ",join_widget,"blank_label");
	join_layout->addWidget( blank_label, 2, 0, Qt::AlignRight );
		
	QLabel* blank2_label = new QLabel(" ",join_widget,"blank2_label");
	join_layout->addWidget( blank2_label, 2, 1, Qt::AlignRight );
		
	QLabel* gap_label = new QLabel("Gap Handling ",join_widget,"gap_label");
	join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );
		
	_gap = new QComboBox(false,join_widget,"gap");
	_gap->insertItem(LINEAR_INTERP_GAP, -1 );
	_gap->insertItem(ZERO_FILL_GAP, -1 );
	_gap->insertItem(FILL_GAP, -1 );
	_gap->insertItem(NEAREST_NEIGHBOUR_GAP, -1 );
	_gap->insertItem(REMOVE_GAP, -1 );
	connect(_gap, SIGNAL(activated(int)), _join_listview, SLOT(setGap()));
	join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );
		
	QLabel* gapSize_label = new QLabel("Gap threshold (sec) ",join_widget,"gapSize_label");
	join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight );
		
	_gap_field = new QguWLinput( join_widget, "gap_field" );  
	_gap_field->acceptNumber = TRUE;
	connect(_gap_field, SIGNAL(textChanged(const QString &)), _join_listview, SLOT(setGapField()));
	QToolTip::add( _gap_field, \
	"minimum time interval in seconds to be treated as a gap if empty");
	join_layout->addWidget( _gap_field, 4, 1 );

	opts_layout->addWidget(join_widget);
	
	//// Time option
	
	QGroupBox * time_widget=new QGroupBox("Time Source", opts_widget);
	
	QGridLayout * time_layout=new QGridLayout(time_widget);
	time_layout->setSpacing(2);
	time_layout->setColStretch(1, 10);
	
	 QLabel* TimeInterval_label = new QLabel("Time Interval ",time_widget,"TimeInterval_label");
	time_layout->addWidget( TimeInterval_label, 0, 0, Qt::AlignRight );
		
	_TimeIvlSelect = new QComboBox(false,time_widget,"TimeIvlSelect");
	_TimeIvlSelect->insertItem("from Data", 0 );
	_TimeIvlSelect->insertItem("from Object", 1 );
	time_layout->addWidget( _TimeIvlSelect, 0, 1, Qt::AlignLeft );
	connect(_TimeIvlSelect, SIGNAL(activated(int)), this, SLOT(setTTivl(int)));
		
	IvlSource_label = new QLabel("  ",time_widget,"IvlSource_label");
	time_layout->addWidget( IvlSource_label, 1, 0, Qt::AlignRight );
		
	_IvlSource = new QguWLinput( time_widget, "IvlSource" );  
 	_IvlSource->setMinimumWidth(200);
 	_IvlSource->setMaxLength(100);
	_IvlSource->acceptTIvlObject = true;
	_IvlSource->active = false;
		
	QToolTip::add( _IvlSource, "not active");
	time_layout->addWidget( _IvlSource, 1, 1, Qt::AlignLeft );
	
	connect(_join_listview, SIGNAL(pasteDetected()), _IvlSource, SLOT(paste()));
	connect(_join_listview, SIGNAL(raiseWindow()), this, SLOT(bringToFront()));
	connect(_IvlSource, SIGNAL(validated_OK()), this, SLOT(setTimeFeedback()));

	time_layout->addWidget(new QLabel("", time_widget, "blank"), 2, 0); //space
	
	QLabel* TimeTags_label = new QLabel("Time Tags ",time_widget,"TimeTags_label");
	time_layout->addWidget( TimeTags_label, 3, 0, Qt::AlignRight );
		
	_TimeTagSelect = new QComboBox(false,time_widget,"TimeTagSelect");
	_TimeTagSelect->insertItem("Create Regular", 0 );
	_TimeTagSelect->insertItem("from Object", 1 );
	time_layout->addWidget( _TimeTagSelect, 3, 1, Qt::AlignLeft );
	connect(_TimeTagSelect, SIGNAL(activated(int)), this, SLOT(setTTsource(int)));
		
	TagSource_label = new QLabel("Time Tag Object ",time_widget,"TagSource_label");
	time_layout->addWidget( TagSource_label, 4, 0, Qt::AlignRight );
		
	_TagSource = new QguWLinput( time_widget, "TagSource" );  
 	_TagSource->setMinimumWidth(200);
 	_TagSource->setMaxLength(100);
	_TagSource->acceptTTagObject = true;
	connect(_join_listview, SIGNAL(pasteDetected()), _TagSource, SLOT(paste()));
	connect(_TagSource, SIGNAL(validated_OK()), this, SLOT(setTimeFeedback()));
	QToolTip::add( _TagSource, "drag and drop object here");
	time_layout->addWidget( _TagSource, 4, 1, Qt::AlignLeft );

	opts_layout->addWidget(time_widget);

	opts_widget->adjustSize();
	
	_top_splitter->addWidget(opts_widget);
	_top_splitter->adjustSize();
	
	main_layout->addWidget(_top_splitter);
	
	//// FEEDBACK
	
	QGroupBox * feedback_widget=new QGroupBox("Time Tag Details", _top_widget);
	QGridLayout * feedback_layout=new QGridLayout(feedback_widget);
	feedback_layout->setSpacing(2);
	feedback_layout->setColStretch(1, 10);
	
	QLabel* Start_label = new QLabel("Start Time: ",feedback_widget,"Start_label");
	feedback_layout->addWidget( Start_label, 0, 0, Qt::AlignRight );

	Start_field = new QLabel("<Not yet set>",feedback_widget,"Start_field");
	feedback_layout->addWidget( Start_field, 0, 1, Qt::AlignLeft );

	QLabel* End_label = new QLabel("End Time: ",feedback_widget,"End_label");
	feedback_layout->addWidget( End_label, 1, 0, Qt::AlignRight );

	End_field = new QLabel("<Not yet set>",feedback_widget,"End_field");
	feedback_layout->addWidget( End_field, 1, 1, Qt::AlignLeft );

	QLabel* Ntags_label = new QLabel("     Number of Time Tags in Interval: ",feedback_widget,"Ntags_label");
	feedback_layout->addWidget( Ntags_label, 0, 2, Qt::AlignRight );

	Ntags_field = new QLabel("<Not yet set>",feedback_widget,"Ntags_field");
	feedback_layout->addWidget( Ntags_field, 0, 3, Qt::AlignLeft );

	QLabel* spacing_label = new QLabel("Time Tag Spacing (sec): ",feedback_widget,"spacing_label");
	feedback_layout->addWidget( spacing_label, 1, 2, Qt::AlignRight );

	spacing_field = new QLabel("<Not yet set>",feedback_widget,"spacing_field");
	feedback_layout->addWidget( spacing_field, 1, 3, Qt::AlignLeft );

	main_layout->addWidget(feedback_widget);
	//// BUTTONS
	
	_buttons_widget=new QWidget(_top_widget);
	buttons_layout=new QBoxLayout(QBoxLayout::LeftToRight, _buttons_widget);

	buttons_layout->setSpacing(2);
	
	//  buttons added in JOIN or EXPORT create windows  
	
	main_layout->addWidget(_buttons_widget);
	
	setCentralWidget(_top_widget);
	
		//// Menus
	
	_menu_bar->clear();
	
	QMenu* file_menu = new QMenu(this);
	QMenu* edit_menu = new QMenu(this);

		
	file_menu->clear();
	file_menu->insertItem("&Save", this, SLOT(save()), Qt::CTRL+Qt::Key_S);
	file_menu->insertItem("Save &As...", this, SLOT(saveAs()), Qt::CTRL+Qt::Key_A);
	file_menu->insertSeparator();
	file_menu->insertItem("Include...", this, SLOT(include()));
	file_menu->insertItem("&Restore", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
	file_menu->insertSeparator();
	file_menu->insertItem("Use Default Settings", this, SLOT(defaultAll()));
	_menu_bar->insertItem("File", file_menu );
		
	edit_menu->clear();
	edit_menu->insertItem("&New Folder", _join_listview, SLOT( makeGroup() ), Qt::CTRL+Qt::Key_N );
	edit_menu->insertItem("&Re-Name Item", _join_listview, SLOT( renameSelectedItem()), Qt::CTRL+Qt::Key_R );
	edit_menu->insertItem("&Edit WL Links", _join_listview, SLOT( editWLLink()), Qt::CTRL+Qt::Key_E );
	edit_menu->insertSeparator();
	edit_menu->insertItem("&Copy", _join_listview, SLOT( copy() ), Qt::CTRL+Qt::Key_C );
	edit_menu->insertItem("Paste", _join_listview, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	edit_menu->insertItem("select &All", _join_listview, SLOT( selectAll() ), Qt::CTRL+Qt::Key_A );
	edit_menu->insertSeparator();
	edit_menu->insertItem("Copy Settings <-",this,SLOT(keepChoices()), Qt::CTRL+Qt::Key_Left);
	edit_menu->insertItem("Paste Settings ->",this,SLOT(applyChoices()), Qt::CTRL+Qt::Key_Right);
	edit_menu->insertSeparator();
	edit_menu->insertItem("Delete Item", _join_listview, SLOT(deleteListItems()), Qt::CTRL+Qt::Key_X);
	edit_menu->insertItem("Clear List", this, SLOT(deleteAllItems()));
	_menu_bar->insertItem("Edit", edit_menu );
		
	options_menu = new QMenu(this);

	options_menu->setCheckable(true);
	options_menu->clear();
	toggleID = options_menu->insertItem("Verbose",this,SLOT(verboseToggle()));
	_menu_bar->insertItem("Options", options_menu );
		
	_menu_bar->insertSeparator();

	QMenu * help = new QMenu( this );

	_menu_bar->insertItem( " Help", help );
	help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->insertSeparator();
	help->insertItem("...Data Export", this, SLOT( launchManExport() ));
	help->insertItem("...Join", this, SLOT( launchManJoin() ));
	help->insertSeparator();
	help->insertItem("Module Version", this, SLOT( launchVersion() ));
	
	
	adjustSize();
	
	status(" ");
}
#endif


void QjuiWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManual %ld\n", this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

void QjuiWindow::launchManExport(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManExport %ld\n", this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

void QjuiWindow::launchManJoin(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManJoin %ld\n", this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_analysis.html");
}

void QjuiWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("DirectImportWindow::launchVersion %ld\n", this);
	fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Join/Export  ";
  text += QString(QJUI_VERSION);
  QMessageBox::information( this, "QSAS information", text.latin1(), "OK" ); 

}

#if QT_VERSION < 0x040000
void QJUImakeWindow( QWidget *parent, const char * Title )
{
	if(QJUI_widget==NULL)
	{
		QJUI_widget = QJUICreatePopup(parent, Title);
		QJUI_widget->setCaption(Title);
		QJUI_widget->JoinBtn = new QPushButton( "Join", QJUI_widget->buttons_layout, "JoinBtn" );
		QJUI_widget->JoinBtn->setMinimumSize(QJUI_widget->JoinBtn->sizeHint());
		QJUI_widget->connect( QJUI_widget->JoinBtn, SIGNAL(clicked()), QJUI_widget, SLOT(DoJoin()) );
		QJUI_widget->buttons_layout->addSpace(0);
		QJUI_widget->dismissBtn = new QPushButton( "Dismiss", QJUI_widget->buttons_layout, "dismissBtn" );
		QJUI_widget->dismissBtn->setMinimumSize(QJUI_widget->dismissBtn->sizeHint());
		QJUI_widget->connect( QJUI_widget->dismissBtn, SIGNAL(clicked()), QJUI_widget, SLOT(close()) );
  
		QJUI_widget->_filename += QString("/saveSet");
		QJUI_widget->SAVE_EXTENSION = QString(".qs_join");
		QJUI_widget->_filename += QJUI_widget->SAVE_EXTENSION;
		QJUI_widget->_TimeTagSelect->setCurrentItem(1);
		QJUI_widget->_TimeTagSelect_Def = 1;
		QJUI_widget->_TimeTagSelect_cpy = 1;
		QJUI_widget->_saveset_filter=QString("Join savesets (*.qs_join)");

	}
	return;
 
} 
#else
void QJUImakeWindow( QWidget *parent, const char * Title )
{
  if(QJUI_widget==NULL)
  {
     QJUI_widget = QJUICreatePopup(parent, Title);
     QJUI_widget->setCaption(Title);

     QJUI_widget->JoinBtn = new QPushButton( "Join", QJUI_widget->_buttons_widget, "JoinBtn" );
//      QJUI_widget->JoinBtn->setMinimumSize(QJUI_widget->JoinBtn->sizeHint());
     QJUI_widget->buttons_layout->addWidget(QJUI_widget->JoinBtn);
     QJUI_widget->connect( QJUI_widget->JoinBtn, SIGNAL(clicked()), QJUI_widget, SLOT(DoJoin()) );
     //QJUI_widget->buttons_layout->addSpace(0);
     QJUI_widget->dismissBtn = new QPushButton( "Dismiss", QJUI_widget->_buttons_widget, "dismissBtn" );
//      QJUI_widget->dismissBtn->setMinimumSize(QJUI_widget->dismissBtn->sizeHint());
     QJUI_widget->buttons_layout->addWidget(QJUI_widget->dismissBtn);
     QJUI_widget->connect( QJUI_widget->dismissBtn, SIGNAL(clicked()), QJUI_widget, SLOT(close()) );
  
     QJUI_widget->_filename += QString("/saveSet");
     QJUI_widget->SAVE_EXTENSION = QString(".qs_join");
     QJUI_widget->_filename += QJUI_widget->SAVE_EXTENSION;
     QJUI_widget->_TimeTagSelect->setCurrentItem(1);
     QJUI_widget->_TimeTagSelect_Def = 1;
     QJUI_widget->_TimeTagSelect_cpy = 1;
	QJUI_widget->_saveset_filter=QString("Join savesets (*.qs_join)");
	QJUI_widget->adjustSize();

  }
 return;
 
}
#endif 

#if QT_VERSION < 0x040000
void QEXPmakeWindow( QWidget *parent, const char * Title )
{
	if(QJEX_widget==NULL)
	{
		QJEX_widget = QJUICreatePopup(parent, Title);
		QJEX_widget->setCaption(Title);
		QJEX_widget->WriteBtn = new QPushButton( "Export...", QJEX_widget->buttons_layout, "WriteBtn" );
		QJEX_widget->WriteBtn->setMinimumSize(QJEX_widget->WriteBtn->sizeHint());
		QJEX_widget->connect( QJEX_widget->WriteBtn, SIGNAL(clicked()),  QJEX_widget, SLOT(writeFile()) );
		QJEX_widget->buttons_layout->addSpace(0);
		QJEX_widget->dismissBtn = new QPushButton( "Dismiss", QJEX_widget->buttons_layout, "dismissBtn" );
		QJEX_widget->dismissBtn->setMinimumSize(QJEX_widget->dismissBtn->sizeHint());
		QJEX_widget->connect( QJEX_widget->dismissBtn, SIGNAL(clicked()), QJEX_widget, SLOT(close()) );

		QJEX_widget->_filename += QString("/saveSet");
		QJEX_widget->SAVE_EXTENSION = QString(".qs_exp");
		QJEX_widget->_filename += QJEX_widget->SAVE_EXTENSION;
		QJEX_widget->_TimeTagSelect->setCurrentItem(1);
		QJEX_widget->_TimeTagSelect_Def = 1;     
		QJEX_widget->_TimeTagSelect_cpy = 1;
		QJEX_widget->_TagSource->setText("Leave Empty if already joined");
		QJEX_widget->_saveset_filter=QString("Export savesets (*.qs_exp)");
	}

	return;
}
#else
void QEXPmakeWindow( QWidget *parent, const char * Title )
{
  if(QJEX_widget==NULL)
  {
     QJEX_widget = QJUICreatePopup(parent, Title);
     QJEX_widget->setCaption(Title);
     QJEX_widget->WriteBtn = new QPushButton( "Export...", QJEX_widget->_buttons_widget, "WriteBtn" );
     QJEX_widget->WriteBtn->setMinimumSize(QJEX_widget->WriteBtn->sizeHint());
     QJEX_widget->connect( QJEX_widget->WriteBtn, SIGNAL(clicked()),  QJEX_widget, SLOT(writeFile()) );
     QJEX_widget->buttons_layout->addWidget(QJEX_widget->WriteBtn);
     //QJEX_widget->buttons_layout->addSpace(0);
     QJEX_widget->dismissBtn = new QPushButton( "Dismiss", QJEX_widget->_buttons_widget, "dismissBtn" );
     QJEX_widget->dismissBtn->setMinimumSize(QJEX_widget->dismissBtn->sizeHint());
     QJEX_widget->connect( QJEX_widget->dismissBtn, SIGNAL(clicked()), QJEX_widget, SLOT(close()) );
     QJEX_widget->buttons_layout->addWidget(QJEX_widget->dismissBtn);

     QJEX_widget->_filename += QString("/saveSet");
     QJEX_widget->SAVE_EXTENSION = QString(".qs_exp");
     QJEX_widget->_filename += QJEX_widget->SAVE_EXTENSION;
     QJEX_widget->_TimeTagSelect->setCurrentItem(1);
     QJEX_widget->_TimeTagSelect_Def = 1;     
     QJEX_widget->_TimeTagSelect_cpy = 1;
     QJEX_widget->_TagSource->setText("Leave Empty if already joined");
	QJEX_widget->_saveset_filter=QString("Export savesets (*.qs_exp)");
	QJEX_widget->adjustSize();
  }

  return;
}
#endif

void QjuiShowExport(){
  QSize qsasGUISize = qsasWindow->frameSize();
  QJEX_widget->move(qsasGUISize.width(),45);
  QJEX_widget->show();
}

void QjuiShowJoin(){  
  QSize qsasGUISize = qsasWindow->frameSize();
  QJUI_widget->move(qsasGUISize.width(),60);
  QJUI_widget->show();
}

QjuiWindow * QJUICreatePopup(QWidget *parent, const char * Title)
{
 QjuiWindow * Window = new QjuiWindow(parent,Title);
 Window->move(50,50);
 Window->setDefaults();
 Window->defaultAll();
 
 Window->connect(parent, SIGNAL(saveSessionRequest(QString)),  Window, SLOT(saveSession(QString)) );
 Window->connect(parent, SIGNAL(restoreSessionRequest(QString)),   Window, SLOT(restoreSession(QString)) );
 Window->connect(parent, SIGNAL(clearListRequest()),   Window, SLOT(deleteAllItems()) );
 return Window;
}

 QjuiWindow::~QjuiWindow()
{ 
     delete _IvlSource;
     delete _TagSource;
}

 void QjuiWindow::dragEnterEvent(QDragEnterEvent *event)
{ 
 if(QuSlotDebug())
 {
	printf("QjuiWindow::dragEnterEvent %ld\n", this);
	fflush(stdout);
 }
 
      this->setActiveWindow();
      this->raise();
}


void QjuiWindow::save(Vostream& v_os)
{
	string guiID;
	guiID=QSAS_VERSION_ID;
	v_os << guiID;   
	 
   int with = _join_listview->with;
   
   v_os << _algorithm->currentItem();
   
   _width_field->with = with;
   v_os << *_width_field;
   
   v_os << _gap->currentItem();
   
   _gap_field->with = with;
   v_os << *_gap_field;

   v_os << _TimeIvlSelect->currentItem();

   _IvlSource->with = with;
   v_os << *_IvlSource;

   v_os << _TimeTagSelect->currentItem();

   _TagSource->with = with;
   v_os << *_TagSource;

   v_os << *_join_listview;
}


void QjuiWindow::restore(Vistream& v_is, bool replace)
{
	string guiID; 
	v_is >> guiID;
	char *ptr;
   
	if( QuSlotDebug() ) printf("QjuiWindow::restore set ID =  %s\n---------------------------------\n", guiID.c_str());
  
	if( strstr(guiID.c_str(), "qjui") == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
	}
	else
	{
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}
  
  if(replace) deleteAllItems();

  
   v_is >> _algorithm_cpy;
   _algorithm->setCurrentItem(_algorithm_cpy);

   v_is >> *_width_field;
  
   v_is >> _gap_cpy;
   _gap->setCurrentItem(_gap_cpy);

   v_is >> *_gap_field;

   v_is >> _TimeIvlSelect_cpy;
   _TimeIvlSelect->setCurrentItem(_TimeIvlSelect_cpy);
   setTTivl(_TimeIvlSelect_cpy);
   
   v_is >> *_IvlSource;

   v_is >> _TimeTagSelect_cpy;
   _TimeTagSelect->setCurrentItem(_TimeTagSelect_cpy);
   setTTsource(_TimeTagSelect_cpy);

   v_is >> *_TagSource;

   v_is >> *_join_listview;



}


 void QjuiWindow::saveAs()
 {
 
 if(QuSlotDebug())
 {
	printf("QjuiWindow::saveAs %ld\n", this);
	fflush(stdout);
 }
 

   QString _filter = QString("save (*");
   _filter += SAVE_EXTENSION;
   _filter += QString(")");
   QString fn=QFileDialog::getSaveFileName(_filename, _filter, this, "save", "Select file - Save Join Settings");
   if(!fn.isEmpty())
   {
     int pos = fn.findRev(SAVE_EXTENSION);
     if (pos < 0) {
       pos = fn.findRev(".");
       if (pos > 0) fn.truncate(pos);
       fn += SAVE_EXTENSION;
     }
     _filename=fn;
     save();
   }
   else
     status("saving aborted ",10000);
 }
            
 void QjuiWindow::save()
 { 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::save %ld\n", this);
	fflush(stdout);
 }
 

    _join_listview->with = 0;
    if(_filename.isEmpty())
    {
       saveAs();
       return;
    }
    
    try
    {
      VXDRostream xdr_os(_filename.latin1());
      save(xdr_os);
    }
    catch(Exception &e)
    {
       status(e.msg().c_str(),10000);   
       return;
    }
    status(_filename+" saved",10000);
    return;
 }


void QjuiWindow::saveSession(QString folder){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::saveSession %ld\n", this);
	fflush(stdout);
 }
 

  if(folder.isEmpty()) return;
  // keep original name
  QString fileN = QString(_filename.latin1());
  
  // set session folder and save
     this->_filename = QString(folder.latin1());
     this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;
  save();

  // put file name back
  _filename = QString(fileN.latin1());
}


 void QjuiWindow::deleteAllItems(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::deleteAllItems %ld\n", this);
	fflush(stdout);
 }
 
 
   _join_listview->selectAll();    
   _join_listview->deleteListItems();    
}

 void QjuiWindow::verboseToggle(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::verboseToggle %ld\n", this);
	fflush(stdout);
 }
 
  if ( options_menu->isItemChecked(toggleID) ) 
     options_menu->setItemChecked(toggleID,false);
  else 
     options_menu->setItemChecked(toggleID,true);
  verbose = !verbose;
}
 
 void QjuiWindow::DoJoin(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::DoJoin %ld\n", this);
	fflush(stdout);
 }
 
 
  // keep settings
  keepChoices();
  
  QdObjectSeq_var JoinedList = new QdObjectSeq();
  vector<KVDataBase_var> OptionList; 

  if(alreadyJoined()){
       status("No target time tags or already joined",50000);
       return;
  }
  
  if( joinObjects(JoinedList, OptionList) ){

     status("multijoin finished...",10000);
    
     // put resulting objects on the Working List
     for (int jcount=0; jcount<JoinedList->size(); jcount++)  {

       // skip empty and failed slots quietly
    
       QdObject_var out_obj = JoinedList[jcount];
     
       if( out_obj.is_nil() ) continue;
     
      // Note Multijoin sets Xrefs and time tags on output objects

       // get NEW_NAME back from Options list
     
       QdString_var name_ptr =
         QdString_var::narrow((OptionList[jcount])->get(NEW_NAME));
       if( name_ptr.is_nil() ) {
         status("Failed to get new object name", 10000);
         continue;     
       }
     
        // put joined obj on W List
       try{
         QString joined_name = name_ptr->c_str();
         joined_name += "_J";
         int posn = joined_name.find("/");
         joined_name.remove(0,posn+1);
         QString root_name = name_ptr->c_str();
         root_name.truncate(posn);
         status("putting "+joined_name+" on working list",10000);
         QwuiPutOnWL(out_obj, joined_name.latin1(), root_name.latin1());

       }
       catch(Exception &e){
         status( QString(e.msg().c_str()), 10000 );
         return;
       }
    
      }  // end of: for jcount putting back on Working List
      status("JOIN Finished", 10000);
   }
  
  
} 

 bool QjuiWindow::alreadyJoined(){ 
  
  QjuiListItem *next = (QjuiListItem *) _join_listview->firstChild();
  
  if (next == NULL) return true;
  
  QdTimeSeq_var target_tt;
  target_tt = getTimeTags();
  if(target_tt.is_nil() ){
    // no target set, test against first item to allow export to work without joining
    QdObject_var firstObj = next->QguGetObject();
    target_tt = get_timetags(firstObj);
    if(target_tt.is_nil()) return false;  // no tt, give up, not joined
    next = (QjuiListItem *) next->QguGetNext();   
  }

  while(next){
    
    if (next->getEntryType() == ListTypeEntry ){
        
      QdObject_var obj = next->QguGetObject();
      if (obj.is_nil() ){
        return false;
      }
      if(next->I_algorithm == 1 || !QuIsJoined(obj, target_tt)) return false;  // not joined or boxcar method
    }
    next = (QjuiListItem *) next->QguGetNext();   
  }

  return true;
}


 bool QjuiWindow::joinObjects(QdObjectSeq_var JoinedList,
                              vector<KVDataBase_var> &OptionList){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::joinObjects %ld\n", this);
	fflush(stdout);
 }
 
  JoinedList->clear();
  QdTimeSeq_var target_tt;
  QdObjectSeq_var ToJoinList = new QdObjectSeq();
  QdRScalar_var fillval_ptr;
  
  QjuiListItem * next;
  

  // Get Time tags
     
  target_tt = getTimeTags();
  if(target_tt.is_nil() ){
    status("No Time Tags",50000);
    return false;
  }
     
  // if already
  
  // for each item on Join List, set choices
     
  next = (QjuiListItem *) _join_listview->firstChild();
  while(next){
    bool OK;
    if (next->getEntryType() == ListTypeEntry ){
        
      QdObject_var obj = next->QguGetObject();
      if (obj.is_nil() ){
        status("object no longer exists",10000);
        return true;
      }

      QString ctxt = next->getWLcontext();
      QguListItem * item = qsasWindow->WorkingList->getItemByName(ctxt);
      if (item == NULL) {
        status("cannot locate items name", 10000);
	return false;
      }

      if(disjointSeries(obj))
      {
        (void)QMessageBox::warning(this,"Join","disjoint series - "+ctxt);
	qApp->processEvents();
      }

      KVDataBase_var Join_options = new KVDataBase();

      // fill value

      if (obj->xref_exists("FILLVAL")) {
        QdObject_var fill_xref = obj->get_xref("FILLVAL");
        fillval_ptr=QdRScalar_var::narrow(fill_xref);
      }
      if( fillval_ptr.is_nil() )
          fillval_ptr = new QdRScalar((double) 0.1e-30);	 
      Join_options->set(FILL_VALUE, fillval_ptr); 
      
      // gap tolerance in seconds
      
      QdTime_var gap_ptr = new QdTime(Seconds(next->I_gap_field.toDouble(&OK))); 
      Join_options->set(GAP_TOLERANCE, gap_ptr);
      
      // boxcar width in seconds
      
      QdTime_var boxcar_ptr = new QdTime(Seconds(next->I_width_field.toDouble(&OK)));
      Join_options->set(BOXCAR_WIDTH, boxcar_ptr);
      
      // interpolation method
      
      QdString_var interp_ptr = new QdString( (_algorithm->text(next->I_algorithm)).latin1() );
      Join_options->set(INTERP_METHOD, interp_ptr);
      
     // gap handling option
     
      QdString_var gap_opt_ptr = new QdString( (_gap->text(next->I_gap)).latin1() );
      Join_options->set(GAP_METHOD, gap_opt_ptr);
           
      QString name = item->getWLcontext();
      
      QdString_var new_obj_name = new QdString(name.latin1());
      Join_options->set(NEW_NAME, new_obj_name);
                  
      ToJoinList->push_back(obj);
      OptionList.push_back(Join_options);   
    
    }
    next = (QjuiListItem *) next->QguGetNext();   
  }
  
  
  // Do actual join
     
  try{
    status("performing multijoin...", 10000);
    QuMultiJoin(ToJoinList, target_tt, JoinedList, OptionList);
    return true;
  }
  catch(Exception &e){
    status( QString(e.msg().c_str()), 10000);
    return false;
  }   


}

// ---------------------------------------------------

 void QjuiWindow::writeFile(){ 

 if(QuSlotDebug())
  {
	printf("QjuiWindow::writeFile %ld\n", this);
	fflush(stdout);
 }
 
bool  isTimeSeries = false;
 
   // keep settings for default behaviour
   keepChoices();
   
  QdObjectSeq_var JoinedList = new QdObjectSeq();
  vector<KVDataBase_var> OptionList; 
  bool OK;
  
   // test if already joined first
   
  if( alreadyJoined() ){

     // copy over inputs to outputs and set  name in options list
    
    isTimeSeries = true;
    
    JoinedList->clear();
    QjuiListItem *next = (QjuiListItem *) _join_listview->firstChild();
    while(next){
      if (next->getEntryType() == ListTypeEntry ){
        
        QdObject_var obj = next->QguGetObject();
        if (obj.is_nil() ){
          status("object no longer exists", 50000);
          return;
        }
  
        QString ctxt = next->getWLcontext();
	QguListItem * item = qsasWindow->WorkingList->getItemByName(ctxt);
        if(item == NULL){
          status("Cannot find items name", 50000);
	  return;
	}
  
        KVDataBase_var Join_options = new KVDataBase();

	QString name = item->getWLcontext();
        QdString_var new_obj_name = new QdString(name.latin1());
        Join_options->set(NEW_NAME, new_obj_name);
      
                  
        JoinedList->push_back(obj);
        OptionList.push_back(Join_options);   
      }

      next = (QjuiListItem *) next->QguGetNext();   
    }

  }
  else if(joinObjects(JoinedList, OptionList)) {     // try to multi-join first
    isTimeSeries = true;
    // join was successful, nothing more to do before assembling export QIE structure
    

  }
  else{
        // time tags not set: copy over inputs to outputs
        // for each item on Join List, set new name in options list
    JoinedList->clear();
    int sequence_length = -1;
    QjuiListItem *next = (QjuiListItem *) _join_listview->firstChild();
    while(next){
      if (next->getEntryType() == ListTypeEntry ){
        
        QdObject_var obj = next->QguGetObject();
        if (obj.is_nil() ){
          status("object no longer exists", 50000);
          return;
        }
        
	QdTimeSeq_var tt = get_timetags(obj);
	if(!tt.is_nil()) {
	  // should only be in this loop for data series or join failed
	  // since tags are present, join must have failed
          status("Unable to join to target time tags", 50000);
	  return;
	}
	
	if(sequence_length == -1) sequence_length = obj->sequence_size();
	else if (sequence_length != obj->sequence_size() ){
          status("Data sequences of differing lengths.", 50000);
	  return;	
	}
	
        QString ctxt = next->getWLcontext();
	QguListItem * item = qsasWindow->WorkingList->getItemByName(ctxt);
        if(item == NULL){
          status("Cannot find items name", 50000);
	  return;
	}
  
        KVDataBase_var Join_options = new KVDataBase();

	QString name = item->getWLcontext();
        QdString_var new_obj_name = new QdString(name.latin1());
        Join_options->set(NEW_NAME, new_obj_name);
      
                  
        JoinedList->push_back(obj);
        OptionList.push_back(Join_options);   
      }

      next = (QjuiListItem *) next->QguGetNext();   
    }
  
  }
  if(OptionList.size() == 0) return;
  if(JoinedList->size() == 0) return;

   //  Build QIE Export file object from QJUI join object and Write file

  QexBuildExportQIEObj (JoinedList, OptionList );

  OK = QexWriteFile (this, isTimeSeries);
    

    return;

}


 void QjuiWindow::defaultAll(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::defaultAll %ld\n", this);
	fflush(stdout);
 }
 
 
  _width_field->setText(_width_field_Def);
  _gap->setCurrentItem(_gap_Def);
  _gap_field->setText(_gap_field_Def);
  _algorithm->setCurrentItem(_algorithm_Def);
  _TimeIvlSelect->setCurrentItem(_TimeIvlSelect_Def);
  _IvlSource->setText(_IvlSource_Def);
  _TimeTagSelect->setCurrentItem(_TimeTagSelect_Def);
  _TagSource->setText(_TagSource_Def);
  _join_listview->setItemChoices(-1);

 
}

 void QjuiWindow::setDefaults()
{
  _width_field_Def = QString("60");
  _width_field_cpy = _width_field_Def;
  
  _gap_Def = 0;
  _gap_cpy = _gap_Def;
  
  _gap_field_Def = QString("6");
  _gap_field_cpy = _gap_field_Def;
  
  _algorithm_Def = 0;
  _algorithm_cpy = _algorithm_Def;
  
  _IvlSource_Def = QString(" ");
  _IvlSource_cpy = _IvlSource_Def;
  
  _TimeIvlSelect_Def = 0;
  _TimeIvlSelect_cpy = _TimeIvlSelect_Def;
  
  _TagSource_Def = QString(" ");
  _TagSource_cpy = _TagSource_Def;
  
  // re-set in join or export, but here to allow safe initialisation
  _TimeTagSelect_Def = 1;
  _TimeTagSelect_cpy = _TimeTagSelect_Def;
}

 void QjuiWindow::keepChoices(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::keepChoices %ld\n", this);
	fflush(stdout);
 }
 
 
  _width_field_cpy = _width_field->text();
  _gap_cpy = _gap->currentItem();
  _gap_field_cpy = _gap_field->text();
  _algorithm_cpy = _algorithm->currentItem();
  _IvlSource_cpy = _IvlSource->text();
  _TimeIvlSelect_cpy = _TimeIvlSelect->currentItem();
  _TagSource_cpy = _TagSource->text();
  _TimeTagSelect_cpy = _TimeTagSelect->currentItem();
 
}

 void QjuiWindow::applyChoices(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::applyChoices %ld\n", this);
	fflush(stdout);
 }
 
 
  _width_field->setText(_width_field_cpy);
  _gap->setCurrentItem(_gap_cpy);
  _gap_field->setText(_gap_field_cpy);
  _algorithm->setCurrentItem(_algorithm_cpy);
  _TimeIvlSelect->setCurrentItem(_TimeIvlSelect_cpy);
  _IvlSource->setText(_IvlSource_cpy);
  _TimeTagSelect->setCurrentItem(_TimeTagSelect_cpy);
  _TagSource->setText(_TagSource_cpy);
  _join_listview->setItemChoices(-1);
 
}

 void QjuiWindow::setTTivl(int item){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::setTTivl %ld, %d\n", this, item);
	fflush(stdout);
 }
 
 
  if(item == 0) {
    // from data
    IvlSource_label->setText(" ");
    QToolTip::remove(_IvlSource);
    QToolTip::add( _IvlSource, "not active");
    _IvlSource->active = false;
    _IvlSource->setText(" "); // force signal so that colour updates
  }
  else if(item == 1){
    IvlSource_label->setText("Interval Object");
    QToolTip::remove(_IvlSource);
    QToolTip::add( _IvlSource, "drag and drop object here");
    _IvlSource->active = true;
    _IvlSource->acceptObject = true;
    _IvlSource->acceptNumber = false;
    _IvlSource->setText("");
    _IvlSource->validateInput(_IvlSource->text());
  }
}


 void QjuiWindow::setTTsource(int item){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::setTTsource %ld, %d\n", this, item);
	fflush(stdout);
 }
 
 
  if(item == 0) {
    // create regular
    TagSource_label->setText("Spacing (sec)");
    QToolTip::remove(_TagSource);
    QToolTip::add( _TagSource, "enter spacing in seconds here");
    _TagSource->acceptNumber = true;
    _TagSource->acceptTTagObject = false;
  }
  else if(item == 1){
    TagSource_label->setText("Time Tag Object");
    QToolTip::remove(_TagSource);
    QToolTip::add( _TagSource, "drag and drop object here");
    _TagSource->acceptNumber = false;
    _TagSource->acceptTTagObject = true;
  }
  _TagSource->validateInput(_TagSource->text()); // updates T display if valid
}

 void QjuiWindow::setTimeFeedback(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::setTimeFeedback %ld\n", this);
	fflush(stdout);
 }
 
 
  QdTimeInterval_var ivl;
  int Ntags;
  double Spacing;
  double Tinterval;
  
  // set interval

  ivl = QjuiGetInterval();
 
  if ( ivl.is_nil() ) {
    Ntags_field->setText("??");
    spacing_field->setText("??");
    Start_field->setText("??");
    End_field->setText("??");
    Tinterval = -1.0;
  }
  else{
    Start_field->setText(QString(ivl->start().iso().c_str()));
    End_field->setText(QString(ivl->end().iso().c_str()));
    Tinterval = Seconds(ivl->duration_abs());
  }
  
  // set tags
  
  if( _TimeTagSelect->currentItem() == 0)
  {
    // create regular tags
    bool OK;
    Spacing = _TagSource->text().toDouble(&OK);
    if(!OK || Spacing < 1.e-20) {
      Spacing = -1.0;
      Ntags = -1;
    }
    else
      Ntags = (int) (Tinterval/Spacing + 1.0);
    
  }
  else
  {
    // tags from object
    QdObject_var tag_obj = _TagSource->getSlotObject();
    QdTimeSeq_var ts=get_timetags(tag_obj);
    Spacing = get_time_spacing(ts);
    Ntags = (int) (Tinterval/Spacing + 1.0);
  }
  QString txt;
  if(Ntags > 0) Ntags_field->setText(txt.setNum(Ntags));
  else Ntags_field->setText("??");
  
  if(Spacing > -1) spacing_field->setText(txt.setNum(Spacing));
  else spacing_field->setText("??");
  
}

QdTimeInterval_var QjuiWindow::QjuiGetInterval(){

  QdTimeInterval_var ivl;
  if( _TimeIvlSelect->currentItem() == 0)
  {
    // interval from data
    ivl = QjuiGetIntervalFromData();

  }
  else
  {
    // interval from object
    QdObject_var ivl_obj = QuGetObjByName(_IvlSource->text());
    if ( !ivl_obj.is_nil() ) {
      ivl = QdTimeInterval_var::narrow(ivl_obj);
      if(ivl.is_nil()){
        QdTimeSeq_var ts=get_timetags(ivl_obj);
        if(!ts.is_nil())
            ivl=new QdTimeInterval(ts->get_interval_bounds());
      }
    }

  } 
  return ivl; 
}

QdTimeInterval_var QjuiWindow::QjuiGetIntervalFromData()
{
  QdTimeInterval_var ivl;
  QdTimeInterval_var ivlUnion;
  QjuiListItem * next;
  
  next = (QjuiListItem *) _join_listview->firstChild();  

  while(next){
    if (next->getEntryType() == ListTypeEntry ){
      QdObject_var ivl_obj = QuGetObjByName(next->getWLcontext());
      if ( !ivl_obj.is_nil() ) {
        ivl = QdTimeInterval_var::narrow(ivl_obj);
        if(ivl.is_nil())
        {
          QdTimeSeq_var ts=get_timetags(ivl_obj);
          if(!ts.is_nil())
            ivl=new QdTimeInterval(ts->get_interval_bounds());
        }
      }
      if(ivlUnion.is_nil() ) ivlUnion = ivl;
      else if ( !ivl.is_nil()){
        QdTimeInterval_var ivlNew = new QdTimeInterval(ivlUnion->int_union(*ivl)); 
        ivlUnion = ivlNew;
      }
    }
    next = (QjuiListItem *) next->QguGetNext();   
  }
  return ivlUnion;
}


QdTimeSeq_var QjuiWindow::getTimeTags(){

  QdTimeSeq_var tags;
  QdTimeInterval_var ivl;
  double Spacing;
  double Tinterval;

  // set interval

  ivl = QjuiGetInterval();
  if ( ivl.is_nil() ) return tags;   // will be .is_nil()
  Tinterval = Seconds(ivl->duration_abs());

  // set tags

  if( _TimeTagSelect->currentItem() == 0)
  {
    // create regular tags
    bool OK;
    Spacing = _TagSource->text().toDouble(&OK);
    if(!OK || Spacing < 1.e-20) return tags;

    int Ntags = (int) (Tinterval/Spacing + 1.0);

    tags = new QdTimeSeq();
    tags->assign_regular(Ntags, ivl->start(), Seconds(Spacing) );

  }
  else
  {
    // tags from object
    QdObject_var tag_obj = _TagSource->getSlotObject();
    QdTimeSeq_var ts=get_timetags(tag_obj);
    if(ts.is_nil())  return tags;    // will be .is_nil()

    // truncate to interval if necessary

    if (( ivl->start() != ts->get_interval_bounds().start()) ||
        ( ivl->end()   != ts->get_interval_bounds().end())) {

      SequenceMask msk(ts->get_mask(QdTimeSeq::InRange(ivl->start(),ivl->end())));
      tags = new QdTimeSeq();
      ts->get_subsequence(*tags, msk);

    }
    else tags = new QdTimeSeq(*ts);
  }

  return tags;

}

bool QjuiWindow::disjointSeries(QdObject_var obj)
{
// checks whether obj is disjoint Time Series.
//
//
  if(obj.is_nil())
    return false;

  String objName;

  if(obj->xref_exists(OBJECT_NAME))
  {
    QdString_var objStr=QdString_var::narrow(obj->get_xref(OBJECT_NAME));
    if(!objStr.is_nil())
      objName=*objStr;
  }

#ifdef DIAG
  if(objName!="")
    cout << "checking object " << objName << endl;
#endif

  // fetch time interval associated with this object
  QdTimeInterval_var obj_ivl=get_timeinterval(obj);
  if(obj_ivl.is_nil())
    return false;

  bool result=true;

  // iterate thru' join listview and compare time intervals of each join object
  // if no object is found such that the intersection of the time intervals of
  // both objects is not empty, then the series is disjoint - so return true

  Q_LIST_VIEW_ITEM_ITERATOR it(_join_listview->firstChild());
  int n_series=0;

  while(it.current())
  {
    QjuiListItem* join_item=dynamic_cast<QjuiListItem*>(it.current());
    if(join_item)
    {
      if (join_item->getEntryType() == ListTypeEntry )
      {
        QdObject_var join_obj = QuGetObjByName(join_item->getWLcontext());

	String joinObjName;

	if(join_obj->xref_exists(OBJECT_NAME))
        {
          QdString_var objStr=QdString_var::narrow(join_obj->get_xref(OBJECT_NAME));
          if(!objStr.is_nil())
            joinObjName=*objStr;
        }

#ifdef DIAG
	if(joinObjName!="")
	  cout << "about to compare with " << joinObjName << endl;
#endif

	// ensure we are looking at different objects
	if(join_obj!=obj)
	{
	  n_series++;
          QdTimeInterval_var join_ivl=get_timeinterval(join_obj);
	  if(!join_ivl.is_nil())
	  {
            if(!obj_ivl->int_intersect(*join_ivl).empty())
              return false;
	  }
	}
      }
    }
    ++it;
  }

  if(n_series==0)
    return false;
  else
    return result;
}

void QjuiWindow::messageHandler(QString txt){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::messageHandler %ld\n", this);
	fflush(stdout);
 }


  this->status(txt, 10000);
}

Vostream& operator<<(Vostream& v_os,QjuiListItem& item)
{
   QguListItem& qgu_item=item;
   v_os << qgu_item;
   if(item.getEntryType() == ListTypeEntry){
     v_os << item.I_width_field.latin1();
     v_os << item.I_gap_field.latin1();
     v_os << item.I_algorithm;
     v_os << item.I_gap;
     v_os << item.WLcontext.latin1();
   }
   
   QList<QjuiListItem> lst;
   QjuiListItem *childItem=dynamic_cast<QjuiListItem*>(item.firstChild());
   int cnt=0;
   while(childItem)
   {
     lst.append(childItem);
     cnt++;
     childItem=dynamic_cast<QjuiListItem*>(childItem->nextSibling());
   }
   v_os << cnt;
   for(int i=0;i<cnt;i++)
   {
#if QT_VERSION < 0x040000
     QjuiListItem *nextItem=lst.at(i);
     nextItem->with = qgu_item.with;
     v_os << nextItem;
#else
     QjuiListItem nextItem=lst.at(i);
     nextItem.with = qgu_item.with;
     v_os << &nextItem;
#endif
   }
   return v_os;
}


Vistream& operator>>(Vistream& v_is,QjuiListItem& item)
{
   QguListItem& qgu_item=item;
   v_is >> qgu_item;

   if(qgu_item.getEntryType() == ListTypeEntry){
     string width_str;
     v_is >> width_str;
     item.I_width_field=width_str.c_str();
     string gap_str;
     v_is >> gap_str;
     item.I_gap_field=gap_str.c_str();
     v_is >> item.I_algorithm;
     v_is >> item.I_gap;
     string contxt;
     v_is >> contxt;
     item.WLcontext = contxt.c_str();
   }
   if(qgu_item.getEntryType() == ListTypeGroup){
     qgu_item.setOpen(true);
   }
   
   int cnt;
   v_is >> cnt;
   for(int i=0;i<cnt;i++)
   {
      QjuiListItem *childItem=new QjuiListItem(&item);
      v_is >> *childItem;
   }
   return v_is;
}



Vostream& operator<<(Vostream& v_os,QjuiListView& lv)
{
  int with = lv.with;
#if QT_VERSION < 0x040000
  QList<QjuiListItem> lst;
#else
  QList<QjuiListItem*> lst;
#endif
  
  QjuiListItem *item=dynamic_cast<QjuiListItem*>(lv.firstChild());
  int cnt=0;
  while(item)
  {
    lst.append(item);
    item->with = with;
    cnt++;
    item=dynamic_cast<QjuiListItem*>(item->nextSibling());
  }
  v_os << cnt;
  for(int i=0;i<cnt;i++)
  {
#if QT_VERSION < 0x040000
    QjuiListItem *item=lst.at(i);
    v_os << *item;
#else
    QjuiListItem * item=lst.at(i);
    v_os << *item;
#endif
  }
  return v_os;
} 

Vistream& operator>>(Vistream& v_is,QjuiListView& lv)
{
  
  int cnt;
  v_is >> cnt;
  
  for(int i=0;i<cnt;i++)
  {
    QjuiListItem *item=new QjuiListItem(&lv);
    v_is >> *item;
  }
  lv.listModified();
  qsasWindow->WorkingList->emitListModified();
  return v_is;
} 

