#include <stdio.h>
#include <math.h>

#ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
#endif

#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"


using namespace QSAS;

// local prototypes
extern "C" QplugReturnStatus StripLT (QplugArgList *);
extern "C" QplugReturnStatus StripGT (QplugArgList *);
extern "C" QplugReturnStatus StripNaN (QplugArgList *);
extern "C" QplugReturnStatus StripFills (QplugArgList *);
extern "C" QplugReturnStatus ConvertFills (QplugArgList *);


// -------------------------------------------------

QplugReturnStatus StripGT( QplugArgList *call_list )
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	QdObject_var dobj = QdObject_var::narrow((*call_list)[0]);
	QdObject_var userValue = QdObject_var::narrow((*call_list)[1]);
	
	QdRScalar_var limitVal_ptr;
	QdRScalarSeq_var limitValSeq_ptr;
	QdObject_var obj_out;
   
	if(dobj.is_nil() || userValue.is_nil()  ){
		QplugAppendTextDisplay( "Missing input object in remove greater than filter\n" );
		return QPLUG_FAILURE; 
	}
 
	limitVal_ptr = QdRScalar_var::narrow(userValue);  
	limitValSeq_ptr = QdRScalarSeq_var::narrow(userValue);
	if( limitValSeq_ptr.is_nil()){
		if(limitVal_ptr.is_nil())
		{
			QplugAppendTextDisplay( "Invalid input limit value for remove greater than filter \n" );
			return QPLUG_FAILURE; 
		}
		limitValSeq_ptr = new QdRScalarSeq( dobj->sequence_size(), *limitVal_ptr);
	}
	// we now have a sequence of limit values 
	
  	// create output object

	bool hasTT;

	QdTimeSeq_var ts_in=get_timetags(dobj);
	hasTT = !(ts_in.is_nil());

	QdTimeSeq_var ts_out=new QdTimeSeq;

	// is sequence a matrix sequence
	QdRMatrixSeq_var ms_in = QdRMatrixSeq_var::narrow(dobj);
	if(!ms_in.is_nil())
	{
		if(ms_in->empty()) return QPLUG_FAILURE;

		// initialise matrix output sequence and reserve max space
		QdRMatrixSeq_var ms_out=new QdRMatrixSeq;
		ms_out->reserve(ms_in->size());
    
		// initialise mask
		SequenceMask msk(ms_in->size(), true);

		for(register int i=0;i<ms_in->size();i++)
		{
			double limit = limitValSeq_ptr[i];
			bool valid_data=true;

			// get underlying data for matrix element
			valarray<double> v_in(ms_in[i].data()[ms_in[i].getgslice()]);
			for(register int j=0; j<v_in.size(); j++)
			{
				if(v_in[j] > limit){
					msk[i] = false;
					valid_data=false;
					break;
				}
			}
			// if valid update output matrix sequence
			if(valid_data)
			{
				ms_out->push_back(ms_in[i]);
				if(hasTT) ts_out->push_back(ts_in[i]);
			}
		}

		ms_out->copy_xrefs_from(ms_in);
	  
		if( hasTT){
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ms_out,ts_out);
		}
		applyMaskToXrefs(ms_in, ms_out, msk, ts_out);
	  
		
		call_list->push_back(ms_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;

	}

	// scalar sequence case

	QdRScalarSeq_var ss_in = QdRScalarSeq_var::narrow(dobj);
	if(!ss_in.is_nil())
	{
		if(ss_in->empty()) return QPLUG_FAILURE;

		// initialise mask
		SequenceMask msk(ss_in->size(), true);

		for(int i=0;i<ss_in->size();i++){
			if(ss_in[i] > limitValSeq_ptr[i]) msk[i] = false;
		}

		QdRScalarSeq_var ss_out=new QdRScalarSeq;
		ss_in->get_subsequence(*ss_out,msk);
		ss_out->copy_xrefs_from(ss_in);

		if( !ts_in.is_nil() ){
			ts_in->get_subsequence(*ts_out, msk);
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ss_out,ts_out);
		}
		applyMaskToXrefs(ss_in, ss_out, msk, ts_out);
		
		call_list->push_back(ss_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}


   // string and other input types not cleaned - return input object

   QplugAppendTextDisplay( " limit filter does not handle this data type\n" );
   return QPLUG_FAILURE;

}


QplugReturnStatus StripLT( QplugArgList *call_list )
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	QdObject_var dobj = QdObject_var::narrow((*call_list)[0]);
	QdObject_var userValue = QdObject_var::narrow((*call_list)[1]);
	
	QdRScalar_var limitVal_ptr;
	QdRScalarSeq_var limitValSeq_ptr;
	QdObject_var obj_out;
   
	if(dobj.is_nil() || userValue.is_nil()  ){
		QplugAppendTextDisplay( "Invalid input object in remove less than filter\n" );
		return QPLUG_FAILURE; 
	}
 
	limitVal_ptr = QdRScalar_var::narrow(userValue);  
	limitValSeq_ptr = QdRScalarSeq_var::narrow(userValue);
	if( limitValSeq_ptr.is_nil()){
		if(limitVal_ptr.is_nil())
		{
			QplugAppendTextDisplay( "Invalid input limit value for remove less than filter \n" );
			return QPLUG_FAILURE; 
		}
		limitValSeq_ptr = new QdRScalarSeq( dobj->sequence_size(), *limitVal_ptr);
	}
	// we now have a sequence of limit values 
	
  	// create output object

	bool hasTT;

	QdTimeSeq_var ts_in=get_timetags(dobj);
	hasTT = !(ts_in.is_nil());

	QdTimeSeq_var ts_out=new QdTimeSeq;

	// is sequence a matrix sequence
	QdRMatrixSeq_var ms_in = QdRMatrixSeq_var::narrow(dobj);
	if(!ms_in.is_nil())
	{
		if(ms_in->empty()) return QPLUG_FAILURE;

		// initialise matrix output sequence and reserve max space
		QdRMatrixSeq_var ms_out=new QdRMatrixSeq;
		ms_out->reserve(ms_in->size());
    
		// initialise mask
		SequenceMask msk(ms_in->size(), true);

		for(register int i=0;i<ms_in->size();i++)
		{
			double limit = limitValSeq_ptr[i];
			bool valid_data=true;

			// get underlying data for matrix element
			valarray<double> v_in(ms_in[i].data()[ms_in[i].getgslice()]);
			for(register int j=0; j<v_in.size(); j++)
			{
				if(v_in[j] < limit){
					msk[i] = false;
					valid_data=false;
					break;
				}
			}
			// if valid update output matrix sequence
			if(valid_data)
			{
				ms_out->push_back(ms_in[i]);
				if(hasTT) ts_out->push_back(ts_in[i]);
			}
		}

		ms_out->copy_xrefs_from(ms_in);
	  
		if( hasTT){
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ms_out,ts_out);
		}
		applyMaskToXrefs(ms_in, ms_out, msk, ts_out);
	  
		call_list->push_back(ms_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}

	// scalar sequence case

	QdRScalarSeq_var ss_in = QdRScalarSeq_var::narrow(dobj);
	if(!ss_in.is_nil())
	{
		if(ss_in->empty()) return QPLUG_FAILURE;

		// initialise mask
		SequenceMask msk(ss_in->size(), true);

		for(int i=0;i<ss_in->size();i++){
			if(ss_in[i] < limitValSeq_ptr[i]) msk[i] = false;
		}

		QdRScalarSeq_var ss_out=new QdRScalarSeq;
		ss_in->get_subsequence(*ss_out,msk);
		ss_out->copy_xrefs_from(ss_in);

		if( !ts_in.is_nil() ){
			ts_in->get_subsequence(*ts_out, msk);
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ss_out,ts_out);
		}
		applyMaskToXrefs(ss_in, ss_out, msk, ts_out);
		
		call_list->push_back(ss_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}


   // string and other input types not cleaned - return input object

	QplugAppendTextDisplay( " limit filter does not handle this data type\n" );
	return QPLUG_FAILURE;


}

QplugReturnStatus StripNaN( QplugArgList *call_list )
{
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data object

	QdObject_var dobj = QdObject_var::narrow((*call_list)[0]);
	
	QdObject_var obj_out;
   
	if(dobj.is_nil()  ){
		QplugAppendTextDisplay( "Invalid input object in strip NaN filter\n" );
		return QPLUG_FAILURE; 
	}
 	
  	// create output object

	bool hasTT;

	QdTimeSeq_var ts_in=get_timetags(dobj);
	hasTT = !(ts_in.is_nil());

	QdTimeSeq_var ts_out=new QdTimeSeq;

	// is sequence a matrix sequence
	QdRMatrixSeq_var ms_in = QdRMatrixSeq_var::narrow(dobj);
	if(!ms_in.is_nil())
	{
		if(ms_in->empty()) return QPLUG_FAILURE;

		// initialise matrix output sequence and reserve max space
		QdRMatrixSeq_var ms_out=new QdRMatrixSeq;
		ms_out->reserve(ms_in->size());
    
		// initialise mask
		SequenceMask msk(ms_in->size(), true);

		for(register int i=0;i<ms_in->size();i++)
		{
			bool valid_data=true;

			// get underlying data for matrix element
			valarray<double> v_in(ms_in[i].data()[ms_in[i].getgslice()]);
			for(register int j=0; j<v_in.size(); j++)
			{
				if( !finite( v_in[j]) ){
					msk[i] = false;
					valid_data=false;
					break;
				}
			}
			// if valid update output matrix sequence
			if(valid_data)
			{
				ms_out->push_back(ms_in[i]);
				if(hasTT) ts_out->push_back(ts_in[i]);
			}
		}

		ms_out->copy_xrefs_from(ms_in);
	  
		if( hasTT){
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ms_out,ts_out);
		}
		applyMaskToXrefs(ms_in, ms_out, msk, ts_out);
	  
		call_list->push_back(ms_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}

	// scalar sequence case

	QdRScalarSeq_var ss_in = QdRScalarSeq_var::narrow(dobj);
	if(!ss_in.is_nil())
	{
		if(ss_in->empty()) return QPLUG_FAILURE;

		// initialise mask
		SequenceMask msk(ss_in->size(), true);

		for(int i=0;i<ss_in->size();i++){
			if( !finite(ss_in[i]) ) msk[i] = false;
		}

		QdRScalarSeq_var ss_out=new QdRScalarSeq;
		ss_in->get_subsequence(*ss_out,msk);
		ss_out->copy_xrefs_from(ss_in);

		if( !ts_in.is_nil() ){
			ts_in->get_subsequence(*ts_out, msk);
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ss_out,ts_out);
		}
		applyMaskToXrefs(ss_in, ss_out, msk, ts_out);
		
		call_list->push_back(ss_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}


   // string and other input types not cleaned - return input object

	QplugAppendTextDisplay( " limit filter does not handle this data type\n" );
	return QPLUG_FAILURE;


}



QplugReturnStatus StripFills( QplugArgList *call_list )
{
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data object

	QdObject_var dobj = QdObject_var::narrow((*call_list)[0]);
	
	QdObject_var obj_out;
	QdObject_var fill_xref;

	if(dobj.is_nil()  ){
		QplugAppendTextDisplay( "Invalid input object in strip Fills filter\n" );
		return QPLUG_FAILURE; 
	}
 	
	// get Fillval
	
    if (dobj->xref_exists("FILLVAL"))  fill_xref = dobj->get_xref("FILLVAL");
	else{
		QplugAppendTextDisplay( "No Fillval attribute in object, using -1.0e31\n" );
		fill_xref = new QdRScalar( -10.e+30); // in case xref not set
	}

  	// create output object

	bool hasTT;

	QdTimeSeq_var ts_in=get_timetags(dobj);
	hasTT = !(ts_in.is_nil());

	QdTimeSeq_var ts_out=new QdTimeSeq;

	// is sequence a matrix sequence
	QdRMatrixSeq_var ms_in = QdRMatrixSeq_var::narrow(dobj);
	if(!ms_in.is_nil())
	{
		if(ms_in->empty()) return QPLUG_FAILURE;

		// initialise matrix output sequence and reserve max space
		QdRMatrixSeq_var ms_out=new QdRMatrixSeq;
		ms_out->reserve(ms_in->size());
    
		// initialise mask
		SequenceMask msk(ms_in->size(), true);

		// get Fill array
		QdRMatrix_var fill_m = QdRMatrix_var::narrow(fill_xref);

		// if not array create matrix of fill values
		// this assumes the MatrixSeq is homogeneous i.e. each matrix
		// element has the same no. and sizes of dimensions

		if(fill_m.is_nil())
		{
			QdRScalar_var fill_v = QdRScalar_var::narrow(fill_xref);
			if(fill_v.is_nil()) return QPLUG_FAILURE;
			fill_m = new QdRMatrix( ms_in[0].dimdata(), fill_v->scalar());

		}
		// get underlying data for fill matrix
		valarray<double> v_fill(fill_m->data()[fill_m->getgslice()]);


		for(register int i=0;i<ms_in->size();i++)
		{
			bool valid_data=true;

			// get underlying data for matrix element
			valarray<double> v_in(ms_in[i].data()[ms_in[i].getgslice()]);
			for(register int j=0; j<v_in.size(); j++)
			{
				const double& dfill = v_fill[j%v_fill.size()];
				if( QuIsMatch( v_in[j], dfill) ){
					msk[i] = false;
					valid_data=false;
					break;
				}
			}
			// if valid update output matrix sequence
			if(valid_data)
			{
				ms_out->push_back(ms_in[i]);
				if(hasTT) ts_out->push_back(ts_in[i]);
			}
		}

		ms_out->copy_xrefs_from(ms_in);
	  
		if( hasTT){
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ms_out,ts_out);
		}
		applyMaskToXrefs(ms_in, ms_out, msk, ts_out);
	  
		call_list->push_back(ms_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}

	// scalar sequence case

	QdRScalarSeq_var ss_in = QdRScalarSeq_var::narrow(dobj);
	if(!ss_in.is_nil())
	{
		if(ss_in->empty()) return QPLUG_FAILURE;
		
        QdRScalar_var fill_v = QdRScalar_var::narrow(fill_xref);
		if( fill_v.is_nil() ) return QPLUG_FAILURE;
		
		double fill = fill_v->scalar();
		
		// initialise mask
		SequenceMask msk(ss_in->size(), true);

		for(int i=0;i<ss_in->size();i++){
			if( QuIsMatch(ss_in[i], fill) ) msk[i] = false;
		}

		QdRScalarSeq_var ss_out=new QdRScalarSeq;
		ss_in->get_subsequence(*ss_out,msk);
		ss_out->copy_xrefs_from(ss_in);

		if( !ts_in.is_nil() ){
			ts_in->get_subsequence(*ts_out, msk);
			ts_out->copy_xrefs_from(ts_in);
			set_timetags(ss_out,ts_out);
		}
		applyMaskToXrefs(ss_in, ss_out, msk, ts_out);
		
		call_list->push_back(ss_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}


   // string and other input types not cleaned - return input object

	QplugAppendTextDisplay( " limit filter does not handle this data type\n" );
	return QPLUG_FAILURE;


}

// -------------------------------------------------

QplugReturnStatus ConvertFills( QplugArgList *call_list )
{
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	QdObject_var dobj = QdObject_var::narrow((*call_list)[0]);
	QdObject_var userValue = QdObject_var::narrow((*call_list)[1]);
	
	
	QdRScalar_var newVal_ptr;
	QdObject_var out_ptr;
   
	if(dobj.is_nil() || userValue.is_nil()  ){
		QplugAppendTextDisplay( "Missing input object in remove greater than filter\n" );
		return QPLUG_FAILURE; 
	}
	 
	newVal_ptr = QdRScalar_var::narrow(userValue);
	if( newVal_ptr.is_nil() ){
		QplugAppendTextDisplay( "Invalid input target value for Fillval Conversion \n" );
		return QPLUG_FAILURE; 
	}
 
	// create output object

	QdObject_var fill_xref;
	double fill;
	bool hasTT = false;
	QdObject_var obj_out;
  
	double newValue = (double) *newVal_ptr;
	QdRScalar_var fill_out = new QdRScalar( newValue );

    if (dobj->xref_exists("FILLVAL"))  fill_xref = dobj->get_xref("FILLVAL");

    fill = -10.e+30; // in case xref not set
    
    QdTimeSeq_var ts_in=get_timetags(dobj);
    hasTT = !(ts_in.is_nil());

    // is sequence a matrix sequence
    QdRMatrixSeq_var ms_in=QdRMatrixSeq_var::narrow(dobj);
    if(!ms_in.is_nil())
    {
		if(ms_in->empty())
		{
			QplugAppendTextDisplay( "Empty input matrix sequence \n" );
			return QPLUG_FAILURE; 
		}
		// if so, do we have a single fill-value or a matrix of different fill
		// values for each dimension

		QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

		// ...either way create matrix of fill values
		// this assumes the MatrixSeq is homogeneous i.e. each matrix
		// element has the same no. and sizes of dimensions

		if(fill_m.is_nil())
		{
			QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
			if(fill_v.is_nil()) fill_v = new QdRScalar(fill);

			fill_m=new QdRMatrix(ms_in[0].dimdata(),fill_v->scalar());

		}

		// initialise matrix output sequence and reserve max space
		QdRMatrixSeq_var ms_out = Factory::clone(ms_in);
	 
		// get underlying data for fill matrix
		valarray<double> v_fill(fill_m->data()[fill_m->getgslice()]);

		for(register int i=0;i<ms_out->size();i++)
		{ 
			// create valarray<double> of results for assigning into matrix (gslice_array)
			bool update=false;       
			valarray<double> v_out(ms_out[i].data()[ms_out[i].getgslice()]);
			for(register int j=0;j<v_out.size();j++)
			{
				const double& dfill = v_fill[j%v_fill.size()];
				if( QuIsMatch( v_out[j], dfill) ) {
					update=true;
					v_out[j] = newValue;  // newValue is a double
				}
			}
			// assign results into gslice array (only if need to)
			if(update) ms_out[i].data()[ms_out[i].getgslice()] = v_out;
		}

		ms_out->copy_xrefs_from(ms_in);
		ms_out->change_xref(FILL_VALUE, fill_out);
	 

		if( hasTT){
			set_timetags(ms_out,ts_in);
		}	  
		call_list->push_back(ms_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}

    // scalar sequence case

	QdRScalarSeq_var ss_in=QdRScalarSeq_var::narrow(dobj);
	if(!ss_in.is_nil())
	{
		if(ss_in->empty())
		{
			QplugAppendTextDisplay( "Empty input sequence \n" );
			return QPLUG_FAILURE; 
		}

		QdRScalar_var fill_v = QdRScalar_var::narrow(fill_xref);
		if( !fill_v.is_nil()) fill = (double) *fill_v;

		QdRScalarSeq_var ss_out = Factory::clone(ss_in);
	 
		for(int i=0;i<ss_in->size();i++){
			if( QuIsMatch(ss_out[i], fill) ) ss_out[i] = newValue;
		}

		ss_out->copy_xrefs_from(ss_in);
		ss_out->change_xref(FILL_VALUE, fill_out);

		if( hasTT ){
			set_timetags(ss_out, ts_in);
		}	
	  
		call_list->push_back(ss_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}


	QdRMatrix_var m_in=QdRMatrix_var::narrow(dobj);
	if(!m_in.is_nil())
    {

		// if so, do we have a single fill-value or a matrix of different fill
		// values for each dimension

		QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

		// ...either way create matrix of fill values
		// this assumes the MatrixSeq is homogeneous i.e. each matrix
		// element has the same no. and sizes of dimensions

		if(fill_m.is_nil())
		{
			QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
			if(fill_v.is_nil()) fill_v = new QdRScalar(fill);

			fill_m=new QdRMatrix(m_in->dimdata(),fill_v->scalar());

		}


		// iterator for fill values
		RMatrix::iterator fitr=fill_m->begin();
      
		QdRMatrix_var m_out = Factory::clone(m_in);
		m_out->copy_xrefs_from(m_in);
		m_out->change_xref(FILL_VALUE, fill_out);

		if( hasTT){
			set_timetags(m_out,ts_in);
		}
	 
		// iterator for matrix
		RMatrix::iterator itr=m_out->begin();
		int j=0;
		while(itr!=m_out->end() )
		{
			double& d=*itr++;
			if( QuIsMatch(d, *(fitr+j++) ) ) d = newValue;
		}

		call_list->push_back(m_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;

	}

	QdRScalar_var s_in=QdRScalar_var::narrow(dobj);
	if(!s_in.is_nil())
	{
		QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
		if( !fill_v.is_nil()) fill = (double) *fill_v;
	 
		QdRScalar_var s_out = Factory::clone(s_in);
		s_out->copy_xrefs_from(s_in);
		s_out->change_xref(FILL_VALUE, fill_out);

		if( hasTT){
			set_timetags(s_out,ts_in);
		}

		if( QuIsMatch( *s_out, fill)) *s_out = newValue;
		
		call_list->push_back(s_out);
		QplugAppendTextDisplay ("Filter: Done\n");
		return QPLUG_SUCCESS;
	}


	// string and other input types not cleaned - return input object

	QplugAppendTextDisplay (" Fillval Conversion not handled for this data type\n");
	return QPLUG_FAILURE;
 
 }
