
// -----------------------------------------------------------------------------------------
//
// Calculate Q-matrix for MVA of any vector Quantity
//
// SEH, Mar 27, 2003
//      Sep 13, 2003  - cloned fromm massflow and modified 
//      Sep 26, 2005  - MVAV version
//
// Inputs :
//     V = any vector time series, e.g., magnetic field   (n x 3)     
//
// -----------------------------------------------------------------------------------------

#include "Qdos.h"
#include "qplug_if.h"
#include "qdutil.h"

//#include "QsasUtils.h"
#include "GenericMinVar.h"
#include "GenericResidue.h"

// -----------------------------------------------------------------------------------------
//
// Returns the Q_matrix for MVAV (Minimum Variance of Velcity)
//
// SEH, 26 Sep, 2005, initiated from Q_MVAB_calc
// Doxygen info :
/*! 
  \brief Calculates the covariance matrix of the velocity.
  \param V -  vector time series with plasma velocity in units of km/s
  \param U -  holds the calculated velocity on successful return
  \param nestsize - number of elements in B and V.

*/ 

QdRMatrix_var Q_MVAV_calc(QdRMatrixSeq_var V, QdRMatrix_var U, int nestsize)
{
  int i,j,k;
  double normfac = (double) nestsize;
  QdRMatrix_var Q  = new QdRMatrix(Dim(3,3), 0.00);
  QdRMatrixSeq_var dV  = new QdRMatrixSeq(nestsize,RMatrix(Dim(3)));
  QdRMatrix_var aveV  = new QdRMatrix(Dim(3), 0.0);
  bool PO = true;                           // PrintOut to screen ? 


  for (k=0; k<nestsize; k++) {              // find <B>
    for (i=0; i<3; i++) {
      (*aveV)[i] += (*V)[k][i] / normfac;;
    }
  }

  if (PO)printf("Q_MVAV : ave V : [%E  %E  %E]\n",(double)(*aveV)[0], (double)(*aveV)[1], (double)(*aveV)[2]);
  for (k=0; k<nestsize; k++) {             // find delta B
    for (i=0; i<3; i++) {
      (*dV)[k][i] = (double)(*V)[k][i] - (double)(*aveV)[i];
    }
  }

  for (k=0; k<nestsize; k++) {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) (*Q)[i][j] += (*dV)[k][i] * (*dV)[k][j] / normfac;
    }
   }


  // ----- calculate U == ave V ------

  for (i=0; i<3; i++) (*U)[i] = (*aveV)[i];
  if (PO)printf("Q_MVAV : ave V : [%E  %E  %E]\n",(double)(*U)[0], (double)(*U)[1], (double)(*U)[2]);

  return (Q);
}





