#ifndef MOMENTS_UTILS_H
#define MOMENTS_UTILS_H

#include "Qdos.h"
#include "qar.h"
//#include "Moments_plot.h"
#include "triangulation.h"

#define MOM_DENSITY 0x01
#define MOM_VELOCITY 0x02
#define MOM_PRESSURE 0x04
#define MOM_HEATFLUX 0x08

class MomentsParameters
{
    protected:
        
        bool init_data(QdObject_var & data);
        
        bool init_potential(QdObject_var & potential);
        
        bool init_threshold(QdObject_var & threshold);
        
        bool init_gse(QdObject_var & gse);
    
    public:
        MomentsParameters(QdObject_var& obj, QdObject_var& potential_in, QdObject_var& threshold_in, QdObject_var gse_in, QdObject_var direction_in, bool _logData=false, bool _logR=false);
        
        double mass;
        double charge;
        
        enum data_type {MATRIX_SEQ, MATRIX, SCALAR_SEQ, SCALAR};
        
        bool init_ok;
        
        QdRMatrixSeq_var data;
        double dataFillVal;
        bool logData;
        
        QdRMatrix_var e_const;
        QdRMatrixSeq_var e_seq;
        QdRMatrix_var e_minus_const;
        QdRMatrixSeq_var e_minus_seq;
        QdRMatrix_var e_plus_const;
        QdRMatrixSeq_var e_plus_seq;
        double eFillVal;
        bool logE;
        
        QdRMatrix_var phi_const;
        QdRMatrixSeq_var phi_seq;
        QdRMatrix_var phi_minus_matrix;
        QdRMatrixSeq_var phi_minus_matrix_seq;
        QdRScalar_var phi_minus_scalar;
        QdRScalarSeq_var phi_minus_scalar_seq;
        QdRMatrix_var phi_plus_matrix;
        QdRMatrixSeq_var phi_plus_matrix_seq;
        QdRScalar_var phi_plus_scalar;
        QdRScalarSeq_var phi_plus_scalar_seq;
        data_type type_phi_minus;
        data_type type_phi_plus;
        double phiFillVal;
        double phi2rad;
        
        
        QdRMatrix_var theta_const;
        QdRMatrixSeq_var theta_seq;
        QdRMatrix_var theta_minus_matrix;
        QdRMatrixSeq_var theta_minus_matrix_seq;
        QdRScalar_var theta_minus_scalar;
        QdRScalarSeq_var theta_minus_scalar_seq;
        QdRMatrix_var theta_plus_matrix;
        QdRMatrixSeq_var theta_plus_matrix_seq;
        QdRScalar_var theta_plus_scalar;
        QdRScalarSeq_var theta_plus_scalar_seq;
        data_type type_theta_minus;
        data_type type_theta_plus;
        double thetaFillVal;
        double theta2rad;
        
        QdRScalarSeq_var potential;
        QdRScalarSeq_var threshold;
        
        QdRMatrix_var gse_const;
        QdRMatrixSeq_var gse_seq;
        data_type type_gse;
        
        list<Element3D> tetra;
        int time_tetra_cache;
        
        int indexE, indexPhi, indexTheta;
        int maxTime, maxE, maxTheta, maxPhi;
        
        double v_sign;

//         double getIntegratedValue(int time);
        
        double getData(int time, int r, int phi, int theta);
        
        inline double getElementVolume(int time, int v, int phi, int theta)
        {
            double v1(getVMinus(time, v)), v2(getVPlus(time, v)), theta1(getThetaMinus(time, theta)*theta2rad), theta2(getThetaPlus(time, theta)*theta2rad), phi1(getPhiMinus(time, phi)*phi2rad), phi2(getPhiPlus(time, phi)*phi2rad);

            if(v1<0./*==-1*/ || v2<0./*==-1*/ || phi1==phiFillVal || phi2==phiFillVal || theta1==thetaFillVal || theta2==thetaFillVal)
                return -1.;
            else
            {
                return fabs((v2*v2*v2-v1*v1*v1)*(cos(theta1)-cos(theta2))*(phi2-phi1))/3.;
            }
        }
        
        inline double getElementVolume_raw(int time, int v, int phi, int theta)
        {
            double v1(getVMinus_raw(time, v)), v2(getVPlus_raw(time, v)), theta1(getThetaMinus(time, theta)*theta2rad), theta2(getThetaPlus(time, theta)*theta2rad), phi1(getPhiMinus(time, phi)*phi2rad), phi2(getPhiPlus(time, phi)*phi2rad);

            if(v1<0./*==-1*/ || v2<0./*==-1*/ || phi1==phiFillVal || phi2==phiFillVal || theta1==thetaFillVal || theta2==thetaFillVal)
                return -1.;
            else
            {
                return fabs((v2*v2*v2-v1*v1*v1)*(cos(theta1)-cos(theta2))*(phi2-phi1))/3.;
            }
        }
        
        inline double getElementVolumeVelocitySpace(double mass, int time, int r, int phi, int theta)
        {
            mass=mass*mass*mass;
            double r1(getEMinus(time, r)), r2(getEPlus(time, r)), theta1(getThetaMinus(time, theta)*theta2rad), theta2(getThetaPlus(time, theta)*theta2rad), phi1(getPhiMinus(time, phi)*phi2rad), phi2(getPhiPlus(time, phi)*phi2rad);
    
            if(r1==eFillVal || r2==eFillVal || phi1==phiFillVal || phi2==phiFillVal || theta1==thetaFillVal || theta2==thetaFillVal)
                return -1.;
            else
                return sqrt(8./mass)*(sqrt(r2*r2*r2)-sqrt(r1*r1*r1))*(cos(theta1)-cos(theta2))*(phi2-phi1)/3.;
        }
        
        inline double getE(int time, int e)
        {
            double energy=(e_const.is_nil() ? e_seq[time][e] : (*e_const)[e]);
            if (energy < threshold->elt(time)) return eFillVal;
            if((double)potential->elt(time) > 0. && (double)potential->elt(time)<1.e6)
            energy = energy - charge*(double)potential->elt(time);
            if (energy<0.) return eFillVal;
            return energy;
        }
        
        inline double getE_raw(int time, int e)
        {
            double energy=(e_const.is_nil() ? e_seq[time][e] : (*e_const)[e]);
            return energy;
        }
        
        inline double getEPlus(int time, int e)
        {
            double _e=getE(time, e);
            if(_e==eFillVal)
                return eFillVal;
            else
                return _e+(double)(e_plus_const.is_nil() ? e_plus_seq[time][e] : (*e_plus_const)[e]);
        }
        
        inline double getEPlus_raw(int time, int e)
        {
            double _e=getE_raw(time, e);
            if(_e==eFillVal)
                return eFillVal;
            else
                return _e+(double)(e_plus_const.is_nil() ? e_plus_seq[time][e] : (*e_plus_const)[e]);
        }

        inline double getEMinus(int time, int e)
        {
            double _e=getE(time, e);
            if(_e==eFillVal)
                return eFillVal;
            else
                return _e-(double)(e_minus_const.is_nil() ? e_minus_seq[time][e] : (*e_minus_const)[e]);
        }
        
        inline double getEMinus_raw(int time, int e)
        {
            double _e=getE_raw(time, e);
            if(_e==eFillVal)
                return eFillVal;
            else
                return _e-(double)(e_minus_const.is_nil() ? e_minus_seq[time][e] : (*e_minus_const)[e]);
        }

        inline double getTheta(int time, int theta)
        {
            return (theta_const.is_nil() ? theta_seq[time][theta] : (*theta_const)[theta]);
        }
        
        inline double getThetaPlus(int time, int theta)
        {
            double _theta=getTheta(time, theta);
            if(_theta==thetaFillVal)
                return thetaFillVal;
            else
                switch(type_theta_plus)
                {
                    case MATRIX_SEQ :
                        return _theta+theta_plus_matrix_seq[time](theta);
                        break;
                    
                    case MATRIX :
                        return _theta+(*theta_plus_matrix)(theta);
                        break;
                        
                    case SCALAR_SEQ :
                        return _theta+theta_plus_scalar_seq[time];
                        break;
                    
                    default :
                        return _theta+*theta_plus_scalar;
                        break;
                }
        }

        inline double getThetaMinus(int time, int theta)
        {
            double _theta=getTheta(time, theta);
            if(_theta==thetaFillVal)
                return thetaFillVal;
            else
                switch(type_theta_minus)
                {
                    case MATRIX_SEQ :
                        return _theta-theta_minus_matrix_seq[time](theta);
                        break;
                        
                    case MATRIX :
                        return _theta-(*theta_minus_matrix)(theta);
                        break;
                            
                    case SCALAR_SEQ :
                        return _theta-(double)theta_minus_scalar_seq[time];
                        break;
                        
                    default :
                        return _theta-(double)(*theta_minus_scalar);
                        break;
                }
        }

        inline double getPhi(int time, int phi)
        {
            return (phi_const.is_nil() ? phi_seq[time][phi] : (*phi_const)[phi]);
        }

        inline double getPhiPlus(int time, int phi)
        {
            double _phi=getPhi(time, phi);
            if(_phi==phiFillVal)
                return phiFillVal;
            else
                switch(type_phi_plus)
                {
                    case MATRIX_SEQ :
                        return _phi+phi_plus_matrix_seq[time](phi);
                        break;
                        
                    case MATRIX :
                        return _phi+(*phi_plus_matrix)(phi);
                        break;
                            
                    case SCALAR_SEQ :
                        return _phi+(double)phi_plus_scalar_seq[time];
                        break;
                        
                    default :
                        return _phi+(double)(*phi_plus_scalar);
                        break;
                }
        }

        inline double getPhiMinus(int time, int phi)
        {
            double _phi=getPhi(time, phi);
            if(_phi==phiFillVal)
                return phiFillVal;
            else
                switch(type_phi_minus)
                {
                    case MATRIX_SEQ :
                        return _phi-phi_minus_matrix_seq[time](phi);
                        break;
                            
                    case MATRIX :
                        return _phi-(*phi_minus_matrix)(phi);
                        break;
                                
                    case SCALAR_SEQ :
                        return _phi-(double)phi_minus_scalar_seq[time];
                        break;
                            
                    default :
                        return _phi-(double)(*phi_minus_scalar);
                        break;
                }
        }
        
        inline double E2V(double e)
        {
            if(e!=eFillVal)
            {
                return sqrt(e/(mass)*2.);
            }
            else
            {
                return -1.e31;
            }
        }
        
        inline double getV(int time, int v)
        {
            //return E2V((e_const.is_nil() ? e_seq[time][v] : (*e_const)[v]));
            return E2V(getE(time, v));
        }
        
        inline double getV_raw(int time, int v)
        {
            return E2V(getE_raw(time, v));
        }
        
        inline double getVPlus(int time, int v)
        {
            return E2V(getEPlus(time, v));
        }
        
        inline double getVPlus_raw(int time, int v)
        {
            return E2V(getEPlus_raw(time, v));
        }

        inline double getVMinus(int time, int v)
        {
            return E2V(getEMinus(time, v));
        }
        
        inline double getVMinus_raw(int time, int v)
        {
            return E2V(getEMinus_raw(time, v));
        }
        
        QdRMatrix_var computeSlice(int resolution, int time, QdRMatrix_var& CoM, QdRMatrix_var& direction, double offset);
        
        QdRMatrix_var computeValuesAlongVector(int time, QdRMatrix_var& CoM, QdRMatrix_var& direction, double v_min, double v_max, int nb_values);
        
        QdRMatrix_var computePAD(int time, QdRMatrix_var& CoM, QdRMatrix_var& direction, double v_min, double v_max, int nb_v, int nb_theta);
        
        void buildTetra(int time); // TODO use triangulation2 where possible
        
        void buildTetra2(int time);
        
        //void plot(char* filename, int time);
        
        void triangulation2(int time, list<Element3D>& tetra); //TODO fill center gap + GSE
        
        inline void convertToGSE(int time, double& x, double& y, double& z)
        {
            double _x, _y, _z;
            switch(type_gse)
            {
                case SCALAR:
                case SCALAR_SEQ:
                    break;

                case MATRIX:
                    _x=(*gse_const)(0, 0)*x+(*gse_const)(0, 1)*y+(*gse_const)(0, 2)*z;
                    _y=(*gse_const)(1, 0)*x+(*gse_const)(1, 1)*y+(*gse_const)(1, 2)*z;
                    _z=(*gse_const)(2, 0)*x+(*gse_const)(2, 1)*y+(*gse_const)(2, 2)*z;
                    x=_x;
                    y=_y;
                    z=_z;
                    break;
                
                case MATRIX_SEQ:
                    _x=(*gse_seq)[time](0, 0)*x+(*gse_seq)[time](1, 0)*y+(*gse_seq)[time](2, 0)*z;
                    _y=(*gse_seq)[time](0, 1)*x+(*gse_seq)[time](1, 1)*y+(*gse_seq)[time](2, 1)*z;
                    _z=(*gse_seq)[time](0, 2)*x+(*gse_seq)[time](1, 2)*y+(*gse_seq)[time](2, 2)*z;
                    x=_x;
                    y=_y;
                    z=_z;
                
                default: break;
            }

        }
        
        void getMomentsSummed(int flags, QdRScalarSeq_var& density, QdRMatrixSeq_var& velocity, QdRMatrixSeq_var& pressure, QdRMatrixSeq_var& heat, QdRScalarSeq_var& temperature);
        
        void getMomentsIntegrated(int flags, QdRScalarSeq_var& density, QdRMatrixSeq_var& velocity, QdRMatrixSeq_var& pressure, QdRMatrixSeq_var& heat, QdRScalarSeq_var& temperature/*, QdRMatrixSeq_var& pad*/);
        
        void getMomentsIntegratedO(int flags, QdRScalarSeq_var& density, QdRMatrixSeq_var& velocity, QdRMatrixSeq_var& pressure, QdRMatrixSeq_var& heat, QdRScalarSeq_var& temperature);
};

void plot_slice(QdRMatrix_var& slice);

void getRotationMatrix(QdRMatrix_var& direction, QdRMatrix_var& rotation);

#endif
