/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/


/* ensure cdf.h is included before declaractions dependent on CDF types */


#if !defined(___cdf_h___)
#include <cdf.h>
#endif
 
#ifndef _QRTN_H_  
#define _QRTN_H_

/* c++ compliance... */

  
/* include files */

#include <stddef.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <time.h> 
#include <dirent.h>  
#include <math.h>
#include <ctype.h>
#include <sys/param.h>

#include "qdutil.h" 
#include "qprofile.h"
#include "Qdos.h" 

using namespace QSAS;

/* environment variables */

#define MISSING 0
#define EXISTS 1

/* frames known */

#define GSE 0   /* Geocentric Solar Ecliptic */
#define GSM 1   /* Geocentric Solar Magnetic */
#define GEOC 2  /* Geographic (geocentric) */
#define SM 3    /* Solar Magnetic */
#define GEI 4   /* Geocentric Inertial */
#define MAGD 5  /* Geomagnetic (dipole) */
#define HAE 6   /* Heliospheric Aries Ecliptic */
#define HEE 7   /* Heliospheric Earth Ecliptic */
#define HEEQ 8  /* Heliospheric Earth Equatorial */
#define GSEQ 9 /* Geocentric Solar Equatorial */
#define UNKNOWN -1 


/* messages */

#define QRTN_OK 0         /* must be 0 for use of |= in accumulating errors */
#define QRTN_WARNING -1
#define QRTN_ERROR 1
#define NOT_YET_IMPLEMENTED 2
#define UNKNOWN_FRAME 3
#define SAME_FRAME 4
#define BAD_MDT_FILE 6
#define BAD_EPOCH 7
#define BAD_AXIS 8
#define CANNOT_ROTATE 9
#define QRTN_INIT_ERROR 10
#define CANNOT_GET_XYZ 11

/* sizes */

#define N_YEAR_MODELS 23
#define MAX_N_SYS 20
#define SYS_NAME_LEN 36
#define N_ROTNS 6 /* max number of rotns for each system */


/* Structures */

 typedef struct QrSVector
  {
    int IsPositionVector;
    int in_frame;
    int out_frame;
    int n_recs;
    QdRMatrixSeq_var ptr;
    QdTimeSeq_var t;
    
  } QrVector;


/* prototypes */

long QrDoRotation(QrVector *vec);

/***********************************************************************/

 char * QrErrorMessage(long err_flag);

/***********************************************************************/

void QrSetFrameAttr(int frame_flag,
                    QdObject_var dobj);

/***********************************************************************/

int QrGetFrameFromAttr(QdObject_var dobj);

/***********************************************************************/

long QrSetAttrInfo();


/***********************************************************************/

long QrGetModel(double year);

/***********************************************************************/

long QrUpdateModel(int MJDdate, double ut);

/***********************************************************************/

int QrMJD(int year, int month, int day);

/***********************************************************************/

long QrEpochToMJD_UT(double epoch, int * MJDdate, double * ut);

/***********************************************************************/

long QrInitQRTN();

/***********************************************************************/

long QrGetMagPole();

/***********************************************************************/

long QrGetTerms ();

/***********************************************************************/

long QrBuildTran(double (*T)[3][3], int n_terms, int *list_rtns);

/***********************************************************************/
       
long QrMultVecbyMatx(double (*XX)[3], double T[3][3]);

/***********************************************************************/

long QrMultMtxByMtx(double (*T)[3][3], double S[3][3]);
 
/***********************************************************************/
 
long QrMakeMatrix(double theta, int axis, double (*T)[3][3]);

/***********************************************************************/

 int  QrGetAxis(int i);
 
/***********************************************************************/

void QrUnitMatrix(double (*T)[3][3] );

/***********************************************************************/

void QrNullMatrix(double (*T)[3][3] );

/***********************************************************************/

double QrDegToRadMod2pi(double angle);

/***********************************************************************/

double QrGST(double JulianC, double ut);

/***********************************************************************/

double QrSunLong(double JulianC, double ut);

/***********************************************************************/

 double QrObliquity(double JulianC);

/***********************************************************************/

long QrGSM_to_GSE(double *psi,
                   double *mu,
                   double latitude,
                   double longitude);

/***********************************************************************/

double QrSunEqator(int MJDdate);

/***********************************************************************/

long QrSunEarth(double *theta , double *phi);

/***********************************************************************/

long QrGetRotAngles(int MJDdate, double ut);

/***********************************************************************/

 double QrGetAngle(int i);

/***********************************************************************/
         
long QrMakeMatx(int in_frame,
                int out_frame,
                double (*T)[3][3]);

/***********************************************************************/

int QrOriginsDiffer(int in_frame, 
                    int out_frame);

/***********************************************************************/

int QrIsPositionVector( QdObject_var dobj);

/***********************************************************************/


#endif
