//  QbTimeWidgets.h
//
// Time Edition Widgets
//
// November 2008 A Rochel
//

#include <iostream>

#include "TimeUtils.h"
#include "Factory.h"
#include "StringRep.h"
#include "qdutil.h"
#include "qwui.h"

#if QT_VERSION < 0x040000
#include "QbTimeWidgets.h"
#define WIDGET_NAME QbDateTimeWidget
#else
#include "QbTimeWidgets_qt4.h"
#define WIDGET_NAME QbDateTimeLineEdit
#endif

////////////////// QbDurationWidget ////////////////////

QbDurationWidget::QbDurationWidget(QWidget * parent)
	: QLineEdit(parent)
{
	setAcceptDrops(true);
	//QRegExp rx("^(\\s*\\-?\\d+\\.?\\d*[dhms]\\s*)(\\s\\-?\\d+\\.?\\d*[dhms]\\s*)*$");
	QRegExp rx("^(\\s*\\-?\\d*\\.?\\d*[dhms]\\s*)(\\s\\-?\\d*\\.?\\d*[dhms]\\s*)*$");
	validator=new QRegExpValidator(rx, this);
	setValidator(validator);
	
	active=true;
	
	activeColor=QColorGroup(QApplication::palette().active()).light();
	inactiveColor=QColorGroup(QApplication::palette().active()).midlight();
	activeText=QColorGroup(QApplication::palette().active()).text();
	inactiveText=QColorGroup(QApplication::palette().inactive()).text();
	
	QFontMetrics font_mt(qApp->font());
	int font_width = font_mt.width("n");
	setMinimumWidth(font_width*23);
}

bool QbDurationWidget::getDuration(Time& res)
{
	QString current_text=text();
	int length=current_text.length();
	
	if(validator->validate(current_text,length)!=QValidator::Acceptable)
		return false;
	
	double d(0.), m(0.), h(0.), s(0.);
	
#if QT_VERSION < 0x040000
	QStringList sl=QStringList::split(" ", current_text, false);
#else
	QStringList sl=current_text.split(" ", QString::SkipEmptyParts);
#endif
	
	for (QStringList::iterator iter = sl.begin(); iter != sl.end(); ++iter)
	{
		if((*iter).endsWith("d"))
		{
#if QT_VERSION < 0x040000
			(*iter).truncate((*iter).length()-1);
#else
			(*iter).chop(1);
#endif
			d+=(*iter).toDouble();
		}
		else if((*iter).endsWith("h"))
		{
#if QT_VERSION < 0x040000
			(*iter).truncate((*iter).length()-1);
#else
			(*iter).chop(1);
#endif
			h+=(*iter).toDouble();
		}
		else if((*iter).endsWith("m"))
		{
#if QT_VERSION < 0x040000
			(*iter).truncate((*iter).length()-1);
#else
			(*iter).chop(1);
#endif
			m+=(*iter).toDouble();
		}
		else if((*iter).endsWith("s"))
		{
#if QT_VERSION < 0x040000
			(*iter).truncate((*iter).length()-1);
#else
			(*iter).chop(1);
#endif
			s+=(*iter).toDouble();
		}
	}
	
	double seconds=d*24.*60.*60.+h*60.*60.+m*60.+s;
	
	res=QDOS::Seconds(seconds);
	
	return true;
}

void QbDurationWidget::setDuration(const Time& t)
{
	vector<string> srep;
	t.split_srep(srep);
	
	QString res;
	int year=QString((srep[0]).c_str()).toInt();
	int month=QString((srep[2]).c_str()).toInt();
	int day=QString((srep[5]).c_str()).toInt();
	QDate date(year, month, day);

	int days=-date.daysTo(QDate(2000,1,1));
	int h=QString((srep[6]).c_str()).toInt();
	int m=QString((srep[7]).c_str()).toInt();
	double s=QString((srep[8]).c_str()).toInt()+0.001*QString((srep[9]).c_str()).toInt();


	if(days!=0) res+=QString("%1d ").arg(days);
	
	if(h!=0)
		res+=QString("%1h ").arg(h);
	if(m!=0)
		res+=QString("%1m ").arg(m);
	if(s!=0.)
		res+=QString("%1s ").arg(s);
	
	setText(res);

}

void QbDurationWidget::focusInEvent(QFocusEvent *e)
{
	QLineEdit::focusInEvent(e);
	emit getFocus();
}

void QbDurationWidget::setActive(bool active)
{
	if(this->active!=active)
	{
		if(active)
		{
			setAcceptDrops(true);
			setPaletteBackgroundColor(activeColor);
			setPaletteForegroundColor(activeText);
		}
		else
		{
			setAcceptDrops(false);
			setPaletteBackgroundColor(inactiveColor);
			setPaletteForegroundColor(inactiveText);
		}
		this->active=active;
	}
}

void QbDurationWidget::dropEvent(QDropEvent *event)
{
	QdObject_var droppedObject;
#if QT_VERSION < 0x040000
	QString txt;
	QTextDrag::decode(event, txt);
	if(!txt) return;
	droppedObject = QuGetObjByName(txt);
#else
	droppedObject  = QuGetObjByName(event->mimeData()->text());
#endif
	
	if(droppedObject.is_nil()) return;
	
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		QDOS::Time t=int_var->end()-int_var->start();
		setDuration(t);
		return;
	}
	
	if(!ts_var.is_nil())
	{
		QDOS::Time t=ts_var->elt(ts_var->size()-1)-ts_var->elt(0);
		setDuration(t);
		return;
	}
	
	std::cout << "Dropped type not handled" << std::endl;
	
	return;

}

void QbDurationWidget::dragEnterEvent(QDragEnterEvent *event)
{
	if(!active) return;
	this->setActiveWindow();
	this->raise();
#if QT_VERSION < 0x040000
	event->accept(QTextDrag::canDecode(event));
#else
	if (event->mimeData()->hasText()) event->acceptProposedAction();
#endif
}

////////////////// QbDateTimeWidget ////////////////////

// #if QT_VERSION < 0x040000

WIDGET_NAME::WIDGET_NAME ( QWidget * parent) : QLineEdit(parent)
{
	QString rx_string("^("); // start of string
	rx_string+=QString("\\d{0,4}-"); // years
	rx_string+=QString("((0?[1-9])|(1[0-2]))-"); // months
	rx_string+=QString("((0?[1-9])|([12]\\d)|(3[01]))\\s"); // days in month
	rx_string+=QString("(((0|1)?\\d)|(2[0-3])):"); // hours
	rx_string+=QString("([0-5]?\\d):"); // min
	rx_string+=QString("([0-5]?\\d)\\."); // sec
	rx_string+=QString("[0-9]{0,3}"); // ms
	rx_string+=QString(")$"); // end of string
	QRegExp rx(rx_string);
	validator=new QRegExpValidator(rx, this);
	setValidator(validator);
	
	active=true;
	hasBeenEdited=false;

	connect(this, SIGNAL(textChanged(const QString &)), this , SLOT(slotValueChanged()));
	
	activeColor=QColorGroup(QApplication::palette().active()).light();
	inactiveColor=QColorGroup(QApplication::palette().active()).midlight();
	activeText=QColorGroup(QApplication::palette().active()).text();
	inactiveText=QColorGroup(QApplication::palette().inactive()).text();
	
	QFontMetrics font_mt(qApp->font());
	int font_width = font_mt.width("n");
	setMinimumWidth(font_width*23);
}

WIDGET_NAME::WIDGET_NAME ( const QDateTime & datetime, QWidget * parent) : QLineEdit(parent)
{
	QString rx_string("^("); // start of string
	rx_string+=QString("\\d{0,4}-"); // years
	rx_string+=QString("((0?[1-9])|(1[0-2]))-"); // months
	rx_string+=QString("((0?[1-9])|([12]\\d)|(3[01]))\\s"); // days in month
	rx_string+=QString("(((0|1)?\\d)|(2[0-3])):"); // hours
	rx_string+=QString("([0-5]?\\d):"); // min
	rx_string+=QString("([0-5]?\\d)\\."); // sec
	rx_string+=QString("[0-9]{0,3}"); // ms
	rx_string+=QString(")$"); // end of string
	QRegExp rx(rx_string);

	validator=new QRegExpValidator(rx, this);
	setValidator(validator);
	
	QString current=datetime.toString("yyyy-MM-dd hh:mm:ss.zzz");
	try
	{
		Time t(current.latin1());
		previousTime=t;
		setText(current);
	} catch(Exception & e){}
	
	active=true;
	hasBeenEdited=false;

	connect(this, SIGNAL(textChanged(const QString &)), this , SLOT(slotValueChanged()));
	
	activeColor=QColorGroup(QApplication::palette().active()).light();
	inactiveColor=QColorGroup(QApplication::palette().active()).midlight();
	activeText=QColorGroup(QApplication::palette().active()).text();
	inactiveText=QColorGroup(QApplication::palette().inactive()).text();
	
	QFontMetrics font_mt(qApp->font());
	int font_width = font_mt.width("n");
}

void WIDGET_NAME::keyPressEvent(QKeyEvent * e)
{
	QLineEdit::keyPressEvent(e);
	if(e->key()==Qt::Key_Up)
	{
		addToCurrent(1);
	}
	else if(e->key()==Qt::Key_Down)
	{
		addToCurrent(-1);
	}
}

void WIDGET_NAME::wheelEvent(QWheelEvent* e)
{
	if(e->delta()>0)
	{
		addToCurrent(+1);
	}
	else
	{
		addToCurrent(-1);
	}
}

void WIDGET_NAME::addToCurrent(int delta)
{
	int pos=cursorPosition();
	QString t=text();
	QStringList l=QStringList::split(QRegExp(":|-|\\s|\\."), t);
	int selected=0;
	if(l.size()<6) return;
	int boundaries[6];
	
	for(int i=0; i<6; ++i)
	{
		boundaries[i]=0;
		for(int j=0; j<=i; ++j)
		{
			boundaries[i]+=l[j].length()+1;
		}
	}
	
	if(pos<boundaries[0])
		selected=0;
	else if(pos>=boundaries[0] && pos<boundaries[1])
		selected=1;
	else if(pos>=boundaries[1] && pos<boundaries[2])
		selected=2;
	else if(pos>=boundaries[2] && pos<boundaries[3])
		selected=3;
	else if(pos>=boundaries[3] && pos<boundaries[4])
		selected=4;
	else if(pos>=boundaries[4] && pos<boundaries[5])
		selected=5;
	else selected=6;
	
	vector<string> decomp;
	previousTime.split_srep(decomp);
	
	int y, M, d, h, m, s, ms;
	y=QString(decomp[0].c_str()).toInt();
	M=QString(decomp[2].c_str()).toInt();
	d=QString(decomp[5].c_str()).toInt();
	h=QString(decomp[6].c_str()).toInt();
	m=QString(decomp[7].c_str()).toInt();
	s=QString(decomp[8].c_str()).toInt();
	ms=QString(decomp[9].c_str()).toInt();
	
	switch(selected)
	{
		case 0:
			y+=delta;
			break;
		case 1:
			M+=delta;
			break;
		case 2:
			d+=delta;
			break;
		case 3:
			h+=delta;
			break;
		case 4:
			m+=delta;
			break;
		case 5: 
			s+=delta;
			break;
		default:
			ms+=delta;
			break;
	}
	
	char iso[25];
	sprintf(iso, "%04d-%02d-%02d %02d:%02d:%02d.%03d", y, M, d, h, m, s, ms);

	try
	{
		Time newTime(iso);
		setText(newTime.iso().c_str()); // Not iso directly: passing through the Time constructor ensures the time is correct, else exception raised.
		setCursorPosition(pos);
	}
	catch(Exception &){/*Invalid time, do nothing*/}
}

Time WIDGET_NAME::time()
{
	Time res;
	try
	{
		res=Time(text().latin1());
		previousTime=res;
	}
	catch(Exception& e)
	{
		res=previousTime;
		setTime(res);
	}
	return res;
}

void WIDGET_NAME::setTime(const Time& t)
{
	this->blockSignals(true);
	previousTime=t;
	setText(t.iso().c_str());
	this->blockSignals(false);
}

void WIDGET_NAME::slotValueChanged()
{
	try
	{
		QDOS::Time test(text().latin1());
		previousTime=test;
	}
	catch(Exception & e)
	{
		setTime(previousTime);
		return;
	}
	hasBeenEdited=true;
	emit(valueChanged());
}

void WIDGET_NAME::setActive(bool active)
{
	if(this->active!=active)
	{
		if(active)
		{
			setAcceptDrops(true);
			setPaletteBackgroundColor(activeColor);
			setPaletteForegroundColor(activeText);
		}
		else
		{
			setAcceptDrops(false);
			setPaletteBackgroundColor(inactiveColor);
			setPaletteForegroundColor(inactiveText);
		}
		this->active=active;
	}
}

void WIDGET_NAME::focusInEvent(QFocusEvent *e)
{
	QLineEdit::focusInEvent(e);
	emit getFocus();
}

double WIDGET_NAME::value()
{
	return time().get_time_rep();
}

void WIDGET_NAME::setDouble(double value)
{
	Time slotTime;
	slotTime.set_time_rep(value);
	setTime(slotTime);
}

void WIDGET_NAME::mouseDoubleClickEvent ( QMouseEvent * event )
{
	int pos=cursorPosition();
	QString t=text();
	QStringList l=QStringList::split(QRegExp(":|-|\\s|\\."), t);
	int selected=0;
	if(l.size()<6) return;
	int boundaries[6];
	
	for(int i=0; i<6; ++i)
	{
		boundaries[i]=0;
		for(int j=0; j<=i; ++j)
		{
			boundaries[i]+=l[j].length()+1;
		}
	}
	
	if(pos<boundaries[0])
		selected=0;
	else if(pos>=boundaries[0] && pos<boundaries[1])
		selected=1;
	else if(pos>=boundaries[1] && pos<boundaries[2])
		selected=2;
	else if(pos>=boundaries[2] && pos<boundaries[3])
		selected=3;
	else if(pos>=boundaries[3] && pos<boundaries[4])
		selected=4;
	else if(pos>=boundaries[4] && pos<boundaries[5])
		selected=5;
	else selected=6;
	
	setSelection((selected==0?0:boundaries[selected-1]), l[selected].length());
}

#if QT_VERSION < 0x040000
void WIDGET_NAME::dropEvent(QDropEvent * event)
{
	QString txt;
	QTextDrag::decode(event, txt);
	if(!txt) return;
	droppedObject = QuGetObjByName(txt);
	
	if(droppedObject.is_nil()) return;
	
	QdTime_var time_var=QdTime_var::narrow(droppedObject);
	
	if(!time_var.is_nil()) // no time object can be dragged at the moment, but we never know... Anyway it's trivial
	{
		setText(time_var->iso().c_str());
		return;
	}
	
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil() || !ts_var.is_nil())
	{
		Q_POPUP_MENU * selectPopup=new Q_POPUP_MENU;
		selectPopup->insertItem("Start", this, SLOT(droppedObjStart()));
		selectPopup->insertItem("Centre", this, SLOT(droppedObjCentre()));
		selectPopup->insertItem("End", this, SLOT(droppedObjEnd()));
		selectPopup->insertItem("Cancel");
		
		selectPopup->exec(QCursor::pos());
		return;
	}
	
	std::cout << "Dropped type not handled" << std::endl;
	
	return;
}

void WIDGET_NAME::dragEnterEvent(QDragEnterEvent * e)
{
	if(!active) return;
	this->setActiveWindow();
	this->raise();
	e->accept(QTextDrag::canDecode(e));
}

void WIDGET_NAME::droppedObjStart()
{
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		setText(int_var->start().iso().c_str());
		return;
	}
	
	if(!ts_var.is_nil())
	{
		setText(ts_var->elt(0).iso().c_str());
		return;
	}
}

void WIDGET_NAME::droppedObjCentre()
{
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		Time t=(int_var->start()+int_var->end())/2.;
		setText(t.iso().c_str());
		return;
	}
	
	if(!ts_var.is_nil())
	{
		Time t=(ts_var->elt(0)+ts_var->elt(ts_var->size()-1))/2.;
		setText(t.iso().c_str());
		return;
	}
}

void WIDGET_NAME::droppedObjEnd()
{
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		setText(int_var->end().iso().c_str());
		return;
	}
	
	if(!ts_var.is_nil())
	{
		setText(ts_var->elt(ts_var->size()-1).iso().c_str());
		return;
	}
}
#endif

////////////////// QbDateTimeWidget ////////////////////////

#if QT_VERSION >= 0x040000
QbDateTimeWidget::QbDateTimeWidget(const QDateTime& dt, QWidget* parent):
		QAbstractSpinBox(parent)
{
	editor=new QbDateTimeLineEdit(dt, parent);
	setLineEdit(editor);
	setAcceptDrops(true);
	
	connect(editor, SIGNAL(getFocus()), this, SIGNAL(getFocus()));
	connect(editor, SIGNAL(valueChanged()), this, SIGNAL(valueChanged()));
	connect(editor, SIGNAL(textEdited(const QString&)), this, SIGNAL(textEdited(const QString&)));
	
	QFontMetrics font_mt(qApp->font());
	int font_width = font_mt.width("n");
	setMinimumWidth(font_width*25);
}

QbDateTimeWidget::QbDateTimeWidget(QWidget* parent):
		QAbstractSpinBox(parent)
{
	editor=new QbDateTimeLineEdit(parent);
	setLineEdit(editor);
	setAcceptDrops(true);
	
	connect(editor, SIGNAL(getFocus()), this, SIGNAL(getFocus()));
	connect(editor, SIGNAL(valueChanged()), this, SIGNAL(valueChanged()));
	connect(editor, SIGNAL(textEdited(const QString&)), this, SIGNAL(textEdited(const QString&)));
	
	QFontMetrics font_mt(qApp->font());
	int font_width = font_mt.width("n");
	setMinimumWidth(font_width*25);
}

void QbDateTimeWidget::stepBy(int steps)
{
	editor->addToCurrent(steps);
}

QAbstractSpinBox::StepEnabled QbDateTimeWidget::stepEnabled () const
{
	return QAbstractSpinBox::StepUpEnabled|QAbstractSpinBox::StepDownEnabled;
}


void QbDateTimeWidget::dropEvent(QDropEvent * event)
{
	droppedObject  = QuGetObjByName(event->mimeData()->text());
	
	if(droppedObject.is_nil()) return;
	
	QdTime_var time_var=QdTime_var::narrow(droppedObject);
	
	if(!time_var.is_nil()) // no time object can be dragged at the moment, but we never know... Anyway it's trivial
	{
		editor->setText(time_var->iso().c_str());
		return;
	}
	
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil() || !ts_var.is_nil())
	{
		Q_POPUP_MENU * selectPopup=new Q_POPUP_MENU;
		selectPopup->insertItem("Start", this, SLOT(droppedObjStart()));
		selectPopup->insertItem("Centre", this, SLOT(droppedObjCentre()));
		selectPopup->insertItem("End", this, SLOT(droppedObjEnd()));
		selectPopup->insertItem("Cancel");
		
		selectPopup->exec(QCursor::pos());
		return;
	}
	
	std::cout << "Dropped type not handled" << std::endl;
	
	return;
}

void QbDateTimeWidget::dragEnterEvent(QDragEnterEvent * e)
{
	if(!editor->active) return;
	this->setActiveWindow();
	this->raise();
	if (e->mimeData()->hasText()) e->acceptProposedAction();
}

void QbDateTimeWidget::droppedObjStart()
{
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		editor->setText(int_var->start().iso().c_str());
		return;
	}
	
	if(!ts_var.is_nil())
	{
		editor->setText(ts_var->elt(0).iso().c_str());
		return;
	}
}

void QbDateTimeWidget::droppedObjCentre()
{
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		Time t=(int_var->start()+int_var->end())/2.;
		editor->setText(t.iso().c_str());
		return;
	}
	
	if(!ts_var.is_nil())
	{
		Time t=(ts_var->elt(0)+ts_var->elt(ts_var->size()-1))/2.;
		editor->setText(t.iso().c_str());
		return;
	}
}

void QbDateTimeWidget::droppedObjEnd()
{
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
	if(!int_var.is_nil())
	{
		editor->setText(int_var->end().iso().c_str());
		return;
	}
	
	if(!ts_var.is_nil())
	{
		editor->setText(ts_var->elt(ts_var->size()-1).iso().c_str());
		return;
	}
}
#endif

////////////////// QbTimeIntervalWidget ////////////////////

QbTimeIntervalWidget::QbTimeIntervalWidget(QWidget* parent)
	: QWidget(parent)
{
	layout=new QGridLayout(this);

	setAcceptDrops(true);
	
	layout->addWidget(new QLabel("Start time:", this), 0, 0);
	
#if QT_VERSION >= 0x040000 && QT_VERSION < 0x040300
	_start_time_field = new QbDateTimeLineEdit(QDateTime::currentDateTime(), this);
#else
	_start_time_field = new QbDateTimeWidget(QDateTime::currentDateTime(), this);
#endif

	connect(_start_time_field, SIGNAL(getFocus()), this, SLOT(startFocus()));
	connect(_start_time_field, SIGNAL(valueChanged()), this , SLOT(updateFields()));
	
	layout->addWidget(_start_time_field, 1, 0);

	layout->addWidget(new QLabel("End time:", this), 0, 1);
	
#if QT_VERSION >= 0x040000 && QT_VERSION < 0x040300
	_end_time_field = new QbDateTimeLineEdit(QDateTime::currentDateTime().addDays(7), this);
#else
	_end_time_field = new QbDateTimeWidget(QDateTime::currentDateTime().addDays(7), this);
#endif
	connect(_end_time_field, SIGNAL(getFocus()), this, SLOT(endFocus()));
	connect(_end_time_field, SIGNAL(valueChanged()), this , SLOT(updateFields()));
	
	layout->addWidget(_end_time_field, 1, 1);
	
	layout->addWidget(new QLabel("Centre time:", this), 3, 0);
	
#if QT_VERSION >= 0x040000 && QT_VERSION < 0x040300
	_centre_time_field = new QbDateTimeLineEdit(QDateTime::currentDateTime(), this);
#else
	_centre_time_field = new QbDateTimeWidget(QDateTime::currentDateTime(), this);
#endif

	connect(_centre_time_field, SIGNAL(getFocus()), this, SLOT(centreFocus()));
	connect(_centre_time_field, SIGNAL(valueChanged()), this , SLOT(updateFields()));
	
	layout->addWidget(_centre_time_field, 4, 0);

	
	layout->addWidget(new QLabel("Duration:", this), 3, 1);
	
	_duration_field = new QbDurationWidget(this);
	_duration_field->setAcceptDrops(false);
	QToolTip::add( _duration_field, \
			"Any combination of d, h, m, s e.g. 10d 2h 3m 1.001s ");
	layout->addWidget(_duration_field, 4, 1);

	connect(_duration_field, SIGNAL(getFocus()), this, SLOT(durationFocus()));
	connect(_duration_field, SIGNAL(textChanged(const QString &)), this, SLOT(updateFields()));

	QLabel * lbl=new QLabel("Type into a field to make it active and change its contents\nor\nDrop a time interval/time tagged object on an active slot to change the slot\nor\nDrop an object elsewhere set the whole interval", this);
	
	lbl->setAlignment(Qt::AlignCenter);
	
#if QT_VERSION < 0x040000
	layout->addMultiCellWidget(lbl, 5, 5, 0, 1);
#else
	layout->addWidget(lbl, 5, 0, 1, 2);
#endif
// layout->addWidget(new QbDateTimeWidget3(this), 6, 0);
	layout->setColStretch(0,1);
	layout->setColStretch(1,1);
	layout->setSpacing(4);


	allowUpdate=true;
	
	current=1;
	previous=2;
	updateSelection();
}

void QbTimeIntervalWidget::dropEvent(QDropEvent *event)
{
#if QT_VERSION < 0x040000
	QString txt;
	QTextDrag::decode(event, txt);
	if(!txt) return;
	QdObject_var xref_obj = QuGetObjByName(txt);
#else
	QdObject_var xref_obj = QuGetObjByName(event->mimeData()->text());
#endif
	setTimeInterval(xref_obj);
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(xref_obj);
	emit intervalUpdated();

}

void QbTimeIntervalWidget::setTimeInterval(QdObject_var obj)
{
	if(obj.is_nil()) return;

	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(obj);
	if(!int_var.is_nil())
	{
		int p(previous), c(current);
		previous=0;
		current=0;
		_start_time_field->setTime(int_var->start());
		_end_time_field->setTime(int_var->end());
		updateCentreDuration();
		previous=p;
		current=c;
		emit intervalUpdated();
		return;
	}
	
	QdTimeSeq_var ts_var=get_timetags(obj);
	
	if(!ts_var.is_nil())
	{
		int p(previous), c(current);
		previous=0;
		current=0;
		_start_time_field->setTime(ts_var->elt(0));
		_end_time_field->setTime(ts_var->elt(ts_var->size()-1));
		updateCentreDuration();
		previous=p;
		current=c;
		emit intervalUpdated();
		return;
	}
	
	return;	
}

void QbTimeIntervalWidget::setTimeInterval(const QDOS::TimeInterval& obj)
{
	int p(previous), c(current);
	previous=0;
	current=0;
	_start_time_field->setTime(obj.start());
	_end_time_field->setTime(obj.end());
	updateCentreDuration();
	previous=p;
	current=c;
}

QdTimeInterval_var QbTimeIntervalWidget::getTimeInterval()
{
	return new QdTimeInterval(_start_time_field->time(), _end_time_field->time());
}

Time QbTimeIntervalWidget::getDuration()
{
	Time res;
	_duration_field->getDuration(res);
	return res;
}

void QbTimeIntervalWidget::dragEnterEvent(QDragEnterEvent *event)
{
	this->setActiveWindow();
	this->raise();
#if QT_VERSION < 0x040000
	event->accept(QTextDrag::canDecode(event));
#else
	if (event->mimeData()->hasText()) event->acceptProposedAction();
#endif
}

void QbTimeIntervalWidget::updateStartEnd()
{
	Time centre=_centre_time_field->time();
	Time duration;
	if(_duration_field->getDuration(duration))
	{
		Time start=centre-duration/2.;
		Time end=centre+duration/2.;
		_start_time_field->setTime(start);
		_end_time_field->setTime(end);
	}
}

void QbTimeIntervalWidget::updateStartCentre()
{
	Time end=_end_time_field->time();
	Time duration;
	if(_duration_field->getDuration(duration))
	{
		Time start=end-duration;
		Time centre=end-duration/2.;
		_start_time_field->setTime(start);
		_centre_time_field->setTime(centre);
	}
}

void QbTimeIntervalWidget::updateStartDuration()
{
	Time end=_end_time_field->time();
	Time centre=_centre_time_field->time();
	_start_time_field->setTime(end-2.*(end-centre));
	Time duration=(end-centre)*2.;
	_duration_field->setDuration(duration);
}

void QbTimeIntervalWidget::updateEndCentre()
{
	Time start=_start_time_field->time();
	Time duration;
	Time test=duration;
	if(_duration_field->getDuration(duration))
	{
		Time end=start+duration;
		Time centre=start+duration/2.;
		_end_time_field->setTime(end);
		_centre_time_field->setTime(centre);
	}
}

void QbTimeIntervalWidget::updateEndDuration()
{
	Time start=_start_time_field->time();
	Time centre=_centre_time_field->time();
	_end_time_field->setTime(start+2.*(centre-start));
	Time duration=(centre-start)*2.;
	_duration_field->setDuration(duration);
}

void QbTimeIntervalWidget::updateCentreDuration()
{
	Time start=_start_time_field->time();
	Time end=_end_time_field->time();
	_centre_time_field->setTime((start+end)/2.);
	Time duration=end-start;
	_duration_field->setDuration(duration);
}

void QbTimeIntervalWidget::startFocus()
{
	if(current!=1)
	{
		previous=current;
		current=1;
		updateSelection();
	}
}

void QbTimeIntervalWidget::endFocus()
{
	if(current!=2)
	{
		previous=current;
		current=2;
		updateSelection();
	}
}

void QbTimeIntervalWidget::centreFocus()
{
	if(current!=3)
	{
		previous=current;
		current=3;
		updateSelection();
	}
}

void QbTimeIntervalWidget::durationFocus()
{
	if(current!=4)
	{
		previous=current;
		current=4;
		updateSelection();
	}
}

void QbTimeIntervalWidget::updateSelection()
{
	if(current !=0 && previous !=0)
	{
		_start_time_field->setActive(current==1 || previous==1);
		_end_time_field->setActive(current==2 || previous==2);
		_centre_time_field->setActive(current==3 || previous==3);
		_duration_field->setActive(current==4 || previous==4);
	}
}

void QbTimeIntervalWidget::updateFields()
{
	if(current==0 || previous==0) return; // Changed by drop
	int c(current), p(previous);
	current=0;
	previous=0;

	if(c==1 || p==1) // One of the parameters is start
	{
		if(c==2 || p==2)
			updateCentreDuration();
		else if(c==3 || p==3)
			updateEndDuration();
		else updateEndCentre();
	}
	else if(c==2 || p==2) // One of the params is end, the other is not start
	{
		if(c==3 || p==3)
			updateStartDuration();
		else updateStartCentre();
	}
	else updateStartEnd();
	
	previous=p;
	current=c;
	
	emit intervalUpdated();
}




