//  qbu.cc
//
// Data Slice Widget
//
// April 2006: Tony Allen
//

#include <stdlib.h>

#include <qpushbutton.h>
#include <qlayout.h>
#include "qslot.h"
#include "qwui.h"

using namespace QSAS;

// ----------------------------------------------------------------------------------

DataSliceWindow::DataSliceWindow(QWidget *parent, const char *name ) 
  : QMainWindow(parent,name)
#if QT_VERSION < 0x040000
{

 if(QuSlotDebug())
 {
  printf("DataSliceWindow::DataSliceWindow \n");
  fflush(stdout);
 }
  
   this->setPalette(qApp->palette());

   QFontMetrics qgu_font_mt(qApp->font());
   int char_width = qgu_font_mt.width("n");
   
   QWidget::setWFlags(Qt::WDestructiveClose);
  
   msgBar = statusBar();
   
   QVGroupBox *topVB = new QVGroupBox( this, "TopWidget" );
   setCentralWidget( topVB );

   QHGroupBox *slotPanel = new QHGroupBox(topVB, "slot");
   slotPanel->setTitle("Data Object");
   
   objField = new QguWLinput(slotPanel,"objField");
   objField->acceptObjectPlus = true; 
   objField->acceptTimeSubset = true;
   objField->acceptTimeSlice = true;
   objField->acceptTIvlSeqObject = true;
   objField->setEnabled(true);
   objField->setMinimumWidth(char_width*55);

   connect(objField, SIGNAL(validated_OK()), this, SLOT(setNewName()));
   
   menuBar()->insertSeparator();   
   QPopupMenu * help = new QPopupMenu( this );
   menuBar()->insertItem( "  Help ", help );
//   help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), CTRL+Key_M );
   help->insertSeparator();
 //help->insertItem("...Slice", this, SLOT( launchManSlice() ));
   help->insertSeparator();
//   help->insertItem("Module Version", this, SLOT( launchVersion() ));
 
   QHGroupBox *name_panel=new QHGroupBox(topVB, "name");
   
   QLabel *name_label = new QLabel("Save as ", name_panel);
   name_field = new QguWLinput(name_panel, "name_field");
   name_field->acceptText = true;
   name_field->setEnabled(true);
   name_field->setMinimumWidth(char_width*50);
   userSet = false;
   connect( name_field,  SIGNAL( textChanged(const QString &) ), this, SLOT( setUserInput(const QString &) ) );

  QHGroupBox *buttons_layout=new QHGroupBox(topVB, "buttons");
    
  saveBtn = new QPushButton( "Save ", buttons_layout, "saveBtn" );
  saveBtn->setMinimumSize(saveBtn->sizeHint());
  connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptSlice()) );
  saveBtn->setFocusPolicy(STRONG_FOCUS);

  dismissBtn = new QPushButton( "Dismiss", buttons_layout, "dismissBtn" );
  dismissBtn->setMinimumSize(dismissBtn->sizeHint());
  connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
  dismissBtn->setDefault(true);
  
  this->setMinimumSize(this->sizeHint());
}
#else
{

	if(QuSlotDebug())
	{
		printf("DataSliceWindow::DataSliceWindow \n");
		fflush(stdout);
	}
  
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->acceptObjectPlus = true; 
	objField->acceptTimeSubset = true;
	objField->acceptTimeSlice = true;
	objField->acceptTIvlSeqObject = true;
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(validated_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->acceptText = true;
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( textChanged(const QString &) ), this, SLOT( setUserInput(const QString &) ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptSlice()) );
	saveBtn->setFocusPolicy(STRONG_FOCUS);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}
#endif

void DataSliceWindow::setUserInput(const QString &txt){
  if(name_field->lineBox->hasFocus()){
     if(txt.isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void DataSliceWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate();
   
}

void DataSliceWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().latin1());
   
   if(!objName.isEmpty()){
       QString newName = QString(objName);
       QdObject_var obj = QuGetObjByName(objName);
       QdTimeIntervalSeq_var test_tis=QdTimeIntervalSeq_var::narrow(obj);
       
     int ptr = newName.findRev("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.find(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += "_S";
     if(!test_tis.is_nil()) newName="Time Intervals/"+newName;
     this->name_field->setText(newName);
   }
   

}

void DataSliceWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptSlice();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void DataSliceWindow::acceptSlice(){
      
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::acceptSlice \n");
	fflush(stdout);
 }


  QString sliceString = objField->text();
  
  QdObject_var slicedObject = QuGetObjByNameAndComp(sliceString);
  // this may be the original untouched object, so duplicate to be safe...
  
  QString newName = name_field->text();
  if( !(slicedObject.is_nil() ) ) QwuiPutOnWL(slicedObject, newName.latin1(), "Data");
  else QuMsgBar("NULL object after slice", true);

  this->close();
  return;
}


// class TimeSliceWindow
// ------------------------

TimeSliceWindow::TimeSliceWindow(QString WLcontext, const char * source, QWidget *parent, const char *name, 
                                     bool acceptTSubset, bool acceptTSlice, bool modal )
  : QDialog(parent,name,modal)
#if QT_VERSION < 0x040000
{  
      
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::TimeSliceWindow \n");
	fflush(stdout);
 }

  QdObject_var dobj = QuGetObjByName(WLcontext);

  QWidget::setWFlags(Qt::WDestructiveClose);

  QVBoxLayout *slice_window_top_layout=new QVBoxLayout(this);
  slice_window_top_layout->setSpacing(4);

  QMenuBar * menu_bar = new QMenuBar(this, "menuBar");
  slice_window_top_layout->addWidget(menu_bar);

  Q_POPUP_MENU * help = new Q_POPUP_MENU( this );
  menu_bar->insertItem( " Help", help );
 // help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), CTRL+Key_M );
  help->insertSeparator();
  menu_bar->setFixedSize(menu_bar->sizeHint());

  this->setCaption(WLcontext);

  QFrame * underline = new QFrame(this, "underline");
  underline->setFrameStyle(QFrame::HLine | QFrame::Raised);
  underline->setLineWidth(1);
  slice_window_top_layout->addWidget(underline);
  
  QString feedbackMsg;
  Q_V_GROUP_BOX *componentBox;
  if ( strcmp(source, "T") == 0 ){
	feedbackMsg = QString("Subset or slice a time sequence");
    vectorChoice = 0;  
  }
  else if ( strcmp(source, "V") == 0 ||  strcmp(source, "V+") == 0 ){
	
	componentBox = new Q_V_GROUP_BOX( "Select Component", this, "componentBox");
    slice_window_top_layout->addWidget(componentBox);
    vectorChoice = new QComboBox(componentBox, "vectorChoice"); 
	if ( strcmp(source, "V+") == 0 ) vectorChoice->insertItem("This Object");	
    vectorChoice->insertItem(" r (mag) ");
    vectorChoice->insertItem(" x ");
    vectorChoice->insertItem(" y ");
    vectorChoice->insertItem(" z ");
    vectorChoice->insertItem(" Lat (deg) ");
    vectorChoice->insertItem(" theta (deg) ");
    vectorChoice->insertItem(" phi (deg) ");
    vectorChoice->insertItem(" Lat (rad) ");
    vectorChoice->insertItem(" theta (rad) ");
    vectorChoice->insertItem(" phi (rad) ");
	vectorChoice->adjustSize();
	feedbackMsg = QString("Input is a vector sequence"); 

   QFontMetrics slice_font_mt(qApp->font());
   int char_height = slice_font_mt.height();
   componentBox->setInsideMargin((int)(char_height*.7)+1); // add 1 to ensure rounding never gives zero
   componentBox->setFixedHeight((int)(char_height*3.5)+1);
  }
  else {
	feedbackMsg = QString("Input is a scalar sequence");
    vectorChoice = 0;
  }
  
  if( !dobj.is_nil() &&(acceptTSlice || acceptTSubset) ) {
      slicer = new timeSlider(dobj, acceptTSlice, acceptTSubset, this);
      slice_window_top_layout->addWidget(slicer);
	  slicer->setTitle("Time Slice Options");
  }
  else slicer = 0;
   
   
  buttons_layout=new QHGroupBox(this, "buttons");

  QPushButton *acceptBtn = new QPushButton( "Accept", buttons_layout, "acceptButton" );
  acceptBtn->setMinimumSize(acceptBtn->sizeHint());
  connect( acceptBtn, SIGNAL(clicked()), this, SLOT(acceptSelection()) );
  acceptBtn->setFocusPolicy(STRONG_FOCUS);
  acceptBtn->setDefault(true);

  QPushButton *setDefaultBtn = new QPushButton( "Reset", buttons_layout, "setDefaultBtn" );
  setDefaultBtn->setMinimumSize(setDefaultBtn->sizeHint());
  connect( setDefaultBtn, SIGNAL(clicked()), this, SLOT(setDefaults()) );

  QPushButton *dismissBtn = new QPushButton( "Cancel", buttons_layout, "dismissBtn" );
  dismissBtn->setMinimumSize(dismissBtn->sizeHint());
  connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );

  
  buttons_layout->setAlignment(Qt::AlignHCenter);
  buttons_layout->setTitle(feedbackMsg);
  slice_window_top_layout->addWidget(buttons_layout);
  buttons_layout->setMinimumSize(buttons_layout->sizeHint());

  this->setMinimumSize(this->sizeHint());

}
#else
{  
      
	if(QuSlotDebug())
	{
		printf("TimeSliceWindow::TimeSliceWindow \n");
		fflush(stdout);
	}

	QdObject_var dobj = QuGetObjByName(WLcontext);

	setAttribute(Qt::WA_DeleteOnClose);

	this->setCaption(WLcontext);
	QString feedbackMsg;
	
	QVBoxLayout * mainLayout=new QVBoxLayout;
	mainLayout->setSpacing(4);
	
	QGroupBox * componentBox;
	if ( strcmp(source, "T") == 0 ){
		feedbackMsg = QString("Subset or slice a time sequence");
		vectorChoice = 0;  
	}
	else if ( strcmp(source, "V") == 0 ||  strcmp(source, "V+") == 0 )
	{
	
		componentBox = new QGroupBox( "Select Component", this, "componentBox");
		QVBoxLayout * componentLayout=new QVBoxLayout;
		
		vectorChoice = new QComboBox(componentBox, "vectorChoice"); 
		if ( strcmp(source, "V+") == 0 ) vectorChoice->insertItem("This Object");	
		vectorChoice->insertItem(" r (mag) ");
		vectorChoice->insertItem(" x ");
		vectorChoice->insertItem(" y ");
		vectorChoice->insertItem(" z ");
		vectorChoice->insertItem(" Lat (deg) ");
		vectorChoice->insertItem(" theta (deg) ");
		vectorChoice->insertItem(" phi (deg) ");
		vectorChoice->insertItem(" Lat (rad) ");
		vectorChoice->insertItem(" theta (rad) ");
		vectorChoice->insertItem(" phi (rad) ");
		vectorChoice->adjustSize();
		componentLayout->addWidget(vectorChoice);
		
		feedbackMsg = QString("Input is a vector sequence"); 

		QFontMetrics slice_font_mt(qApp->font());
		int char_height = slice_font_mt.height();
// 		componentBox->setInsideMargin((int)(char_height*.7)+1); // add 1 to ensure rounding never gives zero
// 		componentBox->setFixedHeight((int)(char_height*3.5)+1);
		
		componentBox->setLayout(componentLayout);
		mainLayout->addWidget(componentBox);
	}
	else {
		feedbackMsg = QString("Input is a scalar sequence");
		vectorChoice = 0;
	}
	
	if( !dobj.is_nil() &&(acceptTSlice || acceptTSubset) ) {
		slicer = new timeSlider(dobj, acceptTSlice, acceptTSubset, this);
		slicer->setTitle("Time Slice Options");
		mainLayout->addWidget(slicer);
	}
	else slicer = 0;
	
	buttons_groupbox=new QGroupBox(this);
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	QPushButton *acceptBtn = new QPushButton( "Accept");
	acceptBtn->setMinimumSize(acceptBtn->sizeHint());
	connect( acceptBtn, SIGNAL(clicked()), this, SLOT(acceptSelection()) );
	acceptBtn->setFocusPolicy(STRONG_FOCUS);
	acceptBtn->setDefault(true);
	buttons_layout->addWidget(acceptBtn);

	QPushButton *setDefaultBtn = new QPushButton( "Reset");
	setDefaultBtn->setMinimumSize(setDefaultBtn->sizeHint());
	connect( setDefaultBtn, SIGNAL(clicked()), this, SLOT(setDefaults()) );
	buttons_layout->addWidget(setDefaultBtn);

	QPushButton *dismissBtn = new QPushButton( "Cancel");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	buttons_groupbox->setAlignment(Qt::AlignHCenter);
	buttons_groupbox->setTitle(feedbackMsg);
	buttons_groupbox->setMinimumSize(buttons_layout->sizeHint());
	
	buttons_groupbox->setLayout(buttons_layout);
	mainLayout->addWidget(buttons_groupbox);
	
	setLayout(mainLayout);
	
}
#endif


void TimeSliceWindow::updateFeedback(){
  if(QuSlotDebug())
  {
	printf("TimeSliceWindow::updateFeedback %ld\n", this);
	fflush(stdout);
  }

  QString newTitle;
  if( vectorChoice != 0)  {
     newTitle =  QString("  Output is %1 component of a vector sequence").arg(vectorChoice->currentText());
  }
  else  newTitle =  QString("  Output is a scalar sequence") ;
   
  if (slicer != 0){
     if(slicer->sliceChoice->currentItem() == slicer->Isubset){
	   int nrec = slicer->toSlider->record - slicer->fromSlider->record + 1;
	    newTitle += QString(", subset of %1 records  ").arg(nrec);
	 }
     if(slicer->sliceChoice->currentItem() == slicer->Iaverage){
	   int nrec = slicer->toSlider->record - slicer->fromSlider->record + 1;
	    newTitle += QString(", averaged over %1 records to single value  ").arg(nrec);
	 }
	 if(slicer->sliceChoice->currentItem() == slicer->Islice){
	    newTitle += QString(", value at record %1  ").arg(slicer->atSlider->record);
	 }
  }
  else  newTitle += "  ";
   
#if QT_VERSION < 0x040000
  buttons_layout->setTitle( newTitle);
#else
  buttons_groupbox->setTitle( newTitle);
#endif

}

void TimeSliceWindow::dismiss(){
  close();
}

void TimeSliceWindow::setDefaults(){
    
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::setDefaults \n");
	fflush(stdout);
  }

   if( vectorChoice != 0)  vectorChoice->setCurrentItem(0);
   
   if (slicer != 0){
      if(slicer->sliceChoice->currentItem() == slicer->Isubset){
         slicer->fromSlider->adjustRec(0);
         slicer->toSlider->adjustRec(slicer->toSlider->maxRecord);
	  }
      if(slicer->sliceChoice->currentItem() == slicer->Iaverage){
         slicer->fromSlider->adjustRec(0);
         slicer->toSlider->adjustRec(slicer->toSlider->maxRecord);
	  }
	  if(slicer->sliceChoice->currentItem() == slicer->Islice){
         slicer->atSlider->adjustRec(0);
	  }
   }

}

void TimeSliceWindow::setToLastChoice(){
  
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::setToLastChoice \n");
	fflush(stdout);
 }

  if(vectorChoice != 0) {
     QguComp_e component = parentSlot->getComponent();
	 if ( component >= 0 && component <= 10){
	     if(vectorChoice->text(0) != "This Object" ) vectorChoice->setCurrentItem(component -1);
	     else vectorChoice->setCurrentItem(component);
	}
  }
  
   dimTriple sliceTriple = parentSlot->parseTimeSlice();
   if(sliceTriple.size() == 3 && slicer != 0){
      if(sliceTriple[0] == 0 ) {
	     slicer->sliceChoice->setCurrentItem(slicer->Isubset);
         slicer->fromSlider->recBar->setValue(sliceTriple[1]);
         slicer->toSlider->recBar->setValue(sliceTriple[2]);
	  }
	  else if(sliceTriple[0] == 1) {
	     slicer->sliceChoice->setCurrentItem(slicer->Iaverage);
		 slicer->fromSlider->recBar->setValue(sliceTriple[1]);
		 slicer->toSlider->recBar->setValue(sliceTriple[2]);
	  }
	  else if(sliceTriple[0] == 2) {
	     slicer->sliceChoice->setCurrentItem(slicer->Islice);
		 slicer->atSlider->recBar->setValue(sliceTriple[1]);
		 slicer->updateTimeSlider(slicer->Islice);
	  }
   }

}


void TimeSliceWindow::acceptSelection(){
  
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::acceptSelection \n");
	fflush(stdout);
 }

  QString newText = parentSlot->textObject();
  QString cmpText = "";
  
  if ( vectorChoice != 0 && vectorChoice->currentText() != QString("This Object"))  {
    cmpText += " [";
	cmpText += vectorChoice->currentText();
    cmpText += "]";
  }
  
  if(slicer != 0 && !(slicer->isDefault()) ){
    
     cmpText += " < ";

     // Time slice/subset available
	 if(slicer->sliceChoice->currentItem() == slicer->Isubset){
		QString start = slicer->fromSlider->recField->text();      
		QString end   = slicer->toSlider->recField->text();
	 	cmpText += start;
	 	cmpText += ",";
	 	cmpText += end;
	 }
	 else if(slicer->sliceChoice->currentItem() == slicer->Iaverage){
		QString start = slicer->fromSlider->recField->text();      
		QString end   = slicer->toSlider->recField->text();
	 	cmpText += "Ave ";
	 	cmpText += start;
	 	cmpText += ",";
	 	cmpText += end;
	 }
	 else if(slicer->sliceChoice->currentItem() == slicer->Islice){
		QString atT = slicer->atSlider->recField->text();      
	 	cmpText += atT;
	 }
	 
     cmpText += " >";
  }


  newText += cmpText;
  parentSlot->setText(newText);
  parentSlot->reValidate();
  parentSlot->emitCmpSelected(cmpText.latin1(), "SLICE");
  
  close();
}

// class SelectTivlWindow
// ------------------------

SelectTivlWindow::SelectTivlWindow(QString WLcontext, const char * source, QWidget *parent, const char *name, bool modal )
    : QDialog(parent,name,modal), record(0)
{  
      
    if(QuSlotDebug())
    {
		printf("SelectTivlWindow::SelectTivlWindow \n");
		fflush(stdout);
	}

    QdObject_var dobj = QuGetObjByName(WLcontext);
    
    obj=QdTimeIntervalSeq_var::narrow(dobj);
    
    max_rec=obj->sequence_size()-1;

#if QT_VERSION < 0x040000
    QWidget::setWFlags(Qt::WDestructiveClose);
#else
    setAttribute(Qt::WA_DeleteOnClose);
#endif
    
    QVBoxLayout * main_layout=new QVBoxLayout(this);
    main_layout->setSpacing(4);
    
    slider=new TimeIntervalSlider("", this, "slider");
    slider->maxRecord=max_rec;
    slider->recBar->setMaxValue(max_rec);
    slider->obj=obj;
    slider->updateTitle();
    main_layout->addWidget(slider);
    
    QHBoxLayout * buttons_layout=new QHBoxLayout(main_layout);
    validateBtn=new QPushButton("OK", this, "OKBtn");
    validateBtn->setEnabled(true);
    validateBtn->setFocusPolicy(STRONG_FOCUS);
    
    cancelBtn=new QPushButton("Cancel", this, "CancelBtn");
    cancelBtn->setEnabled(true);
    cancelBtn->setFocusPolicy(STRONG_FOCUS);
    
    buttons_layout->addWidget(validateBtn);
    buttons_layout->addWidget(cancelBtn);

    connect(validateBtn, SIGNAL(clicked()), this, SLOT(validated()));
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(cancelled()));
    
    this->setMinimumSize(this->sizeHint());

}

void SelectTivlWindow::validated()
{
    if(QuSlotDebug())
    {
		printf("SelectTivlWindow::validated \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    QString cmpText = "";
    
    cmpText += " < ";
    cmpText += tr("%1").arg(slider->record);
    cmpText += " >";

    newText += cmpText;
    parentSlot->setText(newText);
    parentSlot->reValidate();
    parentSlot->emitCmpSelected(cmpText.latin1(), "IVL");
    
    close();
}

void SelectTivlWindow::cancelled()
{
    if(QuSlotDebug())
    {
		printf("TimeSliceWindow::acceptSelection \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    parentSlot->setText(newText);
    parentSlot->reValidate();
  
    close();
}

// class SelectTivlSubsetWindow
// ------------------------

SelectTivlSubsetWindow::SelectTivlSubsetWindow(QString WLcontext, const char * source, QWidget *parent, const char *name, bool modal )
    : QDialog(parent,name,modal), record_start(0)
{  
      
    if(QuSlotDebug())
    {
		printf("SelectTivlWindow::SelectTivlWindow \n");
		fflush(stdout);
	}

    QdObject_var dobj = QuGetObjByName(WLcontext);
    
    obj=QdTimeIntervalSeq_var::narrow(dobj);
    
    max_rec=obj->sequence_size()-1;
    record_end=max_rec;

#if QT_VERSION < 0x040000
    QWidget::setWFlags(Qt::WDestructiveClose);
#else
    setAttribute(Qt::WA_DeleteOnClose);
#endif
    
    QVBoxLayout * main_layout=new QVBoxLayout(this);
    main_layout->setSpacing(4);

    slider1=new TimeIntervalSlider("From", this, "slider1");
    slider1->maxRecord=max_rec;
    slider1->recBar->setMaxValue(max_rec);
    slider1->obj=obj;
    slider1->updateTitle();
    main_layout->addWidget(slider1);
    
    slider2=new TimeIntervalSlider("To", this, "slider2");
    slider2->maxRecord=max_rec;
    slider2->record=max_rec;
    slider2->recBar->setMaxValue(max_rec);
    slider2->recBar->setValue(max_rec);
    slider2->obj=obj;
    slider2->updateTitle();
    main_layout->addWidget(slider2);

    QHBoxLayout * buttons_layout=new QHBoxLayout(main_layout);
    validateBtn=new QPushButton("OK", this, "OKBtn");
    validateBtn->setEnabled(true);
    validateBtn->setFocusPolicy(STRONG_FOCUS);
    
    cancelBtn=new QPushButton("Cancel", this, "CancelBtn");
    cancelBtn->setEnabled(true);
    cancelBtn->setFocusPolicy(STRONG_FOCUS);
    
    buttons_layout->addWidget(validateBtn);
    buttons_layout->addWidget(cancelBtn);

    connect(validateBtn, SIGNAL(clicked()), this, SLOT(validated()));
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(cancelled()));
    
    this->setMinimumSize(this->sizeHint());

}

void SelectTivlSubsetWindow::validated()
{
    if(QuSlotDebug())
    {
		printf("SelectTivlWindow::validated \n");
		fflush(stdout);
	}
    
    QString newText = parentSlot->textObject();
    QString cmpText = "";
    
    int record1=slider1->record;
    int record2=slider2->record;
    if(record1>record2)
    {
        int temp=record1;
        record1=record2;
        record2=record1;
    }

        cmpText += " < ";
        cmpText += tr("%1").arg(record1)+ ","+tr("%1").arg(record2);
        cmpText += " >";

    newText += cmpText;
    parentSlot->setText(newText);
    parentSlot->reValidate();
    parentSlot->emitCmpSelected(cmpText.latin1(), "IVL");
    
    close();
}

void SelectTivlSubsetWindow::cancelled()
{
    if(QuSlotDebug())
    {
		printf("TimeSliceWindow::acceptSelection \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    parentSlot->setText(newText);
    parentSlot->reValidate();
  
    close();
}
