/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>  
#include <math.h>
#include <ctype.h>
#include <sys/param.h>

#include <qfiledialog.h> 
#include <qglobal.h>
#include <qfont.h>
#include <qwidget.h>

#include "qt_version.h"
#include "qwui.h"
#include "qslot.h"
#include "qdutil.h"
// #include "Qtetra_gui.h"
#include "QCCWindow.h"

// Qtetra_options *qtet_widget;
// Qtetra_dialog *qtetWindow;
QCCWindow * qccWindow;

QFont defaultFont;

extern qwuiWindow* qsasWindow;

/// \var  extern vector<QguListItem *> dragList
/// \brief Global variable that contains the list of items being dragged
///
extern vector<QguListItem *> dragList;

/// \var  extern QguListView * dragFromList
/// \brief Global variable that defines the list view from which the items are being dragged
///
extern QguListView * dragFromList;

//extern void QcalcShowExport();
// End Calculator


 using namespace QSAS;

 
// QSAS main window is a class on its own

#if QT_VERSION < 0x040000
QHBox *qwuiWindow::toggleBar;
#else
QWidget *qwuiWindow::toggleBar;
#endif

qwuiWindow::qwuiWindow(QWidget *parent, const char *name) 
   :  QguWindow(parent,name)
{
	double qsas_height = QSAS_profiles->current()->Height; 
	if(qsas_height == 0) qsas_height = 48.; // default if no env set or is bad
	
	// ensure QSAS_HOME env is set
	char * qsas_home_env = getenv("QSAS_HOME"); 
	if( qsas_home_env != NULL) {
        qsas_home = QString(qsas_home_env);
		qsas_home=qsas_home.remove(QString("\""));
	}
	else{   
		bool ok;
		QString user_choice = QuTextInputDialog("Environment QSAS_HOME not set", 
                                   "Please provide the full path to QSAS_HOME Directory",
				   "/usr/local/QSAS_2", &ok, this);
		if( !ok || user_choice.isEmpty() ) user_choice = QString(".");
		qsas_home = QString(user_choice.latin1());   // deep copy
	}

	QguWindow::_saveset_filter=QString("WL savesets (*.qs_WL)");
  
	help_window = new QswHelpWindow();
	
	// Menu insertion moved to qsas_main to avoid circular dependencies
	
	QFontMetrics qsas_font_mt(qApp->font());
	int char_width = qsas_font_mt.width("M");
	int char_height = qsas_font_mt.height();
   
	resize( char_width*48, (int)(char_height*qsas_height) );

	centralW = new QWidget(this);
	centralBox = new QVBoxLayout(centralW);
	
#if QT_VERSION < 0x040000
	toggleBar = new QHBox(centralW);
#else
	toggleBar = new QWidget(centralW);
	QHBoxLayout * toggleLayout=new QHBoxLayout(toggleBar);
#endif
	centralBox->addWidget(toggleBar);

	StripBadData = checkBadData(toggleBar);
	StripBadData->setChecked(FALSE);
	StripBadData->setMaximumHeight(18);
   
	StripFills = checkFills( toggleBar);
	StripFills->setChecked(FALSE);
	StripFills->setMaximumHeight(18);
   
#if QT_VERSION >= 0x040000
	toggleLayout->add(StripBadData);
	toggleLayout->add(StripFills);
#if QT_VERSION < 0x040300
	toggleLayout->setMargin(0);
#else
	toggleLayout->setContentsMargins(0,0,0,0);
#endif
#endif
   
	WorkingList = new QwlListView( centralW, "WorkingList");

	centralBox->addWidget(WorkingList);
  
	WorkingList->setRootIsDecorated(true);
	WorkingList->setMinimumHeight(char_height*10);
	WorkingList->setColumnWidthMode(Obj_Name_, Q_LIST_VIEW::Manual );
	WorkingList->setColumnWidth(Obj_Name_, char_width*45 );
       
	setAcceptDrops(true);
   
	this->_sessionFolder = QString(this->_filename.latin1());
	this->_sessionFolder += QString("/Session.qss");
	this->_filename += QString("/saveSet");
	this->SAVE_EXTENSION = QString(".qs_WL");
	this->_filename += this->SAVE_EXTENSION;
	this->fromSession = false;
  
	connect(WorkingList, SIGNAL(raiseWindow()), this, SLOT(bringToFront()));
	connect(WorkingList, SIGNAL(listModified()),this,SIGNAL(WLmodified()));
	
	setCentralWidget( centralW );

}


void qwuiWindow::makeWindows()
{
	WorkingList->addColumn("     Working List   ");

	WorkingList->Constant = new QwlListItem(WorkingList,  "Constants", ListTypeGroup); 
	WorkingList->Constant->setOpen(TRUE);
	WorkingList->Constant->setLocked(TRUE);    

	QguListItem *spacer_0 = new QguListItem(WorkingList, "-------------------------------------------------------", ListTypeNull); 
   
	WorkingList->setSelected(spacer_0,false);  
	spacer_0->setSelectable(false);  
	spacer_0->setLocked(true); 
 
	WorkingList->Time = new QwlListItem(WorkingList,  "Time Intervals", ListTypeGroup); 
	WorkingList->Time->setOpen(TRUE);
	WorkingList->Time->setLocked(TRUE);  
   
	QguListItem *spacer_1 = new QguListItem(WorkingList, "-------------------------------------------------------", ListTypeNull); 
	WorkingList->setSelected(spacer_1,false);  
	spacer_1->setSelectable(false);  
	spacer_1->setLocked(true);      
   
	WorkingList->Data = new QwlListItem(WorkingList,  "Data", ListTypeGroup);
	WorkingList->Data->setOpen(TRUE);
	WorkingList->Data->setLocked(TRUE);   
  
	// prevent further changes to top level
	WorkingList->setRootLocked(true);

	setStandardConstants( WorkingList->Constant );   
	WorkingList->clearSelection();

	/* create some test time interval objects to put in the list */
	QdTimeInterval_var ti_1 = new QdTimeInterval;  
	ti_1->set( QDOS::Time("2001-09-04 00:00:00"), QDOS::Time("2001-09-05 00:00:00") );
	QwuiPutOnWL((QdObject_var)ti_1, "SampleTI", "Time Intervals");

	WorkingList->clearSelectedList();

// create window for qtet plugin to access QApplication (static QT linkage)
	qccWindow = new QCCWindow(); 

}
 
void qwuiWindow::save(Vostream & v_os){

	if(QuSlotDebug())
	{
		printf("qwuiWindow::saveWithVostream %ld\n", this);
		fflush(stdout);
	}
  
	string guiID;
	guiID=QSAS_VERSION_ID;
	v_os << guiID;   

	int with = WorkingList->with;
      
	v_os << StripBadData->isChecked();
	v_os << StripFills->isChecked();

	v_os << *WorkingList;
  
}

void qwuiWindow::restore(Vistream & v_is, bool replace)
{
	string guiID; 
	v_is >> guiID;
	char *ptr;

      // clear list for true restore
    if(replace)  this->deleteAllDataItems();
    if(replace)  this->deleteAllTimeIntervalItems();
    
	if(QuSlotDebug())
	{
		printf("QwuiWindow::restore set ID =  %s\n   --------------------------------------\n", guiID.c_str());
		fflush(stdout);
	}
  
	if( strstr(guiID.c_str(), "qwui") == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
	}
	else
	{
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}

	bool sbd;
	v_is >> sbd;
	StripBadData->setChecked(sbd);
    
	bool sf;
	v_is >> sf;
	StripFills->setChecked(sf);
   
	this->blockSignals(true);
	WorkingList->blockSignals(true);
	v_is >> *WorkingList;
	WorkingList->blockSignals(false);
	this->blockSignals(false);

}

void qwuiWindow::saveSession(){
 
 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveSession %ld\n", this);
	fflush(stdout);
 }
 
  QuMsgBar(" ", false);

// ensure default is set
   if(_sessionFolder.isEmpty())
  {
     saveSessionAs();
     return;
  }

// ensure default has right filter
  int posn = _sessionFolder.findRev(".qss");
  if (posn < 0 ){
     saveSessionAs();
     return;  
  }

// ensure default exists still
// user might delete it

  QDir sessionDir = QDir(_sessionFolder);
  if( !sessionDir.exists() ){
    bool OK = sessionDir.mkdir(_sessionFolder);
    if (!OK){
      QuMsgBar("Failed to create directory", _sessionFolder, true);
      return;
    }
  }
  // remove trailing '/'

  if (_sessionFolder[_sessionFolder.length()-1] == '/') _sessionFolder.truncate(_sessionFolder.length()-1);
 
  emit saveSessionRequest(_sessionFolder);
  
    // do save of WL
  QString fileN = QString(_filename.latin1());
  
  // set session folder and save
  this->_filename = QString(_sessionFolder.latin1());
  this->_filename += QString("/saveSet");
  this->_filename += this->SAVE_EXTENSION;
  saveWL();
 
  // put file name back
  _filename = QString(fileN.latin1());  
}
  
void qwuiWindow::saveSessionAs(){
 
  if(QuSlotDebug())
  {
	printf("qwuiWindow::saveSessionAs %ld\n", this);
	fflush(stdout);
  }
 
  QString fn;
  QuMsgBar(" ", false);
  
      // use path only
  int posn = _sessionFolder.findRev(".qss");
  
  if (posn > 0) {
     _sessionFolder.truncate(posn);
    posn = _sessionFolder.findRev("/");
    if (posn > 0) _sessionFolder.truncate(posn+1);
  }
  
  SessionFolderDialog *dialog = new SessionFolderDialog(this,"FolderSession");
  dialog->setDir(_sessionFolder);
  if(dialog->exec() == QDialog::Accepted){
    fn = dialog->getSessionFolder();
  
    if(fn.isEmpty())
    {
       QuMsgBar("Not .qss directory", true);
    }
    else
    {
       _sessionFolder=fn;
       if (_sessionFolder[_sessionFolder.length()-1] == '/') _sessionFolder.truncate(_sessionFolder.length()-1);
       saveSession();
	 
	  // set QSAS main window title
	  QString capt = QString(fn.latin1());
	  int pos_end = capt.findRev(".qss");
	  capt.truncate(pos_end);
	  int pos_strt = capt.findRev("/");
	  capt.remove(0,pos_strt+1);

	  this->setCaption(" '" + capt + "'   " + QString(QSAS_VERSION_ID));

    }
  }
 
}
 
  
void qwuiWindow::restoreSession(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::restoreSession %ld\n--------------------------------\n\n", this);
	fflush(stdout);
 }

  QuMsgBar(" ", false);
	
	// ensure GUIs don't try to access item that has been deleted
	// when they get the WL update signal
	WorkingList->clearSelectedList();
	
	
    // use path only
   int posn = _sessionFolder.findRev(".qss");
   if (posn > 0) {
     _sessionFolder.truncate(posn);
     posn = _sessionFolder.findRev("/");
     if (posn > 0) _sessionFolder.truncate(posn);
   }

    QString fn=QFileDialog::getExistingDirectory(_sessionFolder, this,
              "Restore Session", "Select session folder to restore", true );
   if(!fn.isEmpty())
   {
	   QApplication::setOverrideCursor( Qt::waitCursor );
#if QT_VERSION < 0x040000
	   qApp->processEvents(2000);
#else
	   qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
	  if (fn[fn.length()-1] == '/')  fn.truncate(fn.length()-1);

      _sessionFolder = QString(fn.latin1());

      // restore WL
      // keep original name
      QString fileN = QString(_filename.latin1());

     // set session folder and restore
     this->_filename = QString(fn.latin1());
	 this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;
     this->fromSession = true;

	// tell plot and calc GUIs to clear their listviews to minimise wasted revalidation
	emit clearListRequest();

	// empty the working list first
	WorkingList->blockSignals(true);
	deleteAllDataItems();
	deleteAllTimeIntervalItems();
	WorkingList->blockSignals(false);
	
	#if QT_VERSION < 0x040000
	qApp->processEvents(2000);
	#else
	qApp->processEvents(QEventLoop::AllEvents, 2000);
	#endif

	WorkingList->blockSignals(true);	
	QguWindow::restore(true);
	WorkingList->blockSignals(false);

	// restore each GUI
	emit restoreSessionRequest(fn);
   	 
     status(fn+" restored",10000);

     // put file name back
     _filename = QString(fileN.latin1());

     // set QSAS main window title
     QString capt = QString(_sessionFolder.latin1());
     int pos_end = capt.findRev(".qss");
     capt.truncate(pos_end);
     int pos_strt = capt.findRev("/");
     capt.remove(0,pos_strt+1);

     this->setCaption(QString(QSAS_VERSION_ID) + "   '" + capt + "'"  );
     QApplication::restoreOverrideCursor();
#if QT_VERSION < 0x040000
     qApp->processEvents(2000);
#else
     qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
   }
   else
     status("Restore session aborted",10000);
}

void qwuiWindow::saveWL(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveWL %ld\n", this);
	fflush(stdout);
 }


  WorkingList->with = 1;

  if(_filename.isEmpty())
  {
     saveWLAs();
     return;
  }
    
  try
  {
    VXDRostream xdr_os(_filename.latin1());
    save(xdr_os);
  }
  catch(Exception &e)
  {
    status(e.msg().c_str(),10000);   
     return;
  }
  status(_filename+" saved",10000);
  return;   
}

void qwuiWindow::saveWLAs(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveWLAs %ld\n", this);
	fflush(stdout);
 }
 
  
  QString _filter = QString("save (*");
  _filter += SAVE_EXTENSION;
  _filter += QString(")");
  QString fn=QFileDialog::getSaveFileName(_filename, _filter, this, "save", "Select file - Save WL Settings");
  if(!fn.isEmpty())
  {
    int pos = fn.findRev(SAVE_EXTENSION);
    if (pos < 0) {
      pos = fn.findRev(".");
      if (pos > 0) fn.truncate(pos);
      fn += SAVE_EXTENSION;
    }
    _filename=fn;
    saveWL();
  }
  else
    status("saving aborted ",10000); 
}


void qwuiWindow::makeGroup()
{ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::makeGroup %ld\n", this);
	fflush(stdout);
 }
 
 
   (void) WorkingList->makeGroup();
}

void qwuiWindow::issueReName(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::issueReName %ld\n", this);
	fflush(stdout);
 }
 
 
   (void) WorkingList->renameSelectedItem();
}

void qwuiWindow::issueDuplicate(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::issueDuplicate %ld\n", this);
	fflush(stdout);
 }
 
   (void) WorkingList->duplicateItem();
}

void qwuiWindow::selectAllData(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::selectAllData %ld\n", this);
	fflush(stdout);
 }
 
  
  QwlListItem *nextItem;
  
  WorkingList->clearSelectedList();

  nextItem = (QwlListItem *) WorkingList->Data->firstChild();
  while(nextItem){
   WorkingList->AddToSelectedList(nextItem);
   nextItem = (QwlListItem *) nextItem->nextSibling();
  }
   WorkingList->refreshSelectedList();  

}

void qwuiWindow::selectAllTimeIntervals(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::selectAllData %ld\n", this);
	fflush(stdout);
 }
 
  
  QwlListItem *nextItem;
  
  WorkingList->clearSelectedList();

  nextItem = (QwlListItem *) WorkingList->Time->firstChild();
  while(nextItem){
   WorkingList->AddToSelectedList(nextItem);
   nextItem = (QwlListItem *) nextItem->nextSibling();
  }
   WorkingList->refreshSelectedList();  

}

void qwuiWindow::deleteItemTT(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::deleteItemTT %ld\n", this);
	fflush(stdout);
 }
 
  WorkingList->deleteItemTT();
}

void qwuiWindow::clearCache(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::clearCache %ld\n", this);
	fflush(stdout);
 }
 
  this->selectAllData();
  (void) WorkingList->clearCacheSelected();
}

void qwuiWindow::clearCacheSelected(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::clearCacheSelected %ld\n", this);
	fflush(stdout);
 }
 
   (void) WorkingList->clearCacheSelected();
}

void qwuiWindow::deleteListItem(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::deleteListItem %ld\n", this);
	fflush(stdout);
 }
 
  
   (void) WorkingList->deleteItems(false);
}

void qwuiWindow::deleteAllDataItems(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::deleteAllDataItems %ld\n", this);
	fflush(stdout);
 }
 
  this->selectAllData();
  this->deleteListItem();
}

void qwuiWindow::deleteAllTimeIntervalItems(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::deleteAllTimeIntervalItems %ld\n", this);
	fflush(stdout);
 }
 
  this->selectAllTimeIntervals();
  this->deleteListItem();
}


void qwuiWindow::setPreferences(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::setPreferences %ld\n", this);
	fflush(stdout);
 }
 
  
  QuMsgBar("TBD: Window to set user preferences\n global settings", false);
  
}


void qwuiWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManual %ld\n", this);
	fflush(stdout);
 }
 
   qsasWindow->help_window->manEntry("index.html");
}

void qwuiWindow::launchManMainW(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManMainW %ld\n", this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_main.html");
}

void qwuiWindow::launchManSampleW(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManSampleW %ld\n", this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_intro.html");
}

void qwuiWindow::launchManDataH(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManDataH %ld\n", this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

void qwuiWindow::launchManSaveRestore(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManSaveRestore %ld\n", this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_save_restore.html");
}

void qwuiWindow::launchVersion(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchVersion %ld\n", this);
	fflush(stdout);
 }
 
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n";
  text += QString(QSAS_VERSION_DATE);
  text += "\nworking list:  ";
  text += QString(QWUI_VERSION_ID);
  text += "\nqgu list  ";
  text += QString(QGU_VERSION);
  text += "\nqslot  slot:  ";
  text += QString(QSLOT_VERSION);
  Qwui_msgBox(text.latin1());

}

void qwuiWindow::launchContact(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchContact %ld\n", this);
	fflush(stdout);
 }

  qsasWindow->help_window->manEntry("ContactUs.html");


}


void qwuiWindow::closeEvent( QCloseEvent* ce)
{     
    switch( QMessageBox::information( this, "Qwui Window",
                                          "Exit Qsas Main Window?",
                                          "Exit", "Save Session", "Cancel",
                                          0, 1 ) ) 
    {
        case 0:
		{
			ce->accept();
			break;
		}
        case 1:
		{
			saveSession();
            ce->ignore();	    
            break;
		}
        case 2:
        default: 
        {
		    ce->ignore();
            break;
		}
    }

}

 void qwuiWindow::dragMoveEvent(QDragMoveEvent *event)
{   
       this->setActiveWindow();
       this->raise();
}



void qwuiWindow::Qwui_msgBox( const char *  text)
{
   QMessageBox::information( this, "QSAS information", text, "OK" ); 
   return;
}


// WL List View Widget

// QwlListView class
// ------------------


QwlListView::QwlListView(QWidget *parent, const char *name) 
   :  QguListView(parent,name)
{
   RButtonMenu = new Q_POPUP_MENU( this );
   
   // Rt Button menu populated in qsasMenus.cc later

   setAcceptDrops(true);
   setFocusPolicy(WHEEL_FOCUS);


}

QwlListView::~QwlListView()
{  
}
 
 
QwlListItem *QwlListView::insertOnWL(QguListItem *head, 
                                QString name,
                                QguListType type, 
				bool lock, 
				QguListItem *after){
				
  QwlListItem *newItem = new QwlListItem(head, name, type, after);
  if (lock)   newItem->setLocked(true);

  return newItem;
}

void QwlListView::dropEvent(QDropEvent *event)
{
  QString txt;
  QwlListItem *newItem;

 if(QuSlotDebug())
  cout << "QwlListView::dropEvent " << dynamic_cast<QwlListItem*>(currentItem()) << endl;
 

  QwlListItem *dropAtItem = (QwlListItem *)currentItem() ;  
  
  // this method is overloaded here for the WL itself
  
  if (dragList.size() == 0)  return;
  
  if (this != dragFromList ) return; // cannot drag to WL from other list

#if QT_VERSION < 0x040000
  if ( QTextDrag::decode(event, txt) )
#else
  if (event->mimeData()->hasText())
#endif
  {  
	  
#if QT_VERSION >= 0x040000
	  txt=event->mimeData()->text();
#endif
  
    // special case of single click on item
  
    if (dropAtItem && txt == dropAtItem->nameContext() ) return;
  
    //  copy drag text to clipboard to allow repeat paste
#if QT_VERSION < 0x040000
    QApplication::clipboard()->setData( new QTextDrag(txt) );
#else
    QMimeData * clipData=new QMimeData;
    clipData->setText(txt);
    QApplication::clipboard()->setMimeData(clipData);
#endif
     
     // don't allow drop onto item that is on list or has ancestor on it
     //  localList and dragList are same here
    if ( isOnSelectedList(dropAtItem) || findAncestorOnList(dropAtItem) ) {
      refreshSelectedList();
      return;
    }

    // if drop into list at top level or undetermined
    if( dropAtItem == NULL ) 
        dropAtItem = this->Data;
    
    this->insertListItems(dropAtItem);

    clearSelectedList();
    dragList.clear();
    
  } // end if on decode event 
}



void QwlListView::insertListItems(QwlListItem * dropAtItem){
    int i;

    for ( i=0; i<dragList.size(); i++ ){   
      if ( dragList[i]->isLocked() ){
        QuMsgBar("A drag item is locked", false);
	return;
      }
    }
    
    for ( i=0; i<dragList.size(); i++ ){  
      QwlListItem * item = (QwlListItem *)dragList[i];
      insertDragItem(dropAtItem,item);
    }

}

void QwlListView::insertDragItem(QwlListItem * dropAt, QwlListItem *item){
  QwlListItem * itemParent;
  QwlListItem * folder;
  QwlListItem * sameItem;
  QwlListItem * itAbove;
  
  // only allow drag from this List

  if( item->listView() != this) return;
  
  if( item == 0) return;
  
  itemParent = (QwlListItem *)item->parent();  
  if(isOnSelectedList(itemParent) ) return;
  
  // keep these for later use
  QString oldContext = item->nameContext();
        
  // if name already exists in target folder reject move
  if (dropAt->getEntryType() == ListTypeEntry)
     folder = (QwlListItem *) dropAt->safeParent();
  else if (dropAt->getEntryType() == ListTypeGroup)
     folder = dropAt;
  else if (dropAt->getEntryType() == ListTypeNull){
    // on separator
    QwlListItem * itAbove = (QwlListItem *)dropAt->itemAbove();
    if(itAbove->parent()){
      if(itAbove->parent()->parent())
        folder = (QwlListItem *)itAbove->parent()->parent();
      else
        folder = (QwlListItem *)itAbove->parent();
    }
    else
      folder = itAbove;
  }
  else return;   
  
  if ( (sameItem = (QwlListItem *)((QwlListItem *)folder->firstChild())->findName(item->text(Obj_Name_))) != 0 ) {
    if(sameItem != item) {
      QuMsgBar(item->text(Obj_Name_).latin1(), "Item not transferred; name exists in target", true) ;
      return;
    }
  }
	 
    QguMoveItem(item, dropAt);    
        
    // if we leave the drag-from folder empty then delete it 
    //  if (itemParent != 0) itemParent->removeIfEmpty();
  
    refreshWLcontext();
 
} // end insertDragItem



void QwlListView::contentsMouseDoubleClickEvent(QMouseEvent *event)
{
  // disable mouse down on right and middle button
  // forces launch to be from left button and protects
  // from two mouse down events causing drag problems
     
   if( event->button() == Qt::RightButton) return;   
   if( event->button() == Qt::MidButton) return;
   
   if(event->state() == Qt::ControlButton || event->state() == Qt::ShiftButton)
         return;

   clearSelectedList();
   
   QwlListItem *atItem = (QwlListItem *)currentItem() ;  
   if(atItem  == NULL) return;

   if(atItem->getEntryType() == ListTypeGroup) {
      // open/close group view
      Q_LIST_VIEW::contentsMouseDoubleClickEvent(event);
   }
   else{
     emit viewObjRequest();
   }

}


void QwlListView::refreshWLcontext()
{
  
   ((QwlListItem *)this->firstChild())->refreshWLcontext();

   // notify other windows
    
    emitListModified();
    
    return;
}

void QwlListView::renameSelectedItem(){
     
 if(QuSlotDebug())
 {
	printf("QwlListView::renameSelectedItem %ld\n", this);
	fflush(stdout);
 }
 
 
  QwlListItem *selected_item;
  
  selected_item = (QwlListItem *)this->selectedItem();
  if (selected_item == NULL) return;

  reNameItem(selected_item);
  refreshWLcontext();
  
}

void QwlListView::duplicateItem(){
     
 if(QuSlotDebug())
 {
	printf("QwlListView::duplicateItem %ld\n", this);
	fflush(stdout);
 }
   
  QwlListItem *selected_item = (QwlListItem *) this->selectedItem();
  if (selected_item == NULL) return;

  QdObject_var dobj =  selected_item->QguGetObject();
  
  QdObject_var newObj = Factory::clone( (QdObject_var) dobj);
  newObj->copy_xrefs_from(dobj);
  
  QString newName = QString(selected_item->text(Obj_Name_).latin1());
  int tokLen = newName.findRev("/");   
  if( tokLen >= 0) newName.remove(0,tokLen+1);
  
  newName += "_copy";
  QwuiPutOnWL(newObj, newName.latin1(), "Data");
  
}

 bool QwlListView::reNameItem(QwlListItem * item)
{
  QString userNameChoice; 
  bool ok;

  // change text of this item
      
  // launch getText box with current item
  
  userNameChoice = QuTextInputDialog("Object Name", "Enter new object name",  
                      item->text(Obj_Name_).latin1(), &ok, item->listView());
    
  if (ok && !userNameChoice.isEmpty() ){
    if(userNameChoice.find("}") >= 0) {
      // loop until user doesn't use '}'
      userNameChoice = QuTextInputDialog("Rename", " '}' not allowed in name",
                         userNameChoice.latin1(), &ok, item->listView());
    }
  
    // if name is a path remove object and add new
    
    if(userNameChoice.find('/') != -1 ){
      
      QdObject_var keep_object = item->QguGetObject();
      if(keep_object.is_nil()) return false;
      
      // add at new location (don't replace existing object if clash)
      QString rootFolder = QString((this->root(item))->text(Obj_Name_).latin1());
      QString newCtxt = QwuiPutOnWL(keep_object, userNameChoice, rootFolder.latin1(), false);

      // delete old name context and list item
      // don't remove if we failed to put on WL
      // also don't remove if user gave it the same name as it's already
      // been removed by QwuiPutOnWL call to QuSetObjByName()

      if(!(newCtxt.isEmpty()) && newCtxt != item->getWLcontext()){
        QuDeleteObject(item->getWLcontext());
        delete item;
      }
      
      return true;
    }
    
    // otherwise try to rename in this folder
    while(ok && item->nameAlreadyUsed(userNameChoice) ){
       userNameChoice = QuTextInputDialog("Object Name", "Name used", 
                          item->text(Obj_Name_).latin1(), &ok, item->listView());    
    
    } 
    if (ok) {     
      item->setText( Obj_Name_, userNameChoice );
      refreshWLcontext(); // updates WLcontext and object list
    }
    else return false;
  }
  else return false;
  
  item->listView()->setCurrentItem(item);
  emitListModified();
  return true;
  
}


void QwlListView::paste(){
     
 if(QuSlotDebug())
 {
	printf("QwlListView::paste %ld\n", this);
	fflush(stdout);
 }
 
 
  // do nothing - paste not allowed in WL (use duplicate)
}

void QwlListView::insertEntries(QwlListItem *dropAt, QwlListItem *item){

 
  // if item is a folder, it gets ignored
  
  if(item->getEntryType() == ListTypeEntry){
    
    if(dropAt == 0 ) {
       QwlListItem * newItem = new QwlListItem(this, item->getWLcontext(), 
	                    ListTypeEntry);    
    }
    else if(dropAt->getEntryType() == ListTypeEntry){
      if( dropAt->parent() ){
         QwlListItem * newItem = new QwlListItem((QguListItem *)dropAt->parent(), item->getWLcontext(), 
	                    ListTypeEntry);
         if(newItem != NULL) newItem->moveItem(dropAt);
      }
      else{
        QwlListItem * newItem = new QwlListItem(this, item->getWLcontext(), 
	                    ListTypeEntry);          
      }
    } 
    else{
       QwlListItem * newItem = new QwlListItem(dropAt, item->getWLcontext(), 
	                    ListTypeEntry);
    }
  }

}



void QwlListView::makeGroup(){

 if(QuSlotDebug())
 {
	printf("QwlListView::makeGroup %ld\n", this);
	fflush(stdout);
 }
 
 
  QwlListItem *newFolder;
  bool ok;

  // make new group item in folder or parent of selected item
    // if nothing is selected put at root (if allowed)
    if(localList.size() == 0){  
      if( this->rootIsLocked() )
        // root is locked, ie Working List, so there must be a first child
        newFolder =  new QwlListItem((QwlListItem*)this->firstChild(), QString("Group"), ListTypeGroup, 0);
      else{ 
        newFolder =  new QwlListItem(this, QString("Group"), ListTypeGroup, 0);
      } 
            
      ok = reNameItem(newFolder);  // launch re-name text box
      if(!ok) 
         delete newFolder;
      else 
         setSelectedList(newFolder);
      return;
    }
    
    // else something is selected, find where to put folder
    QwlListItem *commonRoot = (QwlListItem *)findCommonFolder();   
    if(commonRoot != NULL)
    {
      newFolder =  new QwlListItem(commonRoot, QString("Group"), ListTypeGroup);    
    }
    else if( !this->rootIsLocked() ){
      newFolder =  new QwlListItem(this, QString("Group"), ListTypeGroup);    
    }
    else return;  // cannot find where to put folder that is allowed
    
    ok = reNameItem(newFolder);  // launch re-name text box
    if(!ok) {
      delete newFolder;
      return;
    }
    
    // move selected items to this new folder
    
      for (int i=0; i<localList.size(); i++){

	if(!localList[i])
	  continue;
	if(localList[i]->listView()!=this) // not really needed as localList
	  continue;
      
        // move item from selection 
	QwlListItem * movingItem = (QwlListItem *)localList[i];
	QwlListItem * parentItem = (QwlListItem *)movingItem->parent();
	
	// move from old folder or listView
	if (parentItem == 0) this->takeItem(movingItem);
	else                 parentItem->takeItem(movingItem);
	
        newFolder->insertItem(movingItem);
	
      }
      refreshWLcontext();
}


void QwlListView::deleteItemTT(){

 if(QuSlotDebug())
 {
	printf("QwlListView::deleteItemTT %ld\n", this);
	fflush(stdout);
 }
 
  for(int i=0; i<localList.size(); i++)
  {
    QwlListItem * mouseDownItem = (QwlListItem *)localList[i];
       
    if( mouseDownItem != NULL && mouseDownItem->getEntryType() == ListTypeEntry){
      QString itemContext = mouseDownItem->getWLcontext();
    
      QdObject_var dobj = QuGetObjByName(itemContext.latin1());
      if (dobj->xref_exists(TIME_TAGS)) dobj->delete_xref(TIME_TAGS);
      if (dobj->xref_exists(DEPEND_0)) dobj->delete_xref(DEPEND_0);  
    }
  } 

}


void QwlListView::deleteListItems()
{
  this->deleteItems(true);
}

void QwlListView::deleteItems(bool askFirst)
{
  
  // ensure that selectionChanged() is not emitted while deleting
  clearSelection();
  
  vector<QwlListItem *> folderList; 
  
   // first warn user if non-empty folder is selected or locked
  for(int i=0; i<localList.size(); i++)
  {
    QwlListItem * mouseDownItem = (QwlListItem *)localList[i];
    if(  mouseDownItem->isLocked() ) {
      QuMsgBar("Cannot delete locked item", true);
      return;
    }
       
    if(  mouseDownItem != NULL 
      && mouseDownItem->getEntryType() == ListTypeGroup){
    
      if( mouseDownItem->firstChild() != NULL && askFirst) {
        // launch dialogue
        switch( QMessageBox::information( this, "Object Lists",
                                        "Non-empty folder selected",
                                        "Delete", "Cancel",
                                           0,         1 ) ) 
        {
          case 0:
	    break;   // drop through and delete
          case 1:
            return;  // don't delete
        }
      }
    }
  } // end for over Selected List testing
  
  // delete entry type items first
  for(int i=0; i<localList.size(); i++)
  {
    QwlListItem * mouseDownItem = (QwlListItem *)localList[i];
    if (mouseDownItem != NULL && mouseDownItem->getEntryType() == ListTypeGroup)
       folderList.push_back(mouseDownItem);
       
    if( mouseDownItem != NULL && mouseDownItem->getEntryType() == ListTypeEntry){
      
      QuDeleteObject(mouseDownItem->nameContext());
      delete mouseDownItem;
    }
  } // end for over Selected List deleting entries
      
  // delete folders - which are now empty
  for(int i=0; i<folderList.size(); i++)
  {
    QwlListItem * mouseDownItem = (QwlListItem *)folderList[i];
    
    if( mouseDownItem->firstChild() == NULL ) 
       delete mouseDownItem;
    
  } // end for over folder List

  // ensure no deleted item is selected 
  
  localList.clear();
  emitListModified();
}


void QwlListView::clearCacheSelected()
{
  
  // ensure that selectionChanged() is not emitted while clearing
  clearSelection();
    
  for(int i=0; i<localList.size(); i++)
  {
    QwlListItem * mouseDownItem = (QwlListItem *)localList[i];
       
    if( mouseDownItem != NULL && mouseDownItem->getEntryType() == ListTypeEntry){
       QdObject_var dobj = QuGetObjByName(mouseDownItem->WLcontext);
	   if( dobj.is_nil() ) continue;
	   
	   vector<string> sl;
	   dobj->list_xref_names(sl);
	   for(int i=0; i<sl.size(); i++)
	   {
	      cout << " cached obj = " << sl[i].c_str() << endl;
		     if( sl[i][0] == '_') dobj->delete_xref(sl[i].c_str());
	   }
    }
  } // end for over Selected List deleting caches
      
  return;
}

bool QwlListView::nameUsedAtWLroot(QString newName){

  if(this->Constant->nameUsed(newName) )
    return true;

  if( this->Time->nameUsed(newName) )
    return true;

  if( this->Data->nameUsed(newName) )
    return true;

  return false;
}



Vostream& operator<<(Vostream& v_os,QwlListView& lv)
{
 
 // treat each top level folder separately as they are not deleted
 // so are not save/restored

  QwlListItem *data = dynamic_cast<QwlListItem*>(lv.findItem("Data", Obj_Name_));
  if( data != 0){
#if QT_VERSION < 0x040000
    QList<QwlListItem> lst;
#else
    QList<QwlListItem*> lst;
#endif
  
    QwlListItem *item=dynamic_cast<QwlListItem*>(data->firstChild());
    int cnt=0;
    while(item)
    {
      lst.append(item);
      cnt++;
      item=dynamic_cast<QwlListItem*>(item->nextSibling());
    }
    v_os << cnt;

    for(int i=0;i<cnt;i++)
    {
      QwlListItem *item=lst.at(i);
      v_os << *item;
    }        
  }
  
  QwlListItem *ti = dynamic_cast<QwlListItem*>(lv.findItem("Time Intervals", Obj_Name_));
  if( ti != 0){
#if QT_VERSION < 0x040000
    QList<QwlListItem> lst;
#else
    QList<QwlListItem*> lst;
#endif
  
    QwlListItem *item=dynamic_cast<QwlListItem*>(ti->firstChild());
    int cnt=0;
    while(item)
    {
      lst.append(item);
      cnt++;
      item=dynamic_cast<QwlListItem*>(item->nextSibling());
    }

    v_os << cnt;
    for(int i=0;i<cnt;i++)
    {
      QwlListItem *item=lst.at(i);
      v_os << *item;
    }  
       
  }
  
  QwlListItem *consts = dynamic_cast<QwlListItem*>(lv.findItem("Constants", Obj_Name_));
  if( consts != 0){
#if QT_VERSION < 0x040000
    QList<QwlListItem> lst;
#else
    QList<QwlListItem*> lst;
#endif
  
    QwlListItem *item=dynamic_cast<QwlListItem*>(consts->firstChild());
    int cnt=0;
    while(item)
    {
      lst.append(item);
      cnt++;
      item=dynamic_cast<QwlListItem*>(item->nextSibling());
    }
    v_os << cnt;
    for(int i=0;i<cnt;i++)
    {
      QwlListItem *item=lst.at(i);
      v_os << *item;
    }    
  }

  return v_os;
} 

Vistream& operator>>(Vistream& v_is,QwlListView& lv)
{
    int cnt;
    
    QwlListItem *data = dynamic_cast<QwlListItem*>(lv.findItem("Data", Obj_Name_)); 
    if(data == 0) return v_is; // if "Data" item is lost, defensive programming only
    
    // find last entry in list
    QwlListItem *last_d_item = (QwlListItem *) data->firstChild();
    while( last_d_item != 0){
	    last_d_item = (QwlListItem *) last_d_item->nextSibling();
    }   

    v_is >> cnt;  
    for(int i=0;i<cnt;i++)
    {
      // create empty objects and fill
	 QwlListItem *d_item;
      d_item=new QwlListItem(data, "", ListTypeEntry, last_d_item);
	 
      v_is >> *d_item;
	 last_d_item = d_item; // always append next item to keep correct order
    }
 
      
    QwlListItem *ti = dynamic_cast<QwlListItem*>(lv.findItem("Time Intervals", Obj_Name_)); 
    if (ti == 0) return v_is; // defensive only, should never happen

    QwlListItem *sample_ti = dynamic_cast<QwlListItem*>(lv.getItemByName("Time Intervals/SampleTI" ));
    if(sample_ti) delete sample_ti;
   
   // find last entry in list
    QwlListItem *last_t_item = (QwlListItem *) ti->firstChild();
    while( last_t_item != 0){
	    last_t_item = (QwlListItem *) last_t_item->nextSibling();
    }
    
    v_is >> cnt;  
    for(int i=0;i<cnt;i++)
    {      
      // create empty objects and fill
      QwlListItem *t_item=new QwlListItem(ti, "", ListTypeEntry, last_t_item);
      v_is >> *t_item;
	 last_t_item = t_item;
    }
 
  
    QwlListItem *consts = dynamic_cast<QwlListItem*>(lv.findItem("Constants", Obj_Name_)); 
    if (consts == 0) return v_is; // defensive only
    
    consts->QwlDeleteChildren();
    QwlListItem *last_c_item = (QwlListItem *) consts->firstChild();
 
    v_is >> cnt;  
    for(int i=0;i<cnt;i++)
    {
      // create empty objects and fill
      QwlListItem *c_item=new QwlListItem(consts, "",  ListTypeEntry, last_c_item);
      v_is >> *c_item;
    }    
 

    return v_is;
} 



// QwlListItem class
// ------------------

QwlListItem::QwlListItem(QguListView *parent,  QString txt,
   QguListType type, QguListItem *after)
   :  QguListItem(parent, txt, type, after)
{
   if(type == ListTypeEntry)
     WLcontext = nameContext();

} 

QwlListItem::QwlListItem(QguListItem *parent,  QString txt, 
   QguListType type, QguListItem *after)
   :  QguListItem(parent, txt, type, after)
{
   if(type == ListTypeEntry)
     WLcontext = nameContext();

} 


QwlListItem::~QwlListItem()
{  
}

QString  QwlListItem::addItemAtAnchor ( QString name, bool reuse)
{
  // utility function used by QwuiPutOnWL() and should not
  // be called directly.
  
 QString NullStr = "";
 QwlListItem *newItem;
 QguListView * listV = (QguListView*)this->listView();
 
 if ( name.isNull() || name.isEmpty() ) return NullStr;
 
 // break path
 int posn = name.find("/");

 // special case of name starting with / allowing full path
 if(posn == 0){
   QString rootFolder = QString(name.latin1());
   rootFolder.remove(0,1);
   
   name.remove(0,1);
   posn = rootFolder.find("/");
   if(posn == -1) 
      return this->addItemAtAnchor(name, reuse);
   rootFolder.truncate(posn);
   QwlListItem *rootItem = (QwlListItem *)listV->getItemByName(rootFolder);

   if(rootItem == NULL) // no valid root folder, revert to local path
      return this->addItemAtAnchor(name, reuse);
   
   name.remove(0,posn+1);
   // root folder found, redirect insert to there
   return rootItem->addItemAtAnchor(name, reuse);
 }
 
 // else path is relative to this
 if (posn != -1){
   QString folder = QString(name.latin1());
   folder.truncate(posn);
   QString fullPath = QString(folder.latin1());
   fullPath.prepend("/");
   fullPath.prepend(this->getWLcontext());
   name.remove(0,posn+1);
   QwlListItem *folderItem = (QwlListItem *)listV->getItemByName(fullPath);
   if(folderItem != NULL){
     return folderItem->addItemAtAnchor(name, reuse);
   }
   else{
     QwlListItem *newFolderItem = (QwlListItem *) new QwlListItem(this, folder, ListTypeGroup);
     return newFolderItem->addItemAtAnchor(name, reuse);
   }
 }
 
 // else name only, no path to remove, insert here
 if ( this->nameUsed(name) ) {
   if(reuse){
     // use existing item with name in this folder
     return dynamic_cast<QguListItem*>(this->firstChild())->findName(name)->nameContext(); 
   }
   else{
     name.append("A");
     return this->addItemAtAnchor(name);
   }
 }
 else{
   //create item with this name
    newItem = (QwlListItem *) new QwlListItem(this, name, ListTypeEntry);
 }

 if (newItem == NULL) return QString("No Item");
 
 newItem->WLcontext = newItem->nameContext();
 
  // return full naming context for qdos Object list use
 return newItem->nameContext();

}


void QwlListItem::QwlDeleteChildren()
{  
  QwlListItem *next = dynamic_cast<QwlListItem*>(this->firstChild());
  while (next){
    next->QwlDeleteChildren();
    if(next->getEntryType() == ListTypeEntry){
      QuDeleteObject( QString(next->nameContext()) );
    }
    QwlListItem *last = next;
    next = dynamic_cast<QwlListItem*>(last->nextSibling());
    delete last;
  }
}

Vostream& operator<<(Vostream& v_os,QwlListItem& item)
{
  v_os << item.text(Obj_Name_).latin1();
  v_os << item.getEntryType();
  
  if(item.getEntryType()==ListTypeEntry)
  {
	if( !item.WLcontext.isEmpty()) 
	{
	     v_os << item.WLcontext.latin1();
		QdObject_var obj=item.QguGetObject();
		string type=obj->object_type();
		v_os << type;
		v_os << obj;
	}
  }

#if QT_VERSION < 0x040000
  QList<QwlListItem> lst;
#else
  QList<QwlListItem*> lst;
#endif
   QwlListItem *childItem=dynamic_cast<QwlListItem*>(item.firstChild());
   int cnt=0;
   while(childItem)
   {
     lst.append(childItem);
     cnt++;
     childItem=dynamic_cast<QwlListItem*>(childItem->nextSibling());
   }
   v_os << cnt;
   for(int i=0;i<cnt;i++)
   {
     QwlListItem *nextItem=lst.at(i);
     v_os << *nextItem;
   }
   return v_os;
}


Vistream& operator>>(Vistream& v_is,QwlListItem& item)
{
  // don't use the qgu base operator to avoid duplicate name on WL
   string text;
   v_is >> text;  
   // ensure still unique (in case include not restore was used)
   int append_cnt=0;   
   while( item.nameAlreadyUsed(QString(text.c_str())) ){ 
     if(append_cnt == 0) text+="_";
     text += "R";
     append_cnt++;
   }
   item.setText(Obj_Name_,text.c_str());
   int it=0;
   v_is >> it;
   item.entryType=(QguListType)it;

   if( item.getEntryType() == ListTypeEntry){
       string name_ctxt;
       v_is >> name_ctxt;
	  if(append_cnt >0)
	  {
	     name_ctxt += "_";
		for(int r_i=0; r_i < append_cnt; r_i++) name_ctxt +="R";
	  }
       string obj_type;
       v_is >> obj_type;
       QdObject_var obj=QdObject::create(obj_type);
       v_is >> obj;
       item.WLcontext = QString(name_ctxt.c_str());
       QuSetObjByName(obj, QString(name_ctxt.c_str()) );
   }
   else if(item.getEntryType() == ListTypeGroup){
     item.setOpen(true);
   }
   int cnt=0;
   v_is >> cnt;
   for(int i=0;i<cnt;i++)
   {
      QwlListItem *childItem=new QwlListItem(&item);
      v_is >> *childItem;
   }
   return v_is;
}

  
#if QT_VERSION < 0x040000
SessionFolderDialog::SessionFolderDialog( QWidget *parent, const char *name ) 
    : QFileDialog(parent,name,true)
{
    this->setCaption("Select a .qss folder for this Save Session");
    this->setMode(QFileDialog::DirectoryOnly);  
    this->setFilter("session dir (*.qss)");
  
    label = new QLabel("Select an existing .qss folder and click OK - Will overwrite contents of this folder.\n (or Create New Folder from menu bar first, hit return then select as above)", this);
    this->addWidgets(0, label, 0);
}
#else
SessionFolderDialog::SessionFolderDialog( QWidget *parent, const char *name ) 
    : QFileDialog(parent) 
{
    this->setCaption("Select a .qss folder for this Save Session");
    this->setFileMode(QFileDialog::DirectoryOnly);  
    this->setFilter("session dir (*.qss)");
    this->layout()->addWidget(new QWidget());//blank
    this->layout()->addWidget(new QLabel("Select an existing .qss folder and click OK - Will overwrite contents of this folder.\n (or Create New Folder from menu bar first, hit return then select as above)", this));
}
#endif

QString SessionFolderDialog::getSessionFolder(){
  QString dirName = QString(selectedFile().latin1());
  
  if( !dirName.isEmpty()){
    int pos = dirName.findRev(".qss");

    if (pos < 0 )  dirName = QString::null;
    if( pos != dirName.length()-4 ) {
      // some versions of QT add / to directories
	 pos = dirName.findRev(".qss/");
      if( pos != dirName.length()-5 ) dirName = QString::null;
    }
  } 
  return dirName;
}


//---------------------------------------------------------

QString QwuiPutOnWL(QdObject_var obj, const char * objName,
                  const char * List, bool replace)
{
 QString context;
 bool OK;

 // trap blank name error
 if(strlen(objName) < 1) return "";
 
 // tolerate full path without the leading '/' and allow over-ride default top level  //location

 QString pathAndName = QString(objName);
 QString listRoot = QString(List);

 int posn, posn2;
  posn =  pathAndName.find("Data/");
  posn2 =  pathAndName.find("/Data/");

  if (posn == 0 ) {
    listRoot = "Data";
    pathAndName.remove(0,5);
  }
  if (posn2 == 0 ) {
    listRoot = "Data";
    pathAndName.remove(0,6);
  }

  posn =  pathAndName.find("Constants/");
  posn2 =  pathAndName.find("/Constants/");

  if (posn == 0 ) {
    listRoot = "Constants";
    pathAndName.remove(0,9);
  }
  if (posn2 == 0 ) {
    listRoot = "Constants";
    pathAndName.remove(0,10);
  }

  posn =  pathAndName.find("Time Intervals/");
  posn2 =  pathAndName.find("/Time Intervals/");

  if (posn == 0 ) {
    listRoot = "Time Intervals";
    pathAndName.remove(0,15);
  }
  if (posn2 == 0 ) {
    listRoot = "Time Intervals";
    pathAndName.remove(0,16);
  }

#ifdef QT_VERSION
#if QT_VERSION <0x030000
  pathAndName.replace(QRegExp("\["), "(");
  pathAndName.replace(QRegExp("\]"), ")");
#else
  pathAndName.replace("[", "(");
  pathAndName.replace("]", ")");

#endif // end of #if
#endif // end of #ifdef
  // remove and '[' or ']' from WL name to prevent calculator problems


  // place data object on selected root on working list

  if( listRoot == "Data" ){
    context = ((QwlListItem *)(qsasWindow->WorkingList->Data))->addItemAtAnchor(pathAndName.latin1(), replace);
    OK = QuSetObjByName(obj,context);
  }
  else if( listRoot == "Time Intervals" ){
    context = ((QwlListItem *)(qsasWindow->WorkingList->Time))->addItemAtAnchor(pathAndName.latin1(), replace);
    OK = QuSetObjByName(obj,context);
  }
  else if( listRoot == "Constants" ){
    context = ((QwlListItem *)(qsasWindow->WorkingList->Constant))->addItemAtAnchor(pathAndName.latin1(), replace);
    OK = QuSetObjByName(obj,context);
  }

  if (!OK) {
    QwlListItem *item = (QwlListItem *)qsasWindow->WorkingList->getItemByName(context);
    delete item;
  }
  qsasWindow->WorkingList->emitListModified();
  
  return context;

}

//---------------------------------------------------------

QSize qsasGUISize(){ 
   return qsasWindow->frameSize();
}

