//  qdi.cc
//
// Direct Import Widget
//
// Sept 2002: Tony Allen
//

#include <stdlib.h>
#include "qcheckbox.h"
#if QT_VERSION < 0x040000
#include "qbu.h"
#include "qdi.h"
#else
#include "qbu_qt4.h"
#include "qdi_qt4.h"
#endif
#include "qie_qsas.h"
#include "qie_caa.h"
#include "qwui.h"

extern qwuiWindow* qsasWindow;

vector <int> importedVarNos;
vector <QdObject_var> importedVars;


// -----------------------------------------------------------------------


DirectImportWindow::DirectImportWindow( QWidget *parent, const char *name ) 
  : QMainWindow(parent,name)
#if QT_VERSION < 0x040000
{

   msgBar = statusBar();
 
   menuBar()->insertSeparator();   
   Q_POPUP_MENU * help = new Q_POPUP_MENU( this );
   menuBar()->insertItem( " Help", help );
   help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
   help->insertSeparator();
   help->insertItem("...Data Import", this, SLOT( launchManDirectImport() ));
   help->insertSeparator();
   help->insertItem("Module Version", this, SLOT( launchVersion() ));
      
   QVGroupBox *_topV = new QVGroupBox( this, "TopWidget" );

   setCentralWidget( _topV );
   
   QHGroupBox *list_panel = new QHGroupBox(_topV, "lists");

   varList = new Q_LIST_VIEW(list_panel, "varList");
   varList->addColumn("Variables", 200);
   varList->setAcceptDrops(false);
   varList->setDragAutoScroll(true);
   varList->setSorting(-1);

   connect( varList, SIGNAL(clicked(QListViewItem*)), this, SLOT(clearGattrList()) );
   connect( varList, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(varDoubleClick(QListViewItem*)) );

   activeList = varList;
   
   GattrList = new Q_LIST_VIEW(list_panel, "GattrList");
   GattrList->addColumn("Global Attributes", 200);
   GattrList->setAcceptDrops(false);
   GattrList->setDragAutoScroll(true);
   connect( GattrList, SIGNAL(clicked(QListViewItem*)), this, SLOT(clearVarList()) );
   connect( GattrList, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(GattrDoubleClick(QListViewItem*)) );


   QVGroupBox *subset_panel = new QVGroupBox("Subset on input", _topV, "subset" );

   
   fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
   toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");
   
   QHGroupBox *button_panel = new QHGroupBox(_topV, "buttons");
   
   getVarBtn = new QPushButton( "Get Data", button_panel, "getVarBtn" );
   getVarBtn->setMinimumSize(getVarBtn->sizeHint());
   getVarBtn->setDefault(TRUE);
   
   connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );

   browseVarBtn = new QPushButton( "Browse Data", button_panel, "browseVarBtn" );
   browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
   connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );

   dismissBtn = new QPushButton( "Dismiss", button_panel, "dismissBtn" );
   dismissBtn->setMinimumSize(dismissBtn->sizeHint());
   connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );

   QieOpt = (QiOptions *) NULL;
   QieContents = (QiCDFContents *) NULL;

   QWidget::setWFlags(Qt::WDestructiveClose);
}
#else
{

	// Status bar
	
	msgBar = statusBar();
	
	// Menu
	
	menuBar()->insertSeparator();   
	QMenu * help = new QMenu( this );
	menuBar()->insertItem( " Help", help );
	help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->insertSeparator();
	help->insertItem("...Data Import", this, SLOT( launchManDirectImport() ));
	help->insertSeparator();
	help->insertItem("Module Version", this, SLOT( launchVersion() ));
	
	// Main widget
	
	QGroupBox * topWidget=new QGroupBox(this);
	QVBoxLayout * topLayout=new QVBoxLayout;
	
	// List panel
	
	QHBoxLayout * listLayout=new QHBoxLayout;
	
	varList = new Q_LIST_VIEW(topWidget, "varList");
	varList->addColumn("Variables", 200);
	varList->setAcceptDrops(false);
	varList->setDragAutoScroll(true);
	varList->setSorting(-1);
	connect( varList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(clearGattrList()) );
	connect( varList, SIGNAL(doubleClicked(Q3ListViewItem*)), this, SLOT(varDoubleClick(Q3ListViewItem*)) );
	activeList = varList;
	listLayout->addWidget(varList);
	
	GattrList = new Q_LIST_VIEW(topWidget, "GattrList");
	GattrList->addColumn("Global Attributes", 200);
	GattrList->setAcceptDrops(false);
	GattrList->setDragAutoScroll(true);
	connect( GattrList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(clearVarList()) );
	connect( GattrList, SIGNAL(doubleClicked(Q3ListViewItem*)), this, SLOT(GattrDoubleClick(Q3ListViewItem*)) );
	listLayout->addWidget(GattrList);
	
	topLayout->addLayout(listLayout);
	
	// Subset panel
	
	QGroupBox *subset_panel=new QGroupBox(topWidget);
	QVBoxLayout * subset_layout=new QVBoxLayout;
	
	fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
	subset_layout->addWidget(fromSlider);
	
	toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");
	subset_layout->addWidget(toSlider);
	
	subset_panel->setLayout(subset_layout);
	topLayout->addWidget(subset_panel);
	
	// Buttons
	
	QHBoxLayout * button_layout=new QHBoxLayout;
	
	getVarBtn = new QPushButton( "Get Data");
	getVarBtn->setMinimumSize(getVarBtn->sizeHint());
	getVarBtn->setDefault(TRUE);
	connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );
	button_layout->addWidget(getVarBtn);

	browseVarBtn = new QPushButton( "Browse Data");
	browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
	connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );
	button_layout->addWidget(browseVarBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	button_layout->addWidget(dismissBtn);

	
	topLayout->addLayout(button_layout);
	
	
	topWidget->setLayout(topLayout);
	setCentralWidget(topWidget);
	
	QieOpt = (QiOptions *) NULL;
	QieContents = (QiCDFContents *) NULL;
	setAttribute(Qt::WA_DeleteOnClose);
}
#endif

void DirectImportWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   getVarBtn->isDefault()) {
		 getVarBtn->setDefault(false);
		 browseVarBtn->setDefault(true);
	  }
	  else if(browseVarBtn->isDefault()) {
		 browseVarBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 getVarBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(getVarBtn->isDefault()) getData();
	 else if (browseVarBtn->isDefault()) browseData();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void DirectImportWindow::launchManual(){ 

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::launchManual %ld\n", this);
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

void DirectImportWindow::launchManDirectImport(){ 

 if(QuSlotDebug())
 {
	printf("DirectImportWindow::launchManDirectImport %ld\n", this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}


void DirectImportWindow::launchVersion(){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::launchVersion %ld\n", this);
  fflush(stdout);
  }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Direct Import:  ";
  text += QString(QIE_VERSION);
  QMessageBox::information( this, "QSAS information", text.latin1(), "OK" ); 

}


void DirectImportWindow::getFileContents(const char *file){

char * ptr;
char * ptr2;
long error;
char *fnamecpy;
int openType;

  QieOpt = QiFreeOptionsObj(QieOpt);                 // always safe
  QieContents = QiFreeCDFContentsObj(QieContents);   // always safe

  QieOpt = (QiOptions *) QiMakeOptionsObj();
  QieContents = (QiCDFContents *) QiMakeCDFContentsObj();

   /* keep global copy of path to add as xref later */
   
    Selected_File_Path = QString(file);
 
    /* parse copy of file name into path, name, extn */
    fnamecpy = QiNewStr(file);

    if( (ptr = strrchr(fnamecpy, '.')) != NULL) {
	  if( strcmp(ptr, ".gz") == 0){
	     // find real extension
	     if( (ptr2 = strstr(fnamecpy, ".cef.gz")) != NULL) ptr = ptr2;
	  }
      QieContents->io_f_extn = QiNewStr(ptr);
      ptr[0] = '\0';
    }
    if( (ptr = strrchr(fnamecpy, '/')) != NULL) {
      QieContents->io_f_name = QiNewStr(&(ptr[1]));
      ptr[0] = '\0';
      QieContents->io_f_path = QiNewStr(fnamecpy);
    }
    else{
      QieContents->io_f_name = QiNewStr(fnamecpy);
      QieContents->io_f_path = QiNewStr("./");
    }
    if( !QistrNULL(fnamecpy) ) free (fnamecpy);

    /* set up options object */


    QieOpt->fp_display = stdout;

    if(QuSlotDebug())
       QieOpt->debug_choice = 'f';
    else
       QieOpt->debug_choice = 'n';
	  
    // check for a recognised file extension and read in data to Contents 

    if (strcmp(QieContents->io_f_extn, ".cdf") == 0 || 
        strcmp(QieContents->io_f_extn, ".CDF") == 0){
        openType = TYPE_CDF;
        QieOpt->f_type = UNSET;  // cdf interface does not use this
    }
    else if (strcmp(QieContents->io_f_extn, ".qft") == 0){
      openType = TYPE_FLAT;
      QieOpt->f_type = TABULAR;
    }
    else if (strcmp(QieContents->io_f_extn, ".cef") == 0 || strcmp(QieContents->io_f_extn, ".cef.gz") == 0){
      openType = TYPE_FLAT;
      int ask = -1; // this tells cefRead2() we are asking not setting flag
      if(cefRead2(ask)) 
         QieOpt->f_type = CAA;
      else
         QieOpt->f_type = EXCHANGE;
    }
    else if (strcmp(QieContents->io_f_extn, ".qfd") == 0){
      openType = TYPE_FLAT;
      QieOpt->f_type = DELIMITED;
    }
    else if (strcmp(QieContents->io_f_extn, ".qfh") == 0){
      openType = TYPE_HEADER;
      QieOpt->f_type = UNSET;
    }
    else{
      openType = TYPE_FLAT;
      QieOpt->f_type = UNSET;

    }

    switch(openType){
    
      case TYPE_CDF:
     
     	error=QiGetCSDSgenCDF(file, QieContents);
        if(error != QMW_OK){
          QuMsgBar( "ERROR reading file ", QString(file), true);
          msgBar->message(QString(QiErrStr(error)) );
	  return;
        }     
      
        break;

      case TYPE_FLAT:
        
        if(QieOpt->f_type == CAA ) initQIE(true);
	    else initQIE(false);

        error=QiGetCSDSgenFlat(QieContents, QieOpt);

        if( error != QMW_OK){
          QuMsgBar( "ERROR reading file ", QString(file), true);
          msgBar->message(QString(QiErrStr(error)) );
	      return;
        }

        // strip out any incomplete variables 

        if (QiMakeSafe(QieContents) == NULL) {
          QuMsgBar( "ERROR reading file ", QString(file), true);
          msgBar->message(QString(QiErrStr(error)) );
	      return;
        }
        
        break;


      case TYPE_HEADER:
        error=QiGetCSDSgenFlat(QieContents, QieOpt);
        if(error != QMW_OK){
          QuMsgBar( "ERROR reading file ", QString(file), true);
          msgBar->message(QString(QiErrStr(error)) );
	  return;
        }
        
        break;

      default:

        QuMsgBar( "Unknown file type ", QString(file), true);
        msgBar->message(QString(QiErrStr(error)) );
	return;
    }

    // Set up display if file read OK
    bool first = true;
    GattrList->clear();
    for(int i=0; i<QieContents->num_g_attrs; i++){
      Q_LIST_VIEW_ITEM * newItem = new Q_LIST_VIEW_ITEM (GattrList, QieContents->g_attr[i]->name);
    }
    vector <Q_LIST_VIEW_ITEM *> metaList; 
    vector <Q_LIST_VIEW_ITEM *> dataList;    
    varList->clear();
    Q_LIST_VIEW_ITEM * sepItem = new Q_LIST_VIEW_ITEM (varList, "   ------------ Support Data ----------");

    
    Q_LIST_VIEW_ITEM * topItem = new Q_LIST_VIEW_ITEM (varList, "     ---------------- Data --------------");

#ifdef QT_VERSION
#if QT_VERSION <0x030000
    topItem->setSelectable(false);
    sepItem->setSelectable(false);
#else
    topItem->setEnabled(false);
    sepItem->setEnabled(false);
#endif // end of #if
#endif // end of #ifdef

    for(int i=0; i<QieContents->n_vars; i++){
      bool isData = true;
      if(QieContents->vardata[i] == NULL) continue;
      for (int j=0; j< QieContents->vardata[i]->num_v_attrs; j++){
	   if( strcmp(QieContents->vardata[i]->attribute[j].name, "PARAMETER_TYPE") == 0) {   //  CAA standard
	     if (QuStrCmp( (char *)QieContents->vardata[i]->attribute[j].data, "Support_Data")) {
		  isData = false;
		  break;
		}
	   }
	   if( strcmp(QieContents->vardata[i]->attribute[j].name, "VAR_TYPE") == 0) {   // ISTP/CSDS standard
	     if (QuStrCmp( (char *)QieContents->vardata[i]->attribute[j].data, "metadata") ) {
		  isData = false;
		  break;
	     }
	     else if (QuStrCmp( (char *)QieContents->vardata[i]->attribute[j].data, "support_data") ) { // added for Themis
		  isData = false;
		  break;
	     }
	   }
	 }
      Q_LIST_VIEW_ITEM * newItem = new Q_LIST_VIEW_ITEM (varList, QieContents->vardata[i]->name);
      if(isData){
	    dataList.push_back(newItem);
	 }
	 else{
	    metaList.push_back(newItem);
	 }
    }
    
 
    // now move metadata
    for(int i=0; i<metaList.size(); i++){
       metaList[i]->moveItem(sepItem);
    }
    
   for(int i=0; i<dataList.size(); i++){
       dataList[i]->moveItem(topItem);
    }
    

  if(QieContents->n_recs <= 1) {
     fromSlider->recBar->setRange(0, 0);
     toSlider->recBar->setRange(0, 0);
     getVarBtn->setEnabled(false);
	 browseVarBtn->setEnabled(false);
     fromSlider->maxRecord = 0;
     toSlider->maxRecord = 0;
	 msgBar->message("WARNING: No records were read, empty file?" );
  }
  else{
     fromSlider->recBar->setRange(0, QieContents->n_recs-1);
     toSlider->recBar->setRange(0, QieContents->n_recs-1);
     fromSlider->maxRecord = QieContents->n_recs-1;
     toSlider->maxRecord = QieContents->n_recs-1;
     toSlider->recBar->setValue(toSlider->maxRecord);
  }
  // get time line and keep globally for this file

  int time_var_num = QieContents->time_var_num;
  int n_recs;
  if( time_var_num >= 0){
    tt =  QdTimeSeq_var::narrow (QdiVarToObj ( QieContents, time_var_num ));
    fromSlider->ttags = tt;
    toSlider->ttags = tt;
  }
  
  // now ttags are set, add time feedback to title
  fromSlider->updateTitle();
  toSlider->updateTitle();
 
}

void DirectImportWindow::clearVarList(){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::clearVarList %ld\n", this);
  fflush(stdout);
 }
 

  Q_LIST_VIEW_ITEM * next = varList->firstChild();
  while(next){
    varList->setSelected(next, false);
    next->listView()->update();
    next = next->nextSibling();
  }
  
  varList->clearSelection();
  activeList = GattrList;
}

void DirectImportWindow::clearGattrList(){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::clearGattrList %ld\n", this);
  fflush(stdout);
 }
 

  Q_LIST_VIEW_ITEM * next = GattrList->firstChild();
  while(next){
    GattrList->setSelected(next, false);
    next->listView()->update();
    next = next->nextSibling();
  }
  
  GattrList->clearSelection();
  activeList = varList;
}

void DirectImportWindow::browseData()
{

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::browseData %ld\n", this);
  fflush(stdout);
 }
 

  QdObject_var  dobj;
  QdObject_var  nil_obj;
  long vnum;
  DataBrowseWindow *QBROWSER_widget;

  // browse either list 
  
  if( activeList == varList) {
    
    Q_LIST_VIEW_ITEM *item = varList->currentItem();
     if(item == 0){
       msgBar->message( "No variable selected");
       return;
     }   
    vnum = QdiFindVarNum(QieContents, item->text(Obj_Name_).latin1() );
  
    if(vnum == -1 || QieContents->vardata[vnum]->max_rec_num < 0) {
      msgBar->message( "No Data Object selected to browse");
      return;
    }

    dobj = qie2qdos(vnum);
     
    //    open browser
    if( !dobj.is_nil() ){
    
      QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, 
                        QieContents->vardata[vnum]->name, dobj, ""); 
      if(QBROWSER_widget != 0) {
         QBROWSER_widget->disallowSaves = true;
	 if(QBROWSER_widget->browser != 0)
	     QBROWSER_widget->editMenu->setEnabled(false);
	     QBROWSER_widget->browser->parentsBtn->setEnabled(false);
      }
    }
  }
  else if( activeList == GattrList){
     Q_LIST_VIEW_ITEM *item = GattrList->currentItem();
     if(item == 0){
       msgBar->message( "No G Attr selected");
       return;
     }
    
     // open G attr
     long Anum = QdiFindGattrNum(QieContents, item->text(Obj_Name_).latin1() );
     dobj = QdiGattrToObj(QieContents, Anum);

     //    open browser
     if( !dobj.is_nil() ){
        QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, 
                        QieContents->g_attr[Anum]->name, dobj, "");
        if(QBROWSER_widget != 0) {
	  QBROWSER_widget->disallowSaves = true;
	  if(QBROWSER_widget->browser != 0)
	     QBROWSER_widget->editMenu->setEnabled(false);
	     QBROWSER_widget->browser->parentsBtn->setEnabled(false);
	}
     }
  }
  

}

void DirectImportWindow::varDoubleClick(Q_LIST_VIEW_ITEM* item){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::varDoubleClick %ld, item %ld\n", this, item);
  fflush(stdout);
 }
 
  if(item == 0) return;
  activeList = varList;
  varList->setCurrentItem(item);
  getData();

}

void DirectImportWindow::GattrDoubleClick(Q_LIST_VIEW_ITEM* item){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::GattrDoubleClick %ld, item %ld\n", this, item);
  fflush(stdout);
 }
 

  if(item == 0) return;
  activeList = GattrList;
  GattrList->setCurrentItem(item);
  browseData();

}

void DirectImportWindow::getData()
{

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::getData %ld\n", this);
  fflush(stdout);
 }

  QdObject_var  dobj;
  long vnum;

  // import from either list 
  
  if( activeList == varList) {
  
    Q_LIST_VIEW_ITEM *item = varList->currentItem();
    if(item == 0) {
      msgBar->message( "No variable selected");
      return;
    }
   
    vnum = QdiFindVarNum(QieContents, item->text(Obj_Name_).latin1() );
    if(vnum == -1 || QieContents->vardata[vnum]->max_rec_num < 0) {
      msgBar->message( "No Data Object selected");
      return;
    }
    
     dobj = qie2qdos(vnum);
     
//    Add object name to WL
    if( !dobj.is_nil() ){
      char * sanitizedVarName = QuSanitizeName(QieContents->vardata[vnum]->name);
      QwuiPutOnWL((QdObject_var) dobj, sanitizedVarName, "Data");
      free(sanitizedVarName);
    } 
  }
  else if(activeList == GattrList){

     Q_LIST_VIEW_ITEM *item = GattrList->currentItem();
     if(item == 0){
       msgBar->message( "No G Attr selected");
       return;
     }
    
     // open G attr
     long Anum = QdiFindGattrNum(QieContents, item->text(Obj_Name_).latin1() );
     dobj = QdiGattrToObj(QieContents, Anum);

     //   put G attr on WL
     if( !dobj.is_nil() ){
       char * sanitizedGattrName = QuSanitizeName(QieContents->g_attr[Anum]->name);
       QwuiPutOnWL((QdObject_var) dobj, sanitizedGattrName, "Data");
       free(sanitizedGattrName);
     }

  }
}



QdObject_var DirectImportWindow::qie2qdos(long vnum)
{
  QdObject_var  dobj;
  QdObject_var  nil_obj;
     
  importedVarNos.clear();  // clear list of xref vars read
  importedVars.clear();  // clear list of xref vars read
  
  try{

    dobj =  QdiVarToObj ( QieContents, vnum );  

    if ( is_empty(dobj) ) {
      msgBar->message( "Failed to read variable");
      return nil_obj;
    }
 
  }
  catch(Exception &e){
    QString Serr = QString(e.msg().c_str());
    Serr += QString(" variable number %1").arg(vnum);
    msgBar->message(Serr ); 
    return nil_obj;
  }
  // add to list of variables read in this request
  importedVarNos.push_back(vnum);
  importedVars.push_back(dobj);
  
  // get time line (will be needed by some attributes, so set now
  
  //   QcvGetTimeVar returns ...
  //   epoch if DEPEND_0 exists and points to an epoch
  //   -2 if DEPEND_0 points to a non-epoch var
  //   -1 if DEPEND_0 does NOT exist but QieContents->time_var_num exists
  //   -2 if varnum is itself an epoch 
  //   -2 if this variable is not record varying (e.g. most xrefs)
  //   -1 if var is record varying, but time tags not explicitly set

  int  time_var_num = QdiGetTimeVar(QieContents, vnum);
  if( time_var_num == -1) time_var_num = QieContents->time_var_num; // use first tt from file
  if( time_var_num >= 0 ){

      // get time line for object (based on Depend_0, not epoch in file) subset records (fromRec, toRec)
      QdTimeSeq_var tt_for_var =  QdTimeSeq_var::narrow (QdiVarToObj ( QieContents, time_var_num ));
      if( !tt_for_var.is_nil() ){
	    // add time tags object to list of vars read in
         importedVarNos.push_back(time_var_num);
 	    importedVars.push_back(tt_for_var);
	    set_timetags(dobj,  tt_for_var);
	    // set xrefs for time tags
	    QdiSet_VAttrs(tt_for_var, QieContents, time_var_num);
      }
  }


  // Add Xrefs to V attributes for dobj itself
  // NOTE: attributes cannot be set in QdiVarToObj() as this can result in infinite loops for
  // metadata that refers back to data (e.g. time dependent delta plus for time tags)

  QdiSet_VAttrs(dobj, QieContents, vnum);
  
  // Add full file name and path to allow browsing global metadata

  QdStringSeq_var file_seq = new QdStringSeq(1);
  file_seq[0] = QdString(Selected_File_Path.latin1()); 
  dobj->change_xref("Parents", (QdObject_var) file_seq); 
    
     
  return dobj;
 
}

void DirectImportWindow::close(){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::close %ld\n", this);
  fflush(stdout);
 }
 

  QieOpt = QiFreeOptionsObj(QieOpt);            // always safe
  QieContents = QiFreeCDFContentsObj(QieContents);   // always safe
 
  QMainWindow::close();
}

//*************************************************************

QdObject_var  DirectImportWindow::QdiVarToObj ( QiCDFContents* info, int vnum){

int  i,j,k;
int seq_size;
QdObject_var  dobj;

  if (vnum > info->n_vars - 1 || vnum < 0) return dobj;
  if(info->vardata[vnum] == 0) return dobj;

  bool intOK;
  int fromRec = fromSlider->recField->text().toInt(&intOK);
  if(!intOK) fromRec = 0;
  
  int toRec = toSlider->recField->text().toInt(&intOK);
  if(!intOK) toRec = toSlider->maxRecord;

  if(info->vardata[vnum]->rec_vary == NOVARY){
    toRec = 0;
    fromRec = 0;
  }

  if( info->vardata[vnum]->max_rec_num >0 && info->vardata[vnum]->rec_vary == VARY)
      seq_size = (int)(info->vardata[vnum]->max_rec_num +1);
  else
      seq_size = 1;
      
  
// test for Data Object is TimeTags object

if( info->vardata[vnum]->data_type == CDF_EPOCH ) {
   if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2){
     // assume to be a time interval (NRV) sequences of Ti not handled yet
    
	 double *ep = (double *)info->vardata[vnum]->data;
	 QdTimeInterval_var ti = new QdTimeInterval(); 

	 Time *iv_start = new QDOS::Time(cdfEpoch(ep[0]));
	 Time *iv_end = new QDOS::Time(cdfEpoch(ep[1]));
	 ti->set(*iv_start, *iv_end );
	    
     // This is a time sequence object so broaden to the QdObject
     dobj = ti;
  }
  else{ // treat as time sequence
 
    QdTimeSeq_var ts = new QdTimeSeq();
    double *ep = (double *)info->vardata[vnum]->data;
  	for (int i = fromRec; i<= toRec; i++){
       ts->push_back( cdfEpoch(ep[i]) );
    }
   
    // This is a time sequence object so broaden to the QdObject
    dobj = ts;
  }
}
else if( info->vardata[vnum]->data_type == ISO_TIME ) {

  if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2){
    // assume to be a time interval (NRV) sequences of Ti not handled yet
    
	 char *sp = (char *)info->vardata[vnum]->data;
	 char sp_0[ISO_TIME_LEN+1];
	 char sp_1[ISO_TIME_LEN+1];
	 strncpy(sp_0, sp, ISO_TIME_LEN);
	 strncpy(sp_1, &(sp[ISO_TIME_LEN]), ISO_TIME_LEN);
	 
	 int i = ISO_TIME_LEN-1;
	 while(sp_0[i] == ' ' && i>0) {sp_0[i] = '\0'; i--;} // remove white space
	 Time iv_start(sp_0);
	 
	 i = ISO_TIME_LEN-1;
	 while(sp_1[i] == ' ' && i>0) {sp_1[i] = '\0'; i--;} // remove white space
	 Time iv_end(sp_1);

	 QdTimeInterval_var ti = new QdTimeInterval(iv_start, iv_end); 
	 
     // This is a time sequence object so broaden to the QdObject
     dobj = ti;
  }
  else{ // treat as time sequence
 
     QdTimeSeq_var ts = new QdTimeSeq();
     char *sp = (char *)info->vardata[vnum]->data;
     int s_ptr=0;
     double epochValue;
     // skip strings up to start record
     for (int i=0; i<fromRec; i++)      s_ptr += ISO_TIME_LEN;
   
     for (int i = fromRec; i <= toRec; i++){
       epochValue = QiISOStringToEpoch(&(sp[s_ptr]));
       ts->push_back( cdfEpoch( epochValue) );
       s_ptr += ISO_TIME_LEN;
     }
   
     // This is a time sequence object so broaden to the QdObject
     dobj = ts;
  }
  
}

else{ 
  
  DataBuffer_var data_buffer; 
  // data object
  
  // Sequence of objects

  if(info->vardata[vnum]->data_type == CDF_CHAR ||
     info->vardata[vnum]->data_type == CDF_UCHAR) {
    
    StringVDataBuffer *sdb_p = new StringVDataBuffer();
      
   // if an array set up matrix with dimensions
    int n_bytes_elem = (int) info->vardata[vnum]->num_elems;

    vector<int> dims;
   
    int n_elems=1; 
    for(int i=0;i<info->vardata[vnum]->num_dims;i++)
    {
      int sz=info->vardata[vnum]->dim_sizes[i];
      dims.push_back(sz );
      n_elems*=sz;
    }
    
    sdb_p->set_dims( dims );
    sdb_p->set_majority( DBUFFER_ROW_MAJOR );
   
    const char* p = (const char*)info->vardata[vnum]->data;
    
    for( int r=fromRec; r <= toRec; r++ )
    {
      const char* next = p + r*n_bytes_elem*n_elems;
      for(int i=0;i<n_elems;i++)
      {
        int actualLen=1;
	for (int kk=1; kk<n_bytes_elem; kk++) {
	   if( *(next+i*n_bytes_elem+kk) != '\0') actualLen++;
	}
        string xstr(next+i*n_bytes_elem,actualLen);
        sdb_p->push_back(xstr);
      }
    }
    data_buffer=sdb_p;
    dobj = Factory::clone(data_buffer);
  }
  else{
      // cast all other data types to double 

   // create new matrix sequence object with dimensionality:  info->vardata[vnum]->num_dims
   //  based on array of sizes info->vardata[vnum]->dim_sizes
     
    NumericVDataBuffer *ndb_p = new NumericVDataBuffer();
      
    int n_length = 1;
    for( j=0; j<info->vardata[vnum]->num_dims; j++){
       n_length *= (int) info->vardata[vnum]->dim_sizes[j];
    }

   // if an array set up matrix with dimensions
   
    vector<int> dims;
    for(int i=0;i<info->vardata[vnum]->num_dims;i++)
      dims.push_back( info->vardata[vnum]->dim_sizes[i] );
    
    ndb_p->set_dims( dims );
    ndb_p->set_majority( DBUFFER_ROW_MAJOR );
    
    // if not it will default to a Scalar sequence
    
    int next = fromRec*n_length;
    void *p = info->vardata[vnum]->data;
    
    for( int r = fromRec; r <= toRec; r++ )
    {

// for numeric types use element by element static casting
//   and let ndb_p->push_back(double) do the conversion to double

      
      // skip array with any fill value
      
      switch( info->vardata[vnum]->data_type )
      {
        case CDF_BYTE:
        case CDF_INT1:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((char *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_UINT1:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((unsigned char *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_INT2:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((short *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_UINT2:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((unsigned short *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_INT4:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((int *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_UINT4:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((unsigned int *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_REAL4:
        case CDF_FLOAT:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *( ((float *)p+next) ) );
	    next++;
	  }
          break;
        case CDF_REAL8:
        case CDF_DOUBLE:
          for(int i=0; i<n_length; i++){
            ndb_p->push_back( *((double *)p+next)  );
	    next++;
	  }
          break;
	default:
	  printf("QdiVarToObj: unknown data type\n");
	  fflush(stdout);
	  break;
      }
    }  // loop over records

    data_buffer=ndb_p;
    dobj = Factory::clone(data_buffer);
      
  } // end else on string vs double
    
  if ( dobj.is_nil() ) {
     return dobj;
   }

  
} // end else on data or TT


 return dobj;
 
}


// Utility call for qwui
//----------------------

 void QdiLaunchWindow(QString file, QWidget *parent)
{
  DirectImportWindow *QDI_widget = new DirectImportWindow(parent, "import");
  QSize qsasGUISize = qsasWindow->frameSize();
  QDI_widget->move(qsasGUISize.width(),20);  

  QDI_widget->show();
  
  if ( !file.isNull() ){  // open file and get data
 
     QDI_widget->setCaption(file);

     // open file
     QDI_widget->getFileContents(file.latin1());
     

  }
  else QDI_widget->msgBar->message("no file selected?");

  return;
}

// interface functions
//--------------------------

long QdiGetTimeVar(QiCDFContents *info, int vnum){

 char * depend;
 long i, ii;
 long epoch_var;
 
 // time variables don't take time lines...

 if(vnum == info->time_var_num) return -2; // don't attach tt to tt
 
 // if NRV doesn't take time line
 if( info->vardata[vnum]->rec_vary == NOVARY ) return -2;

 // set default in case no epoch type Depend_0 found
 
 if (info->time_var_num >=0) epoch_var = -1;  // tt exist for file
 
 // look for Depend_0
 
 for(i=0; i<info->vardata[vnum]->num_v_attrs; i++){
   if(strcmp(((info->vardata[vnum]->attribute)+i)->name, "DEPEND_0") == 0){
     /* find var pointed to */
     depend = (char *)((info->vardata[vnum]->attribute)+i)->data;
     
     // find this variable
     for (ii=0; ii<info->n_vars; ii++){
       if(info->vardata[ii] == 0) continue;
       if( strncmp(depend, info->vardata[ii]->name,  
		   strlen(info->vardata[ii]->name)) == 0){
	  if(info->vardata[ii]->data_type == CDF_EPOCH || info->vardata[ii]->data_type == ISO_TIME)    
	      epoch_var = ii;
          else
              // if Depend_0 points to non-epoch, not a time series
     	      epoch_var = -2; 
       }       
     }  // end for on vars  
   } 
 } // end for on attrs
 
 return epoch_var;   /*  may be -1 or -2 flag */
 
}

//*************************************************************

QdObject_var  QdiGattrToObj ( QiCDFContents* info, int Anum){

int seq_size;
QdObject_var  dobj;
QdObject_var  nil_obj;
long n_length;

  if (Anum > info->num_g_attrs - 1 || Anum < 0) return nil_obj;	

  seq_size = (int)(info->g_attr[Anum]->num_entries );
  if(seq_size <= 0) return nil_obj;
    
  // Sequence of objects

  DataBuffer_var data_buffer;

  if(info->g_attr[Anum]->entry[0].data_type == CDF_CHAR ||
     info->g_attr[Anum]->entry[0].data_type == CDF_UCHAR) {
      
   StringVDataBuffer *sdb_p = new StringVDataBuffer();      

   for(int r=0;r<seq_size;r++)
   {
      const char *ptr = (const char *) info->g_attr[Anum]->entry[r].data;
      int n_elems =  (int)info->g_attr[Anum]->entry[r].num_elems;

      string xstr(ptr,n_elems);
      sdb_p->push_back(xstr); 
   }   
   data_buffer=sdb_p;
   dobj = Factory::clone(data_buffer);
  }
  else if(info->g_attr[Anum]->entry[0].data_type == ISO_TIME) {
      
    QdTimeSeq_var ts = new QdTimeSeq();
    for (int r = 0; r<seq_size; r++){
       char *ptr = (char *) info->g_attr[Anum]->entry[r].data;
       double epochValue = QiISOStringToEpoch(ptr);
       ts->push_back( cdfEpoch( epochValue) );
    }
   
   // This is a time sequence object so broaden to the QdObject
    dobj = ts;

  }
  else{
      // cast all other data types to double 

    NumericVDataBuffer *ndb_p = new NumericVDataBuffer();      
    // if an array set up matrix with dimensions
    if(info->g_attr[Anum]->entry[0].num_elems>1)
    { 
      vector<int> dims;
      dims.push_back( info->g_attr[Anum]->entry[0].num_elems );
      ndb_p->set_dims( dims );
      ndb_p->set_majority( DBUFFER_ROW_MAJOR );
    }
    for( int r=0; r<seq_size; r++ )
    {

      n_length =  info->g_attr[Anum]->entry[r].num_elems;
      
      void *p =  info->g_attr[Anum]->entry[r].data;
      
      switch( info->g_attr[Anum]->entry[r].data_type )
      {
        case CDF_BYTE:
        case CDF_INT1:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((char *)p+i) ) );
          break;
        case CDF_UINT1:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((unsigned char *)p+i) ) );
          break;
        case CDF_INT2:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((short *)p+i) ) );
          break;
        case CDF_UINT2:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((unsigned short *)p+i) ) );
          break;
        case CDF_INT4:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((int *)p+i) ) );
          break;
        case CDF_UINT4:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((unsigned int *)p+i) ) );
          break;
        case CDF_REAL4:
        case CDF_FLOAT:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((float *)p+i) ) );
          break;
        case CDF_REAL8:
        case CDF_DOUBLE:
        case CDF_EPOCH:
          for(int i=0; i<n_length; i++)
            ndb_p->push_back( *( ((double *)p+i) ) );
          break;
      }
      
    }  // loop over records
    data_buffer=ndb_p;
    dobj = Factory::clone(data_buffer);
      
  } // end else on string vs double
    
 return dobj;
 
}

//*******************************************************************

void   DirectImportWindow::QdiSet_VAttrs(QdObject_var dobj, QiCDFContents * info, int vnum)
{
 char * ptr_name;
 int ptr_num;
 
 QiCDFVariable * var =  info->vardata[vnum];

 for(int i=0; i<var->num_v_attrs; i++){

 // get either the attribute or the var pointed to by _ptr type attrs
 
  if( ( ptr_num=Qdi_DistinctVarExists( info, vnum, i) ) >= 0 ) {

      bool readAlready = false;
  	 for (int j=0; j < importedVarNos.size(); j++){
	     if(ptr_num == importedVarNos[j]) {
	        readAlready = true;
		   ptr_num = j;
	     }
	 }

     // we've found a variable pointed to by this attribute
	QdObject_var a_obj;
	if(readAlready){
	   a_obj = importedVars[ptr_num];
	}
	else{

	   a_obj = QdiVarToObj(info, ptr_num);

        if( a_obj.is_nil() ){
            continue;
        }
	   importedVarNos.push_back(ptr_num);
	   importedVars.push_back(a_obj);
	   
	   // set tt for xref object (may differ from parent)
	   int  time_var_num = QdiGetTimeVar(QieContents, ptr_num);
	   QdTimeSeq_var tt_for_xref;
        if( time_var_num >= 0 ){
           // has this time var already been read in
		 bool ttAlreadyRead = false;
		 for(int j=0; j<importedVarNos.size(); j++){
		    if(time_var_num == importedVarNos[j]) {
		       ttAlreadyRead = true;
			  time_var_num = j;
			  break;
		    }
		 }
		 if(ttAlreadyRead){
		 	 tt_for_xref = QdTimeSeq_var::narrow(importedVars[time_var_num]);
	           if( !tt_for_xref.is_nil() ) set_timetags(a_obj, tt_for_xref);
		 }
		 else{
              // get time line for object (based on Depend_0, not epoch in file) subset records (fromRec, toRec)
              tt_for_xref =  QdTimeSeq_var::narrow (QdiVarToObj ( QieContents, time_var_num ));
              if( !tt_for_xref.is_nil() ){
	            set_timetags(a_obj,  tt_for_xref);
	   		  importedVarNos.push_back(time_var_num);
	   		  importedVars.push_back(tt_for_xref);
	            // set xrefs for time tags
	            QdiSet_VAttrs(tt_for_xref, QieContents, time_var_num);
              }
		 }
        }
	   else if(time_var_num == -1 && a_obj->sequence_size() == dobj->sequence_size()){
	       // no Depend_0 so use parent object tt
	       tt_for_xref = get_timetags(dobj);
	       if( !tt_for_xref.is_nil() ) set_timetags(a_obj,  tt_for_xref);
	   }

        // xref attributes were not set by QdiVarToObj
	   QdiSet_VAttrs(a_obj, info, ptr_num);

        QuSetTxtAttr (PTR_VAR_NAME,  (char *) (var->attribute[i]).data, a_obj); 
     }
	
     try{
       dobj->change_xref( QdiAttrName((var->attribute[i]).name) , a_obj);  
     }
     catch(Exception &e){
       QuMsgBar( QString(e.msg().c_str()), (var->attribute[i]).name, true ); 
     }
     

  }
  else if(ptr_num == -1) { // this really is an attribute, so add it as Xref
     try{
        QdiAttrToXref(dobj, var->attribute[i]);
     }
     catch(Exception &e){
       QuMsgBar( QString(e.msg().c_str()), (var->attribute[i]).name, true ); 
     }

  } 
  // else ptr_num == -2 is DEPEND_0 which is skipped
   
 } // end for over attributes by number

  return;
}


//*******************************************************************
     
int Qdi_DistinctVarExists(QiCDFContents *info, int vnum, int attr) {
     
   // if attribute is text, find if it points to a variable
   
   if( (info->vardata[vnum]->attribute[attr]).data_type != CDF_CHAR && 
       (info->vardata[vnum]->attribute[attr]).data_type != CDF_UCHAR)
           return -1;

   int n_elems = (int) strlen( (char *)(info->vardata[vnum]->attribute[attr]).data);
      
   char * ptr = (char *) (info->vardata[vnum]->attribute[attr]).data;
   char entry[n_elems+1];
   strncpy(entry, ptr, n_elems);
   entry[n_elems] = '\0';

   if ( strcmp((info->vardata[vnum]->attribute[attr]).name, "DEPEND_0") == 0)
   return -2;   // epoch done already      

   // find the var referred to 
   for (int k=0; k<info->n_vars; k++){
     if(info->vardata[k] == 0) continue;
     if (strcmp((info->vardata[k])->name, entry) == 0) {
       if( k == vnum ) return -1; // ignore if the same variable
       return k;
     }
   }
   return -1;
}


//*******************************************************************

char * QdiAttrName(char * in_name){
 static char attr_name[ATTR_NAME_LEN+1];

 char * ptr;
 
 // this function is benign if attribute name is not
 //  of ptr type and can safely be called for all attributes
 
 strncpy(attr_name, in_name, ATTR_NAME_LEN); // ensure we return something
 
 // look for ISTP/CSDS names and clean up 
 if (strcmp("LABL_PTR_1", in_name) == 0){
   strcpy(attr_name, "LABLAXIS");
   return &attr_name[0];
 }
 if (strcmp("UNIT_PTR", in_name) == 0){
   strcpy(attr_name, "UNITS");
   return &attr_name[0];
 }
 if (strcmp("FORM_PTR", in_name) == 0){
   strcpy(attr_name, "FORMAT");
   return &attr_name[0];
 }
 
 // look for generic _ptr names, i.e. SI_conversion_ptr
 
 ptr = strstr (attr_name, "_ptr");
 if (ptr == NULL)  ptr = strstr (attr_name, "_PTR");
 if (ptr != NULL) ptr[0] = '\0'; // truncate name
 
 // if no ptr type name found return original name
 
 return &attr_name[0];
}


//*******************************************************************


void QdiAttrToXref(QdObject_var dobj, QiSVarAttribute attribute){
  QdObject_var a_obj;
  
    // switch on data type
    // either string, ISO_TIME or cast to double
    
    DataBuffer_var data_buffer;
    if( attribute.data_type == CDF_CHAR || 
       attribute.data_type == CDF_UCHAR){
      
       StringVDataBuffer* sdb_p=new StringVDataBuffer();
       vector<int> dims;
       if( attribute.num_elems > 1){
           dims.push_back( (int) attribute.num_elems );
	   sdb_p->set_dims( dims );
           sdb_p->set_majority( DBUFFER_ROW_MAJOR );
       }
       
       int n_length = (int) attribute.num_elems;
        
       char *p = ( char*) attribute.data;
       
       for(int i=0; i<n_length; i++){
         
         int n_elems = (int)strlen(p);
         
         string sdata(p,n_elems);

         sdb_p->push_back(sdata);
         p += n_elems + 1;       // jump string and terminator       
	 
       }
       
       data_buffer=sdb_p;
       a_obj = Factory::clone(data_buffer);

      
    }
    else if( attribute.data_type == ISO_TIME) {
      
      char *sp = (char *)attribute.data;
      double epochValue = QiISOStringToEpoch(sp);
      QdTime_var tv = new QdTime(cdfEpoch( epochValue));
  
      a_obj = tv;

    }
    else if( attribute.data_type == CDF_EPOCH ){
      
      double *dp = (double *)attribute.data;
      QdTime_var tv = new QdTime(cdfEpoch( *dp));
  
      a_obj = tv;

    }
    else{
      // cast all other data types to double matrix

    NumericVDataBuffer *ndb_p = new NumericVDataBuffer();
    vector<int> dims;
      if( attribute.num_elems > 1)
        dims.push_back( (int) attribute.num_elems );

    ndb_p->set_dims( dims );
    ndb_p->set_majority( DBUFFER_ROW_MAJOR );

    int n_length = (int) attribute.num_elems;


// for numeric types use element by element static casting
//   and let ndb_p->push_back(double) do the conversion to double

      void *p = attribute.data;

      switch( attribute.data_type )
      {
        case CDF_BYTE:
        case CDF_INT1:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<char*>(p) + i ) );
          break;
        case CDF_UINT1:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<unsigned char*>(p) + i ) );
          break;
        case CDF_INT2:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<short*>(p) + i ) );
          break;
        case CDF_UINT2:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<unsigned short*>(p) + i ) );
          break;
        case CDF_INT4:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<int*>(p) + i ) );
          break;
        case CDF_UINT4:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<unsigned int*>(p) + i ) );
          break;
        case CDF_REAL4:
        case CDF_FLOAT:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<float*>(p) + i ) );
          break;
        case CDF_REAL8:
        case CDF_DOUBLE:
          for(int i=0; i<n_length; i++)
	    ndb_p->push_back( *( static_cast<double*>(p) + i ) );
          break;
      }

    data_buffer=ndb_p;
    a_obj = Factory::clone(data_buffer);


  } // end else on string vs double



   dobj->change_xref(attribute.name, a_obj);

   return;
}


long  QdiFindVarNum(QiCDFContents *QieCont, const char *name ){
  for(int i=0; i< QieCont->n_vars; i++){
    if(QieCont->vardata[i] == NULL) continue;
    if( strcmp(QieCont->vardata[i]->name, name) == 0 ) return i;
  }
  return -1;
}

long  QdiFindGattrNum(QiCDFContents *QieCont, const char *name ){
  for(int i=0; i< QieCont->num_g_attrs; i++){
    if( strcmp(QieCont->g_attr[i]->name, name) == 0 ) return i;
  }
  return -1;
}

// -------------------------------------------------------------
// ---- QDOS 2 Native QIE interface ----------------------------
// -------------------------------------------------------------

// -----------------------------------------------------------------------


DirectAccessWindow::DirectAccessWindow( QWidget *parent, const char *name ) 
  : QMainWindow(parent,name)
#if QT_VERSION < 0x040000
{

   msgBar = statusBar();
 
   menuBar()->insertSeparator();   
   QPopupMenu * help = new QPopupMenu( this );
   menuBar()->insertItem( " Help", help );
   help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
   help->insertSeparator();
   help->insertItem("...Data Import", this, SLOT( launchManDirectImport() ));
   help->insertSeparator();
   help->insertItem("Module Version", this, SLOT( launchVersion() ));
      
   QVGroupBox *_topV = new QVGroupBox( this, "TopWidget" );
   setCentralWidget( _topV );
   
   QHGroupBox *list_panel = new QHGroupBox(_topV, "lists");

   varList = new Q_LIST_VIEW(list_panel, "varList");
   varList->addColumn("Variables", 200);
   varList->setAcceptDrops(false);
   varList->setDragAutoScroll(true);
   varList->setSorting(-1);

   connect( varList, SIGNAL(clicked(QListViewItem*)), this, SLOT(setSliderForItem(QListViewItem*)) );
   connect( varList, SIGNAL(clicked(QListViewItem*)), this, SLOT(clearGattrList()) );
   connect( varList, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(varDoubleClick(QListViewItem*)) );

   activeList = varList;
   
   GattrList = new Q_LIST_VIEW(list_panel, "GattrList");
   GattrList->addColumn("Global Attributes", 200);
   GattrList->setAcceptDrops(false);
   GattrList->setDragAutoScroll(true);

   connect( GattrList, SIGNAL(clicked(QListViewItem*)), this, SLOT(clearVarList()) );
   connect( GattrList, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(GattrDoubleClick(QListViewItem*)) );

   subset_panel = new QVGroupBox("Subset on input", _topV, "subset" );

   useTcheck = new QCheckBox(subset_panel, "useTcheck");
   useTcheck->setText("Get time for subsetting ");
   useTcheck->setDown(false);
   connect(useTcheck, SIGNAL(clicked( )), this, SLOT(tRangeToggle( )) );
   
   fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
   toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");

   
   QHGroupBox *button_panel = new QHGroupBox(_topV, "buttons");

   getVarBtn = new QPushButton( "Get Data", button_panel, "getVarBtn" );
   getVarBtn->setMinimumSize(getVarBtn->sizeHint());
   getVarBtn->setDefault(TRUE);
   
   connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );

   browseVarBtn = new QPushButton( "Browse Data", button_panel, "browseVarBtn" );
   browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
   connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );

   dismissBtn = new QPushButton( "Dismiss", button_panel, "dismissBtn" );
   dismissBtn->setMinimumSize(dismissBtn->sizeHint());
   connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );

   QieDataCont = new QieData();

   QWidget::setWFlags(Qt::WDestructiveClose);
}
#else
{
	// Status bar
	
	msgBar = statusBar();
	
	// Menu
	
	menuBar()->insertSeparator();   
	QMenu * help = new QMenu( this );
	menuBar()->insertItem( " Help", help );
	help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->insertSeparator();
	help->insertItem("...Data Import", this, SLOT( launchManDirectImport() ));
	help->insertSeparator();
	help->insertItem("Module Version", this, SLOT( launchVersion() ));
	
	// Top widget
	
	QWidget * topWidget=new QWidget(this);
	QVBoxLayout * topLayout=new QVBoxLayout;
	
	// List panel
	
	QHBoxLayout * listLayout=new QHBoxLayout;
	
	varList = new Q_LIST_VIEW(topWidget, "varList");
	varList->addColumn("Variables", 200);
	varList->setAcceptDrops(false);
	varList->setDragAutoScroll(true);
	varList->setSorting(-1);
	connect( varList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(setSliderForItem(Q3ListViewItem*)) );
	connect( varList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(clearGattrList()) );
	connect( varList, SIGNAL(doubleClicked(Q3ListViewItem*)), this, SLOT(varDoubleClick(Q3ListViewItem*)) );
	activeList = varList;
	listLayout->addWidget(varList);
	
	GattrList = new Q_LIST_VIEW(topWidget, "GattrList");
	GattrList->addColumn("Global Attributes", 200);
	GattrList->setAcceptDrops(false);
	GattrList->setDragAutoScroll(true);
	connect( GattrList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(clearVarList()) );
	connect( GattrList, SIGNAL(doubleClicked(Q3ListViewItem*)), this, SLOT(GattrDoubleClick(Q3ListViewItem*)) );
	listLayout->addWidget(GattrList);
	
	topLayout->addLayout(listLayout);
	
	// Subset panel
	
	subset_panel=new QGroupBox(topWidget);
	QVBoxLayout * subset_layout=new QVBoxLayout;
	
	useTcheck = new QCheckBox;
	useTcheck->setText("Get time for subsetting ");
	useTcheck->setDown(false);
	connect(useTcheck, SIGNAL(clicked( )), this, SLOT(tRangeToggle( )) );
	subset_layout->addWidget(useTcheck);
	
	fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
	subset_layout->addWidget(fromSlider);
	
	toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");
	subset_layout->addWidget(toSlider);
	
	subset_panel->setLayout(subset_layout);
	topLayout->addWidget(subset_panel);
	
	// Buttons
	
	QHBoxLayout * button_layout=new QHBoxLayout;
	
	getVarBtn = new QPushButton( "Get Data");
	getVarBtn->setMinimumSize(getVarBtn->sizeHint());
	getVarBtn->setDefault(TRUE);
	connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );
	button_layout->addWidget(getVarBtn);

	browseVarBtn = new QPushButton( "Browse Data");
	browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
	connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );
	button_layout->addWidget(browseVarBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	button_layout->addWidget(dismissBtn);
	
	topLayout->addLayout(button_layout);
	
	topWidget->setLayout(topLayout);
	setCentralWidget(topWidget);
	
	QieDataCont = new QieData();
	setAttribute(Qt::WA_DeleteOnClose);
	
	//////////
// 	msgBar = statusBar();
//  
// 	menuBar()->insertSeparator();   
// 	Q_POPUP_MENU * help = new Q_POPUP_MENU( this );
// 	menuBar()->insertItem( " Help", help );
// 	help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
// 	help->insertSeparator();
// 	help->insertItem("...Data Import", this, SLOT( launchManDirectImport() ));
// 	help->insertSeparator();
// 	help->insertItem("Module Version", this, SLOT( launchVersion() ));
//       
// 	Q_V_GROUP_BOX *_topV = new Q_V_GROUP_BOX( this, "TopWidget" );
// 	setCentralWidget( _topV );
//    
// 	Q_H_GROUP_BOX *list_panel = new Q_H_GROUP_BOX(_topV, "lists");
// 
// 	varList = new Q_LIST_VIEW(list_panel, "varList");
// 	varList->addColumn("Variables", 200);
// 	varList->setAcceptDrops(false);
// 	varList->setDragAutoScroll(true);
// 	varList->setSorting(-1);
// #if QT_VERSION < 0x040000
// 	connect( varList, SIGNAL(clicked(QListViewItem*)), this, SLOT(setSliderForItem(QListViewItem*)) );
// 	connect( varList, SIGNAL(clicked(QListViewItem*)), this, SLOT(clearGattrList()) );
// 	connect( varList, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(varDoubleClick(QListViewItem*)) );
// #else
// 	connect( varList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(setSliderForItem(Q3ListViewItem*)) );
// 	connect( varList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(clearGattrList()) );
// 	connect( varList, SIGNAL(doubleClicked(Q3ListViewItem*)), this, SLOT(varDoubleClick(Q3ListViewItem*)) );
// #endif
// 	activeList = varList;
//    
// 	GattrList = new Q_LIST_VIEW(list_panel, "GattrList");
// 	GattrList->addColumn("Global Attributes", 200);
// 	GattrList->setAcceptDrops(false);
// 	GattrList->setDragAutoScroll(true);
// #if QT_VERSION < 0x040000
// 	connect( GattrList, SIGNAL(clicked(QListViewItem*)), this, SLOT(clearVarList()) );
// 	connect( GattrList, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(GattrDoubleClick(QListViewItem*)) );
// #else
// 	connect( GattrList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(clearVarList()) );
// 	connect( GattrList, SIGNAL(doubleClicked(Q3ListViewItem*)), this, SLOT(GattrDoubleClick(Q3ListViewItem*)) );
// #endif
// 
// 	subset_panel = new Q_V_GROUP_BOX("Subset on input", _topV, "subset" );
// 
// 	useTcheck = new QCheckBox(subset_panel, "useTcheck");
// 	useTcheck->setText("Get time for subsetting ");
// 	useTcheck->setDown(false);
// 	connect(useTcheck, SIGNAL(clicked( )), this, SLOT(tRangeToggle( )) );
//    
// 	fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
// 	toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");
// 
//    
// 	Q_H_GROUP_BOX *button_panel = new Q_H_GROUP_BOX(_topV, "buttons");
// 
// 	getVarBtn = new QPushButton( "Get Data", button_panel, "getVarBtn" );
// 	getVarBtn->setMinimumSize(getVarBtn->sizeHint());
// 	getVarBtn->setDefault(TRUE);
//    
// 	connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );
// 
// 	browseVarBtn = new QPushButton( "Browse Data", button_panel, "browseVarBtn" );
// 	browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
// 	connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );
// 
// 	dismissBtn = new QPushButton( "Dismiss", button_panel, "dismissBtn" );
// 	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
// 	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
// 
// 	QieDataCont = new QieData();
// #if QT_VERSION < 0x040000
// 	QWidget::setWFlags(Qt::WDestructiveClose);
// #else
// 	setAttribute(Qt::WA_DeleteOnClose);
// #endif
}
#endif

// ------------------------------------------

void DirectAccessWindow::close(){

 if(QuSlotDebug())
 {
  printf("DirectAccesstWindow::close %ld\n", this);
  fflush(stdout);
 }
 
 QieDataCont->inFile.closeFile();
 
  delete QieDataCont;
  
  QMainWindow::close();
}

// ------------------------------------------

void DirectAccessWindow::tRangeToggle(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::tRangeToggle %ld\n", this);
  fflush(stdout);
 }
 
 
  QApplication::setOverrideCursor( Qt::waitCursor );
	
  // get time variables (note they are cached from earlier calls)
  for(int i=0; i<QieDataCont->QieVars.size(); i++){
	if(QieDataCont->QieVars[i].datatype == "ISO_TIME")  QieDataCont->readVarData(i, 0, -1);
  }
	
  QApplication::restoreOverrideCursor();

  
  if(QieDataCont->time_var_num != -1){
     fromSlider->ttags = QdTimeSeq_var::narrow(QieDataCont->QieVars[QieDataCont->time_var_num].Vobj);
     toSlider->ttags   = QdTimeSeq_var::narrow(QieDataCont->QieVars[QieDataCont->time_var_num].Vobj);
  }

  updateOnSelection();
  useTcheck->setEnabled(false); // prevent being unset
}

// ------------------------------------------

void DirectAccessWindow::launchManual(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchManual %ld\n", this);
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

// ------------------------------------------

void DirectAccessWindow::launchManDirectImport(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchManDirectImport %ld\n", this);
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

// ------------------------------------------

void DirectAccessWindow::launchVersion(){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchVersion %ld\n", this);
  fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Direct Access:  ";
  text += QString(QIE_VERSION);
  QMessageBox::information( this, "QSAS information", text.latin1(), "OK" ); 

}

// ------------------------------------------

int DirectAccessWindow::getFileContents(const char *file){

	int nRecs=0;
	
   // keep global copy of path to add as xref later 
   
	Selected_File_Path = QString(file);

	if(QuSlotDebug())  setDebug(true);
	
	if( QieDataCont->queryFile(file) ){
		// file pointer is at start of data, we need number of records for sliders
		
		nRecs = QieDataCont->readDataRecords();
		
		// Set up display if file read OK
		
		// Set up global list
		GattrList->clear();
		for(int i=0; i<QieDataCont->QieGlobals.size(); i++){
			if(QieDataCont->QieGlobals[i].is_nil() ) continue;
			QdString_var Gname;
			try{
				Gname = QdString_var::narrow(QieDataCont->QieGlobals[i]->get_xref("name"));
			}
			catch(Exception& e){
				continue;
			}
			if( Gname.is_nil() ) continue;
			Q_LIST_VIEW_ITEM * newItem = new Q_LIST_VIEW_ITEM (GattrList, Gname->c_str());
		}
		// set up variable list
		vector <Q_LIST_VIEW_ITEM *> metaList; 
		vector <Q_LIST_VIEW_ITEM *> dataList;    
		varList->clear();
		Q_LIST_VIEW_ITEM * sepItem = new Q_LIST_VIEW_ITEM (varList, "   ------------ Support Data ----------");
		Q_LIST_VIEW_ITEM * topItem = new Q_LIST_VIEW_ITEM (varList, "   ---------------- Data --------------");

#ifdef QT_VERSION
#if QT_VERSION <300
		topItem->setSelectable(false);
		sepItem->setSelectable(false);
#else
		topItem->setEnabled(false);
		sepItem->setEnabled(false);
#endif // end of #if
#endif // end of #ifdef

		
		for(int i=0; i<QieDataCont->QieVars.size(); i++){
			if( QieDataCont->QieVars[i].Vobj.is_nil() ) continue;
			bool isData = true;
			if(QieDataCont->QieVars[i].nRecs == -1){
			    if(QieDataCont->QieVars[i].recVary)
				{
			       QieDataCont->QieVars[i].nRecs = nRecs; // set here as flat files don't know it
				}
				else
				{
				   QieDataCont->QieVars[i].nRecs = 1;
				}
		    }
			QdString_var typeXref;
			if(QieDataCont->QieVars[i].Vobj->xref_exists("PARAMETER_TYPE"))
				 typeXref = QdString_var::narrow(QieDataCont->QieVars[i].Vobj->get_xref("PARAMETER_TYPE"));
			else if (QieDataCont->QieVars[i].Vobj->xref_exists("VAR_TYPE")) 
				typeXref = QdString_var::narrow(QieDataCont->QieVars[i].Vobj->get_xref("VAR_TYPE"));

			if ( !typeXref.is_nil() ){
				if ( *typeXref == QdString("Support_Data") ) isData = false;
				if ( *typeXref == QdString("support_data") ) isData = false;
				if ( *typeXref == QdString("metadata") ) isData = false;
			}
			Q_LIST_VIEW_ITEM * newItem = new Q_LIST_VIEW_ITEM (varList, QieDataCont->QieVars[i].name.c_str());
			if(isData){
				dataList.push_back(newItem);
			}
			else{
				metaList.push_back(newItem);
			}
			
		}
		// now move items into correct parts of list
		for(int i=metaList.size()-1; i>=0; i--){
			metaList[i]->moveItem(sepItem);
		}
		for(int i=dataList.size()-1; i>=0; i--){
			dataList[i]->moveItem(topItem);
		}
		
	}
	else{
		getVarBtn->setEnabled(false);
		browseVarBtn->setEnabled(false);
	}
	
	return nRecs;
}

// ------------------------------------------

void DirectAccessWindow::setSliders(int nRecs){
	// set initial values for sliders
	if( !(getVarBtn->isEnabled()) ) return; // not enabled if file read failed
	 
	if(nRecs == 0) {
		fromSlider->recBar->setRange(0, 0);
		toSlider->recBar->setRange(0, 0);
		getVarBtn->setEnabled(false);
		browseVarBtn->setEnabled(false);
		fromSlider->maxRecord = 0;
		toSlider->maxRecord = 0;
		msgBar->message("WARNING: No records were read, corrupt file?" );
	}
	else{
		fromSlider->recBar->setRange(0, nRecs-1);
		toSlider->recBar->setRange(0, nRecs-1);
		fromSlider->maxRecord = nRecs-1;
		toSlider->maxRecord = nRecs-1;
		
		fromSlider->recBar->setValue(0);
		toSlider->recBar->setValue(toSlider->maxRecord);
	}
	fromSlider->updateTitle();
	toSlider->updateTitle();

}

void DirectAccessWindow::updateOnSelection(QieVariable *V){

	fromSlider->updateTitle();
	toSlider->updateTitle();
	
	if(V == 0 ) return;
	if( !V->ts.is_nil() && V->ts->size() > 0){
	   int seqLength = V->ts->size();
	   string newTitle = string("Subset on input  -------    (")+V->ts[0].iso().c_str() ;
	   newTitle += "   -  ";
	   newTitle += V->ts[seqLength-1].iso().c_str();
	   newTitle +=") ";
		subset_panel->setTitle(newTitle.c_str());
	}
	else if( !(toSlider->ttags.is_nil()) && toSlider->ttags->size() > 0){
	   int seqLength = toSlider->ttags->size();
	   string newTitle = string("Subset on input  -------    (")+toSlider->ttags[0].iso().c_str() ;
	   newTitle += "   -  ";
	   newTitle += toSlider->ttags[seqLength-1].iso().c_str();
	   newTitle +=") ";
		subset_panel->setTitle(newTitle.c_str());
	}
	else{
	   string newTitle = string("Subset on input  -------   from range (0,  ")+int_to_string(V->nRecs)+") ";
	   subset_panel->setTitle(newTitle.c_str());
	}
	
}


// ------------------------------------------

void DirectAccessWindow::setSliderForItem(Q_LIST_VIEW_ITEM *item){
		
	int nRecs;
	int vnum;
	QieVariable *V;
	
	if(item == 0) return;

	// This slot is only used for data files with sparse records,
	// i.e. files where different variables may have different time tags
	
	bool intOK;
	int fromRec = fromSlider->recField->text().toInt(&intOK);
	if (!intOK) fromRec = 0;
	int toRec   = toSlider->recField->text().toInt(&intOK);
	if (!intOK) toRec = toSlider->maxRecord;

	if( activeList == varList) {
	
     		if(item != 0){
			vnum = QieDataCont->findVarNumByName('V', item->text(Obj_Name_).latin1() );
			if( vnum == -1 ) return;
		}
		V = &(QieDataCont->QieVars[vnum]);
		nRecs = V->nRecs;
	}
	else return;
			 
	if(nRecs == 0) {
		fromSlider->recBar->setRange(0, 0);
		toSlider->recBar->setRange(0, 0);
		getVarBtn->setEnabled(false);
		browseVarBtn->setEnabled(false);
		fromSlider->maxRecord = 0;
		toSlider->maxRecord = 0;
		msgBar->message("WARNING: No records were read, corrupt file?" );
	}
	else if(nRecs == 1) {
		fromSlider->recBar->setRange(0, 0);
		toSlider->recBar->setRange(0, 0);
		getVarBtn->setEnabled(true);
		browseVarBtn->setEnabled(true);
		fromSlider->maxRecord = 0;
		toSlider->maxRecord = 0;
		msgBar->message("WARNING: single entry for variable" );
	}
	else{
		fromSlider->maxRecord = nRecs-1;
		toSlider->maxRecord = nRecs-1;
		getVarBtn->setEnabled(true);
		browseVarBtn->setEnabled(true);
		fromSlider->recBar->setRange(0, nRecs-1);
		toSlider->recBar->setRange(0, nRecs-1);
		msgBar->message(" " );

	}

	// reset current slider positions to where they were if safe
	if(toRec == 0) toSlider->recBar->setValue(toSlider->maxRecord);
	else if(toRec > toSlider->maxRecord) toSlider->recBar->setValue(toSlider->maxRecord);
	else toSlider->recBar->setValue(toRec);
		
	if(fromRec > fromSlider->maxRecord) fromSlider->recBar->setValue(0);
	else fromSlider->recBar->setValue(fromRec);

	QdTimeSeq_var ttv = QieDataCont->getTTobj(vnum);
	// attach correct tt for variable
	fromSlider->ttags = ttv;
	toSlider->ttags   = ttv;

	// now ttags are set, add time feedback to title
	updateOnSelection(V);
	fromSlider->setEnabled(true);
	toSlider->setEnabled(true);
}

// -------------------------------------------------------

void DirectAccessWindow::browseData()
{

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::browseData %ld\n", this);
  fflush(stdout);
 }
 

  QdObject_var  dobj;
  QdObject_var  nil_obj;
  int vnum;
  DataBrowseWindow *QBROWSER_widget;

  // browse either list 
  
  if( activeList == varList) {
    
    Q_LIST_VIEW_ITEM *item = varList->currentItem();
     if(item == 0){
       msgBar->message( "No variable selected");
       return;
     }   
    vnum = QieDataCont->findVarNumByName('V', item->text(Obj_Name_).latin1() );
  
    if( vnum == -1 ) {
      msgBar->message( "No Data Object selected to browse");
      return;
    }

	// read data in from file - counting records left us at the end of the file
	
	QieDataCont->readVarData(vnum, 0, toSlider->maxRecord);

    dobj = QieDataCont->QieVars[vnum].Vobj;
     
    //    open browser
    if( !dobj.is_nil() ){
    
      QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, item->text(Obj_Name_).latin1(), dobj, ""); 
      if(QBROWSER_widget != 0) {
			QBROWSER_widget->disallowSaves = true;
			if(QBROWSER_widget->browser != 0) QBROWSER_widget->editMenu->setEnabled(false);
			QBROWSER_widget->browser->parentsBtn->setEnabled(false);
      }
    }
  }
  else if( activeList == GattrList){
     Q_LIST_VIEW_ITEM *item = GattrList->currentItem();
     if(item == 0){
       msgBar->message( "No G Attr selected");
       return;
     }
    
     // open G attr
     int Anum = QieDataCont->findVarNumByName('G', item->text(Obj_Name_).latin1() );
     dobj = QieDataCont->QieGlobals[Anum];

     //    open browser
     if( !dobj.is_nil() ){
        QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, item->text(Obj_Name_).latin1(), dobj, "");
        if(QBROWSER_widget != 0) {
			QBROWSER_widget->disallowSaves = true;
			if(QBROWSER_widget->browser != 0) QBROWSER_widget->editMenu->setEnabled(false);
			QBROWSER_widget->browser->parentsBtn->setEnabled(false);
		}
     }
  }
  

}

// -------------------------------------------------------------

void DirectAccessWindow::clearVarList(){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::clearVarList %ld\n", this);
  fflush(stdout);
 }
 

  Q_LIST_VIEW_ITEM * next = varList->firstChild();
  while(next){
    varList->setSelected(next, false);
    next->listView()->update();
    next = next->nextSibling();
  }
  
  varList->clearSelection();
  activeList = GattrList;
}

// -----------------------------------------------------------

void DirectAccessWindow::clearGattrList(){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::clearGattrList %ld\n", this);
  fflush(stdout);
 }
 

  Q_LIST_VIEW_ITEM * next = GattrList->firstChild();
  while(next){
    GattrList->setSelected(next, false);
    next->listView()->update();
    next = next->nextSibling();
  }
  
  GattrList->clearSelection();
  activeList = varList;
}

// ----------------------------------------------

void DirectAccessWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   getVarBtn->isDefault()) {
		 getVarBtn->setDefault(false);
		 browseVarBtn->setDefault(true);
	  }
	  else if(browseVarBtn->isDefault()) {
		 browseVarBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 getVarBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(getVarBtn->isDefault()) getData();
	 else if (browseVarBtn->isDefault()) browseData();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// ---------------------------------------------

void DirectAccessWindow::varDoubleClick(Q_LIST_VIEW_ITEM* item){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::varDoubleClick %ld, item %ld\n", this, item);
  fflush(stdout);
 }
 
  if( !(getVarBtn->isEnabled()) ) return; // get data disabled
  if(item == 0) return;
  activeList = varList;
  varList->setCurrentItem(item);
  getData();

}

// ---------------------------------------------

void DirectAccessWindow::GattrDoubleClick(Q_LIST_VIEW_ITEM* item){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::GattrDoubleClick %ld, item %ld\n", this, item);
  fflush(stdout);
 }
 

  if(item == 0) return;
  activeList = GattrList;
  GattrList->setCurrentItem(item);
  browseData();

}

// ------------------------------------------------

void DirectAccessWindow::getData()
{

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::getData %ld\n", this);
  fflush(stdout);
 }

  QdObject_var  dobj;
  int vnum;
  int anum;

  // import from either list 
  
  if( activeList == varList) {
  
    Q_LIST_VIEW_ITEM *item = varList->currentItem();
    if(item == 0) {
      msgBar->message( "No variable selected");
      return;
    }
	bool intOK;
	int fromRec = fromSlider->recField->text().toInt(&intOK);
	if(!intOK) fromRec = 0;
  
	int toRec = toSlider->recField->text().toInt(&intOK);
	if(!intOK) toRec = toSlider->maxRecord;

    vnum = QieDataCont->findVarNumByName('V', item->text(Obj_Name_).latin1() );
    if(vnum == -1 ) {
      msgBar->message( "No Data Object selected");
      return;
    }

    if( !(QieDataCont->QieVars[vnum].recVary) ){
	toRec = 0;
	fromRec = 0;
    }
	
    QieDataCont->readVarData(vnum, fromRec, toRec);

     dobj = QieDataCont->QieVars[vnum].Vobj;
     
     // Add parents xref for QSAS browse window use
     QdStringSeq_var file_seq = new QdStringSeq(1);
     file_seq[0] = QdString(QieDataCont->inFile.name); 

     dobj->change_xref("Parents", (QdObject_var) file_seq); 
  
//    Add object name to WL
    if( !dobj.is_nil() ){
      char * sanitizedVarName = QuSanitizeName(item->text(Obj_Name_).latin1());
      QwuiPutOnWL((QdObject_var) dobj, sanitizedVarName, "Data");
      free(sanitizedVarName);
    }
  }
  else if(activeList == GattrList){

     Q_LIST_VIEW_ITEM *item = GattrList->currentItem();
     if(item == 0){
       msgBar->message( "No G Attr selected");
       return;
     }
    
     // open G attr
     anum = QieDataCont->findVarNumByName('G', item->text(Obj_Name_).latin1() );
     dobj = QieDataCont->QieGlobals[anum];

     //   put G attr on WL
     if( !dobj.is_nil() ){
       char * sanitizedGattrName = QuSanitizeName(item->text(Obj_Name_).latin1());
       QwuiPutOnWL((QdObject_var) dobj, sanitizedGattrName, "Data");
       free(sanitizedGattrName);
     }

  }
}



// Utility call for qwui
//----------------------

 void QdaLaunchWindow(QString file, QWidget *parent)
{
  // direct access window
  
	DirectAccessWindow *QDA_widget = new DirectAccessWindow(parent, "import");
	QSize qsasGUISize = qsasWindow->frameSize();
	QDA_widget->move(qsasGUISize.width(),20);
	
	if ( !file.isNull() ){  // open file and get data
 
		QDA_widget->setCaption(file);
		
		QDA_widget->msgBar->message("PLEASE WAIT, Counting data records..." );
		
		// open file
		int nRecs = QDA_widget->getFileContents(file);

		QDA_widget->show();

		// set sliders
		
		QDA_widget->setSliders(nRecs);
		QDA_widget->msgBar->message(" " );

	}
	else {
		QDA_widget->msgBar->message("no file selected?");
		QDA_widget->show();
	}

  return;
}
