//  qexui.cc
//
// Data Export Widget
//
// Nov 2001: Tony Allen
//

#include "qexuiW.h"
#include <qpalette.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qpushbutton.h>
#include <qsize.h>

#include <stdlib.h>

#include "qdutil.h"
#include "qwui.h"
#include "qie_qsas.h"

extern QiCDFContents * QiConts;
extern QiOptions * qi_options;
extern bool isTimeSeries;

#define  COMMAdelim 0
#define  SPACEdelim 1

#define  NEWLINEdelim 0
#define  DOLLARdelim 1
#define  HASHdelim 2
#define  TABdelim 3
 
const int ask = -1; // this tells cefWrite2() we are asking not setting the value

extern  qwuiWindow* qsasWindow;

#if QT_VERSION < 0x040000
DataExportWindow::DataExportWindow( QWidget *parent, const char *name ) 
  : QFileDialog(parent,name,true)
{
  this->setPalette(qApp->palette());

  QStringList ext;
  if(cefWrite2(ask))
    ext.append("Cluster Exchange Format for CAA, cef2 (*.cef)");
  else
    ext.append("Cluster Exchange Format, cef1 (*.cef)");
  ext.append("Common Data Format (*.cdf)");
  ext.append("ASCII Tabular (*.qft)");
  ext.append("ASCII Delimited (*.qfd)");
  setFilters(ext);
  
  _OverWrite = new QCheckBox("Overwrite file", this, "Overwrite");
  _OverWrite->setChecked(FALSE);
  _OverWrite->setMaximumHeight(18);

  addWidgets(0, _OverWrite, 0);  
  
   QWidget* _top_widget = new QWidget(this,"top_widget");

   _top_gl = new QGridLayout( _top_widget, 1, 1 );
   _top_gl->setSpacing(4);
   _top_gl->setMargin(4);

  QVBoxLayout *opts_layout=new QVBoxLayout(); 
  _top_gl->addLayout(opts_layout,0,0);


  options_groupbox =
     new QGroupBox( 1, Qt::Vertical, _top_widget, "options_groupbox");
  
  options_groupbox->setTitle("ASCII File Settings");

  opts_layout->addWidget(options_groupbox);

  QWidget* options_widget =
     new QWidget(options_groupbox,"options_widget");

  QGridLayout* options_layout = new QGridLayout( options_widget, 5, 2);
  options_layout->setSpacing(4);
  options_layout->setColStretch(1,10);

  QLabel* DataDelim_label = new QLabel("Data Delimiter: ",options_widget,"DataDelim_label");
  options_layout->addWidget( DataDelim_label, 0, 0, Qt::AlignRight );

  _DataDelim = new QComboBox(false,options_widget,"DataDelim");
  _DataDelim->insertItem("Comma", COMMAdelim );
  _DataDelim->insertItem("Space", SPACEdelim );
  options_layout->addWidget( _DataDelim, 0, 1, Qt::AlignLeft );

  QLabel* RecDelim_label = new QLabel("Record Delimiter: ",options_widget,"RecDelim_label");
  options_layout->addWidget( RecDelim_label, 1, 0, Qt::AlignRight );

  _RecDelim = new QComboBox(false,options_widget,"RecDelim");
  _RecDelim->insertItem("Newline", NEWLINEdelim );
  _RecDelim->insertItem("Dollar", DOLLARdelim );
  _RecDelim->insertItem("Hash", HASHdelim );
  _RecDelim->insertItem("Tab", TABdelim );
  options_layout->addWidget( _RecDelim, 1, 1, Qt::AlignLeft );

  _RecNumOn = new QCheckBox( "Record Numbering", options_widget, "RecNumOn" );
  _RecNumOn->setChecked(FALSE);
  options_layout->addWidget( _RecNumOn, 4, 0, Qt::AlignLeft );

  _AttachHead = new QCheckBox( "Attached Header", options_widget, "RecNumOn" );
  _AttachHead->setChecked(TRUE);
  options_layout->addWidget( _AttachHead, 3, 0, Qt::AlignLeft );

  addRightWidget( _top_widget );
  
  this->setMinimumSize(this->sizeHint() * 1.25);

}
#else
DataExportWindow::DataExportWindow( QWidget *parent, const char *name ) 
	: QFileDialog(parent)
{
	this->setPalette(qApp->palette());

	QStringList ext;
	if(cefWrite2(ask))
		ext.append("Cluster Exchange Format for CAA, cef2 (*.cef)");
	else
		ext.append("Cluster Exchange Format, cef1 (*.cef)");
	ext.append("Common Data Format (*.cdf)");
	ext.append("ASCII Tabular (*.qft)");
	ext.append("ASCII Delimited (*.qfd)");
	setFilters(ext);
	
	_OverWrite = new QCheckBox("Overwrite file", this, "Overwrite");
	_OverWrite->setChecked(FALSE);
	_OverWrite->setMaximumHeight(18);
	
	options_groupbox = new QGroupBox( "ASCII File Settings", this);
	QGridLayout* options_layout = new QGridLayout( options_groupbox);
	options_layout->setSpacing(4);
	options_layout->setColStretch(1,10);
	
	QLabel* DataDelim_label = new QLabel("Data Delimiter: ");
	options_layout->addWidget( DataDelim_label, 0, 0, Qt::AlignRight );
	_DataDelim = new QComboBox(false,options_groupbox);
	_DataDelim->insertItem("Comma", COMMAdelim );
	_DataDelim->insertItem("Space", SPACEdelim );
	options_layout->addWidget( _DataDelim, 0, 1, Qt::AlignLeft );
	
	QLabel* RecDelim_label = new QLabel("Record Delimiter: ",options_groupbox);
	options_layout->addWidget( RecDelim_label, 1, 0, Qt::AlignRight );

	_RecDelim = new QComboBox(false,options_groupbox);
	_RecDelim->insertItem("Newline", NEWLINEdelim );
	_RecDelim->insertItem("Dollar", DOLLARdelim );
	_RecDelim->insertItem("Hash", HASHdelim );
	_RecDelim->insertItem("Tab", TABdelim );
	options_layout->addWidget( _RecDelim, 1, 1, Qt::AlignLeft );

	_RecNumOn = new QCheckBox( "Record Numbering", options_groupbox);
	_RecNumOn->setChecked(FALSE);
	options_layout->addWidget( _RecNumOn, 4, 0, Qt::AlignLeft );

	_AttachHead = new QCheckBox( "Attached Header", options_groupbox);
	_AttachHead->setChecked(TRUE);
	options_layout->addWidget( _AttachHead, 3, 0, Qt::AlignLeft );
	
	
	this->layout()->addWidget(new QWidget());//blank
	this->layout()->addWidget(_OverWrite);
	this->layout()->addWidget(new QWidget());//blank
	
	this->layout()->addWidget(new QWidget());//blank
	this->layout()->addWidget(options_groupbox);
	  
	this->setMinimumSize(this->sizeHint() * 1.25);
	
}
#endif

void DataExportWindow::writeFile(){

 if(QuSlotDebug())
 {
	printf("DataExportWindow::writeFile %ld\n", this);
	fflush(stdout);
 }
 
 
 // get extension from filter
  QString ext = selectedFilter();
  int posn = ext.findRev(")");
  ext.truncate(posn);
  posn = ext.find(".");
  ext.remove(0,posn);
  
  QiConts->io_f_extn = QiNewStr(ext.latin1()); 
   
  
  QString name_text =  selectedFile(); 
  
  QiConts->io_f_path = QexNewPath(name_text);
  QiConts->io_f_name = QexNewName(name_text);
  
	   
  // overwrite file toggle
  
  if(_OverWrite->isChecked() )
     qi_options->priority = REPLACE;
  else 
     qi_options->priority = WARN;
  
  if(isTimeSeries)
    qi_options->object_type = TS;      
  else
    qi_options->object_type = DS;      
  
  qi_options->attr_delim = ',';      // not accessible via Qexui

  // data delimiter
  
  if ( _DataDelim->currentItem() == COMMAdelim )
      qi_options->data_delim = ',';
  else if ( _DataDelim->currentItem() == SPACEdelim )
      qi_options->data_delim = ' ';
  else   {
      qi_options->data_delim = ' '; // set something, but not used
  }


  if ( _RecDelim->currentItem() == DOLLARdelim )
      qi_options->rec_end = '$';
  else if ( _RecDelim->currentItem() == NEWLINEdelim )
      qi_options->rec_end = '\n';
  else if ( _RecDelim->currentItem() == HASHdelim )
      qi_options->rec_end = '#';
  else if ( _RecDelim->currentItem() == TABdelim )
      qi_options->rec_end = '\t';
  else   {
      qi_options->rec_end = '\n'; // set something, but not used
  }


  qi_options->debug_choice = 'w';    // write errs to Qexui msg area

  qi_options->time_sep_attr = ' ';
  qi_options->time_sep_data = ' ';

  // ########## qi_options->header_path = (char *)  TBI later?  ######
  // ########## qi_options->header_name = (char *)  TBI later?  ######

  if( _RecNumOn->isChecked() )
    qi_options->rec_numbering = NUM_ON;
  else 
    qi_options->rec_numbering = NUM_OFF;
    
  if( _AttachHead->isChecked() )
    qi_options->header = ATTACHED;
  else 
    qi_options->header = DETACHED;
    
  // write data file
  
  int status_int;

  //  Call appropriate QiWrite routine for selected output file type.

  if ( strcmp( QiConts->io_f_extn,  ".cdf" ) == 0) {
    qi_options->f_type = UNSET;
    status_int = QiWriteCSDSgenCDF(QiConts, qi_options);     
  }
  else if ( strcmp( QiConts->io_f_extn,  ".cef" ) == 0) {
    if(cefWrite2(ask))
      qi_options->f_type = CAA; 
    else  
      qi_options->f_type = EXCHANGE;
    status_int = QiWriteCSDSgenFlat(QiConts, qi_options);  

  }
  else if ( strcmp( QiConts->io_f_extn,  ".qft" ) == 0) {
    qi_options->f_type = TABULAR;
    status_int = QiWriteCSDSgenFlat(QiConts, qi_options);  
  }
  else if ( strcmp( QiConts->io_f_extn,  ".qfd" ) == 0) {
    qi_options->f_type = DELIMITED;
    status_int = QiWriteCSDSgenFlat(QiConts, qi_options);  
  }
  else{
    qi_options->f_type = UNSET;
    status_int = QiWriteCSDSgenFlat(QiConts, qi_options);  
  }  
    
  if (status_int != QMW_OK){
      emit QexMessage(QString(QiErrStr(status_int))); 
      if(status_int == FILE_EXISTS) QuMsgBar("File NOT over-written", true);
  }
  else{
    QString txt = "File written to ";
    txt += QString(QiConts->io_f_path);
    txt += QString(QiConts->io_f_name);
    txt += QString(QiConts->io_f_extn);
    emit QexMessage(txt); 
  }
  
}


void DataExportWindow::initName(){
#if QT_VERSION < 0x040000
  setSelection(QexDefaultName(QiConts) );
#else
  selectFile(QexDefaultName(QiConts));
#endif 
 
  setDir(QDir::home());
}



void QEXPopupPopup( QWidget *parent )
{
  if(QEX_widget==NULL)
  {
     QEXCreatePopup(parent);
  }
  QEX_widget->initName(); 
  QObject::connect(QEX_widget, SIGNAL(QexMessage(QString )), parent, SLOT(messageHandler(QString)));
  if( QEX_widget->exec() == QDialog::Accepted){
    QEX_widget->writeFile();
    
  }
}


void QEXCreatePopup(QWidget *parent)
{
 QSize qsasGUISize = qsasWindow->frameSize();
 QEX_widget = new DataExportWindow(parent,"Data_Export_Window");
 QEX_widget->setCaption("Write File");
 QEX_widget->move(qsasGUISize.width(),10);
 QEX_widget->setMode(QFileDialog::AnyFile);

}

void QexBuildExportQIEObj (QdObjectSeq_var ObjList,
                           vector<KVDataBase_var> &OptionList){ 

 int jcount;
 int var_n=0;
 char epoch_str[]="time_tags";
 float *bigdata;
 int n_returned;
 QiCDFVariable* var;
 
   //  ***************** make empty contents object for output *******************

  QiConts = (QiCDFContents*) QiMakeCDFContentsObj();
  QiConts->vardata = (QiCDFVariable**)
      QiMakeQiVariablePtrs( (long) MAX_N_VARS );
  QiConts->n_vars = 0;
  QiConts->num_g_attrs = (long) 0;

 QiInitGlobalContents (QiConts);

 try{
   QiConts->n_recs = (ObjList[0])->sequence_size();
 }
 catch(Exception &e){
   QuMsgBar( QString(e.msg().c_str()), true ); 
   return;
 }
 if( QiConts->n_recs <1 )  return;

  // create empty options object
  qi_options = (QiOptions *) QiMakeOptionsObj();


 //  If Time series type, add timeline as first variable

   QdTimeSeq_var tt = get_timetags( ObjList[0] );
   if( !tt.is_nil() ){
      
     // add the time line data

     var_n = QexDataToCDFfi(QiConts, (QdObject_var) tt); 
	if( var_n == -1) return;

     QiConts->vardata[var_n]->name = QiNewStr(epoch_str);
     QiConts->time_var_num = var_n;
 
   }
 
 // add joined objects to export file-object 

  for (jcount=0; jcount < ObjList->size(); jcount++)  {
    if ( ObjList[jcount].is_nil() ) continue;  // skip empty returns
    
    var_n = QexDataToCDFfi( QiConts, ObjList[jcount]);
    if( var_n == -1) return;   

    QdString_var name_ptr = QdString_var::narrow((OptionList[jcount])->get(NEW_NAME));

    QString QSname = QString(name_ptr->c_str());
    int posn = QSname.findRev("/");
    if(posn !=-1) QSname.remove(0,posn+1);
    QiConts->vardata[var_n]->name = QiNewStr(QSname.latin1() );
    // add the attributes  

     if( !tt.is_nil() ) QexAddXrefsToAttrs(QiConts, var_n, ObjList[jcount], epoch_str);
     else QexAddXrefsToAttrs(QiConts, var_n, ObjList[jcount], "");

    //  qdos 2 does not support globals
 
    
  }  // end for jcount 
    
  if( !tt.is_nil() ){
		     
	// add the epoch variable Xrefs as attributes    (done last to stop thm being used as file name 
     QexAddXrefsToAttrs(QiConts, QiConts->time_var_num, (QdObject_var) tt, "");   

  }
   
  return;


}

int  QexDataToCDFfi(QiCDFContents* qicdf, QdObject_var obj_var){

    char * s_ptr;
    double * d_ptr;
    QiCDFVariable *v;

    int var_n = qicdf->n_vars; // number of vars is index of next var
    
    if (var_n >= MAX_N_VARS) {
      QuMsgBar("Exceeded maximum allowed number of Variables", true);
      return -1;
    }
    
    qicdf->vardata[var_n] = (QiCDFVariable*) QiMakeQiVariable();
     v = (QiCDFVariable*) qicdf->vardata[var_n];

     v->number = (long) var_n;
    
    int n_recs = obj_var->sequence_size();
    
    v->max_rec_num = (long) (n_recs -1);
    if (v->max_rec_num > 0) {v->rec_vary = VARY;} else {v->rec_vary = NOVARY;}
    
    qicdf->n_vars++; // increment var number 
	    
    // Find data type

    QdStringSeq_var s_seq = QdStringSeq_var::narrow(obj_var);
    if( !s_seq.is_nil() ){

       // string sequence type
       v->data_type = CDF_CHAR; 
       int str_len = s_seq->maxlen();  
       v->num_elems = str_len; 
       v->num_dims = 0;
        
       s_ptr = (char *) malloc( n_recs*str_len +1);
       v->data = (void *) s_ptr;
    
       if(n_recs != s_seq->sequence_size())     QuMsgBar("WARNING: wrong data sequence length", false);
       for(int n=0; n < n_recs; n++){
          sprintf(s_ptr, "%*.*s", str_len, str_len, s_seq[n].c_str() );
          s_ptr += str_len; 
       }
       return var_n;
    }

 
   QdString_var s_obj = QdString_var::narrow(obj_var);
   if( !s_obj.is_nil() ){
 
      // string  type
      v->data_type = CDF_CHAR; 
      int str_len = s_obj->size();
      v->num_elems = str_len;
      v->num_dims = 0;
        
      s_ptr = (char *) malloc( str_len +1);
      v->data = (void *) s_ptr;
     
      sprintf(s_ptr, "%*.*s", str_len, str_len, s_obj->c_str() );  
    
      return var_n;
   }
 
   QdSMatrix_var sms_obj = QdSMatrix_var::narrow(obj_var);
   if( !sms_obj.is_nil() ){

       QdSMatrix &smat = *sms_obj; 

      // string  type
      v->data_type = CDF_CHAR;
      int str_len = 0; 
      for(QdSMatrix::iterator itr=smat.begin(); itr !=smat.end(); itr++){
        if(str_len < itr->size())
          str_len = itr->size(); 
      }      
      v->num_elems = str_len;  
    
      v->num_dims = (sms_obj->dims()).size();
      int n_in_array = 1;
      for(int j=0; j<v->num_dims; j++) {
        v->dim_sizes[j] = (sms_obj->dims())[j];
        v->dim_varys[j] = VARY;
        n_in_array *= v->dim_sizes[j];
      }

      s_ptr = (char *) malloc( (str_len +1)*n_in_array);
      v->data = (void *) s_ptr;
    
      for(QdSMatrix::iterator itr=smat.begin(); itr !=smat.end(); itr++){
        sprintf(s_ptr, "%*.*s", str_len, str_len, itr->c_str() );
        s_ptr += str_len;
      }      
    
      return var_n;
              
   }
 
   QdTimeSeq_var t_seq = QdTimeSeq_var::narrow(obj_var);
   if( !t_seq.is_nil() ){

      // Time sequence type as epoch
    
      v->data_type = CDF_EPOCH;    
      v->num_elems = (long) 1;
      v->num_dims = 0;
        
      d_ptr = (double *) malloc( n_recs*sizeof(double) );
      v->data = (void *) d_ptr;
    
      if(n_recs != t_seq->sequence_size())   QuMsgBar("WARNING: wrong data sequence length", false);
      for(int n=0; n < n_recs; n++){
        double epch = cdfEpoch(t_seq[n]);
        memcpy(d_ptr, (void *) &epch, (size_t)sizeof(double) );
        d_ptr++;
      }
      return var_n;
   }
 
   QdTime_var t_obj = QdTime_var::narrow(obj_var);
   if( !t_obj.is_nil() ){

      // Time object type as epoch
    
      v->data_type = CDF_EPOCH;
      v->num_elems = (long) 1;
      v->num_dims = 0;
        
      d_ptr = (double *) malloc(sizeof(double) );
      v->data = (void *) d_ptr;
      double epch = cdfEpoch(*t_obj);   
      memcpy(d_ptr, (void *) &epch, (size_t)sizeof(double) );
      
      return var_n;
   }

   /***** NOT in QDOS yet...

   QdTimeIntervalSeq_var ti_seq = QdTimeIntervalSeq_var::narrow(obj_var);
   if( !ti_seq.is_nil() ){

      // Time interval sequence type as epoch
    
      v->data_type = CDF_EPOCH;
      v->num_elems = (long) 1;
    
      v->num_dims = 1;
      v->dim_sizes[0] = 2;
        
      d_ptr = (double *) malloc( n_recs*sizeof(double)*2);
      v->data = (void *) d_ptr;
    
      for(int n=0; n < ti_seq->sequence_size(); n++){
        double epch1 =  cdfEpoch(ti_seq[n].start());    
        memcpy(d_ptr, (void *) &epch1, (size_t)sizeof(double) );
        d_ptr++;
        double epch2 =  cdfEpoch(ti_seq[n].end());    
        memcpy(d_ptr, &epch2, (size_t)sizeof(double) );
        d_ptr++;
      }
      return var_n;
   }
  *******/

   QdTimeInterval_var ti_obj = QdTimeInterval_var::narrow(obj_var);
   if( !ti_obj.is_nil() ){

      // Time interval sequence type as epoch
      v->data_type = CDF_EPOCH;
      v->num_elems = (long) 1;
      v->num_dims = 1;
      v->dim_sizes[0] = 2;
      v->dim_varys[0] = VARY;
        
      d_ptr = (double *) malloc( sizeof(double)*2);
      v->data = (void *) d_ptr;
    
      double epch1 = cdfEpoch(ti_obj->start());
      memcpy(d_ptr, (void *) &epch1, (size_t)sizeof(double) );
      d_ptr++;
      double epch2 = cdfEpoch(ti_obj->end());
      memcpy(d_ptr, (void *) &epch2, (size_t)sizeof(double) );
      return var_n;
   }

   QdRScalarSeq_var r_seq = QdRScalarSeq_var::narrow(obj_var);
   if( !r_seq.is_nil() ){

      // Time interval sequence type as epoch
    
      v->data_type = CDF_DOUBLE;
      v->num_elems = (long) 1;
      v->num_dims = 0;
        
      d_ptr = (double *) malloc( n_recs*sizeof(double));
      v->data = (void *) d_ptr;
    
      if(n_recs != r_seq->sequence_size())   QuMsgBar("WARNING: wrong data sequence length", false);
      for(int n=0; n < n_recs; n++){
        double d = r_seq[n]; 
        memcpy(d_ptr, (void *) &d, (size_t)sizeof(double) );
        d_ptr++;
      }
      return var_n;
   }

   QdRScalar_var r_obj = QdRScalar_var::narrow(obj_var);
   if( !r_obj.is_nil() ){

      // scalar object
    
      v->data_type = CDF_DOUBLE;
      v->num_elems = (long) 1;
      v->num_dims = 0;
        
      d_ptr = (double *) malloc( sizeof(double));
      v->data = (void *) d_ptr;
       
      double d = *r_obj; 
      memcpy(d_ptr, (void *) &d, (size_t)sizeof(double) );
      
      return var_n;
   }

   QdRMatrixSeq_var m_seq = QdRMatrixSeq_var::narrow(obj_var);
   if( !m_seq.is_nil() ){

      // real matrix sequence type
    
      v->data_type = CDF_DOUBLE;
      v->num_elems = (long) 1;
      v->num_dims = (m_seq[0].dims()).size();

      int n_in_array = 1;
      for(int j=0; j<v->num_dims; j++) {
        v->dim_sizes[j] = (m_seq[0].dims())[j];
        v->dim_varys[j] = VARY;
        n_in_array *= v->dim_sizes[j];
      }
       
      d_ptr = (double *) malloc( n_recs*sizeof(double)*n_in_array);
      v->data = (void *) d_ptr;
    
      if(n_recs != m_seq->sequence_size())   QuMsgBar("WARNING: wrong data sequence length", false);
      for(int n=0; n < n_recs; n++){
        const QdRMatrix &mat = m_seq[n];
        for (QdRMatrix::iterator itr = mat.begin(); itr != mat.end(); itr++, d_ptr++) {
           memcpy(d_ptr, (void *) &(*itr), (size_t)sizeof(double) );
        }
      }
      return var_n;
   }

   QdRMatrix_var m_obj = QdRMatrix_var::narrow(obj_var);
   if( !m_obj.is_nil() ){

      // Real matrix type 
    
      v->data_type = CDF_DOUBLE;
      v->num_elems = (long) 1;   
      v->num_dims = (m_obj->dims()).size();
 
      int n_in_array = 1;
      for(int j=0; j<v->num_dims; j++) {
        v->dim_sizes[j] = (m_obj->dims())[j];
        v->dim_varys[j] = VARY;
        n_in_array *= v->dim_sizes[j];
      }
       
      d_ptr = (double *) malloc( sizeof(double)*n_in_array);
      v->data = (void *) d_ptr;
          
        QdRMatrix &mat = *m_obj;
        for(QdRMatrix::iterator itr=mat.begin(); itr !=mat.end(); itr++, d_ptr++){
           memcpy(d_ptr, (void *) &(*itr), (size_t)sizeof(double) );
        }

      return var_n;
   }
    
   // Else Data Type not supported yet if we get here, warn and return
 
  
    QuMsgBar("Data Type Not yet supported for Export", true); 
    return -1;
 }

void QexAddXrefsToAttrs(QiCDFContents* qicdf, int var_n, QdObject_var obj, const char * tt_name){

   int num_xrefs;
   QiCDFVariable *var;
   StringRep string_rep;
   static string str;

   string_rep.set_token( StringRep::SR_TOKEN_STRING_DELIMITER, "" );

   var = qicdf->vardata[var_n];
  
   var->num_v_attrs = 0;
   
   QdStringSeq_var xref_names = new QdStringSeq;
   obj->list_xref_names( *xref_names);
   if( xref_names.is_nil() ) return;
   
   num_xrefs = xref_names->size();
   if( num_xrefs <= 0 ) return;
   
   for (int n=0; n<num_xrefs ; n++){
      
      if(strcmp(xref_names[n].c_str(), PARENT_FILES) == 0) {
         //  qsas does not keep global attributes and cannot export them
         continue;
      }
      if(xref_names[n][0] == '_') {
         //  internal name beginning '_' is excluded
         continue;
      }
      if(strcmp(xref_names[n].c_str(), OBJECT_NAME) == 0) {
         //  Used internally for housekeeping - reserved name
         continue;
      }
      if(strcmp(xref_names[n].c_str(), DEPEND_0) == 0) {
	 // Ignore DEPEND_0 set by import as export may change time tags
	 // This is set below
         continue;
      }
   
     if(strcmp(xref_names[n].c_str(), TIME_TAGS) != 0) { // not time tag object, handled elsewhere
         QdObject_var attr_obj = obj->get_xref(xref_names[n].c_str());
         if( attr_obj.is_nil() ) continue;

         // if Xref has a PTR_VAR_NAME xref it came from ptr attribute
         if(attr_obj->xref_exists(PTR_VAR_NAME) ){
		 
		 try{
			// add ptr type xref to new meta var
			QdString_var a_ptr = QdString_var::narrow(attr_obj->get_xref(PTR_VAR_NAME));
			str = xref_names[n];
		    
			if ( QexNameExists(a_ptr->c_str(), qicdf) ) {
			     continue;
			}
			QexAddAttribute( var,  a_ptr, str.c_str() );
      
			// now add metadata object itself as variable

			int n_xref = QexDataToCDFfi(qicdf, attr_obj);
			if (n_xref == -1) return;
		   
			qicdf->vardata[n_xref]->name = QiNewStr(a_ptr->c_str() );
		 
			 // recursively add xrefs of xrefs
			QexAddXrefsToAttrs(qicdf, n_xref, attr_obj, tt_name);
			 
		}
		catch(Exception &e){
			// silently use attribute as held
			e.print_msg();
			QexAddAttribute( var,  attr_obj, xref_names[n].c_str());
		}
	}
        else{
           // regular xref to turn to attribute
           QexAddAttribute( var,  attr_obj, xref_names[n].c_str());
        }
     }
   }
   // explicitly add DEPEND_0 according to time line actually used
   
   if(strcmp(tt_name, "") != 0) {      // skip this if no DEPEND_0 to be set
       
      QdString_var attr_obj = new QdString(tt_name);
      QexAddAttribute( var,  attr_obj, "DEPEND_0");
   }
   
   return;
}


void   QexAddAttribute(QiCDFVariable *v,  
                       QdObject_var attr_obj, 
                       const char *name)
{
   char * s_ptr;
   double * d_ptr;
   static string attr;
    
   if (v->num_v_attrs >= MAX_N_ATTRS) return;
   if( strcmp(QiToUpper(name), "TIME_TAGS") == 0 ) return;
   if( strcmp(QiToUpper(name), "PARENTS") == 0 ) return;

   (v->attribute[v->num_v_attrs]).name = (char *) malloc(strlen(name)+1);
   strcpy((v->attribute[v->num_v_attrs]).name, name);
     
   (v->attribute[v->num_v_attrs]).number = v->num_v_attrs;

// Find data type

  QdString_var s_obj = QdString_var::narrow(attr_obj);
  if(!s_obj.is_nil()){

    // string type
    (v->attribute[v->num_v_attrs]).data_type = CDF_CHAR;
    
    int str_len = s_obj->size();
    (v->attribute[v->num_v_attrs]).num_elems = 1;
        
    s_ptr = (char *) malloc( str_len +1);
    (v->attribute[v->num_v_attrs]).data = (void *) s_ptr;
     
    sprintf(s_ptr, "%*.*s", str_len, str_len, s_obj->c_str() );  
           
    v->num_v_attrs++;

    return;
 }
 
  QdSMatrix_var smat_obj = QdSMatrix_var::narrow(attr_obj);
  if(!smat_obj.is_nil()){

    // string type
    (v->attribute[v->num_v_attrs]).data_type = CDF_CHAR;
    
    (v->attribute[v->num_v_attrs]).num_elems = smat_obj->size();
    
    int total_len=0;
    SMatrix::iterator itr=smat_obj->begin();
    while (itr != smat_obj->end() ){
      total_len += itr->size()+1;
      itr++;
    }
    
    s_ptr = (char *) malloc( total_len +1);
    (v->attribute[v->num_v_attrs]).data = (void *) s_ptr;
    
    int s_posn=0;
    itr=smat_obj->begin();
    while (itr != smat_obj->end() ){  
      int str_len = itr->size();
      sprintf(&(s_ptr[s_posn]), "%*.*s", str_len, str_len, itr->c_str() );  
      s_posn += str_len;
      s_ptr[s_posn++] = '\0';
      itr++;
    }
          
    v->num_v_attrs++;

    return;
 }
 
  QdRMatrix_var rmat_obj = QdRMatrix_var::narrow(attr_obj);
  if(!rmat_obj.is_nil()){

    // double type
    (v->attribute[v->num_v_attrs]).data_type = CDF_DOUBLE;
    
    (v->attribute[v->num_v_attrs]).num_elems = rmat_obj->size();
    
     
    d_ptr = (double *) malloc( sizeof(double) * (v->attribute[v->num_v_attrs]).num_elems);
    (v->attribute[v->num_v_attrs]).data = (void *) d_ptr;
    
    RMatrix::iterator itr=rmat_obj->begin();
    while (itr != rmat_obj->end() ){  
      *d_ptr = (double) *itr;
      itr++;
      d_ptr++;
    }
          
    v->num_v_attrs++;

    return;
 }
 
  QdTimeInterval_var ti_obj = QdTimeInterval_var::narrow(attr_obj);
  if(!ti_obj.is_nil()){

    // turn interval to string type
    (v->attribute[v->num_v_attrs]).data_type = CDF_CHAR;
    
    StringRep string_rep;
    string_rep.set_token( StringRep::SR_TOKEN_STRING_DELIMITER, "" );
    attr = string_rep(attr_obj);
  
    int str_len = strlen(attr.c_str());
    (v->attribute[v->num_v_attrs]).num_elems = 1;
        
    s_ptr = (char *) malloc( str_len +1);
    (v->attribute[v->num_v_attrs]).data = (void *) s_ptr;
     
    sprintf(s_ptr, "%*.*s", str_len, str_len, attr.c_str() );  

    v->num_v_attrs++;
    
    return;
 }
 
 QdRScalar_var r_obj = QdRScalar_var::narrow(attr_obj);
 if( !r_obj.is_nil() ){

    // scalar object
    
    (v->attribute[v->num_v_attrs]).data_type = CDF_DOUBLE;
    (v->attribute[v->num_v_attrs]).num_elems = (long) 1;    
        
    d_ptr = (double *) malloc( sizeof(double));
    (v->attribute[v->num_v_attrs]).data = (void *) d_ptr;
       
    *d_ptr = (double) *r_obj;

    v->num_v_attrs++;
      
    return;
 } 

 QdRMatrix_var m_obj = QdRMatrix_var::narrow(attr_obj);
 if( !m_obj.is_nil() ){

    //  matrix object
    int num_dims = (m_obj->dims()).size();

    int n_in_array = 1;
    for(int j=0; j<num_dims; j++) {
      n_in_array *= (m_obj->dims())[j];
    }
    (v->attribute[v->num_v_attrs]).data_type = CDF_DOUBLE;
    (v->attribute[v->num_v_attrs]).num_elems = (long) n_in_array;    
       
    d_ptr = (double *) malloc( sizeof(double)*n_in_array);
    (v->attribute[v->num_v_attrs]).data = (void *) d_ptr;
          
    QdRMatrix &mat = *m_obj;
    for(QdRMatrix::iterator itr=mat.begin(); itr !=mat.end(); itr++, d_ptr++){
       *(d_ptr) = (double) *itr;
    }

    v->num_v_attrs++;
      
    return;
 }


 QuMsgBar(name, "Xref cannot be written as an attribute", true);
 return;
}

bool QexNameExists(const char *name, QiCDFContents* qicdf){

  for(int i=0; i<qicdf->n_vars; i++){
    if( strcmp(name, qicdf->vardata[i]->name) == 0) return true;
  }
  return false;
  
}

bool QexWriteFile (QWidget *parent, bool isTS){
 
  isTimeSeries = isTS;
  
  if (QiConts == NULL) return false;
  if (QiConts->vardata[0] == NULL) return false;
  
  QEXPopupPopup(parent);

  //  Clean up 

  QiConts = QiFreeCDFContentsObj(QiConts);
  qi_options = QiFreeOptionsObj(qi_options);

  return true;

}

char * QexNewPath(QString text){
  char *path;
  
  QString working = QString(text.latin1());
  
  int posn = working.findRev('/' );
  if (posn == -1) 
    path = QiNewStr("./");
  else{
    working.truncate(posn+1);
    path = QiNewStr(working.latin1());
  }
  return path;
}

char * QexNewName(QString text){
  char *name;
  
  QString working = QString(text.latin1());
  int posn = working.findRev('/');
  if (posn != -1) 
  {
    working.remove(0,posn+1);
  }
  
  // remove extension if present
                  posn = working.findRev(".cef",-1, false);
  if (posn == -1) posn = working.findRev(".cdf",-1, false);
  if (posn == -1) posn = working.findRev(".qft",-1, false);
  if (posn == -1) posn = working.findRev(".qfd",-1, false);
  if (posn != -1) {
    working.truncate(posn);  
  }

  name = QiNewStr(working.latin1());
  
  return name;
}

QString QexDefaultName(QiCDFContents *QiConts){

  if(QiConts == NULL) return QString("QSASdataFile");
  if(QiConts->vardata == NULL) return QString("QSASdataFile");
  if(QiConts->vardata[0] == NULL ) return QString("QSASdataFile");
  
  for(int i=0; i<QiConts->n_vars; i++){
    if (QiConts->vardata[i] == NULL ) continue;
    if(QiConts->vardata[i]->data_type != CDF_EPOCH
       && QiConts->vardata[i]->rec_vary == VARY)
       return QString(QiConts->vardata[i]->name);
  }
  // no non-time data var found, use first
  return QString(QiConts->vardata[0]->name);
 
}
