
#include <qstring.h>
#include <qfiledialog.h> 

#include "qsasMenus.h"
#include "qplug_if.h"
#include "qie_VOtable.h"

 extern qwuiWindow* qsasWindow;
 
// for fortran dependent plugins under cygwin
extern "C" QplugReturnStatus  Tsyganenko(  QplugArgList *);
extern "C" QplugReturnStatus  WaveletDenoise(  QplugArgList *);

 QplugArgList* qsasPlugArgs;

void AddQwuiMenus(){

   static QplugArgList staticArgList;
   
   // these are here because cygwin/windows does not allow forward declaration
   
   static mainMenu * file = new mainMenu( qsasWindow );
   qsasWindow->_menu_bar->insertItem( " File ", file );
   file->insertFileItems();
   
   static mainMenu * edit = new mainMenu( qsasWindow );
   qsasWindow->_menu_bar->insertItem( " Edit ", edit );
   edit->insertEditItems();
   

  // Add to test the Object Calculator
   static mainMenu *eCalculate = new mainMenu( qsasWindow );
   qsasWindow->_menu_bar->insertItem( " Analysis ", eCalculate );
   eCalculate->insertCalcItems();
   
   static mainMenu *plugins = new mainMenu( qsasWindow );
   qsasWindow->_menu_bar->insertItem( " Plug-Ins ", plugins );
   plugins->insertPluginMenu();
     
   static mainMenu * plot = new mainMenu( qsasWindow );
   qsasWindow->_menu_bar->insertItem( " Plot    ", plot );
   plot->insertPlotItems();

   static mainMenu * profiles = new mainMenu(qsasWindow);
   qsasWindow->_menu_bar->insertItem( " Profiles ", profiles );
   profiles->insertProfilesItems();
   
   qsasWindow->_menu_bar->insertSeparator();   
   
   static mainMenu * help = new mainMenu( qsasWindow );
   qsasWindow->_menu_bar->insertItem(" Help ", help );
   help->insertHelpItems();
   
   // populate time and maths menus
   qsasWindow->timeOps->addFolderToMenu("TimeOperations");
   qsasWindow->filterOps->addFolderToMenu("Filtering");
   qsasWindow->statOps->addFolderToMenu("Statistics");
   qsasWindow->mathOps->addFolderToMenu("Arithmetic");
   qsasWindow->unitOps->addFolderToMenu("UnitOperations");
   qsasWindow->eventTableOps->addFolderToMenu("EventTables");
   
   // populate plugin menu
   plugins->refreshPlugins(true);
   
   // Add Rt Click Menu for WL
   edit->setRtButtonMenu();
   
   // for static plugins (cygwin missing dynamic libg2c.dll) 
   
   qsasPlugArgs = &staticArgList;
   
 }

// class mainMenu
// -----------------------

#if QT_VERSION < 0x040000
 mainMenu::mainMenu(QWidget *parent, const char *name)
    :  Q_POPUP_MENU(parent,name)
#else
mainMenu::mainMenu(QWidget *parent, const char *name)
    :  Q_POPUP_MENU(parent)
#endif
{        
   // initialise NULL pointer    
    objTypeSelector = NULL;
#if QT_VERSION >= 0x040000
    setTearOffEnabled(true);
#else
    insertTearOffHandle();
#endif
}

// -----------------------

void mainMenu::menuOp(int id)
{ 

  QString pluginName = QString(this->text(id).latin1()); //  force deep copy
  
  if( pluginName == QString("Browse") ){
	pluginName = QFileDialog::getOpenFileName(
                       ".","*.qtpl", this, "openQTPL", "Select qtpl template");
    QtplOpenByName(pluginName, qsasWindow);
    return;
  }
  if( pluginName == QString("Refresh Menu") ){
    this->refreshPlugins(false);
    return;
  }
  
}


void mainMenu::refreshPlugins(bool first)
{ 
  for(int i=0; i<qsasWindow->pluginFolders.size(); i++){
    qsasWindow->pluginFolders[i]->clear();
    if(first) {
#if QT_VERSION < 0x040000
      this->insertItem(qsasWindow->pluginFolders[i]->name(), qsasWindow->pluginFolders[i]);
#else
      this->insertItem(qsasWindow->pluginFolders[i]->title().latin1(), qsasWindow->pluginFolders[i]);
#endif
    }
#if QT_VERSION < 0x040000
    qsasWindow->pluginFolders[i]->addFolderToMenu(qsasWindow->pluginFolders[i]->name(), first);
#else
    qsasWindow->pluginFolders[i]->addFolderToMenu(qsasWindow->pluginFolders[i]->title().latin1(), first);
#endif
  }

  if(first){
    this->insertSeparator();
    this->insertItem("Browse" );
    this->insertItem("Refresh Menu" );
  }
}

void mainMenu::insertFileItems(){
#if QT_VERSION < 0x040000
   this->insertItem("Data Selector", this, SLOT( openDS() ));
   this->insertItem("&Open Data File", this, SLOT( openCAA() ), Qt::CTRL+Qt::Key_O );
   this->insertItem("&Write Data File", this, SLOT( openExport() ), Qt::CTRL+Qt::Key_W );
   this->insertSeparator();
   this->insertItem("Open Data File (old interface)", this, SLOT( openQIE() ) );
   this->insertItem("Load Event Table", this, SLOT(openEvtTable()));
   this->insertSeparator();
   this->insertItem("Save Working List", qsasWindow, SLOT( saveWL() ) );
   this->insertItem("Save Working List As...", qsasWindow, SLOT( saveWLAs() ) );
   this->insertItem("Restore Working List...", qsasWindow, SLOT( restore() ) );
   this->insertItem("Include in WL...", qsasWindow, SLOT( include() ) );
   this->insertSeparator();
   this->insertItem("&Save Session", qsasWindow, SLOT( saveSession() ), Qt::CTRL+Qt::Key_S );
   this->insertItem("Save Session As...", qsasWindow, SLOT( saveSessionAs() ), Qt::SHIFT+Qt::CTRL+Qt::Key_S );
   this->insertItem("&Restore Session", qsasWindow, SLOT( restoreSession() ), Qt::CTRL+Qt::Key_R );
   this->insertSeparator();
   this->insertItem("&Quit", qApp, SLOT( closeAllWindows() ), Qt::CTRL+Qt::Key_Q );
   connect(qApp, SIGNAL(lastWindowClosed() ), qApp, SLOT(quit()));
   connect(qsasWindow, SIGNAL(openQdaWindow(const char *  )), this, SLOT(openDataFile(const char * )));
#else
   this->addAction("Data Selector", this, SLOT( openDS() ));
   this->addAction(style()->standardIcon(QStyle::SP_DialogOpenButton, 0, this), "&Open Data File", this, SLOT( openCAA() ), Qt::CTRL+Qt::Key_O );
   this->addAction(style()->standardIcon(QStyle::SP_DialogSaveButton, 0, this), "&Write Data File", this, SLOT( openExport() ), Qt::CTRL+Qt::Key_W );
   this->addSeparator();
   this->addAction("Open Data File (old interface)", this, SLOT( openQIE() ) );
   this->addAction("Load Event Table", this, SLOT(openEvtTable()));
   this->addSeparator();
   this->addAction("Save Working List", qsasWindow, SLOT( saveWL() ) );
   this->addAction("Save Working List As...", qsasWindow, SLOT( saveWLAs() ) );
   this->addAction("Restore Working List...", qsasWindow, SLOT( restore() ) );
   this->addAction("Include in WL...", qsasWindow, SLOT( include() ) );
   this->addSeparator();
   this->addAction("&Save Session", qsasWindow, SLOT( saveSession() ), Qt::CTRL+Qt::Key_S );
   this->addAction("Save Session As...", qsasWindow, SLOT( saveSessionAs() ), Qt::SHIFT+Qt::CTRL+Qt::Key_S );
   this->addAction("&Restore Session", qsasWindow, SLOT( restoreSession() ), Qt::CTRL+Qt::Key_R );
   this->addSeparator();
   this->addAction(style()->standardIcon(QStyle::SP_DialogCloseButton, 0, this), "&Quit", qApp, SLOT( closeAllWindows() ), Qt::CTRL+Qt::Key_Q );
   connect(qApp, SIGNAL(lastWindowClosed() ), qApp, SLOT(quit()));
   connect(qsasWindow, SIGNAL(openQdaWindow(const char *  )), this, SLOT(openDataFile(const char * )));
#endif
}

void mainMenu::insertEditItems(){

   this->insertItem("new &Object", this, SLOT( createObject() ) );
   this->insertItem("new &Folder", qsasWindow, SLOT( makeGroup() ), Qt::CTRL+Qt::Key_F );
   this->insertSeparator();
   this->insertItem("&Duplicate Object", qsasWindow, SLOT( issueDuplicate() ), Qt::CTRL+Qt::Key_D );
   this->insertItem("Extract from Object", this, SLOT( launchSlice() ), Qt::CTRL+Qt::ALT+Qt::Key_G );
   this->insertItem("&Edit Object", this, SLOT( viewObject() ), Qt::CTRL+Qt::Key_E );
   this->insertItem("re-&Name Object", qsasWindow, SLOT( issueReName() ), Qt::CTRL+Qt::Key_N );
   this->insertItem("select &All", qsasWindow, SLOT( selectAllData() ), Qt::CTRL+Qt::Key_A );
   this->insertSeparator();
#if QT_VERSION < 0x040000
   this->insertItem("Delete Object", qsasWindow, SLOT( deleteListItem() ), Qt::CTRL+Qt::Key_X );
#else
   this->insertItem(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), "Delete Object", qsasWindow, SLOT( deleteListItem() ), Qt::CTRL+Qt::Key_X );
#endif
   this->insertItem("Remove object's time tags", qsasWindow, SLOT( deleteItemTT() ));
   this->insertSeparator();
   this->insertItem("Clear Working List", qsasWindow, SLOT( deleteAllDataItems() ));
   this->insertItem("Clear Time Interval List", qsasWindow, SLOT( deleteAllTimeIntervalItems() ));
   this->insertSeparator();
   this->insertItem("Clear Selected Cache", qsasWindow, SLOT( clearCacheSelected() ));
   this->insertItem("Clear Cache", qsasWindow, SLOT( clearCache() ));
  // edit->insertItem("preferences..&.", qsasWindow, SLOT( setPreferences() ), CTRL+Key_Period );

  }
  
void mainMenu::insertHelpItems(){

#if QT_VERSION < 0x040000
    this->insertItem("QSAS &Manual", qsasWindow, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
#else
    this->insertItem(style()->standardIcon(QStyle::SP_MessageBoxInformation, 0, this), "QSAS &Manual", qsasWindow, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
#endif
   this->insertSeparator();
   this->insertItem("...Getting Started", qsasWindow, SLOT( launchManSampleW() ));
   this->insertItem("...Main Window", qsasWindow, SLOT( launchManMainW() ));
   this->insertItem("...Data Handling", qsasWindow, SLOT( launchManDataH() ));
   this->insertItem("...Save and Restore", qsasWindow, SLOT( launchManSaveRestore() ));
   this->insertSeparator();
   this->insertItem("Module Version", qsasWindow, SLOT( launchVersion() ));
   this->insertItem("Contact QSAS Team", qsasWindow, SLOT( launchContact() ));
   
}

void mainMenu::insertCalcItems(){

    this->insertItem("Calculator", this, SLOT( launchCalculator()), Qt::CTRL+Qt::Key_C );
   // End Calculator
   
   // old Arithmetic plugins
   qsasWindow->mathOps = new qtplMenu( qsasWindow );
   this->insertItem( "Simple Maths", qsasWindow->mathOps );

   // Unit conversion plugins
   qsasWindow->unitOps = new qtplMenu( qsasWindow );
   this->insertItem( "Change Units", qsasWindow->unitOps );

   qsasWindow->timeOps = new qtplMenu( qsasWindow );
   qsasWindow->timeOps->insertItem("&Join", this, SLOT( launchJoin() ), Qt::CTRL+Qt::Key_J );
   this->insertItem("Time Ops...", qsasWindow->timeOps);

   qsasWindow->filterOps = new qtplMenu( qsasWindow );
   this->insertItem( "Filtering", qsasWindow->filterOps );

   qsasWindow->statOps = new qtplMenu( qsasWindow );
   this->insertItem( "Statistics", qsasWindow->statOps );
   
   qsasWindow->eventTableOps = new qtplMenu( qsasWindow );
   this->insertItem( "Event Tables", qsasWindow->eventTableOps );
   
   this->insertItem("Extract from Object", this, SLOT( launchSlice() ) );
   
   this->insertItem("Array to Vector", this, SLOT( launchMakeVector() ) );
   
}

void mainMenu::insertPlotItems(){

    this->insertItem("&Plot Layout", this, SLOT( launchPlot() ), Qt::CTRL+Qt::Key_P );
}

void mainMenu::insertPluginMenu(){
   static qtplMenu *geoMenu = new qtplMenu(qsasWindow, "Geophysics");
   qsasWindow->pluginFolders.push_back(geoMenu);
   
   static qtplMenu *analysisMenu = new qtplMenu(qsasWindow, "Analysis"); 
   qsasWindow->pluginFolders.push_back(analysisMenu);
   
   connect(this, SIGNAL(activated(int)),this,  SLOT(menuOp(int)) );
}

void mainMenu::insertProfilesItems()
{
	this->insertItem("Edit profiles...", this, SLOT( launchEditProfile() ));
	this->insertSeparator();

	QString currentName=QSAS_profiles->current()->getName();
	std::vector<QString> names;
	QSAS_profiles->names(names);
	for(int i=0; i<names.size(); ++i)
	{
		if(names[i]==currentName)
		{
			int id=this->insertItem(currentName);
			setItemChecked(id, true);
			checkedProfile=id;
		}
		else
		{
			int id=this->insertItem(names[i]);
			setItemChecked(id, false);
		}
		connect(this, SIGNAL(activated(int)), this, SLOT(launchSelectProfile(int)));
	}
}

void mainMenu::openDS(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openDS %ld\n", this);
  fflush(stdout);
  }
 
 
  QdsShow();

}

void mainMenu::openQIE(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openQIE %ld\n", this);
  fflush(stdout);
 }
 
 
  QString file;
  static QString dir=".";
  
  // get user file choice
  file = QFileDialog::getOpenFileName( dir, "All QSAS data files ( *.cdf *.cef *.qft *.qfd);;*.cdf;;*.cef;;*.qft;;*.qfd;;Try any file (*)", 
                                        this, "fileDialog","Select Data File" );
 
  if ( !file.isEmpty() ) {
  
    // keep chosen directory for next time
    int posn = file.findRev("/");
    if (posn >= 0){
      dir = QString(file.latin1()); // force deep copy
      dir.truncate(posn); 
    } 
    QdiLaunchWindow(file, qsasWindow);
  }
}


void mainMenu::openCAA(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openCAA %ld\n", this);
  fflush(stdout);
 }
 
 
  QString file;
  static QString dir=".";
  
  // get user file choice
  file = QFileDialog::getOpenFileName( dir, "CAA syntax CEF and CDF data files ( *.cef *.cef.gz *.cdf );;*.cef;;*.cef.gz;;*.cdf", 
                                        this, "fileDialog","Select Data File" );
 
  if ( !file.isEmpty() ) {
  
    // keep chosen directory for next time
    int posn = file.findRev("/");
    if (posn >= 0){
      dir = QString(file.latin1()); // force deep copy
      dir.truncate(posn); 
    } 
    QdaLaunchWindow(file, this);
  }
}

void mainMenu::openExport(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openExport %ld\n", this);
  fflush(stdout);
  }
 
 
   QjuiShowExport();
}

void mainMenu::openEvtTable(){ 

    if(QuSlotDebug())
	{
        printf("mainMenu::openEvtTable %ld\n", this);
		fflush(stdout);
	}
    
    QString file;
    static QString dir=".";
    
      // get user file choice
    file = QFileDialog::getOpenFileName( dir, "VOTables ( *.xml);;*.*", 
                                         this, "fileDialog","Select Data File" );
 
    if ( !file.isEmpty() ) {
        QdTimeIntervalSeq_var obj=QiLoadVOTable(file);
        if(!obj.is_nil())
        {
            file=file.remove(".xml");
            int posn = file.findRev("/");
            if (posn >= 0){
                dir = QString(file.latin1()); // force deep copy
                dir.truncate(posn);
                file=file.remove(dir);
            }
            
            file=file.append(" (evt table)");
            
            QwuiPutOnWL((QdObject_var) obj, file, "Time Intervals");
        }
    }

}

void mainMenu::launchPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchPlot %ld\n", this);
  fflush(stdout);
  }
 
  
  QpuiShow();
  
}
void mainMenu::launchJoin()
{ 
   QjuiShowJoin();
}

void mainMenu::launchCalculator()
{
   QcalcShowCalc();
}

void mainMenu::FolderToEventTable()
{
    QwlListItem *atItem = (QwlListItem *) qsasWindow->WorkingList->currentItem() ;
    
    if(atItem  == NULL) return;

    if(atItem->getEntryType() == ListTypeNull || atItem->getEntryType() == ListTypeEntry) {
        return;
    }
    
    if(qsasWindow->WorkingList->localList.size()==0) return;
    
    QdTimeIntervalSeq_var itv(new QdTimeIntervalSeq());
    
    for(vector<QguListItem*>::iterator iter=qsasWindow->WorkingList->localList.begin(); iter!=qsasWindow->WorkingList->localList.end(); iter++)
    {
        QdObject_var temp=(*iter)->QguGetObject();
        QdTimeInterval_var temp_ti=QdTimeInterval_var::narrow(temp);
        
        if(!temp_ti.is_nil())
        {
            itv->push_back(*temp_ti);
        }
        else
        {
            QdTimeIntervalSeq_var temp_tis=QdTimeIntervalSeq_var::narrow(temp);
            if(!temp_tis.is_nil())
            {
                for(vector<TimeInterval>::iterator iter=temp_tis->begin(); iter!=temp_tis->end(); iter++)
                {
                    QdTimeInterval foo(*iter);
                    itv->push_back(foo);
                }
            }
        }
    }
   
    QString name=QString(atItem->nameContext().latin1())+QString(" (evt table)");
    
    QwuiPutOnWL((QdObject_var)itv, name, "Time Intervals");
}


void mainMenu::EventTableToFolder()
{
    QwlListItem *atItem = (QwlListItem *) qsasWindow->WorkingList->currentItem() ;
    
    if(atItem  == NULL) return;
    
    if(atItem->getEntryType() != ListTypeEntry) return;
    
    QdObject_var obj=atItem->QguGetObject();
    QdTimeIntervalSeq_var dobj=QdTimeIntervalSeq_var::narrow(obj);
    
    if(dobj.is_nil())
    {
        return;
    }

    QwlListItem *commonRoot = (QwlListItem *)qsasWindow->WorkingList->findCommonFolder();
    QString commonRootName = commonRoot->nameContext().latin1();
    QString itemName=atItem->nameContext().latin1();
    itemName.remove(commonRootName+QString("/"));
    QString folderName= itemName+QString("_folder");
    QwlListItem *newFolder;

    if(commonRoot != NULL)
    {
        newFolder =  new QwlListItem(commonRoot, QString(folderName), ListTypeGroup);    
    }
    else if( !qsasWindow->WorkingList->rootIsLocked() ){
        newFolder =  new QwlListItem(qsasWindow->WorkingList, QString(folderName), ListTypeGroup);    
    }
    else return;
    qsasWindow->WorkingList->refreshWLcontext();
    int i=dobj->sequence_size()-1;
    for(QdTimeIntervalSeq::reverse_iterator iter=dobj->rbegin(); iter!=dobj->rend(); iter++, i--)
    {
        QdTimeInterval_var temp=new QdTimeInterval(*iter);
        QString name=folderName+QString("/")+itemName+QString().setNum(i);
        QString context;
        context = ((QwlListItem *)(qsasWindow->WorkingList->Time))->addItemAtAnchor(name, true);
        QuSetObjByName(temp, context);
    }
    
    qsasWindow->WorkingList->emitListModified();

}

void mainMenu::viewObject(){

   DataBrowseWindow *QBROWSER_widget;

   QwlListItem *atItem = (QwlListItem *) qsasWindow->WorkingList->currentItem() ;  
   if(atItem  == NULL) return;

   if(atItem->getEntryType() == ListTypeNull) {
      return;
   }
   else if(atItem->getEntryType() == ListTypeEntry) {
     // open browser
     
     qsasWindow->WorkingList->clearSelection();
     
     // launch editor 
     // appropriate edit insert is used depending on dobj type 
     
     QdObject_var nil_obj;

     QdObject_var dobj = atItem->QguGetObject();
     
     QString dobjName = atItem->getWLcontext();
     QString nameOnly = QString(dobjName.latin1());
     int pos = nameOnly.findRev('/');
     nameOnly.remove(0, pos+1);
     if(dobj.is_nil()) {
       QuMsgBar(dobjName.latin1(), " is nil", true);
       return;
     }
     
     QBROWSER_widget = QbuLaunchBrowse(nil_obj, 0, nameOnly, dobj, dobjName);

   }
   
   atItem->setSelected( TRUE );
     
}


void mainMenu::createObject(){ 

 if(QuSlotDebug())
 {
  printf("createObject::createObject %ld\n", this);
  fflush(stdout);
 }
 
  
   static QPushButton *Sbtn;
   static QPushButton *Vbtn;
   static QPushButton *Abtn;
   static QPushButton *tbtn;
   static QPushButton *tibtn;
   static QPushButton *Txtbtn;
   static QPushButton *TxtArrbtn;
   static Q_V_BOX * typeBox;

  if(objTypeSelector == NULL){
    
     objTypeSelector = new QDialog(0, "new object chooser");
     typeBox = new Q_V_BOX(objTypeSelector, "typeBox");
#if QT_VERSION < 0x040000
     typeBox->setMargin(4);
     typeBox->setSpacing(6);
     
     Txtbtn = new QPushButton(" Text Constant  ", typeBox, "Txtbtn");
     TxtArrbtn = new QPushButton(" Text Array   ", typeBox, "TxtArrbtn");
     tbtn = new QPushButton(" Time value ", typeBox, "tbtn");
     tibtn = new QPushButton(" Time Interval  ", typeBox, "tibtn");
     Sbtn = new QPushButton(" Scalar Constant  ", typeBox, "Sbtn");
     Vbtn = new QPushButton(" Vector Constant  ", typeBox, "Vbtn");
     Abtn = new QPushButton(" Array Constant  ", typeBox, "Abtn");
     connect(Txtbtn,SIGNAL(clicked()), this, SLOT(createText()));
     connect(TxtArrbtn,SIGNAL(clicked()), this, SLOT(createTextArr()));
     connect(tbtn,SIGNAL(clicked()), this, SLOT(createTime()));
     connect(tibtn,SIGNAL(clicked()), this, SLOT(createTInterval()));
     connect(Sbtn,SIGNAL(clicked()), this, SLOT(createConstant()));
     connect(Vbtn,SIGNAL(clicked()), this, SLOT(createVConstant()));
     connect(Abtn,SIGNAL(clicked()), this, SLOT(createArray()));
     typeBox->adjustSize();
#else
     QVBoxLayout* typeBoxLayout=new QVBoxLayout;
     typeBoxLayout->setMargin(4);
     typeBoxLayout->setSpacing(6);
     
     Txtbtn = new QPushButton(" Text Constant  ", typeBox, "Txtbtn"); typeBoxLayout->addWidget(Txtbtn);
     TxtArrbtn = new QPushButton(" Text Array   ", typeBox, "TxtArrbtn"); typeBoxLayout->addWidget(TxtArrbtn);
     tbtn = new QPushButton(" Time value ", typeBox, "tbtn"); typeBoxLayout->addWidget(tbtn);
     tibtn = new QPushButton(" Time Interval  ", typeBox, "tibtn"); typeBoxLayout->addWidget(tibtn);
     Sbtn = new QPushButton(" Scalar Constant  ", typeBox, "Sbtn"); typeBoxLayout->addWidget(Sbtn);
     Vbtn = new QPushButton(" Vector Constant  ", typeBox, "Vbtn"); typeBoxLayout->addWidget(Vbtn);
     Abtn = new QPushButton(" Array Constant  ", typeBox, "Abtn"); typeBoxLayout->addWidget(Abtn);
     connect(Txtbtn,SIGNAL(clicked()), this, SLOT(createText()));
     connect(TxtArrbtn,SIGNAL(clicked()), this, SLOT(createTextArr()));
     connect(tbtn,SIGNAL(clicked()), this, SLOT(createTime()));
     connect(tibtn,SIGNAL(clicked()), this, SLOT(createTInterval()));
     connect(Sbtn,SIGNAL(clicked()), this, SLOT(createConstant()));
     connect(Vbtn,SIGNAL(clicked()), this, SLOT(createVConstant()));
     connect(Abtn,SIGNAL(clicked()), this, SLOT(createArray()));
     typeBox->setLayout(typeBoxLayout);
     typeBox->adjustSize();
#endif
     
  }
  
  objTypeSelector->show();  
}

void mainMenu::createArray(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createArray %ld\n", this);
  fflush(stdout);
 }
  
  objTypeSelector->close();
  
  arrTypeSelector = new arrDialog(this, "arrayDimChooser", 0, true);
	
  arrTypeSelector->arrType = QB_DOUBLE;
  arrTypeSelector->show();  
}

 
void mainMenu::createTextArr(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createTextArr %ld\n", this);
  fflush(stdout);
 }
 
  objTypeSelector->close();
    
  arrTypeSelector = new arrDialog(this, "arrTypeSelector", 0, true);
	
  arrTypeSelector->arrType = QB_STRING;
  arrTypeSelector->show();  
}

void mainMenu::createText(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createText %ld\n", this);
  fflush(stdout);
  }
 
   
  QdObject_var nil_obj;
  DataBrowseWindow *QBROWSER_widget;
  QdObject_var dobj;  
  QdString_var sobj = new QdString(" ");
  
  dobj = sobj;

  objTypeSelector->close();

  QString context = QwuiPutOnWL(dobj, "newText", "Constants");
  
  QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "newText", dobj, context);
  
}


void mainMenu::createConstant(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createConstant %ld\n", this);
  fflush(stdout);
 }
 
  
  QdRScalar_var dobj;
  QdObject_var nil_obj;
  DataBrowseWindow *QBROWSER_widget;
  
  dobj = new QdRScalar(0.0);
  objTypeSelector->close();
  QString context = QwuiPutOnWL(dobj, "newScalar", "Constants");
  
  QdString_var SI_ptr = new QdString("1>?");
  dobj->change_xref(SI_CONVERSION, SI_ptr);
  
  QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "newScalar", dobj, context);
  
}

void mainMenu::createVConstant(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createVConstant %ld\n", this);
  fflush(stdout);
 }
 
  Dim mDim = Dim(3);
  
  QdObject_var nil_obj;
  DataBrowseWindow *QBROWSER_widget;
  
  QdRMatrix_var dobj = new QdRMatrix(mDim, 0);

  QString context = QwuiPutOnWL(dobj, "newVector", "Constants");
  
  QdString_var Frame_ptr = new QdString("vector>gse_xyz");
  dobj->change_xref(FRAME, Frame_ptr);
  
  QdString_var SI_ptr = new QdString("1>?");
  dobj->change_xref(SI_CONVERSION, SI_ptr);
  
  QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "newVector", dobj, context);
 

  
  objTypeSelector->close();
  
}

void mainMenu::createTime(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createTime %ld\n", this);
  fflush(stdout);
 }
 
  QdTime_var dobj;
  QdObject_var nil_obj;
  DataBrowseWindow *QBROWSER_widget;
  
  dobj = new QdTime("2001-09-05 00:00:00.000");
  objTypeSelector->close();
  QString context = QwuiPutOnWL(dobj, "new", "Constants");
  
  QdString_var SI_ptr = new QdString("1>s");
  dobj->change_xref(SI_CONVERSION, SI_ptr);
  
  QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "new", dobj, context);
  
}

void mainMenu::createTInterval(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::createTInterval %ld\n", this);
  fflush(stdout);
 }
 
  
  objTypeSelector->close();

  QdTimeInterval_var dobj = new QdTimeInterval;
  
  dobj->set( QDOS::Time("2001-09-04 00:00:00"), QDOS::Time("2001-09-05 00:00:00") );
  QString context = QwuiPutOnWL(dobj, "new", "Time Intervals");

  QdObject_var nil_obj;
  DataBrowseWindow *QBROWSER_widget;
    
  QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "new", dobj, context);
 
  
}

void mainMenu::setRtButtonMenu(){
   
   Q_POPUP_MENU * RButtonMenu = qsasWindow->WorkingList->RButtonMenu;
   
   RButtonMenu->insertItem("New &Folder", qsasWindow->WorkingList, SLOT( makeGroup() ), Qt::CTRL+Qt::Key_F );
   RButtonMenu->insertItem("Re-&Name", qsasWindow->WorkingList, SLOT( renameSelectedItem() ), Qt::CTRL+Qt::Key_N );
   RButtonMenu->insertItem("&Edit Object", this, SLOT( viewObject() ), Qt::CTRL+Qt::Key_E );
   RButtonMenu->insertItem("New Object", this, SLOT( createObject() ) );
   RButtonMenu->insertSeparator(); 
   RButtonMenu->insertItem("&Duplicate", qsasWindow->WorkingList, SLOT( duplicateItem() ), Qt::CTRL+Qt::Key_D );
   RButtonMenu->insertItem("Extract from Object", this, SLOT( launchSlice() ) );
   RButtonMenu->insertItem("Make Array into Vector", this, SLOT( launchMakeVector() ) );
   RButtonMenu->insertItem("Time Interval Folder to Event Table", this, SLOT( FolderToEventTable() ) );
   RButtonMenu->insertItem("Event Table to Time Interval Folder", this, SLOT( EventTableToFolder() ) );
   int id=RButtonMenu->insertItem("Export Event Table", this, SLOT( exportEventTable() ) );
   RButtonMenu->insertSeparator(); 
   RButtonMenu->insertItem("&Copy", qsasWindow->WorkingList, SLOT( copy() ), Qt::CTRL+Qt::Key_C );
   RButtonMenu->insertItem("Move Down", qsasWindow->WorkingList, SLOT( moveItemDown() ) , Qt::CTRL+Qt::Key_Down);
   RButtonMenu->insertItem("Move Up", qsasWindow->WorkingList, SLOT( moveItemUp() ) , Qt::CTRL+Qt::Key_Up);
   RButtonMenu->insertSeparator();
#if QT_VERSION < 0x040000
   RButtonMenu->insertItem("Delete", qsasWindow->WorkingList, SLOT( deleteListItems() ), Qt::CTRL+Qt::Key_X );
#else
   RButtonMenu->insertItem(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), "Delete", qsasWindow->WorkingList, SLOT( deleteListItems() ), Qt::CTRL+Qt::Key_X );
#endif
   RButtonMenu->insertItem("Remove time tags", qsasWindow->WorkingList, SLOT( deleteItemTT() ) );

   // connect signal emitted by double click on WL
   connect(qsasWindow->WorkingList, SIGNAL(viewObjRequest()), this, SLOT( viewObject() ) );
}


void mainMenu::launchSlice(){

 if(QuSlotDebug())
 {
    printf("mainMenu::launchSlice %ld\n", this);
	fflush(stdout);
 }
 
   QwlListItem *atItem = (QwlListItem *) qsasWindow->WorkingList->currentItem() ;  

   if(atItem  == NULL || (atItem!=NULL && atItem->getEntryType() != ListTypeEntry))
   {
      // ask user for selection
       DataSliceWindow *SliceWindow = new DataSliceWindow(0, "SliceWindow");
       QSize qsasGUISize = qsasWindow->frameSize();
       SliceWindow->move(qsasGUISize.width(),75);
       SliceWindow->show();
   }
    else
    {
       QString WLname = atItem->getWLcontext();
  
      // ask user for selection
       DataSliceWindow *SliceWindow = new DataSliceWindow(0, "SliceWindow");
       SliceWindow->setObject(WLname.latin1());
       QSize qsasGUISize = qsasWindow->frameSize();
       SliceWindow->move(qsasGUISize.width(),75);
       SliceWindow->show();
      
   }
}

void mainMenu::launchMakeVector(){

 if(QuSlotDebug())
 {
    printf("mainMenu::launchMakeVector %ld\n", this);
	fflush(stdout);
 }
 
   QwlListItem *atItem = (QwlListItem *) qsasWindow->WorkingList->currentItem() ;  
   if(atItem  == NULL) return;

   if(atItem->getEntryType() == ListTypeEntry) {
    
      QString WLname = atItem->getWLcontext();

      // ask user for selection 
      MakeVectorWindow *toVectorWindow = new MakeVectorWindow(0, "toVectorWindow");
      toVectorWindow->setObject(WLname.latin1());
      QSize qsasGUISize = qsasWindow->frameSize();
      toVectorWindow->move(qsasGUISize.width(),75);
      toVectorWindow->show();  
   }
  
}

void mainMenu::exportEventTable(){

    if(QuSlotDebug())
	{
        printf("mainMenu::exportEventTable %ld\n", this);
		fflush(stdout);
	}
    
    QwlListItem * selected_item = (QwlListItem *)qsasWindow->WorkingList->selectedItem();
    
    if (selected_item == NULL) return;
    
    QdObject_var keep_object = selected_item->QguGetObject();
    
    QdTimeIntervalSeq_var table=QdTimeIntervalSeq_var::narrow(keep_object);
    
    if(!table.is_nil())
    {
        QString fileName=selected_item->text(Obj_Name_).latin1();
        fileName.remove(" (evt table)").append(".xml");
        QString fn=QFileDialog::getSaveFileName(fileName, "*.xml", qsasWindow, "save", "Select file - Save WL Settings");
        if(!fn.isEmpty())
        {
            bool res=QiSaveVOTable(table, fn);
            if(!res) QieMsg ("ERROR", "Export failed\n");
        }
        else
        {
            QieMsg ("ERROR", "Failed selecting the file\n");
        }
    }
    else
    {
        QieMsg ("ERROR", "Wrong object type\n");
    }
}

void mainMenu::openDataFile(const char *file){
   
   QdaLaunchWindow(QString(file), 0);
   
}

void mainMenu::launchEditProfile()
{
	QProfileEditor * edit=new QProfileEditor;
	if(QPUI_widget!=NULL) edit->setQpuiWidget(QPUI_widget);
	connect(edit, SIGNAL(changedNames()), this, SLOT(updateProfileList()));
}

void mainMenu::updateProfileList()
{
	clear();
	insertProfilesItems();
	QSAS_profiles->save();
}

void mainMenu::launchSelectProfile(int id)
{
	if(QSAS_profiles->setCurrent(text(id)))
	{
		setItemChecked(checkedProfile, false);
		checkedProfile=id;
		setItemChecked(id, true);
		
		if(QSAS_profiles->current()->fontSize>=5)
		{
			QFont f;
			f.setPixelSize(QSAS_profiles->current()->fontSize);
			qApp->setFont(f);
		}
		else
		{
			qApp->setFont(defaultFont);
		}
		
	}
	
	QSAS_profiles->save();
}

