/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include <iostream>
#include <stdio.h>

#include "qt_version.h"
#include <qapplication.h>
#include <qpaintdevice.h>
#include <qwindowsstyle.h>
#include <qmotifstyle.h>
#include <qcdestyle.h>
#include <qtooltip.h> 
#include <qglobal.h>
#include <qfont.h>
#include <qsize.h>
#include <qobject.h>
#include <qmainwindow.h>
#include <qmenubar.h> 
#include <qwidget.h> 
#include <qstring.h> 
#include <qstatusbar.h>
#include <qlocale.h>
#include <locale.h>

#if defined(Q_OS_MACX)
#include <qmacstyle_mac.h>
#endif
#include "qie_qsas.h"
#include "qwui.h"
#include "qsasMenus.h"

#include "qprofile.h"

//#include <QCDEStyle>
//#include <QWindowsStyle>
//#include <QCleanlooksStyle>
//#include <QPlastiqueStyle>

// common to all QguListViews so set as a global

 // vector<QguListItem *> dragList;
 // QguListView * dragFromList;
extern  qwuiWindow* qsasWindow;

// *************************************************************************

main( int argc, char **argv )
{
	setlocale(LC_ALL, "C");
	QApplication a( argc, argv );
	QLocale::setDefault(QLocale::c());

	QSAS_profiles = new QProfilesList;
	QSAS_profiles->load();
	
	defaultFont=QFont();
	defaultFont.setPixelSize(DEFAULT_FONT_SIZE);
	
	if(QSAS_profiles->current()->fontSize>=5)
	{
		QFont f;
		f.setPixelSize(QSAS_profiles->current()->fontSize);
		a.setFont(f);
	}

	char * cef2Read = (char *) getenv ("CEF_READ_DEFAULT");
	if(cef2Read == NULL) 
		cefRead2(0);  // false:  default is read cef1 as cef2 can be autodetected
	else if(0 == strcmp(cef2Read,"CEF2")  ) 
		cefRead2(1);
	else
		cefRead2(0);  
  
	char * cef2Write = (char *) getenv ("CEF_WRITE_DEFAULT");
	if(cef2Write == NULL) 
		cefWrite2(1);  // default is write CAA syntax CEF2 files
	else if(0 == strcmp(cef2Write,"CEF2")  ) 
		cefWrite2(1);
	else
		cefWrite2(0);  
  
	// QSAS wide font settings
	QFont qsas_font = QFont(qApp->font());
	QApplication::setFont(qsas_font, TRUE);

	static QColor col_White("#ffffff");
	static QColor col_Light_Grey("#eeeeee");
	static QColor col_Grey("#8c8c8c");
	static QColor col_Black("#000000");
	static QColor col_Green("#228b22");
	static QColor col_Red("#ff3333");
	static QColor col_Yellow("#ffff00");
	static QColor col_Blue("#000099");

	QPalette qsas_palette = QPalette(qApp->palette());
  
	QColorGroup active_cg = QColorGroup(qsas_palette.active()); 
	QColorGroup inactive_cg = QColorGroup(qsas_palette.inactive()); 
	QColorGroup disabled_cg = QColorGroup(qsas_palette.disabled());
	
	// copy Mac set brushes - will be solid colours on other platforms
	QBrush brush_background = QBrush(active_cg.brush(QColorGroup::Background));
	QBrush brush_button = QBrush(active_cg.brush(QColorGroup::Button));
  
	active_cg.setColor(QColorGroup::Light,col_White);
	active_cg.setColor(QColorGroup::Midlight,col_Light_Grey);
	active_cg.setColor(QColorGroup::Mid,col_Grey);
	active_cg.setColor(QColorGroup::Dark,col_Black); 
	active_cg.setColor(QColorGroup::Foreground,col_Black);
	active_cg.setColor(QColorGroup::Base,col_White);
	active_cg.setColor(QColorGroup::Highlight,col_Grey);
	active_cg.setColor(QColorGroup::HighlightedText,col_White);
	active_cg.setColor(QColorGroup::Text,col_Black);
	active_cg.setColor(QColorGroup::Link,col_Red);
	active_cg.setColor(QColorGroup::LinkVisited,col_Yellow);
	active_cg.setColor(QColorGroup::Button,col_Light_Grey);
	active_cg.setColor(QColorGroup::ButtonText,col_Black);
	qsas_palette.setActive(active_cg);

	inactive_cg.setColor(QColorGroup::Light,col_White);
	inactive_cg.setColor(QColorGroup::Mid,col_Grey);
	inactive_cg.setColor(QColorGroup::Midlight,col_Light_Grey);
	inactive_cg.setColor(QColorGroup::Foreground,col_Grey);	  
	inactive_cg.setColor(QColorGroup::Base,col_Light_Grey);
	inactive_cg.setColor(QColorGroup::Highlight,col_Grey);
	inactive_cg.setColor(QColorGroup::HighlightedText,col_Light_Grey);
	inactive_cg.setColor(QColorGroup::Text,col_Grey);
	inactive_cg.setColor(QColorGroup::Button,col_Light_Grey);
	inactive_cg.setColor(QColorGroup::ButtonText,col_Grey);
	qsas_palette.setInactive(inactive_cg);

	disabled_cg.setColor(QColorGroup::Light,col_White);
	disabled_cg.setColor(QColorGroup::Mid,col_Grey);
	disabled_cg.setColor(QColorGroup::Midlight,col_Light_Grey);
	disabled_cg.setColor(QColorGroup::Foreground,col_Grey);	  
	disabled_cg.setColor(QColorGroup::Base,col_Light_Grey);
	disabled_cg.setColor(QColorGroup::Highlight,col_Grey);
	disabled_cg.setColor(QColorGroup::HighlightedText,col_Light_Grey);
	disabled_cg.setColor(QColorGroup::Text,col_Grey);
	disabled_cg.setColor(QColorGroup::Button,col_Light_Grey);
	disabled_cg.setColor(QColorGroup::ButtonText,col_Grey);
	qsas_palette.setDisabled(disabled_cg);

	QApplication::setPalette(qsas_palette, TRUE);
	
	qsasWindow = new qwuiWindow(0, "qsas");
	qsasWindow->WorkingList->blockSignals(true);
	
	printf("QSAS version %s.\n", QSAS_VERSION_ID);
	qsasWindow->setCaption( QSAS_VERSION_ID );

	a.setMainWidget( qsasWindow );
	qsasWindow->setFocusPolicy(WHEEL_FOCUS);

	qsasWindow->move(0,25);     
	qsasWindow->bringToFront();
	
	// create some GUIs now so they can receive restoreSession signal
	QEXPmakeWindow (qsasWindow, "Export");

	QJUImakeWindow (qsasWindow, "Join");

	QDSmakeWindow (qsasWindow, "Data Selection");

	QPUImakeWindow (qsasWindow, "Plot");

	QCALCmakeWindow( qsasWindow, "Calculator" );

	AddQwuiMenus();

	QString txt = "QSAS version ";
	txt += QSAS_VERSION_ID;
	qsasWindow->statusBar()->message(txt);
	
	qsasWindow->show();

	qsasWindow->makeWindows();
	
	qsasWindow->WorkingList->blockSignals(false);

	return a.exec();

}

