#include <cmath>
#include <iostream>
#if QT_VERSION < 0x040000
#include <qpalette.h>
#else
#include <QColorGroup>
#endif
#include "IntervalSlider.h"

IntervalSlider::IntervalSlider(double min, double max, double start, double end)
{
	reset(min, max, start, end);
	
	tracking=0;
	
	setMinimumSize(100, TOTAL_HEIGHT);
	setMouseTracking(true);
	
#if QT_VERSION < 0x040000
	QToolTip::add(this, "Interactive interval selector\nCross pattern: data unavailable\nWhite: data available\nGrey: current interval");
#else
	setToolTip("Interactive interval selector\nCross pattern: data unavailable\nWhite: data available\nGrey: current interval");
#endif
}

IntervalSlider::IntervalSlider(QWidget * parent, double min, double max, double start, double end) : QWidget(parent)
{
	reset(min, max, start, end);
	
// 	this->isTime=isTime;
	tracking=0;
	
	setMinimumSize(100, TOTAL_HEIGHT);
	setMouseTracking(true);
#if QT_VERSION < 0x040000
	QToolTip::add(this, "Interactive interval selector\nCross pattern: data unavailable\nWhite: data available\nGrey: current interval");
#else
	setToolTip("Interactive interval selector\nCross pattern: data unavailable\nWhite: data available\nGrey: current interval");
#endif
}


void IntervalSlider::setValue(double start, double end)
{
	this->start=start;
	this->end=end;
	
	int w=width();
	int h=height();
	start_pos=(int)floor((start-min)/(max-min)*(max_pos-min_pos)+H_MARGIN+0.5);
	end_pos=(int)floor((end-min)/(max-min)*(max_pos-min_pos)+H_MARGIN+0.5);
	
	update();
}

void IntervalSlider::reset(double min, double max, double start, double end)
{
	this->min=min;
	this->max=max;
	if(this->max<this->min)
	{
		double temp=this->max;
		this->max=this->min;
		this->min=temp;
	}
	
	if(this->max==this->min)
	{
		std::cerr << "IntervalSlider: min and max set to the same value. Setting max as min+100." << std::endl;
		this->max=this->min+100.;
	}
	
	this->start=start;
	this->end=end;
	if(this->start<this->min) this->start=this->min;
	if(this->end<this->min) this->end=this->min;
	if(this->start>this->max) this->start=this->max;
	if(this->end>this->max) this->end=this->max;
	
	int w=width();
	int h=height();
	start_pos=(int)floor((start-min)/(max-min)*(max_pos-min_pos)+H_MARGIN+0.5);
	end_pos=(int)floor((end-min)/(max-min)*(max_pos-min_pos)+H_MARGIN+0.5);
	
	update();
}

void IntervalSlider::resizeEvent(QResizeEvent* e)
{
	int w=width();
	int h=height();
	min_pos=H_MARGIN;
	max_pos=w-H_MARGIN;
	start_pos=(int)floor((start-min)/(max-min)*(max_pos-min_pos)+H_MARGIN+0.5);
	end_pos=(int)floor((end-min)/(max-min)*(max_pos-min_pos)+H_MARGIN+0.5);
	
	update();
}

void IntervalSlider::paintEvent(QPaintEvent* e)
{
	QPainter* painter=new QPainter(this);
	
#if QT_VERSION < 0x040000
	int ymin=(int)(height()/2.-SLIDER_HEIGHT/2.);
	int ymax=ymin+SLIDER_HEIGHT;
	
	QPen pen;
	QBrush brush;
	
	// Draw "available" interval
	pen.setWidth(1);
// 	pen.setColor(QColor(255, 100, 0));
// 	brush.setColor(QColor(255, 200, 150));
	pen.setColor(QColor(180, 180, 180));
	brush.setColor(QColor(255, 255, 255));
	brush.setStyle(Qt::SolidPattern);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRect(QRect(QPoint(H_MARGIN, ymin-1), QPoint(width()-H_MARGIN, ymax+1)));
	
	// Draw "unavailable" interval
// 	brush.setColor(QColor(255, 100, 0));
	brush.setColor(QColor(180, 180, 180));
	brush.setStyle(Qt::DiagCrossPattern);
	pen.setStyle(Qt::NoPen);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRect(QRect(QPoint(0, ymin), QPoint(H_MARGIN, ymax)));
	painter->drawRect(QRect(QPoint(width(), ymin), QPoint(width()-H_MARGIN, ymax)));
	
	// Draw "current" interval
	
	pen.setWidth(1);
// 	pen.setColor(QColor(0, 0, 255));
	pen.setColor(QColor(130, 130, 130));
	pen.setStyle(Qt::SolidLine);
// 	brush.setColor(QColor(100, 100, 255));
	brush.setColor(QColor(160, 160, 160));
	brush.setStyle(Qt::SolidPattern);
	painter->setPen(pen);
	painter->setBrush(brush);

	if(start_pos<end_pos)
	{
		QPointArray points(5);
		points.setPoint(0, start_pos, ymin);
		points.setPoint(1, start_pos, ymax);
		points.setPoint(2, end_pos, ymax);
		points.setPoint(3, end_pos+(ymax-ymin)/2, ymin+(ymax-ymin)/2);
		points.setPoint(4, end_pos, ymin);
		
		painter->drawConvexPolygon(points);
	}
	else
	{
		QPointArray points(5);
		points.setPoint(0, start_pos, ymin);
		points.setPoint(1, start_pos, ymax);
		points.setPoint(2, end_pos, ymax);
		points.setPoint(3, end_pos-(ymax-ymin)/2, ymin+(ymax-ymin)/2);
		points.setPoint(4, end_pos, ymin);
		
		painter->drawConvexPolygon(points);
	}
#else
	double ymin=height()/2.-SLIDER_HEIGHT/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	QPen pen;
	QBrush brush;
	QLinearGradient linearGrad1(QPointF(0,  ymin), QPointF(0,  ymax));
	QLinearGradient linearGrad2(QPointF(0,  ymin), QPointF(0,  ymax));
	
	// Draw "available" interval
	pen.setWidthF(1.);
	pen.setColor(QColor(180, 180, 180));
	linearGrad1.setColorAt(0., QColor(255, 255, 255));
	linearGrad1.setColorAt(1., QColor(230, 230, 230));
	brush=QBrush(linearGrad1);
// 	brush.setStyle(Qt::SolidPattern);
	painter->setPen(pen);
	painter->setBrush(brush);
// 	painter->drawRect(QRectF(H_MARGIN, ymin-1), QPointF(width()-H_MARGIN, ymax+1)));
	painter->drawRect(QRectF(H_MARGIN, ymin-1, width()-2*H_MARGIN, ymax+1-(ymin-1)));
	
	// Draw "unavailable" interval
	brush.setColor(QColor(180, 180, 180));
	brush.setStyle(Qt::DiagCrossPattern);
	pen.setStyle(Qt::NoPen);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRect(QRectF(0, ymin, H_MARGIN, ymax-ymin));
	painter->drawRect(QRectF(width()-H_MARGIN, ymin, H_MARGIN, ymax-ymin));
// 	painter->drawRect(QRectF(QPointF(0, ymin), QPointF(H_MARGIN, ymax)));
// 	painter->drawRect(QRectF(QPointF(width(), ymin), QPointF(width()-H_MARGIN, ymax)));
	
	// Draw "current" interval
	
	pen.setWidthF(1.);
	pen.setStyle(Qt::SolidLine);
	pen.setColor(QColor(130, 130, 130));
	linearGrad2.setColorAt(0., QColor(170, 170, 170));
	linearGrad2.setColorAt(1., QColor(150, 150, 150));
	brush=QBrush(linearGrad2);
	painter->setPen(pen);
	painter->setBrush(brush);

	if(start_pos<end_pos)
	{
		const QPointF points[5] =
		{
			QPointF(start_pos, ymin),
			QPointF(start_pos, ymax),
			QPointF(end_pos, ymax),
			QPointF(end_pos+(ymax-ymin)/2., ymin+(ymax-ymin)/2.),
			QPointF(end_pos, ymin)
		};
		painter->drawConvexPolygon(points, 5);
	}
	else
	{
		const QPointF points[5] =
		{
			QPointF(start_pos, ymin),
			QPointF(start_pos, ymax),
			QPointF(end_pos, ymax),
			QPointF(end_pos-(ymax-ymin)/2., ymin+(ymax-ymin)/2.),
			QPointF(end_pos, ymin)
		};
		painter->drawConvexPolygon(points, 5);
	}
	
#endif
	
	painter->end();
}

void IntervalSlider::mouseMoveEvent(QMouseEvent* e)
{
	int diff;
	double ymin=height()/2.-SLIDER_HEIGHT/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	switch(tracking)
	{
		case 1:
			start_pos=e->x();
// 			if(start_pos<min_pos) start_pos=min_pos;
// 			if(start_pos>max_pos) start_pos=max_pos;
			if(start_pos>min_pos-MOUSE_MARGIN && start_pos<min_pos+MOUSE_MARGIN) start_pos=min_pos;
			if(start_pos>max_pos-MOUSE_MARGIN && start_pos<max_pos+MOUSE_MARGIN) start_pos=max_pos;
			start=(start_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
			update();
			emit changed(start, end);
			break;
		case 2:
			end_pos=e->x();
// 			if(end_pos<min_pos) end_pos=min_pos;
// 			if(end_pos>max_pos) end_pos=max_pos;
			if(end_pos>min_pos-MOUSE_MARGIN && end_pos<min_pos+MOUSE_MARGIN) end_pos=min_pos;
			if(end_pos>max_pos-MOUSE_MARGIN && end_pos<max_pos+MOUSE_MARGIN) end_pos=max_pos;
			end=(end_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
			update();
			emit changed(start, end);
			break;
		case 3: 
			diff=end_pos-start_pos;
			end_pos=e->x()+diff-tracking_position;
			start_pos=e->x()-tracking_position;
			if(start_pos>min_pos-MOUSE_MARGIN && start_pos<min_pos+MOUSE_MARGIN)
			{
				start_pos=min_pos;
				end_pos=min_pos+diff;
			}
			if(start_pos>max_pos-MOUSE_MARGIN && start_pos<max_pos+MOUSE_MARGIN)
			{
				start_pos=max_pos;
				end_pos=max_pos+diff;
			}
			if(end_pos>min_pos-MOUSE_MARGIN && end_pos<min_pos+MOUSE_MARGIN)
			{
				end_pos=min_pos;
				start_pos=min_pos-diff;
			}
			if(end_pos>max_pos-MOUSE_MARGIN && end_pos<max_pos+MOUSE_MARGIN)
			{
				end_pos=max_pos;
				start_pos=max_pos-diff;
			}
// 			if(start_pos<min_pos || end_pos<min_pos)
// 			{
// 				if(diff>=0)
// 				{
// 					start_pos=min_pos;
// 					end_pos=min_pos+diff;
// 				}
// 				else
// 				{
// 					end_pos=min_pos;
// 					start_pos=min_pos-diff;
// 				}
// 			}
// 			if(start_pos>max_pos || end_pos>max_pos)
// 			{
// 				if(diff>=0)
// 				{
// 					end_pos=max_pos;
// 					start_pos=max_pos-diff;
// 				}
// 				else
// 				{
// 					start_pos=max_pos;
// 					end_pos=max_pos+diff;
// 				}
// 			}
			end=(end_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
			start=(start_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
			update();
			emit changed(start, end);
			break;
		case 0: // just hovering
		default:
#if QT_VERSION < 0x040000
			if(e->button()==Qt::NoButton)
#else
			if(e->buttons()==0)
#endif
			{
				if(e->y()>=ymin && e->y()<=ymax)
				{
					if(e->x()>=start_pos-MOUSE_MARGIN && e->x()<=start_pos+MOUSE_MARGIN)
					{
						setCursor( Qt::SizeHorCursor);
						return;
					}
					if(e->x()>=end_pos-MOUSE_MARGIN && e->x()<=end_pos+MOUSE_MARGIN)
					{
						setCursor( Qt::SizeHorCursor);
						return;
					}
					if(e->x()>start_pos+MOUSE_MARGIN && e->x()<end_pos-MOUSE_MARGIN || e->x()>end_pos+MOUSE_MARGIN && e->x()<start_pos-MOUSE_MARGIN)
					{
#if QT_VERSION < 0x040000
						setCursor(Qt::PointingHandCursor);
#else
						setCursor(Qt::OpenHandCursor);
#endif
						return;
					}
					setCursor(Qt::ArrowCursor);
				}
				else
				{
					setCursor(Qt::ArrowCursor);
				}
			}
			break;
	}
}

void IntervalSlider::mousePressEvent(QMouseEvent* e)
{
	double ymin=height()/2.-SLIDER_HEIGHT/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	if(tracking==0)
	{
		if(e->y()>=ymin && e->y()<=ymax)
		{
			if(e->x()>=start_pos-MOUSE_MARGIN && e->x()<=start_pos+MOUSE_MARGIN)
			{
				tracking=1;
				return;
			}
			if(e->x()>=end_pos-MOUSE_MARGIN && e->x()<=end_pos+MOUSE_MARGIN)
			{
				tracking=2;
				return;
			}
			if(e->x()>start_pos+MOUSE_MARGIN && e->x()<end_pos-MOUSE_MARGIN || e->x()>end_pos+MOUSE_MARGIN && e->x()<start_pos-MOUSE_MARGIN)
			{
				tracking_position=e->x()-start_pos;
				tracking=3;
				return;
			}
			else
			{
				int diff=abs(start_pos-end_pos);
				if(e->x()<start_pos-MOUSE_MARGIN && e->x()<end_pos-MOUSE_MARGIN)
				{
					start_pos-=diff;
					end_pos-=diff;
					if(start_pos<min_pos || end_pos<min_pos)
					{
						if(diff>=0)
						{
							start_pos=min_pos;
							end_pos=min_pos+diff;
						}
						else
						{
							end_pos=min_pos;
							start_pos=min_pos-diff;
						}
					}
					if(start_pos>max_pos || end_pos>max_pos)
					{
						if(diff>=0)
						{
							end_pos=max_pos;
							start_pos=max_pos-diff;
						}
						else
						{
							start_pos=max_pos;
							end_pos=max_pos+diff;
						}
					}
					end=(end_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
					start=(start_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
					update();
					emit changed(start, end);
					return;
				}
				if(e->x()>start_pos+MOUSE_MARGIN && e->x()>end_pos+MOUSE_MARGIN)
				{
					start_pos+=diff;
					end_pos+=diff;
					if(start_pos<min_pos || end_pos<min_pos)
					{
						if(diff>=0)
						{
							start_pos=min_pos;
							end_pos=min_pos+diff;
						}
						else
						{
							end_pos=min_pos;
							start_pos=min_pos-diff;
						}
					}
					if(start_pos>max_pos || end_pos>max_pos)
					{
						if(diff>=0)
						{
							end_pos=max_pos;
							start_pos=max_pos-diff;
						}
						else
						{
							start_pos=max_pos;
							end_pos=max_pos+diff;
						}
					}
					end=(end_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
					start=(start_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
					update();
					emit changed(start, end);
					return;
				}
			}
		}
	}
}

void IntervalSlider::mouseReleaseEvent(QMouseEvent* e)
{
	tracking=0;
}

void IntervalSlider::keyPressEvent(QKeyEvent* e)
{
	int diff=end_pos-start_pos;
	int delta=(int)fabs(floor((max_pos-min_pos)/20.)+0.5);
	switch(e->key())
	{
		case Qt::Key_Left:
			start_pos-=delta;
			break;
		
		case Qt::Key_Right:
			start_pos+=delta;
			break;
		
		case Qt::Key_Up:
			end_pos+=delta;
			break;
		
		case Qt::Key_Down:
			end_pos-=delta;
			break;
		
		case Qt::Key_PageUp:
			start_pos+=delta;
			end_pos+=delta;
			break;
		
		case Qt::Key_PageDown:
			start_pos-=delta;
			end_pos-=delta;
			break;
		
			default: return;
	}
	
	if(start_pos<min_pos || end_pos<min_pos)
	{
		if(diff>=0)
		{
			start_pos=min_pos;
			end_pos=min_pos+diff;
		}
		else
		{
			end_pos=min_pos;
			start_pos=min_pos-diff;
		}
	}
	if(start_pos>max_pos || end_pos>max_pos)
	{
		if(diff>=0)
		{
			end_pos=max_pos;
			start_pos=max_pos-diff;
		}
		else
		{
			start_pos=max_pos;
			end_pos=max_pos+diff;
		}
	}
	end=(end_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
	start=(start_pos-min_pos)/(double)(max_pos-min_pos)*(max-min)+min;
	update();
	
}

void IntervalSlider::leaveEvent(QEvent * e)
{
	setCursor(Qt::ArrowCursor);
}
