// qpui_axis_options.cc
//
// Plot Interface Implementation - Trace Options 
//
// Janet Barnes
// July 2002
//
// Tony Allen
// April 2008


#include "qpui.h"
#include "qdutil.h"
//#include "QbTimeEditor.h"


#include "qt_version.h"
#if QT_VERSION < 0x040000
#include <qgroupbox.h>
#include <qcolordialog.h>
#include <qtooltip.h>
#include "qbu.h"
#else
#include <QGroupBox>
#include <QColorDialog>
#include <QToolTip>
#include "qbu_qt4.h"
#endif

extern QPalette QPUI_normal_palette;	
extern int qsas_restore_release; 

IntervalEditorWindow::IntervalEditorWindow(QWidget * parent):QMainWindow(parent)
{
	interval=new QdTimeInterval;
	
	QWidget* main_widget=new QWidget(this);
#if QT_VERSION < 0x040000
	QBoxLayout * v_layout=new QBoxLayout(main_widget, QBoxLayout::TopToBottom);
#else
	QBoxLayout * v_layout=new QBoxLayout(QBoxLayout::TopToBottom, main_widget);
#endif
	v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	v_layout->setMargin(QSAS_profiles->current()->GUImargins);
	
	editor=new QbTimeIntervalEditor(interval, main_widget, "editor");
			
	v_layout->addWidget(editor);
	
	QWidget* buttons_widget=new QWidget(main_widget);
#if QT_VERSION < 0x040000
	QBoxLayout * h_layout=new QBoxLayout(buttons_widget, QBoxLayout::LeftToRight);
#else
	QBoxLayout * h_layout=new QBoxLayout(QBoxLayout::LeftToRight, buttons_widget);
#endif
	
	QPushButton* buttonClose=new QPushButton("Apply", buttons_widget);
	h_layout->addWidget(buttonClose, Qt::AlignCenter);
	QPushButton* buttonCancel=new QPushButton("Cancel", buttons_widget);
	h_layout->addWidget(buttonCancel, Qt::AlignCenter);
	
	v_layout->addWidget(buttons_widget);
			
	connect(buttonClose, SIGNAL(clicked()), this, SLOT(apply()));
	connect(buttonCancel, SIGNAL(clicked()), this, SLOT(cancel()));
	
	setCentralWidget(main_widget);
	
	hide();
}
		
void IntervalEditorWindow::setInterval(const QDOS::Time& t1, const QDOS::Time& t2)
{
	try
	{
		interval->set(t1, t2);
		editor->setData(interval);
	}
	catch(Exception& e){}
}

void IntervalEditorWindow::apply()
{
	emit(changesApplied());
	hide();
}

void IntervalEditorWindow::cancel()
{
	hide();
}



//     QpTab CLASS (Base of all plot tabs)
//     ---------------------

void QpTab::setSlotColour(bool active)
{
	QColorGroup g = QColorGroup(GUIpalette.active());
	QColorGroup in_g = QColorGroup(GUIpalette.inactive());
	if(active) 
	{
		in_g.setColor(QColorGroup::Base, g.light());
		g.setColor(QColorGroup::Base, g.light());
	}
	else{
		in_g.setColor(QColorGroup::Base, g.midlight());
		g.setColor(QColorGroup::Base, g.midlight());
	}
	GUIpalette.setActive(g);
    GUIpalette.setInactive(in_g);

}

QpTab::QpTab(QTabWidget *tabWidget,  QString name, QpTabType _tabType):
    QWidget(tabWidget, name)
{
	// Comstructor
	TabType = _tabType;
	GUIpalette = QPalette(qApp->palette());
}

//     QpAxisTab CLASS
//     ---------------------

#if QT_VERSION < 0x040000
QpAxisTab::QpAxisTab(QTabWidget *tabWidget,  QString name, QpListItem *item, QpTabType _tabType):
      QpTab(tabWidget, name, _tabType)
{
	// Comstructor

	editorWindow=new IntervalEditorWindow(this);
	connect(editorWindow, SIGNAL(changesApplied()), this, SLOT(refreshInterval()));
	
	I_Item = item; // will be either frame or panel in inherited classes QpYAxis and QpXAxis (or QpTAxis)
	
	QVBoxLayout *axis_top_layout = new QVBoxLayout(this);
	axis_top_layout->setSpacing(4);

	QGroupBox *axis_scale_groupbox = new QGroupBox(1, Qt::Vertical, this, "axis_scale_groupbox");
	axis_scale_groupbox->setTitle("Scale");

	QWidget *axis_scale_widget = new QWidget(axis_scale_groupbox, "axis_scale_widget");

	QGridLayout *axis_scale_layout = new QGridLayout(axis_scale_widget, 13, 6);
	axis_scale_layout->setSpacing(QSAS_profiles->current()->GUImargins/2);
	axis_scale_layout->setMargin(QSAS_profiles->current()->GUImargins/2);

	QLabel *_axis_autoscale_lbl = new QLabel("Use: ", axis_scale_widget, "autoscale_label");
	axis_scale_layout->addWidget(_axis_autoscale_lbl, 1, 0, Qt::AlignRight);
	
	_axis_autoscale_fld = new QComboBox(false, axis_scale_widget, "axis_autoscale");
	_axis_autoscale_fld->insertItem("Metadata", QP_FROM_META);
	_axis_autoscale_fld->insertItem("Autoscale", QP_FROM_DATA);
	_axis_autoscale_fld->insertItem("User Set", QP_FROM_USER);

	axis_scale_layout->addWidget(_axis_autoscale_fld, 1, 1, Qt::AlignLeft);

	linlog_label = new QLabel("Type: ", axis_scale_widget, "linlog_label");
	axis_scale_layout->addWidget(linlog_label, 1, 3, Qt::AlignRight);
	_axis_linlog_fld = new QComboBox(false, axis_scale_widget, "axis_linlog");
	_axis_linlog_fld->insertItem("Linear", QP_LINEAR);
	_axis_linlog_fld->insertItem("Log",QP_LOG);
	QFontMetrics linlog_fm(_axis_linlog_fld->font());
	int linlog_w = linlog_fm.width("Linear");
	int linlog_h = linlog_fm.height();
	_axis_linlog_fld->setMinimumSize(linlog_w, linlog_h);
#if QT_VERSION < 0x040000
	_axis_linlog_fld->listBox()->setMinimumSize(linlog_w, linlog_h);
#else
	_axis_linlog_fld->setMinimumSize(linlog_w, linlog_h);
#endif
	axis_scale_layout->addWidget(_axis_linlog_fld, 1, 4, Qt::AlignLeft);

	// only show if X or Y axis
	linlog_label->hide();
	_axis_linlog_fld->hide();
	
	t_drop_label = new QLabel("From Object: ", axis_scale_widget, "t_drop_label");
	axis_scale_layout->addMultiCellWidget(t_drop_label, 1, 1, 2, 3, Qt::AlignRight);

	_t_data_fld = new QguWLinput(axis_scale_widget, "t_data");
	_t_data_fld->setMinimumWidth(280);
	axis_scale_layout->addMultiCellWidget(_t_data_fld, 1, 1, 4, 5, Qt::AlignLeft);
	_t_data_fld->acceptTIvlObject = true;
	_t_data_fld->acceptTimeSubset = true;
	
	// only show if t axis
	_t_data_fld->hide();
	t_drop_label->hide();

	QLabel* from_label = new QLabel("From: ", axis_scale_widget, "from_label");
	axis_scale_layout->addWidget(from_label, 3, 0, Qt::AlignRight);
	
	_axis_from_xy_fld = new QguWLinput(axis_scale_widget, "axis_from");
	QFontMetrics axis_from_fm(_axis_from_xy_fld->font());
	int axis_from_w = axis_from_fm.width("YYYYMMDD HH:MM:SS.SSS XXXX");
	int axis_from_h = axis_from_fm.height();
	_axis_from_xy_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_from_xy_fld->lineBox->setPalette(GUIpalette);
	axis_scale_layout->addMultiCellWidget(_axis_from_xy_fld, 3, 3, 1, 2, Qt::AlignLeft);
	
	_axis_from_time_fld = new QbDateTimeWidget(axis_scale_widget);
	QFontMetrics axis_from_time_fm(_axis_from_time_fld->font());
	_axis_from_time_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_from_time_fld->setPalette(GUIpalette);
	axis_scale_layout->addMultiCellWidget(_axis_from_time_fld, 3, 3, 1, 2, Qt::AlignLeft);

	QLabel* to_label = new QLabel("To: ", axis_scale_widget, "to_label");
	axis_scale_layout->addWidget(to_label, 3, 3, Qt::AlignRight);
	
	_axis_to_xy_fld = new QguWLinput(axis_scale_widget, "axis_to");
	_axis_to_xy_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_to_xy_fld->lineBox->setPalette(GUIpalette);
	axis_scale_layout->addMultiCellWidget(_axis_to_xy_fld, 3, 3, 4, 5, Qt::AlignLeft);
	
	_axis_to_time_fld = new QbDateTimeWidget(axis_scale_widget);
	_axis_to_time_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_to_time_fld->setPalette(GUIpalette);
	axis_scale_layout->addWidget(_axis_to_time_fld, 3, 4, Qt::AlignLeft);
	
	_axis_time_edit_fld=new QPushButton("Edit interval", axis_scale_widget);
#if QT_VERSION < 0x040000
	axis_scale_layout->addMultiCellWidget(_axis_time_edit_fld, 5, 5, 2, 3, Qt::AlignRight);
#else
	axis_scale_layout->addWidget(_axis_time_edit_fld, 5, 5, 2, 3, Qt::AlignRight);
#endif

	connect(_axis_time_edit_fld, SIGNAL(clicked()), this, SLOT(launchIntervalEditor()));

	_axis_y_nudge_fld=new QCheckBox("Nudge Y range", axis_scale_widget);
	axis_scale_layout->addMultiCellWidget(_axis_y_nudge_fld, 5, 5, 2, 3, Qt::AlignRight);
	_axis_y_nudge_fld->setChecked(false);
	
	axis_scale_layout->addRowSpacing(0,0);
	axis_scale_layout->addRowSpacing(1, _axis_linlog_fld->height());
	axis_scale_layout->addRowSpacing(2,0);
	axis_scale_layout->addRowSpacing(3, _axis_from_xy_fld->height());
	axis_scale_layout->addRowSpacing(4,0);

	axis_scale_layout->setRowStretch(0,0);
	axis_scale_layout->setRowStretch(1,1);
	axis_scale_layout->setRowStretch(2,0);
	axis_scale_layout->setRowStretch(3,1);
	axis_scale_layout->setRowStretch(4,0);

	QGroupBox *axis_ticks_groupbox = new QGroupBox(1, Qt::Vertical, axis_scale_widget, "axis_ticks_groupbox");
	axis_ticks_groupbox->setTitle("Tickmarks");

	QWidget* axis_ticks_widget = new QWidget(axis_ticks_groupbox, "axis_ticks_widget");

	QGridLayout *axis_ticks_layout = new QGridLayout(axis_ticks_widget, 9, 4);
	axis_ticks_layout->setSpacing(QSAS_profiles->current()->GUImargins/2);
	axis_ticks_layout->setMargin(QSAS_profiles->current()->GUImargins/2);

	QLabel* num_major_ticks_label = new QLabel( "# Major ticks: ", axis_ticks_widget, "num_major_ticks_label");
	axis_ticks_layout->addWidget(num_major_ticks_label, 1, 0, Qt::AlignRight);
	_axis_major_ticks_num_fld = new QSpinBox(axis_ticks_widget, "axis_major_ticks_num");
	_axis_major_ticks_num_fld->setFixedWidth(100);
	axis_ticks_layout->addWidget(_axis_major_ticks_num_fld, 1, 1, Qt::AlignLeft);

	QLabel* major_tickmarks_label = new QLabel("Major tickmarks: ", axis_ticks_widget, "major_ticks_label");
	axis_ticks_layout->addWidget(major_tickmarks_label, 1, 2, Qt::AlignRight);
	_axis_major_tickmarks_fld = new QComboBox(false, axis_ticks_widget, "major_tickmarks");
	_axis_major_tickmarks_fld->insertItem("None",QPUI_NOTICKS);
	_axis_major_tickmarks_fld->insertItem("Inwards",QPUI_IN);
	_axis_major_tickmarks_fld->insertItem("Outwards",QPUI_OUT);
	_axis_major_tickmarks_fld->insertItem("Grid",QPUI_GRID);
	QFontMetrics major_ticks_fm(_axis_major_tickmarks_fld->font());
	int major_ticks_w = major_ticks_fm.width("Outwards");
	int major_ticks_h = major_ticks_fm.height();
	_axis_major_tickmarks_fld->setMinimumSize(major_ticks_w, major_ticks_h);
#if QT_VERSION < 0x040000
	_axis_major_tickmarks_fld->listBox()->setMinimumSize( major_ticks_w, major_ticks_h);
#else
        _axis_major_tickmarks_fld->setMinimumSize( major_ticks_w, major_ticks_h);
#endif
	QToolTip::add( _axis_major_tickmarks_fld, "Select major tickmarks style");
	axis_ticks_layout->addWidget(_axis_major_tickmarks_fld, 1, 3, Qt::AlignLeft);
	_axis_major_tickmarks_fld->setCurrentItem(QPUI_IN);
	
	QLabel* num_minor_ticks_label = new QLabel( "# Minor ticks: ", axis_ticks_widget, "num_minor_ticks_label");
	axis_ticks_layout->addWidget(num_minor_ticks_label, 3, 0, Qt::AlignRight);
	_axis_minor_ticks_num_fld = new QSpinBox(axis_ticks_widget, "axis_minor_ticks_num");
	_axis_minor_ticks_num_fld->setFixedWidth(100);
	axis_ticks_layout->addWidget(_axis_minor_ticks_num_fld, 3, 1, Qt::AlignLeft);

	QLabel* minor_tickmarks_label = new QLabel( "Minor tickmarks: ", axis_ticks_widget, "minor_ticks_label");
	axis_ticks_layout->addWidget(minor_tickmarks_label, 3, 2, Qt::AlignRight);
	_axis_minor_tickmarks_fld = new QComboBox(false, axis_ticks_widget, "minor_tickmarks");
	_axis_minor_tickmarks_fld->insertItem("None",QPUI_NOTICKS);
	_axis_minor_tickmarks_fld->insertItem("Inwards",QPUI_IN);
	_axis_minor_tickmarks_fld->insertItem("Outwards",QPUI_OUT);
	_axis_minor_tickmarks_fld->setMinimumSize(major_ticks_w,major_ticks_h);
#if QT_VERSION <0x040000
	_axis_minor_tickmarks_fld->listBox()->setMinimumSize( major_ticks_w, major_ticks_h);
#else
        _axis_minor_tickmarks_fld->setMinimumSize( major_ticks_w, major_ticks_h);
#endif
	QToolTip::add( _axis_minor_tickmarks_fld, "Select minor tickmarks style");
	axis_ticks_layout->addWidget(_axis_minor_tickmarks_fld, 3, 3, Qt::AlignLeft);
	_axis_minor_tickmarks_fld->setCurrentItem(QPUI_IN);
	
	tick_time_fmt_label=new QLabel("Time ticks format", axis_ticks_widget);
	axis_ticks_layout->addWidget(tick_time_fmt_label, 5, 0, Qt::AlignRight);
	tick_time_fmt_fld=new QLineEdit(axis_ticks_widget);
	setSlotColour(false);
	tick_time_fmt_fld->setPalette(GUIpalette);
	
	axis_ticks_layout->addWidget(tick_time_fmt_fld, 5, 1, Qt::AlignRight);
	tick_time_fmt_auto_fld=new QCheckBox("Auto-format-ticks", axis_ticks_widget);
	axis_ticks_layout->addWidget(tick_time_fmt_auto_fld, 5, 2, Qt::AlignRight);
	tick_time_fmt_auto_fld->setChecked(true);
	
	
	time_labels_fmt_label=new QLabel("Time labels format", axis_ticks_widget);
	axis_ticks_layout->addWidget(time_labels_fmt_label, 7, 0, Qt::AlignRight);
	time_labels_fmt_fld=new QLineEdit(axis_ticks_widget);
	setSlotColour(false);
	time_labels_fmt_fld->setPalette(GUIpalette);
	
	axis_ticks_layout->addWidget(time_labels_fmt_fld, 7, 1, Qt::AlignRight);
	time_labels_fmt_auto_fld=new QCheckBox("Auto-format-label", axis_ticks_widget);
	axis_ticks_layout->addWidget(time_labels_fmt_auto_fld, 7, 2, Qt::AlignRight);
	time_labels_fmt_auto_fld->setChecked(true);
	
	QToolTip::add( tick_time_fmt_fld,
		       QString("Time format as in the strftime C function:\n")+
			QString("%A or %a for week day,\n")+
			QString("%B or %b for month name,\n")+
			QString("%d for day of month as integer,\n")+
			QString("%H for hours,\n")+
			QString("%j for day of year,\n")+
			QString("%m for month as integer,\n")+
			QString("%M for minutes,\n")+
			QString("%S for seconds,\n")+
			QString("%Y or %y for year,\n")+
			QString("...\n")+
			QString("#u to switch to superscript,\n")+
			QString("#d to switch to subscript,\n")+
			QString("# is alt-3 on Mac\n")
		);
	
	QToolTip::add( time_labels_fmt_fld,
		       QString("Time format as in the strftime C function:\n")+
				QString("%A or %a for week day,\n")+
				QString("%B or %b for month name,\n")+
				QString("%d for day of month as integer,\n")+
				QString("%H for hours,\n")+
				QString("%j for day of year,\n")+
				QString("%m for month as integer,\n")+
				QString("%M for minutes,\n")+
				QString("%S for seconds,\n")+
				QString("%Y or %y for year,\n")+
				QString("...F\n")+
				QString("#u to switch to superscript,\n")+
				QString("#d to switch to subscript,\n")+
				QString("# is alt-3 on Mac\n")
		     );
	
	axis_ticks_layout->addRowSpacing(0,0);
	axis_ticks_layout->addRowSpacing(1, _axis_major_tickmarks_fld->height());
	axis_ticks_layout->addRowSpacing(2,0);
	axis_ticks_layout->addRowSpacing(3, _axis_minor_tickmarks_fld->height());

	axis_ticks_layout->setRowStretch(0,0);
	axis_ticks_layout->setRowStretch(1,1);
	axis_ticks_layout->setRowStretch(2,0);
	axis_ticks_layout->setRowStretch(3,1);

	axis_ticks_layout->setColStretch(0,0);
	axis_ticks_layout->setColStretch(1,0);
	axis_ticks_layout->setColStretch(2,0);
	axis_ticks_layout->setColStretch(3,0);

	axis_scale_layout->addMultiCellWidget(axis_ticks_groupbox, 7, 11, 0, 5);

	axis_scale_layout->addRowSpacing(10,0);
	axis_scale_layout->setRowStretch(10,0);

	axis_top_layout->addWidget(axis_scale_groupbox);

	QGroupBox *axis_label_groupbox = new QGroupBox(1, Qt::Vertical, this, "axis_label_groupbox");

	axis_label_groupbox->setTitle("Label");

	QWidget* axis_label_widget = new QWidget(axis_label_groupbox, "axis_label_widget");

	QGridLayout *axis_label_layout = new QGridLayout(axis_label_widget, 9, 5);
	axis_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* text_label = new QLabel("Text: ", axis_label_widget, "text_label");
	axis_label_layout->addWidget(text_label, 1, 0, Qt::AlignRight);
	_label_fld = new QLineEdit(axis_label_widget, "label"); 
	_label_fld->setMinimumWidth(350);
	setSlotColour(false);
	_label_fld->setPalette(GUIpalette);

	axis_label_layout->addMultiCellWidget(_label_fld, 1, 1, 1, 3, Qt::AlignLeft);
		
	_axis_autolabel_fld = new QCheckBox("Auto-label", axis_label_widget, "axis_autolabel");
	_axis_autolabel_fld->setChecked(true);
	axis_label_layout->addWidget(_axis_autolabel_fld, 1, 4, Qt::AlignLeft);

	QLabel* label_char_size_label = new QLabel("Character size: ", axis_label_widget, "label_char_size_label");
	axis_label_layout->addWidget(label_char_size_label, 3, 0, Qt::AlignRight);
	_label_char_size_fld = new FloatSpinBox(0.01, 40.,1, .1, axis_label_widget, "label_char_size");
	_label_char_size_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_char_size_fld, 3, 1, Qt::AlignLeft);
	_label_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	
	QLabel* label_disp_label = new QLabel("Displacement: ", axis_label_widget, "label_disp_label");
	axis_label_layout->addWidget(label_disp_label, 3, 2, Qt::AlignRight);
	_label_disp_fld = new FloatSpinBox(0.01, 40., 1,.5, axis_label_widget, "label_disp");
	_label_disp_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_disp_fld, 3, 3, Qt::AlignLeft);
	_label_disp_fld->setValue(QPUI_LABELS_DISP);

	QLabel* label_rotation_label = new QLabel("Rotation angle: ", axis_label_widget, "label_rotation_label");
	axis_label_layout->addWidget(label_rotation_label, 5, 0, Qt::AlignRight);
	_label_rotation_fld = new FloatSpinBox(0.0, 360.0, 1, 10., axis_label_widget, "label_rotation");
	_label_rotation_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_rotation_fld, 5, 1, Qt::AlignLeft);

	QLabel* font_label = new QLabel("Font: ", axis_label_widget, "font_label");
	axis_label_layout->addWidget(font_label, 7, 0, Qt::AlignRight);
	_label_font_fld = new FontButton(axis_label_widget);
	QToolTip::add( _label_font_fld, "Select font for axis label");
	axis_label_layout->addWidget(_label_font_fld, 7, 1, Qt::AlignLeft);

	QLabel* colour_label = new QLabel("Colour: ", axis_label_widget, "colour_label");
	axis_label_layout->addWidget(colour_label, 7, 2, Qt::AlignRight);
	_label_colour_fld = new QPushButton("Colour...", axis_label_widget, "labels_colour");
	_label_colour_fld->setFixedWidth(100);
	_label_colour_fld->setFixedHeight(30);
	QFont f = _label_colour_fld->font();
	f.setBold(TRUE);
	_label_colour_fld->setFont(f);
	axis_label_layout->addWidget(_label_colour_fld, 7, 3, Qt::AlignLeft);
	

	_event_label_fld = new QPushButton("Add Event label", axis_label_widget, "Add eventlabel");
	axis_label_layout->addWidget(_event_label_fld, 8, 0, Qt::AlignLeft);
	_event_label_fld->hide();
	
	axis_label_layout->addRowSpacing(0, 0);
	axis_label_layout->addRowSpacing(1, _label_fld->height());
	axis_label_layout->addRowSpacing(2, 0);
	axis_label_layout->addRowSpacing(3, _label_char_size_fld->height());
	axis_label_layout->addRowSpacing(4, 0);
	axis_label_layout->addRowSpacing(5, _label_rotation_fld->height());
	axis_label_layout->addRowSpacing(6, 0);
	axis_label_layout->addRowSpacing(7, _label_colour_fld->height());
	axis_label_layout->addRowSpacing(8, 0);

	axis_label_layout->setRowStretch(0,0);
	axis_label_layout->setRowStretch(1,1);
	axis_label_layout->setRowStretch(2,0);
	axis_label_layout->setRowStretch(3,1);
	axis_label_layout->setRowStretch(4,0);
	axis_label_layout->setRowStretch(5,1);
	axis_label_layout->setRowStretch(6,0);
	axis_label_layout->setRowStretch(7,1);
	axis_label_layout->setRowStretch(8,0);

	axis_top_layout->addWidget(axis_label_groupbox);


	frame_type_groupbox = new QGroupBox(1, Qt::Vertical, this, "frame_type_groupbox");

	QWidget* frame_type_widget = new QWidget(frame_type_groupbox, "frame_type_widget");

	QGridLayout *frame_type_layout = new QGridLayout(frame_type_widget,2,1);
	frame_type_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	frame_type_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_frame_type_btn = new QPushButton("Change Frame Type...", frame_type_widget,"frame_type_btn");
	_frame_type_btn->setMinimumWidth(100);
	frame_type_layout->addWidget(_frame_type_btn,0,0,Qt::AlignLeft);
	connect(_frame_type_btn, SIGNAL(pressed()), this, SLOT(launchFrameTypeSelector()) );
	QToolTip::add( _frame_type_btn, "Determines whether plot items will be TS or XY");

	frame_type_layout->addRowSpacing(0,0);
	frame_type_layout->addRowSpacing(1,_frame_type_btn->height());
		
	frame_type_layout->setRowStretch(0,0);
	frame_type_layout->setRowStretch(1,1);

	frame_type_layout->setColStretch(0,0);
	frame_type_layout->setColStretch(1,0);

	axis_top_layout->addWidget(frame_type_groupbox);

	axis_top_layout->addSpacing(4);
	axis_top_layout->addStretch(10);

	_frame_type_btn->hide();
	frame_type_groupbox->hide();
	axis_scale_widget->adjustSize();

	_label_colour = Qt::black;
}
#else // Qt4
QpAxisTab::QpAxisTab(QTabWidget *tabWidget,  QString name, QpListItem *item, QpTabType _tabType):
		QpTab(tabWidget, name, _tabType)
{
	// Comstructor

	editorWindow=new IntervalEditorWindow(this);
	connect(editorWindow, SIGNAL(changesApplied()), this, SLOT(refreshInterval()));
	
	I_Item = item; // will be either frame or panel in inherited classes QpYAxis and QpXAxis (or QpTAxis)
	
	QVBoxLayout *axis_top_layout = new QVBoxLayout(this);
	axis_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *axis_scale_groupbox = new QGroupBox(this);
	axis_scale_groupbox->setTitle("Scale");

// 	QWidget *axis_scale_widget = new QWidget(axis_scale_groupbox, "axis_scale_widget");

	QGridLayout *axis_scale_layout = new QGridLayout;
	axis_scale_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_scale_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel *_axis_autoscale_lbl = new QLabel("Use: ");
	axis_scale_layout->addWidget(_axis_autoscale_lbl, 1, 0, Qt::AlignRight);
	
	_axis_autoscale_fld = new QComboBox(false);
	_axis_autoscale_fld->insertItem("Metadata", QP_FROM_META);
	_axis_autoscale_fld->insertItem("Autoscale", QP_FROM_DATA);
	_axis_autoscale_fld->insertItem("User Set", QP_FROM_USER);

	axis_scale_layout->addWidget(_axis_autoscale_fld, 1, 1, Qt::AlignLeft);

	linlog_label = new QLabel("Type: ");
	axis_scale_layout->addWidget(linlog_label, 1, 3, Qt::AlignRight);
	_axis_linlog_fld = new QComboBox(false);
	_axis_linlog_fld->insertItem("Linear", QP_LINEAR);
	_axis_linlog_fld->insertItem("Log",QP_LOG);
	QFontMetrics linlog_fm(_axis_linlog_fld->font());
	int linlog_w = linlog_fm.width("Linear");
	int linlog_h = linlog_fm.height();
	_axis_linlog_fld->setMinimumSize(linlog_w, linlog_h);

	axis_scale_layout->addWidget(_axis_linlog_fld, 1, 4, Qt::AlignLeft);

	// only show if X or Y axis
	linlog_label->hide();
	_axis_linlog_fld->hide();
	
	t_drop_label = new QLabel("From Object: ");
	axis_scale_layout->addMultiCellWidget(t_drop_label, 1, 1, 2, 3, Qt::AlignRight);

	_t_data_fld = new QguWLinput(axis_scale_groupbox, "t_data_fld");
	_t_data_fld->setMinimumWidth(280);
	axis_scale_layout->addMultiCellWidget(_t_data_fld, 1, 1, 4, 5, Qt::AlignLeft);
	_t_data_fld->acceptTIvlObject = true;
	_t_data_fld->acceptTimeSubset = true;
	
	// only show if t axis
	_t_data_fld->hide();
	t_drop_label->hide();

	QLabel* from_label = new QLabel("From: ");
	axis_scale_layout->addWidget(from_label, 3, 0, Qt::AlignRight);
	
	_axis_from_xy_fld = new QguWLinput(axis_scale_groupbox, "axis_from");
	QFontMetrics axis_from_fm(_axis_from_xy_fld->font());
	int axis_from_w = axis_from_fm.width("YYYYMMDD HH:MM:SS.SSS XXXX");
	int axis_from_h = axis_from_fm.height();
	_axis_from_xy_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_from_xy_fld->lineBox->setPalette(GUIpalette);
	axis_scale_layout->addMultiCellWidget(_axis_from_xy_fld, 3, 3, 1, 2, Qt::AlignLeft);
	
	_axis_from_time_fld = new QbDateTimeWidget(axis_scale_groupbox);
	QFontMetrics axis_from_time_fm(_axis_from_time_fld->font());
	_axis_from_time_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_from_time_fld->setPalette(GUIpalette);
	axis_scale_layout->addMultiCellWidget(_axis_from_time_fld, 3, 3, 1, 2, Qt::AlignLeft);

	QLabel* to_label = new QLabel("To: ");
	axis_scale_layout->addWidget(to_label, 3, 3, Qt::AlignRight);
	
	_axis_to_xy_fld = new QguWLinput(axis_scale_groupbox, "axis_to");
	_axis_to_xy_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_to_xy_fld->lineBox->setPalette(GUIpalette);
	axis_scale_layout->addMultiCellWidget(_axis_to_xy_fld, 3, 3, 4, 5, Qt::AlignLeft);
	
	_axis_to_time_fld = new QbDateTimeWidget(axis_scale_groupbox);
	_axis_to_time_fld->setMinimumSize(axis_from_w, axis_from_h);
	setSlotColour(false);
	_axis_to_time_fld->setPalette(GUIpalette);
	axis_scale_layout->addWidget(_axis_to_time_fld, 3, 4, Qt::AlignLeft);
	
	_axis_time_edit_fld=new QPushButton("Edit interval");
	axis_scale_layout->addMultiCellWidget(_axis_time_edit_fld, 5, 5, 2, 3, Qt::AlignRight);

	connect(_axis_time_edit_fld, SIGNAL(clicked()), this, SLOT(launchIntervalEditor()));

	_axis_y_nudge_fld=new QCheckBox("Nudge Y range");
	axis_scale_layout->addMultiCellWidget(_axis_y_nudge_fld, 5, 5, 2, 3, Qt::AlignRight);
	_axis_y_nudge_fld->setChecked(false);
	
// 	axis_scale_layout->addRowSpacing(0,0);
// 	axis_scale_layout->addRowSpacing(1, _axis_linlog_fld->height());
// 	axis_scale_layout->addRowSpacing(2,0);
// 	axis_scale_layout->addRowSpacing(3, _axis_from_xy_fld->height());
// 	axis_scale_layout->addRowSpacing(4,0);
// 
// 	axis_scale_layout->setRowStretch(0,0);
// 	axis_scale_layout->setRowStretch(1,1);
// 	axis_scale_layout->setRowStretch(2,0);
// 	axis_scale_layout->setRowStretch(3,1);
// 	axis_scale_layout->setRowStretch(4,0);

	QGroupBox *axis_ticks_groupbox = new QGroupBox/*(axis_scale_widget)*/;
	axis_ticks_groupbox->setTitle("Tickmarks");

// 	QWidget* axis_ticks_widget = new QWidget(axis_ticks_groupbox, "axis_ticks_widget");

	QGridLayout *axis_ticks_layout = new QGridLayout;
	
	axis_ticks_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_ticks_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* num_major_ticks_label = new QLabel( "# Major ticks: ");
	axis_ticks_layout->addWidget(num_major_ticks_label, 1, 0, Qt::AlignRight);
	_axis_major_ticks_num_fld = new QSpinBox;
	_axis_major_ticks_num_fld->setFixedWidth(100);
	axis_ticks_layout->addWidget(_axis_major_ticks_num_fld, 1, 1, Qt::AlignLeft);

	QLabel* major_tickmarks_label = new QLabel("Major tickmarks: ");
	axis_ticks_layout->addWidget(major_tickmarks_label, 1, 2, Qt::AlignRight);
	_axis_major_tickmarks_fld = new QComboBox(false);
	_axis_major_tickmarks_fld->insertItem("None",QPUI_NOTICKS);
	_axis_major_tickmarks_fld->insertItem("Inwards",QPUI_IN);
	_axis_major_tickmarks_fld->insertItem("Outwards",QPUI_OUT);
	_axis_major_tickmarks_fld->insertItem("Grid",QPUI_GRID);
	QFontMetrics major_ticks_fm(_axis_major_tickmarks_fld->font());
	int major_ticks_w = major_ticks_fm.width("Outwards");
	int major_ticks_h = major_ticks_fm.height();
	_axis_major_tickmarks_fld->setMinimumSize(major_ticks_w, major_ticks_h);

	QToolTip::add( _axis_major_tickmarks_fld, "Select major tickmarks style");
	axis_ticks_layout->addWidget(_axis_major_tickmarks_fld, 1, 3, Qt::AlignLeft);
	_axis_major_tickmarks_fld->setCurrentItem(QPUI_IN);
	
	QLabel* num_minor_ticks_label = new QLabel( "# Minor ticks: ");
	axis_ticks_layout->addWidget(num_minor_ticks_label, 3, 0, Qt::AlignRight);
	_axis_minor_ticks_num_fld = new QSpinBox;
	_axis_minor_ticks_num_fld->setFixedWidth(100);
	axis_ticks_layout->addWidget(_axis_minor_ticks_num_fld, 3, 1, Qt::AlignLeft);

	QLabel* minor_tickmarks_label = new QLabel( "Minor tickmarks: ");
	axis_ticks_layout->addWidget(minor_tickmarks_label, 3, 2, Qt::AlignRight);
	_axis_minor_tickmarks_fld = new QComboBox(false);
	_axis_minor_tickmarks_fld->insertItem("None",QPUI_NOTICKS);
	_axis_minor_tickmarks_fld->insertItem("Inwards",QPUI_IN);
	_axis_minor_tickmarks_fld->insertItem("Outwards",QPUI_OUT);
	_axis_minor_tickmarks_fld->setMinimumSize(major_ticks_w,major_ticks_h);

	QToolTip::add( _axis_minor_tickmarks_fld, "Select minor tickmarks style");
	axis_ticks_layout->addWidget(_axis_minor_tickmarks_fld, 3, 3, Qt::AlignLeft);
	_axis_minor_tickmarks_fld->setCurrentItem(QPUI_IN);
	
	tick_time_fmt_label=new QLabel("Time ticks format");
	axis_ticks_layout->addWidget(tick_time_fmt_label, 5, 0, Qt::AlignRight);
	tick_time_fmt_fld=new QLineEdit;
	setSlotColour(false);
	tick_time_fmt_fld->setPalette(GUIpalette);
	
	axis_ticks_layout->addWidget(tick_time_fmt_fld, 5, 1, Qt::AlignRight);
	tick_time_fmt_auto_fld=new QCheckBox("Auto-format-ticks");
	axis_ticks_layout->addWidget(tick_time_fmt_auto_fld, 5, 2, Qt::AlignRight);
	tick_time_fmt_auto_fld->setChecked(true);
	
	
	time_labels_fmt_label=new QLabel("Time labels format");
	axis_ticks_layout->addWidget(time_labels_fmt_label, 7, 0, Qt::AlignRight);
	time_labels_fmt_fld=new QLineEdit;
	setSlotColour(false);
	time_labels_fmt_fld->setPalette(GUIpalette);
	
	axis_ticks_layout->addWidget(time_labels_fmt_fld, 7, 1, Qt::AlignRight);
	time_labels_fmt_auto_fld=new QCheckBox("Auto-format-label");
	axis_ticks_layout->addWidget(time_labels_fmt_auto_fld, 7, 2, Qt::AlignRight);
	time_labels_fmt_auto_fld->setChecked(true);
	
	QToolTip::add( tick_time_fmt_fld,
		       QString("Time format as in the strftime C function:\n")+
				       QString("%A or %a for week day,\n")+
				       QString("%B or %b for month name,\n")+
				       QString("%d for day of month as integer,\n")+
				       QString("%H for hours,\n")+
				       QString("%j for day of year,\n")+
				       QString("%m for month as integer,\n")+
				       QString("%M for minutes,\n")+
				       QString("%S for seconds,\n")+
				       QString("%Y or %y for year,\n")+
				       QString("...\n")+
				       QString("#u to switch to superscript,\n")+
				       QString("#d to switch to subscript,\n")+
				       QString("# is alt-3 on Mac\n")
		     );
	
	QToolTip::add( time_labels_fmt_fld,
		       QString("Time format as in the strftime C function:\n")+
				       QString("%A or %a for week day,\n")+
				       QString("%B or %b for month name,\n")+
				       QString("%d for day of month as integer,\n")+
				       QString("%H for hours,\n")+
				       QString("%j for day of year,\n")+
				       QString("%m for month as integer,\n")+
				       QString("%M for minutes,\n")+
				       QString("%S for seconds,\n")+
				       QString("%Y or %y for year,\n")+
				       QString("...F\n")+
				       QString("#u to switch to superscript,\n")+
				       QString("#d to switch to subscript,\n")+
				       QString("# is alt-3 on Mac\n")
		     );

	axis_ticks_groupbox->setLayout(axis_ticks_layout);
	
	axis_scale_layout->addMultiCellWidget(axis_ticks_groupbox, 7, 11, 0, 5);

// 	axis_scale_layout->addRowSpacing(10,0);
// 	axis_scale_layout->setRowStretch(10,0);
	
	axis_scale_groupbox->setLayout(axis_scale_layout);
	
	axis_top_layout->addWidget(axis_scale_groupbox);

	QGroupBox *axis_label_groupbox = new QGroupBox(this);

	axis_label_groupbox->setTitle("Label");

// 	QWidget* axis_label_widget = new QWidget(axis_label_groupbox, "axis_label_widget");

	QGridLayout *axis_label_layout = new QGridLayout;
	axis_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* text_label = new QLabel("Text: ");
	axis_label_layout->addWidget(text_label, 1, 0, Qt::AlignRight);
	_label_fld = new QLineEdit; 
	_label_fld->setMinimumWidth(350);
	setSlotColour(false);
	_label_fld->setPalette(GUIpalette);

	axis_label_layout->addMultiCellWidget(_label_fld, 1, 1, 1, 3, Qt::AlignLeft);
		
	_axis_autolabel_fld = new QCheckBox("Auto-label");
	_axis_autolabel_fld->setChecked(true);
	axis_label_layout->addWidget(_axis_autolabel_fld, 1, 4, Qt::AlignLeft);

	QLabel* label_char_size_label = new QLabel("Character size: ");
	axis_label_layout->addWidget(label_char_size_label, 3, 0, Qt::AlignRight);
	_label_char_size_fld = new FloatSpinBox(0.01, 40.,1, .1, axis_label_groupbox, "label_char_size");
	_label_char_size_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_char_size_fld, 3, 1, Qt::AlignLeft);
	_label_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	
	QLabel* label_disp_label = new QLabel("Displacement: ");
	axis_label_layout->addWidget(label_disp_label, 3, 2, Qt::AlignRight);
	_label_disp_fld = new FloatSpinBox(0.01, 40., 1,.5, axis_label_groupbox, "label_disp");
	_label_disp_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_disp_fld, 3, 3, Qt::AlignLeft);
	_label_disp_fld->setValue(QPUI_LABELS_DISP);

	QLabel* label_rotation_label = new QLabel("Rotation angle: ");
	axis_label_layout->addWidget(label_rotation_label, 5, 0, Qt::AlignRight);
	_label_rotation_fld = new FloatSpinBox(0.0, 360.0, 1, 10., axis_label_groupbox, "label_rotation");
	_label_rotation_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_rotation_fld, 5, 1, Qt::AlignLeft);

	QLabel* font_label = new QLabel("Font: ");
	axis_label_layout->addWidget(font_label, 7, 0, Qt::AlignRight);
	_label_font_fld = new FontButton;
	QToolTip::add( _label_font_fld, "Select font for axis label");
	axis_label_layout->addWidget(_label_font_fld, 7, 1, Qt::AlignLeft);

	QLabel* colour_label = new QLabel("Colour: ");
	axis_label_layout->addWidget(colour_label, 7, 2, Qt::AlignRight);
	_label_colour_fld = new QPushButton("Colour...");
	_label_colour_fld->setFixedWidth(100);
	_label_colour_fld->setFixedHeight(30);
	QFont f = _label_colour_fld->font();
	f.setBold(TRUE);
	_label_colour_fld->setFont(f);
	axis_label_layout->addWidget(_label_colour_fld, 7, 3, Qt::AlignLeft);
	

	_event_label_fld = new QPushButton("Add Event label");
	axis_label_layout->addWidget(_event_label_fld, 8, 0, Qt::AlignLeft);
	_event_label_fld->hide();
	
// 	axis_label_layout->addRowSpacing(0, 0);
// 	axis_label_layout->addRowSpacing(1, _label_fld->height());
// 	axis_label_layout->addRowSpacing(2, 0);
// 	axis_label_layout->addRowSpacing(3, _label_char_size_fld->height());
// 	axis_label_layout->addRowSpacing(4, 0);
// 	axis_label_layout->addRowSpacing(5, _label_rotation_fld->height());
// 	axis_label_layout->addRowSpacing(6, 0);
// 	axis_label_layout->addRowSpacing(7, _label_colour_fld->height());
// 	axis_label_layout->addRowSpacing(8, 0);
// 
// 	axis_label_layout->setRowStretch(0,0);
// 	axis_label_layout->setRowStretch(1,1);
// 	axis_label_layout->setRowStretch(2,0);
// 	axis_label_layout->setRowStretch(3,1);
// 	axis_label_layout->setRowStretch(4,0);
// 	axis_label_layout->setRowStretch(5,1);
// 	axis_label_layout->setRowStretch(6,0);
// 	axis_label_layout->setRowStretch(7,1);
// 	axis_label_layout->setRowStretch(8,0);

	axis_label_groupbox->setLayout(axis_label_layout);
	
	axis_top_layout->addWidget(axis_label_groupbox);


	frame_type_groupbox = new QGroupBox(this);

// 	QWidget* frame_type_widget = new QWidget(frame_type_groupbox, "frame_type_widget");

	QGridLayout *frame_type_layout = new QGridLayout;
	frame_type_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	frame_type_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_frame_type_btn = new QPushButton("Change Frame Type...");
	_frame_type_btn->setMinimumWidth(100);
	frame_type_layout->addWidget(_frame_type_btn,0,0,Qt::AlignLeft);
	connect(_frame_type_btn, SIGNAL(pressed()), this, SLOT(launchFrameTypeSelector()) );
	QToolTip::add( _frame_type_btn, "Determines whether plot items will be TS or XY");

// 	frame_type_layout->addRowSpacing(0,0);
// 	frame_type_layout->addRowSpacing(1,_frame_type_btn->height());
// 		
// 	frame_type_layout->setRowStretch(0,0);
// 	frame_type_layout->setRowStretch(1,1);
// 
// 	frame_type_layout->setColStretch(0,0);
// 	frame_type_layout->setColStretch(1,0);

	frame_type_groupbox->setLayout(frame_type_layout);
	
	axis_top_layout->addWidget(frame_type_groupbox);

	axis_top_layout->addSpacing(4);
	axis_top_layout->addStretch(10);

	_frame_type_btn->hide();
	frame_type_groupbox->hide();
	//axis_scale_widget->adjustSize();

	_label_colour = Qt::black;
}
#endif

//  ---------------------

void QpAxisTab::setAxisAutoScale(int choice)
{
	int axis = this->I_axis;

	if ( QuSlotDebug() )
	{
		printf( "QpAxisTab::setAxisAutoScale axis = %ld \n", axis );
		fflush(stdout);
	}

	if(choice == QP_FROM_USER)
	{
		setSlotColour(true);
	}
	else
	{
		_axis_from_xy_fld->clearFocus();
		_axis_to_xy_fld->clearFocus();
		_axis_from_time_fld->clearFocus();
		_axis_to_time_fld->clearFocus();
		this->setAxisMinMax();
		if(axis == X_AXIS || axis == T_AXIS) refreshYZRanges(); // also tell Y axis and colourscale
#if QT_VERSION < 0x040000
		qApp->processEvents(2000);
#else
		qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
		setSlotColour(false);
	}
	_axis_from_xy_fld->lineBox->setPalette(GUIpalette);
	_axis_to_xy_fld->lineBox->setPalette(GUIpalette);
	_axis_from_time_fld->setPalette(GUIpalette);
	_axis_to_time_fld->setPalette(GUIpalette);
	
	
	// only tell list view to make changes to selected items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Item->I_ListView->setAxisOption(AXIS_AUTOSCALE, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisLinLog(int choice)
{
	int axis = this->I_axis;
	this->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);

	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   	if ( QuSlotDebug() )
	{
		printf( "QpAxisTab::setAxisLinLog axis = %ld \n", axis );
		fflush(stdout);
	}
 
	if(this->isVisible() )
	{
		I_Item->I_ListView->setAxisOption(AXIS_LINLOG, axis, choice);
	}
}

//  ---------------------

void QpAxisTab::setAxisMin()
{
	int axis = this->I_axis;

	if ( QuSlotDebug() )
	{
		printf( "QpAxisTab::setAxisMin axis = %ld \n", axis );
		fflush(stdout);
	}
	
	if(axis!=T_AXIS && _axis_from_xy_fld->lineBox->hasFocus())
	{
		this->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
		setSlotColour(true);
		_axis_from_xy_fld->lineBox->setPalette(GUIpalette);
		_axis_to_xy_fld->lineBox->setPalette(GUIpalette);
		
		if(axis==X_AXIS) refreshYZRanges();
		
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
		
		if(this->isVisible() ) {
			I_Item->I_ListView->setAxisOption(AXIS_MIN, axis, 0, 0.0, this->_axis_from_xy_fld->text());
		}
	}
	else if(axis==T_AXIS && _axis_from_time_fld->hasFocus())
	{
		this->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
		setSlotColour(false);
		_t_data_fld->lineBox->setPalette(GUIpalette);
		setSlotColour(true);
		_axis_from_time_fld->setPalette(GUIpalette);
		_axis_to_time_fld->setPalette(GUIpalette);
		
		refreshYZRanges();
		
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
		
		if(this->isVisible() ) {
			I_Item->I_ListView->setAxisOption(AXIS_MIN, axis, 0, 0.0, this->_axis_from_time_fld->text());
		}
	}
	
}

//  ---------------------

void QpAxisTab::setAxisMax()
{
	int axis = this->I_axis;

	if ( QuSlotDebug() )	printf( "QpAxisTab::setAxisMax axis = %ld \n", axis );
	
	if(axis!=T_AXIS && _axis_to_xy_fld->lineBox->hasFocus())
	{
		this->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
		setSlotColour(true);
		_axis_from_xy_fld->lineBox->setPalette(GUIpalette);
		_axis_to_xy_fld->lineBox->setPalette(GUIpalette);
		
		if(axis==X_AXIS) refreshYZRanges();
		
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
		
		if(this->isVisible() ) {
			I_Item->I_ListView->setAxisOption(AXIS_MAX, axis, 0, 0.0, this->_axis_to_xy_fld->text());
		}
	}
	else if(axis==T_AXIS && _axis_to_time_fld->hasFocus())
	{
		this->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
		setSlotColour(false);
		_t_data_fld->lineBox->setPalette(GUIpalette);
		setSlotColour(true);
		_axis_from_time_fld->setPalette(GUIpalette);
		_axis_to_time_fld->setPalette(GUIpalette);
		
		refreshYZRanges();
		
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
		
		if(this->isVisible() ) {
			I_Item->I_ListView->setAxisOption(AXIS_MAX, axis, 0, 0.0, this->_axis_to_time_fld->text());
		}
	}
}

//  ---------------------

void QpAxisTab::setAxisNumMajorTicks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_NUM_MAJOR_TICKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisNumMinorTicks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisNumMinorTicks axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_NUM_MINOR_TICKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisMajorTickMarks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisMajorTickMarks axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_MAJOR_TICKMARKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisMinorTickMarks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisMinorTickMarks axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_MINOR_TICKMARKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabel()
{	
	int axis = this->I_axis;
	
	if ( QuSlotDebug() )
	{
		printf( "QpAxisTab::setAxisLabel axis = %ld \n", axis );
		fflush(stdout);
	}

	if(_label_fld->hasFocus())
	{
		// only change if user input
		_axis_autolabel_fld->setChecked(false);
	
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
		if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL, axis, 0, 0.0, this->_label_fld->text());
	}

}
//  ---------------------

void QpAxisTab::setAxisAutoLabel( bool choice)
{

	int axis = this->I_axis;
	
	if ( QuSlotDebug() )
	{
		printf( "QpAxisTab::setAxisAutoLabel axis = %ld \n", axis );
		fflush(stdout);
	}
	
	if(choice)
	{
		_label_fld->clearFocus();
	
		this->setAutoLabelText();
	}
	setSlotColour(!choice);
	_label_fld->setPalette(GUIpalette);
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) I_Item->I_ListView->setAxisOption(AXIS_AUTOLABEL, axis, 0, 0.0, "", choice);
	

}


//  ---------------------

void QpAxisTab::setAxisLabelCharSize(double choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL_CHAR_SIZE, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelDisp(double choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelDisp axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL_DISP, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelRotation(double choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelRotation axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL_ROTATION, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelFamily(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelFamily axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL_FONT_FAMILY, axis, choice);
}

void QpAxisTab::setAxisLabelStyle(bool choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelStyle axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL_FONT_STYLE, axis, 0, 0., "", choice);
}

void QpAxisTab::setAxisLabelWeight(bool choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelWeight axis = %ld \n", axis );
   
	if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_LABEL_FONT_WEIGHT, axis, 0, 0., "", choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelColour()
{
	int axis = this->I_axis;
	
	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(_label_colour, &changed, this);
	if(changed)
	{
        QPalette line_palette = QPalette(QPUI_normal_palette);
        line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, c);
        line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, c);
        this->_label_colour_fld->setPalette(line_palette);
        _label_colour = c;
	}

	// only tell list view to make changes to selected trace items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	I_Item->I_ListView->setAxisOption(AXIS_LABEL_COLOUR, axis, 0, 0., "", false, c);
}

//  ---------------------

void QpAxisTab::refreshYZRanges()
{
	// called when X or T axis changes: force update of Y and Z ranges
	QpFrameItem * frameItem = dynamic_cast<QpFrameItem*>(I_Item);
	if(frameItem)
	{
		QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(frameItem->firstChild());
		while(panelItem)
		{
			panelItem->updateYZRange();
			panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
		}
	}
}

// ------------------------------

void QpAxisTab::launchIntervalEditor()
{
	editorWindow->setInterval(_axis_from_time_fld->time(), _axis_to_time_fld->time());
	editorWindow->show();
	
}

void QpAxisTab::refreshInterval()
{
	QdTimeInterval tiv;
	if(editorWindow->editor->get_time_interval(tiv))
	{
		_axis_from_time_fld->setTime(tiv.start());
		_axis_to_time_fld->setTime(tiv.end());
	}
}

// ------------------------------

void QpAxisTab::launchFrameTypeSelector()
{
	// slot invoked by Change Type button
	Q_POPUP_MENU * frameTypePopup = new Q_POPUP_MENU;
	frameTypePopup->insertItem("Time Series", QP_TS);
	frameTypePopup->insertItem("XY Frame", QP_XY);
	
	connect(frameTypePopup, SIGNAL(activated(int)), I_Item->I_ListView, SLOT(setFrameType(int)));

	frameTypePopup->exec(QCursor::pos());
	
}


//     QpXAxisTab CLASS
//     ---------------------

QpXAxisTab::QpXAxisTab(QTabWidget *tabWidget,  QString name, QpFrameItem * frame):
      QpAxisTab(tabWidget, name, (QpListItem*) frame, QP_XT_TAB)
{
	// Comstructor
	if ( QuSlotDebug() )
		printf( "QpXAxisTab::QpXAxisTab name = %s \n", name.latin1() );
	
	I_Frame = frame; 
	I_axis = X_AXIS;
	
	// only show for X or Y axis
	
	_axis_from_xy_fld->show();
	_axis_to_xy_fld->show();
	_axis_from_time_fld->hide();
	_axis_to_time_fld->hide();
	_axis_time_edit_fld->hide();
	_axis_y_nudge_fld->hide();
	
	linlog_label->show();
	_axis_linlog_fld->show();
	_frame_type_btn->show();
	frame_type_groupbox->show();
	frame_type_groupbox->setTitle("Frame type:   X vs Y");
	
	tick_time_fmt_label->hide(); // show only if time axis
	tick_time_fmt_fld->hide(); // show only if time axis
	tick_time_fmt_auto_fld->hide(); // show only if time axis
	time_labels_fmt_label->hide(); // show only if time axis
	time_labels_fmt_fld->hide(); // show only if time axis
	time_labels_fmt_auto_fld->hide(); // show only if time axis

	// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	connect( _axis_linlog_fld, SIGNAL(activated(int)), this, SLOT(setAxisLinLog(int)) );
	
	_axis_from_xy_fld->acceptNumber = true;
	connect( _axis_from_xy_fld, SIGNAL(validated_OK()), this, SLOT(setAxisMin()) );

	_axis_to_xy_fld->acceptNumber = true;
	connect( _axis_to_xy_fld, SIGNAL(validated_OK()), this, SLOT(setAxisMax()) );
	
	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));


}

// -----------------------------------------------

void QpXAxisTab::setAxisMinMax()
{
	// slot called when autoscale is set on or object changed
	if ( QuSlotDebug() )
	{
		printf( "QpXAxisTab::setAxisMinMax this = %ld \n", this );
		fflush(stdout);
	}
	
	double from, to;

	switch( _axis_autoscale_fld->currentItem())
	{
	
		case QP_FROM_META:
		{
			bool found = I_Frame->getXmetaRange(from, to); // plot level call falls through to autoscale if metadata not present
			if(found)
			{
				_axis_from_xy_fld->setText(double_to_string(from).c_str());
				_axis_to_xy_fld->setText(double_to_string(to).c_str());
			}
			else
			{
				_axis_from_xy_fld->setText("");
				_axis_to_xy_fld->setText("");
			}
			break;
		}
		case QP_FROM_DATA:
		{
			QApplication::setOverrideCursor( Qt::waitCursor );
			bool found = I_Frame->getXRange(from, to);
			QApplication::restoreOverrideCursor();
			if(found)
			{
				_axis_from_xy_fld->setText(double_to_string(from).c_str());
				_axis_to_xy_fld->setText(double_to_string(to).c_str());
			}
			else
			{
				_axis_from_xy_fld->setText("");
				_axis_to_xy_fld->setText("");
			}
			break;
			
		}
		Default: case QP_FROM_USER:
		{
			// ensure objects are set for Y range
			I_Frame->getYObjects();
		}
	}
	

}
  
// -------------------------------

void QpXAxisTab::setAutoLabelText()
{	
	if ( QuSlotDebug() )
		printf( "QpXAxisTab::setAutoLabelText this = %ld \n", this );

	// refresh Axis Label
	string _label="X Axis";
	
	// use first valid X axis plot item we can find (dependent axis should all be the same)
	QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(I_Frame->firstChild());
	if(panelItem) 
	{
		QpTraceItem * traceItem = dynamic_cast<QpTraceItem*>(panelItem->firstChild());
		QpSpectrumItem * spectrumItem = dynamic_cast<QpSpectrumItem*>(panelItem->firstChild());
		if(traceItem) traceItem->getLabel(X_AXIS, _label);
		else if(spectrumItem) spectrumItem->getLabel(X_AXIS, _label);
	}
	
	_label_fld->setText(_label.c_str());
}

void QpXAxisTab::interactiveRescale(double start, double end)
{
	_axis_from_xy_fld->setText(double_to_string(start).c_str());
	_axis_to_xy_fld->setText(double_to_string(end).c_str());

    _axis_autoscale_fld->setCurrentItem(QP_FROM_USER);

    refreshYZRanges();
	
	PlotWindow* win=(PlotWindow *) sender();
	if(!win) return;
	
	I_Frame->I_Page->plot(win);
}


//     QpYAxisTab CLASS
//     ---------------------

QpYAxisTab::QpYAxisTab(QTabWidget *tabWidget,  QString name, QpPanelItem * panel):
      QpAxisTab(tabWidget, name, (QpListItem*) panel, QP_Y_TAB)
{
	// Comstructor
	if ( QuSlotDebug() )
		printf( "QpYAxisTab::QpYAxisTab name = %s \n", name.latin1() );
	
	I_Panel = panel; 
	I_axis = Y_AXIS;
	
	// only show for X or Y axis
	linlog_label->show();
	_axis_linlog_fld->show();
	
	_label_rotation_fld->setValue(90.);
	
	_axis_from_xy_fld->show();
	_axis_to_xy_fld->show();
	_axis_from_time_fld->hide();
	_axis_to_time_fld->hide();
	_axis_time_edit_fld->hide();
	_axis_y_nudge_fld->show();
	
	tick_time_fmt_label->hide(); // show only if time axis
	tick_time_fmt_fld->hide(); // show only if time axis
	tick_time_fmt_auto_fld->hide(); // show only if time axis
	time_labels_fmt_label->hide(); // show only if time axis
	time_labels_fmt_fld->hide(); // show only if time axis
	time_labels_fmt_auto_fld->hide(); // show only if time axis
	
		// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	connect( _axis_linlog_fld, SIGNAL(activated(int)), this, SLOT(setAxisLinLog(int)) );
	
	_axis_from_xy_fld->acceptNumber = true;
	connect( _axis_from_xy_fld, SIGNAL(validated_OK()), this, SLOT(setAxisMin()) );
	
	_axis_to_xy_fld->acceptNumber = true;
	connect( _axis_to_xy_fld, SIGNAL(validated_OK()), this, SLOT(setAxisMax()) );
	
	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));

	_event_label_fld->show();
	connect(_event_label_fld, SIGNAL(clicked()), I_Panel->I_ListView, SLOT(makeEventItem()));

	connect(_axis_y_nudge_fld, SIGNAL(stateChanged(int)), this, SLOT(updateNudge()));

}

void QpYAxisTab::updateNudge()
{	
	if ( QuSlotDebug() )
	{
		printf( "QpYAxisTab::updateNudge this = %ld \n", this );
		fflush(stdout);
	}
	setAxisMinMax();
}

// -----------------------------------------------

void QpYAxisTab::setAxisMinMax()
{
	// slot called when autoscale is set on, Y nudge is toggled or object changed
	if ( QuSlotDebug() )
	{
		printf( "QpYAxisTab::setAxisMinMax this = %ld \n", this );
		fflush(stdout);
	}

	double from, to;

	switch( _axis_autoscale_fld->currentItem())
	{
		case QP_FROM_META:
		{
			if(I_Panel->getYmetaRange(from, to))
			{
				_axis_from_xy_fld->setText(double_to_string(from).c_str());
				_axis_to_xy_fld->setText(double_to_string(to).c_str());
			}
			else
			{
				_axis_from_xy_fld->setText("");
				_axis_to_xy_fld->setText("");
			}
			break;
		}
		case QP_FROM_DATA:
		{
			bool found = I_Panel->getYRange(from, to);

			QApplication::restoreOverrideCursor();
			if(found)
			{
				_axis_from_xy_fld->setText(double_to_string(from).c_str());
				_axis_to_xy_fld->setText(double_to_string(to).c_str());
			}
			else
			{
				_axis_from_xy_fld->setText("");
				_axis_to_xy_fld->setText("");
			}
		}
		Default: case QP_FROM_USER:
		{
			// do nothing
		}
	}


}

// -------------------------------

void QpYAxisTab::setAutoLabelText()
{	
	if ( QuSlotDebug() )
		printf( "QpYAxisTab::setAutoLabelText this = %ld \n", this );

	// refresh Axis Label
	string _label="";
	I_Panel->getLabel(Y_AXIS, _label);
	_label_fld->setText(_label.c_str());
}

// -----------------------------------------------

  

//     QpTAxisTab CLASS
//     ---------------------

QpTAxisTab::QpTAxisTab(QTabWidget *tabWidget,  QString name, QpFrameItem * frame):
      QpAxisTab(tabWidget, name, (QpListItem*) frame, QP_XT_TAB)
{
	// Comstructor
	if ( QuSlotDebug() )
		printf( "QpTAxisTab::QpTAxisTab name = %s \n", name.latin1() );
	
	I_Frame = frame; 
	I_axis = T_AXIS;
	
	// only show forT axis
	t_drop_label->show();
	_t_data_fld->show();
	_frame_type_btn->show();
	frame_type_groupbox->show();
	frame_type_groupbox->setTitle("Frame type:   Time Series");
	
	tick_time_fmt_label->show();
	tick_time_fmt_fld->show();
	tick_time_fmt_auto_fld->show();
	time_labels_fmt_label->show();
	time_labels_fmt_fld->show();
	time_labels_fmt_auto_fld->show();

	_axis_from_xy_fld->hide();
	_axis_to_xy_fld->hide();
	_axis_from_time_fld->show();
	_axis_to_time_fld->show();
	_axis_time_edit_fld->show();
	_axis_y_nudge_fld->hide();
	
	connect(_t_data_fld, SIGNAL(validated_OK()), this, SLOT(setTData()));

	// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	
// 	_axis_from_xy_fld->acceptTime = true;
	connect( _axis_from_time_fld, SIGNAL(valueChanged()), this, SLOT(setAxisMin()) );
	
// 	_axis_to_xy_fld->acceptTime = true;
	connect( _axis_to_time_fld, SIGNAL(valueChanged()), this, SLOT(setAxisMax()) );

	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));
	
	connect(tick_time_fmt_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setTickTimeFormat(const QString&)));
	
	connect(tick_time_fmt_auto_fld, SIGNAL(stateChanged(int)), this, SLOT(setTickTimeFormatAuto(int)));
	
	connect(time_labels_fmt_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setTimeLabelsFormat(const QString&)));
	
	connect(time_labels_fmt_auto_fld, SIGNAL(stateChanged(int)), this, SLOT(setTimeLabelsFormatAuto(int)));

}

//  ---------------------

void QpTAxisTab::setTData()
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTData this = %ld \n", this );
		fflush(stdout);
	}
   
	this->setTfromObj();
  // 	this->refreshYZRanges();

	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) {
		setSlotColour(true);
		_t_data_fld->lineBox->setPalette(GUIpalette);
		I_Item->I_ListView->setAxisOption(AXIS_T_DATA, T_AXIS, 0, 0.0, this->_t_data_fld->text());
	}
}

void QpTAxisTab::setTickTimeFormat(const QString& str)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTickTimeFormat this = %ld \n", this );
		fflush(stdout);
	}

	if(tick_time_fmt_fld->hasFocus())
	{
		// only change if user input
		tick_time_fmt_auto_fld->setChecked(false);
	
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
		if(this->isVisible() ) 	I_Item->I_ListView->setAxisOption(AXIS_TICK_TIME_FORMAT, T_AXIS, 0, 0.0, str);
	}

}

void QpTAxisTab::setTickTimeFormatAuto(int state)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTickTimeFormatAuto this = %ld %ld\n", this, state );
		fflush(stdout);
	}
	
	if(state) tick_time_fmt_fld->clearFocus();
	setSlotColour(!state);
	tick_time_fmt_fld->setPalette(GUIpalette);
	
	if(state) tick_time_fmt_fld->setText(" ");

	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) {
		I_Item->I_ListView->setAxisOption(AXIS_TICK_TIME_FORMAT_AUTO, T_AXIS, state, 0.0, "");
	}
}

void QpTAxisTab::setTimeLabelsFormat(const QString& str)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTimeLabelsFormat this = %ld \n", this );
		fflush(stdout);
	}
	
	if(time_labels_fmt_fld->hasFocus())
	{
		// only change if user input
		time_labels_fmt_auto_fld->setChecked(false);
	
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
		if(this->isVisible() ) I_Item->I_ListView->setAxisOption(AXIS_TIME_LABELS_FORMAT, T_AXIS, 0, 0.0, str);
	}
}

void QpTAxisTab::setTimeLabelsFormatAuto(int state)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTimeLabelFormatAuto this = %ld %ld\n", this, state );
		fflush(stdout);
	}
	
	if(state) 
	{
		time_labels_fmt_fld->clearFocus();
		time_labels_fmt_fld->setText(" ");
	}
	setSlotColour(!state);
	time_labels_fmt_fld->setPalette(GUIpalette);
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) {
		I_Item->I_ListView->setAxisOption(AXIS_TIME_LABELS_FORMAT_AUTO, T_AXIS, state, 0.0, "");
	}
}


// ---------------------------

void QpTAxisTab::setTfromObj()
{
	// slot called when t data name field changed on tab
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTfromObj this = %ld \n", this );
		fflush(stdout);
	}
	
	// Fetch timeinterval_var object
	QdTimeInterval_var tint = get_timeinterval(_t_data_fld->getSlotObject());
	if(!tint.is_nil())
	{
		// update range
		_axis_from_time_fld->setTime(tint->start());
		_axis_to_time_fld->setTime(tint->end());
		_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
			
		refreshYZRanges();

	}
}

// -----------------------------------

void QpTAxisTab::setAxisMinMax()
{
	// slot called when autoscale is set on or object changed
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setAxisMinMax this = %ld \n", this );
		fflush(stdout);
	}

	double from, to;

	switch( _axis_autoscale_fld->currentItem())
	{
	
		case QP_FROM_META:
		{
			if(I_Frame->getXmetaRange(from, to))
			{
				Seconds tFrom(from);
				Seconds tTo(to);
				_axis_from_time_fld->setTime(tFrom);
				_axis_to_time_fld->setTime(tTo);
				setSlotColour(false);
				_t_data_fld->lineBox->setPalette(GUIpalette);
				break;
			}
			// else fall through to use data
		}
		case QP_FROM_DATA:
		{
			QApplication::setOverrideCursor( Qt::waitCursor );
			bool found = I_Frame->getXRange(from, to);
			QApplication::restoreOverrideCursor();
			
			if(found)
			{
				Seconds tFrom(from);
				Seconds tTo(to);
				_axis_from_time_fld->setTime(tFrom);
				_axis_to_time_fld->setTime(tTo);
				setSlotColour(false);
				_t_data_fld->lineBox->setPalette(GUIpalette);
			}
			else 
			{
				//_axis_from_time_fld->setText("");
				//_axis_to_time_fld->setText("");
				//TODO
			}
		}
		Default: case QP_FROM_USER:
		{
			// ensure objects are set for Y range
			I_Frame->getYObjects();
		}
	}
}

// -------------------------------

void QpTAxisTab::setAutoLabelText()
{	
	if ( QuSlotDebug() )
		printf( "QpTAxisTab::setAutoLabelText this = %ld \n", this );

	// refresh Axis Label
	_label_fld->setText("Universal Time");
}

void QpTAxisTab::interactiveRescale(double start, double end)
{
	QDOS::Time s;
	s.set_time_rep(start);
	QDOS::Time e;
	e.set_time_rep(end);
	_axis_from_time_fld->setTime(s);
	_axis_to_time_fld->setTime(e);

    _axis_autoscale_fld->setCurrentItem(QP_FROM_USER);

    refreshYZRanges();
	
	PlotWindow* win=(PlotWindow *) sender();
	if(!win) return;
	
	I_Frame->I_Page->plot(win);
}


Vistream& operator>>(Vistream& v_is, QpAxisTab& axis)
{
	// streams from virtual input stream v_is into QpAxisTab base object
	int autoscale;
	v_is >> autoscale;
	axis._axis_autoscale_fld->setCurrentItem(autoscale);
	bool autolabel;
	v_is >> autolabel;
	axis._axis_autolabel_fld->setChecked(autolabel);

	string from;
	v_is >> from;
	axis._axis_from_xy_fld->setText(from.c_str());

	string to;
	v_is >> to;
	axis._axis_to_xy_fld->setText(to.c_str());

	
	string from_time;
	v_is >> from_time;
	try
	{
		axis._axis_from_time_fld->setTime(Time(from_time.c_str()));
	}
	catch(Exception &e)
	{;}
	string to_time;
	v_is >> to_time;

	try
	{
		axis._axis_to_time_fld->setTime(Time(to_time.c_str()));
	}
	catch(Exception &e)
	{;}
	
	if( qsas_restore_release > 20304)
	{
		bool nudgeState;
		v_is >> nudgeState;
		axis._axis_y_nudge_fld->setChecked(nudgeState);
	}
	int majorNTicks;
	v_is >> majorNTicks;
	axis._axis_major_ticks_num_fld->setValue(majorNTicks);
	
	int minorNTicks;
	v_is >> minorNTicks;
	axis._axis_minor_ticks_num_fld->setValue(minorNTicks);
	
	int majorTicks;
	v_is >> majorTicks;
	axis._axis_major_tickmarks_fld->setCurrentItem(majorTicks);

	int minorTicks;
	v_is >> minorTicks;
	axis._axis_minor_tickmarks_fld->setCurrentItem(minorTicks);
	
	string fmt;
	v_is >> fmt;
	axis.tick_time_fmt_fld->setText(fmt.c_str());
	
	int checkState;
	v_is >> checkState;
	if(checkState)
	{
		axis.tick_time_fmt_auto_fld->setChecked(true);
		//axis.tick_time_fmt_fld->setEnabled(false);
		//axis.setSlotColour(false);
		//axis.tick_time_fmt_fld->setPalette(axis.GUIpalette);
		
	}
	else
	{
		axis.tick_time_fmt_auto_fld->setChecked(false);
		//axis.tick_time_fmt_fld->setEnabled(true);
		//axis.setSlotColour(true);
		//axis.tick_time_fmt_fld->setPalette(axis.GUIpalette);
	}
	
	v_is >> fmt;
	axis.time_labels_fmt_fld->setText(fmt.c_str());
	
	v_is >> checkState;
	if(checkState)
	{
		axis.time_labels_fmt_auto_fld->setChecked(true);
	//	axis.time_labels_fmt_fld->setEnabled(false);
		//axis.setSlotColour(false);
		//axis.time_labels_fmt_fld->setPalette(axis.GUIpalette);
	}
	else
	{
		axis.time_labels_fmt_auto_fld->setChecked(false);
	//	axis.time_labels_fmt_fld->setEnabled(true);
	//	axis.setSlotColour(true);
	//	axis.time_labels_fmt_fld->setPalette(axis.GUIpalette);
	}
	
	string label;
	v_is >> label;
	axis._label_fld->setText(label.c_str());

	 
	float charSize;
	v_is >> charSize;
	axis._label_char_size_fld->setValue(charSize);
	
	float disp;
	v_is >> disp;
	axis._label_disp_fld->setValue(disp);
	
	float rotn;
	v_is >> rotn;
	axis._label_rotation_fld->setValue(rotn);
	
	if( qsas_restore_release >= 20306)
	{
		PLINT family, style, weight;
		v_is >> family;
		v_is >> style;
		v_is >> weight;
		switch(family)
		{
			case PL_FCI_SERIF: axis._label_font_fld->setSerifFont(); break;
			case PL_FCI_MONO: axis._label_font_fld->setMonotypeFont(); break;
			case PL_FCI_SCRIPT: axis._label_font_fld->setDecorativeFont(); break;
			default: axis._label_font_fld->setSansFont(); break;
		}
		axis._label_font_fld->setItalic(style==PL_FCI_ITALIC);
		axis._label_font_fld->setBold(weight==PL_FCI_BOLD);
	}
	else
	{
		int font;
		v_is >> font;
		switch(font)
		{
			case 1: // Roman
				axis._label_font_fld->setSerifFont();
				axis._label_font_fld->setItalic(false);
				axis._label_font_fld->setBold(false);
				break;
				
			case 2: // Italic
				axis._label_font_fld->setSerifFont();
				axis._label_font_fld->setItalic(true);
				axis._label_font_fld->setBold(false);
				break;
				
			default: // Normal
				axis._label_font_fld->setSansFont();
				axis._label_font_fld->setItalic(false);
				axis._label_font_fld->setBold(false);
				break;
		}
	}

	
	int r, g, b;
	v_is >> r;
	v_is >> g;
	v_is >> b;
	axis._label_colour = QColor(r,g,b);
  
	return v_is;
}


Vostream& operator<<(Vostream& v_os, QpAxisTab& axis)
{
	// streams QpAxisTab base object to virtual input stream v_os

	v_os << (int) axis._axis_autoscale_fld->currentItem();
	v_os << (bool) axis._axis_autolabel_fld->isChecked();
	

	v_os << string(axis._axis_from_xy_fld->text().latin1());
	v_os << string(axis._axis_to_xy_fld->text().latin1());
	v_os << string(axis._axis_from_time_fld->text().latin1());
	v_os << string(axis._axis_to_time_fld->text().latin1());

	//  introduced at qsas_restore_release == 20305
	v_os << axis._axis_y_nudge_fld->isChecked();

	
	v_os << (int)axis._axis_major_ticks_num_fld->value();
	v_os << (int)axis._axis_minor_ticks_num_fld->value();
	
	v_os << (int)axis._axis_major_tickmarks_fld->currentItem();
	v_os << (int)axis._axis_minor_tickmarks_fld->currentItem();
	
	v_os << string(axis.tick_time_fmt_fld->text().latin1());
	v_os << (int)axis.tick_time_fmt_auto_fld->isChecked();
	v_os << string(axis.time_labels_fmt_fld->text().latin1());
	v_os << (int)axis.time_labels_fmt_auto_fld->isChecked();

	
	v_os << string(axis._label_fld->text().latin1());
	
	v_os << (float)axis._label_char_size_fld->value();
	v_os << (float)axis._label_disp_fld->value();
	v_os << (float)axis._label_rotation_fld->value();
	PLINT family, size, weight;
	axis._label_font_fld->getFontParams(family, size, weight);
	v_os << family;
	v_os << size;
	v_os << weight;
  
	v_os << axis._label_colour.red();
	v_os << axis._label_colour.green();
	v_os << axis._label_colour.blue();

	return v_os;
}

