// qpui_event_options.cc
//
// Plot Interface Implementation - Event Options Tab
//
// Janet Barnes
// Sept 2006
//
// Tony Allen
// April 2008

#include "qpui.h"
#include "TimeUtils.h"

#include <qtooltip.h> 

extern QPalette QPUI_normal_palette;

//     QpEventTab CLASS
//     ----------------------

#if QT_VERSION < 0x040000
QpEventTab::QpEventTab(QTabWidget *tabWidget,  QString name, QpEventItem * _event):
    QpTab(tabWidget, name, QP_EVENT_TAB)
{
	// Constructor
	
	// keep track of item which owns this tab
	I_event = _event;

	QVBoxLayout *event_top_layout = new QVBoxLayout(this);
	event_top_layout->setSpacing(4);

	QGroupBox *event_pos_groupbox = new QGroupBox(1, Qt::Vertical, this, "event_pos_groupbox");

	event_pos_groupbox->setTitle("Event Label Position");

	QWidget *event_pos_widget=new QWidget(event_pos_groupbox, "event_pos_widget");

	QGridLayout *event_pos_layout = new QGridLayout(event_pos_widget, 5, 5);
	event_pos_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	event_pos_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_y_label_fld = new QLabel("Y position: ", event_pos_widget, "y_label");
	event_pos_layout->addWidget(_y_label_fld, 1, 0, Qt::AlignRight);
	_y_data_fld = new QguWLinput(event_pos_widget, "y_data");
	_y_data_fld->setMinimumWidth(300);
	event_pos_layout->addMultiCellWidget(_y_data_fld, 1, 1, 1, 2, Qt::AlignLeft);
	_y_data_fld->acceptNumber = true;
	_y_data_fld->acceptScalarObject = true;
	_y_data_fld->acceptTimeSlice = true;
	connect(_y_data_fld, SIGNAL(validated_OK()), this, SLOT(setYData()));
	// connect(_y_data_fld, SIGNAL(hadDropEvent()), this, SLOT(setYData())); // <<< NOT NEEDED???

	_x_label_fld = new QLabel("T position: ", event_pos_widget, "x_label");
	if(I_event->frameType() == QP_XY) _x_label_fld->setText("X position: ");
		
	event_pos_layout->addWidget(_x_label_fld, 3, 0, Qt::AlignRight);
	_x_data_fld = new QguWLinput(event_pos_widget, "x_data");
	if(I_event->frameType()  == QP_TS)
	{
		_x_data_fld->acceptTime = true;
		_x_data_fld->acceptNumber = false;
		_x_data_fld->acceptScalarObject = false;
		_x_data_fld->acceptTimeObject = true;
		_x_data_fld->acceptTimeSlice = true;
	}
	else
	{
		_x_data_fld->acceptTime = false;
		_x_data_fld->acceptNumber = true;
		_x_data_fld->acceptScalarObject = true;
		_x_data_fld->acceptTimeObject = false;
		_x_data_fld->acceptTimeSlice = true;
	}
	_x_data_fld->setMinimumWidth(300);
	event_pos_layout->addMultiCellWidget(_x_data_fld, 3, 3, 1, 2, Qt::AlignLeft);
	connect(_x_data_fld, SIGNAL(validated_OK()), this, SLOT(setXData()));
	// connect(_x_data_fld,SIGNAL(hadDropEvent()),this,SLOT(setXData())); // NOT NEEDED???

	QPushButton *get_values_pb = new QPushButton("Get Values...", event_pos_widget, "get_values_pb");
	get_values_pb->setFixedWidth(150);
	event_pos_layout->addWidget(get_values_pb, 3, 3, Qt::AlignLeft);
	get_values_pb->setEnabled(true);

	connect(get_values_pb, SIGNAL(clicked()), this, SLOT(getValues()));

	event_pos_layout->addRowSpacing(0, 0);
	event_pos_layout->addRowSpacing(1, _y_data_fld->height());
	event_pos_layout->addRowSpacing(2, 0);
	event_pos_layout->addRowSpacing(3, _x_data_fld->height());
	event_pos_layout->addRowSpacing(4, 0);

	event_pos_layout->setRowStretch(0, 0);
	event_pos_layout->setRowStretch(1, 1);
	event_pos_layout->setRowStretch(2, 0);
	event_pos_layout->setRowStretch(3, 1);
	event_pos_layout->setRowStretch(4, 0);

	event_pos_layout->setColStretch(0, 0);
	event_pos_layout->setColStretch(1, 0);
	event_pos_layout->setColStretch(2, 0);
	event_pos_layout->setColStretch(3, 0);
	event_pos_layout->setColStretch(4, 1);

	event_top_layout->addWidget(event_pos_groupbox);
   
	QGroupBox *event_label_groupbox = new QGroupBox(1, Qt::Vertical, this, "event_label_groupbox");

	event_label_groupbox->setTitle("Label");

	QWidget* event_label_widget = new QWidget(event_label_groupbox, "event_label_widget");

	QGridLayout *event_label_layout = new QGridLayout(event_label_widget, 7, 5);
	event_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	event_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* text_label = new QLabel("Text: ", event_label_widget, "text_label");
	event_label_layout->addWidget(text_label, 1, 0, Qt::AlignRight);
	_event_label_fld = new QbuLineEdit(event_label_widget, "evLabel");
	_event_label_fld->setMinimumWidth(300);
	event_label_layout->addMultiCellWidget(_event_label_fld, 1, 1, 1, 4, Qt::AlignLeft);
	connect( _event_label_fld, SIGNAL(returnPressed()), this, SLOT(setEventLabel()) );
	connect( _event_label_fld, SIGNAL(focusOut()), this, SLOT(setEventLabel()) );

	QLabel* event_label_char_size_label = new QLabel("Character size: ", event_label_widget, "event_char_size_label");
	event_label_layout->addWidget(event_label_char_size_label, 3, 0, Qt::AlignRight);
	_char_size_fld = new FloatSpinBox(0.01, 40., 1,.1, event_label_widget, "_char_size");
	_char_size_fld->setFixedWidth(100);
	event_label_layout->addWidget(_char_size_fld, 3, 1, Qt::AlignLeft);

	connect(_char_size_fld, SIGNAL(valueChanged(double)),this, SLOT(setEventLabelCharSize(double)));
  
	QLabel* font_label = new QLabel("Font: ", event_label_widget, "font_label");
	event_label_layout->addWidget(font_label, 5, 0, Qt::AlignRight);
	_event_label_font_fld = new FontButton;
	QToolTip::add( _event_label_font_fld, "Select font for event label");
	event_label_layout->addWidget(_event_label_font_fld, 5, 1, Qt::AlignLeft);

	connect( _event_label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setEventLabelFamily(int)) );
	connect( _event_label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setEventLabelStyle(bool)) );
	connect( _event_label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setEventLabelWeight(bool)) );

	QLabel* colour_label = new QLabel("Colour: ", event_label_widget, "colour_label");
	event_label_layout->addWidget(colour_label, 5, 2, Qt::AlignRight);
	_event_label_colour_fld = new QPushButton("Colour...", event_label_widget, "labels_colour");
	_event_label_colour_fld->setFixedWidth(100);
	_event_label_colour_fld->setFixedHeight(30);
	QFont f = _event_label_colour_fld->font();
	f.setBold(TRUE);
	_event_label_colour_fld->setFont(f);
	event_label_layout->addWidget(_event_label_colour_fld, 5, 3, Qt::AlignLeft);

	connect(_event_label_colour_fld, SIGNAL(clicked()), this, SLOT(setEventLabelColour()));
  
	event_label_layout->addRowSpacing(0, 0);
	event_label_layout->addRowSpacing(1, _event_label_fld->height());
	event_label_layout->addRowSpacing(2, 0);
	event_label_layout->addRowSpacing(3, _char_size_fld->height());
	event_label_layout->addRowSpacing(4, 0);
	event_label_layout->addRowSpacing(5, _event_label_colour_fld->height());
	event_label_layout->addRowSpacing(6, 0);

	event_label_layout->setRowStretch(0, 0);
	event_label_layout->setRowStretch(1, 1);
	event_label_layout->setRowStretch(2, 0);
	event_label_layout->setRowStretch(3, 1);
	event_label_layout->setRowStretch(4, 0);
	event_label_layout->setRowStretch(5, 1);
	event_label_layout->setRowStretch(6, 1);

	event_label_layout->setColStretch(0, 0);
	event_label_layout->setColStretch(1, 0);
	event_label_layout->setColStretch(2, 0);
	event_label_layout->setColStretch(3, 0);
	event_label_layout->setColStretch(4, 1);

	event_top_layout->addWidget(event_label_groupbox);

	event_top_layout->addSpacing(4);
	event_top_layout->addStretch(10);


}
#else
QpEventTab::QpEventTab(QTabWidget *tabWidget,  QString name, QpEventItem * _event):
		QpTab(tabWidget, name, QP_EVENT_TAB)
{
	// Constructor
	
	// keep track of item which owns this tab
	I_event = _event;

	QVBoxLayout *event_top_layout = new QVBoxLayout(this);
	event_top_layout->setSpacing(4);

	QGroupBox *event_pos_groupbox = new QGroupBox(this);

	event_pos_groupbox->setTitle("Event Label Position");

// 	QWidget *event_pos_widget=new QWidget(event_pos_groupbox, "event_pos_widget");

	QGridLayout *event_pos_layout = new QGridLayout;
	event_pos_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	event_pos_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_y_label_fld = new QLabel("Y position: ");
	event_pos_layout->addWidget(_y_label_fld, 1, 0, Qt::AlignRight);
	_y_data_fld = new QguWLinput(event_pos_groupbox, "y_data");
	_y_data_fld->setMinimumWidth(300);
	event_pos_layout->addMultiCellWidget(_y_data_fld, 1, 1, 1, 2, Qt::AlignLeft);
	_y_data_fld->acceptNumber = true;
	_y_data_fld->acceptScalarObject = true;
	_y_data_fld->acceptTimeSlice = true;
	connect(_y_data_fld, SIGNAL(validated_OK()), this, SLOT(setYData()));
	// connect(_y_data_fld, SIGNAL(hadDropEvent()), this, SLOT(setYData())); // <<< NOT NEEDED???

	_x_label_fld = new QLabel("T position: ");
	if(I_event->frameType() == QP_XY) _x_label_fld->setText("X position: ");
		
	event_pos_layout->addWidget(_x_label_fld, 3, 0, Qt::AlignRight);
	_x_data_fld = new QguWLinput(event_pos_groupbox, "x_data");
	if(I_event->frameType()  == QP_TS)
	{
		_x_data_fld->acceptTime = true;
		_x_data_fld->acceptNumber = false;
		_x_data_fld->acceptScalarObject = false;
		_x_data_fld->acceptTimeObject = true;
		_x_data_fld->acceptTimeSlice = true;
	}
	else
	{
		_x_data_fld->acceptTime = false;
		_x_data_fld->acceptNumber = true;
		_x_data_fld->acceptScalarObject = true;
		_x_data_fld->acceptTimeObject = false;
		_x_data_fld->acceptTimeSlice = true;
	}
	_x_data_fld->setMinimumWidth(300);
	event_pos_layout->addMultiCellWidget(_x_data_fld, 3, 3, 1, 2, Qt::AlignLeft);
	connect(_x_data_fld, SIGNAL(validated_OK()), this, SLOT(setXData()));
	// connect(_x_data_fld,SIGNAL(hadDropEvent()),this,SLOT(setXData())); // NOT NEEDED???

	QPushButton *get_values_pb = new QPushButton("Get Values...");
	get_values_pb->setFixedWidth(150);
	event_pos_layout->addWidget(get_values_pb, 3, 3, Qt::AlignLeft);
	get_values_pb->setEnabled(true);

	connect(get_values_pb, SIGNAL(clicked()), this, SLOT(getValues()));

// 	event_pos_layout->addRowSpacing(0, 0);
// 	event_pos_layout->addRowSpacing(1, _y_data_fld->height());
// 	event_pos_layout->addRowSpacing(2, 0);
// 	event_pos_layout->addRowSpacing(3, _x_data_fld->height());
// 	event_pos_layout->addRowSpacing(4, 0);
// 
// 	event_pos_layout->setRowStretch(0, 0);
// 	event_pos_layout->setRowStretch(1, 1);
// 	event_pos_layout->setRowStretch(2, 0);
// 	event_pos_layout->setRowStretch(3, 1);
// 	event_pos_layout->setRowStretch(4, 0);
// 
// 	event_pos_layout->setColStretch(0, 0);
// 	event_pos_layout->setColStretch(1, 0);
// 	event_pos_layout->setColStretch(2, 0);
// 	event_pos_layout->setColStretch(3, 0);
// 	event_pos_layout->setColStretch(4, 1);

	event_pos_groupbox->setLayout(event_pos_layout);
	
	event_top_layout->addWidget(event_pos_groupbox);
   
	QGroupBox *event_label_groupbox = new QGroupBox(this);

	event_label_groupbox->setTitle("Label");

// 	QWidget* event_label_widget = new QWidget(event_label_groupbox, "event_label_widget");

	QGridLayout *event_label_layout = new QGridLayout;
	event_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	event_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* text_label = new QLabel("Text: ");
	event_label_layout->addWidget(text_label, 1, 0, Qt::AlignRight);
	_event_label_fld = new QbuLineEdit;
	_event_label_fld->setMinimumWidth(300);
	event_label_layout->addMultiCellWidget(_event_label_fld, 1, 1, 1, 4, Qt::AlignLeft);
	connect( _event_label_fld, SIGNAL(returnPressed()), this, SLOT(setEventLabel()) );
	connect( _event_label_fld, SIGNAL(focusOut()), this, SLOT(setEventLabel()) );

	QLabel* event_label_char_size_label = new QLabel("Character size: ");
	event_label_layout->addWidget(event_label_char_size_label, 3, 0, Qt::AlignRight);
	_char_size_fld = new FloatSpinBox(0.01, 40., 1,.1, event_label_groupbox, "_char_size");
	_char_size_fld->setFixedWidth(100);
	event_label_layout->addWidget(_char_size_fld, 3, 1, Qt::AlignLeft);

	connect(_char_size_fld, SIGNAL(valueChanged(double)),this, SLOT(setEventLabelCharSize(double)));
  
	QLabel* font_label = new QLabel("Font: ");
	event_label_layout->addWidget(font_label, 5, 0, Qt::AlignRight);
	_event_label_font_fld = new FontButton;
	QToolTip::add( _event_label_font_fld, "Select font for event label");
	event_label_layout->addWidget(_event_label_font_fld, 5, 1, Qt::AlignLeft);

	connect( _event_label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setEventLabelFamily(int)) );
	connect( _event_label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setEventLabelStyle(bool)) );
	connect( _event_label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setEventLabelWeight(bool)) );
	
	QLabel* colour_label = new QLabel("Colour: ");
	event_label_layout->addWidget(colour_label, 5, 2, Qt::AlignRight);
	_event_label_colour_fld = new QPushButton("Colour...");
	_event_label_colour_fld->setFixedWidth(100);
	_event_label_colour_fld->setFixedHeight(30);
	QFont f = _event_label_colour_fld->font();
	f.setBold(TRUE);
	_event_label_colour_fld->setFont(f);
	event_label_layout->addWidget(_event_label_colour_fld, 5, 3, Qt::AlignLeft);

	connect(_event_label_colour_fld, SIGNAL(clicked()), this, SLOT(setEventLabelColour()));
  
// 	event_label_layout->addRowSpacing(0, 0);
// 	event_label_layout->addRowSpacing(1, _event_label_fld->height());
// 	event_label_layout->addRowSpacing(2, 0);
// 	event_label_layout->addRowSpacing(3, _char_size_fld->height());
// 	event_label_layout->addRowSpacing(4, 0);
// 	event_label_layout->addRowSpacing(5, _event_label_colour_fld->height());
// 	event_label_layout->addRowSpacing(6, 0);
// 
// 	event_label_layout->setRowStretch(0, 0);
// 	event_label_layout->setRowStretch(1, 1);
// 	event_label_layout->setRowStretch(2, 0);
// 	event_label_layout->setRowStretch(3, 1);
// 	event_label_layout->setRowStretch(4, 0);
// 	event_label_layout->setRowStretch(5, 1);
// 	event_label_layout->setRowStretch(6, 1);
// 
// 	event_label_layout->setColStretch(0, 0);
// 	event_label_layout->setColStretch(1, 0);
// 	event_label_layout->setColStretch(2, 0);
// 	event_label_layout->setColStretch(3, 0);
// 	event_label_layout->setColStretch(4, 1);
	
	event_label_groupbox->setLayout(event_label_layout);

	event_top_layout->addWidget(event_label_groupbox);

	event_top_layout->addSpacing(4);
	event_top_layout->addStretch(10);


}
#endif

// -------------------------------------------

void QpEventTab::getValues()
{
	// slot invoked when Get Values... push button is clicked

	if(QuSlotDebug()) cout << "QpEventTab::getValues" << endl;
	double _x_from, _y_from;
	      // clear listview selection to avoid user accidentally applying changes
  // to the wrong panel. Selection is from the plot.
	QpPanelItem *panel_item = I_event->I_Panel;
	QpFrameItem *frame_item = panel_item->I_Frame;
	QpPageItem *page_item = frame_item->I_Page;
    
      // check that the currently selected plot x window is open
	int plot_id=page_item->PageTab->_xwin_num_fld->text().toInt();
	if(plot_id<0)
	{
		(void)QMessageBox::warning( this,"Event Options", "Error! Device not open? Try replotting.");
		return;
	}
    
	PlotWindow* win=page_item->focusWindow(plot_id);
	if(win==NULL)
	{
		(void)QMessageBox::warning( this,"Event Options", "Error! Window id not found.");
		return;
	}
	double x_start, y_start, x_end, y_end;
	win->plot->captureMouse(&x_start, &y_start, &x_end, &y_end);

	y_start=1.-y_start;
	y_end=1.-y_end;
	
	// Get clicks coordinates relatively to the current pane;
	
	double x_offset = frame_item->PropertiesTab->_loffset_fld->value();
	double x_sz = frame_item->PropertiesTab->_width_fld->value();
	double y_sz = frame_item->PropertiesTab->_height_fld->value();
	y_sz /= frame_item->numberOfChildren();
	double y_offset = 1. - frame_item->PropertiesTab->_toffset_fld->value() - ( panel_item->getStackNumber() ) * y_sz;

  
	// calculate new x axis ranges from plot selection coordinates 
	if(frame_item->frameType() == QP_XY)
	{
		double xmin = frame_item->xAxisTab->_axis_from_xy_fld->value();
		double xmax = frame_item->xAxisTab->_axis_to_xy_fld->value();
	
    		// if logarithmic axis, convert to logs first
		if(frame_item->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			xmin = SafeLog10(xmin);
			xmax = SafeLog10(xmax);
		}
	
		double x_range = xmax - xmin;
		_x_from = xmin + x_range * (x_start-x_offset)/x_sz;
		if(frame_item->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			_x_from=pow(10., _x_from);
		}	    
		_x_data_fld->setText(QString("%1").arg(_x_from));
	}
	else if(frame_item->frameType() == QP_TS)
	{
		double tmin = frame_item->tAxisTab->_axis_from_time_fld->value();
		double tmax = frame_item->tAxisTab->_axis_to_time_fld->value();
		double t_range = tmax - tmin;
		_x_from = tmin + t_range * (x_start-x_offset)/x_sz;
   
   	
		Seconds tFrom(_x_from);
		_x_data_fld->setText(tFrom.iso_srep().c_str());

	} 
  
  // calculate new y axis ranges
	if(panel_item->yAxisTab)
	{
		double ymin = panel_item->yAxisTab->_axis_from_xy_fld->value();
		double ymax = panel_item->yAxisTab->_axis_to_xy_fld->value();
    // if logarithmic axis convert to logs first
		if(panel_item->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			ymin = SafeLog10(ymin);
			ymax = SafeLog10(ymax);
		}
		double y_range = ymax - ymin;
		_y_from = ymin + y_range*(y_start-y_offset)/y_sz;
		if(panel_item->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			_y_from = pow(10., _y_from);
		}	    
		_y_data_fld->setText(QString("%1").arg(_y_from));
	}

} 

// -------------------------------------------------

void QpEventTab::setXData()
{
  // slot called when x data name field changed on widget
	// Only used to propogate multiple selection input
 
    if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_X_DATA, 0, 0., this->_x_data_fld->text());
}

// -----------------------------

void QpEventTab::setYData()
{
	// slot called when y data name field changed on widget
	// Only used to propogate multiple selection input
  
   if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_Y_DATA, 0, 0., this->_y_data_fld->text());
}

// -----------------------------

void QpEventTab::setEventLabel()
{
	// slot called when label text field changed on widget
	// Only used to propogate multiple selection input
  
   if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_LABEL, 0, 0., this->_event_label_fld->text());
}

// -----------------------------

void QpEventTab::setEventLabelCharSize(double fValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propogate multiple selection input
  
   if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_LABEL_CHAR_SIZE, 0, fValue);
}

// -----------------------------

void QpEventTab::setEventLabelFamily(int iValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propogate multiple selection input
  
   if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_LABEL_FONT_FAMILY, iValue);
}

void QpEventTab::setEventLabelStyle(bool bValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propogate multiple selection input
  
	if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_LABEL_FONT_STYLE, 0, 0., "", bValue);
}

void QpEventTab::setEventLabelWeight(bool bValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propogate multiple selection input
  
	if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_LABEL_FONT_WEIGHT, 0, 0., "", bValue);
}

// -----------------------------

void QpEventTab::setEventLabelColour()
{
   bool changed = false;
   // ask the user for colour choice
   QColor c = QuGetColour(I_event->_event_label_colour, &changed, this);
   if(changed)
   {
        QPalette line_palette = QPalette(QPUI_normal_palette);
        line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, c);
        line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, c);
        this->_event_label_colour_fld->setPalette(line_palette);
        I_event->_event_label_colour = c;
   }

   // only tell list view to make changes to selected event items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_event->I_ListView->setEventOption(EVENT_LABEL_COLOUR, 0, 0., "", false, c);
}


