// qpui_page.cc
//
// Plot Interface Implementation - Page Object
//
// Janet Barnes
//
// July 2002

#include <stdlib.h>

#include "qpui.h"

//#include <string>



// global to allow counting spectra colour tables
vector <int> colourTablesUsed;

//     QpPageItem CLASS
//     --------------

// static members
double QpPageItem::_width_in_inches[6]={8.3,8.5,7.5,8.5,8.5,0.};
double QpPageItem::_height_in_inches[6]={11.694,9,10.,14,11,0.};

QpPageItem::QpPageItem(QpListView *parent, QString name, QpPageItem *after):
    QpListItem(parent,name,after,ListTypeGroup,"")
{
	_conversion_factors[QPUI_PIXELS]=72.;
	_conversion_factors[QPUI_INCHES]=1.;
	_conversion_factors[QPUI_CM]=2.54;
	_conversion_factors[QPUI_MM]=25.4;
	_conversion_factors[QPUI_PSPOINTS]=72;
	
	// Constructor
	I_Type = QP_PAGE;
	//_plot_id = 0;
	_print_id = 0;
	this->name=name;
	
	// initialise page tab widgets
   	PageTab = new QpPageTab(tabWidget, "PageTab", this);
	PageTab->hide();
	
   	LayoutTab = new QpLayoutTab(tabWidget, "LayoutTab", this);
	LayoutTab->hide();
		
	ValueReturnTab = new QpValueReturnTab(tabWidget, "ValueReturnTab", this);
	ValueReturnTab->hide();


	tabWidget->adjustSize();
}

// ----------------------------------

QpPageItem::~QpPageItem()
{
	// Destructor
	clearWindows();
  	// empty page of frame items
	QpFrameItem * child = dynamic_cast<QpFrameItem*>(this->firstChild());
	while(child)
	{
		QpFrameItem * next = dynamic_cast<QpFrameItem*>(child->nextSibling());
		delete child;
		child = next;
	}
        
	delete ValueReturnTab;
	delete PageTab;
	delete LayoutTab;
}

// ----------------------------------

void QpPageItem::showTabs()
{
	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--) 
	{
		if(tabWidget->page(i))
		{
			tabWidget->page(i)->hide();
			tabWidget->removePage(this->tabWidget->page(i));
		}
	}

	tabWidget->addTab(PageTab, "Page");
	tabWidget->addTab(LayoutTab, "Layout");
	tabWidget->addTab(ValueReturnTab, "Value Return");
	
	if(this->I_ListView->I_LastTab == QP_LAYOUT_TAB)  tabWidget->showPage(LayoutTab);
	else	tabWidget->showPage(PageTab);

}

// ----------------------------------

void QpPageItem::setDefaultOptions()
{
	
	PageTab->_page_size_fld->setCurrentItem(QPUI_A4);
	PageTab->_units_fld->setCurrentItem(QPUI_PIXELS);
	PageTab->_orientation_fld->setCurrentItem(QPUI_PORTRAIT);
	PageTab->_show_ident_fld->setChecked(true);
	PageTab->_xwin_num_fld->setText("-1");
		
	ValueReturnTab->_xaxis_rescale_fld->setChecked(true);
	ValueReturnTab->_xaxis_swap_fld->setChecked(true);
	ValueReturnTab->_xaxis_overwrite_fld->setChecked(true);
	ValueReturnTab->_yaxis_rescale_fld->setChecked(true);
	ValueReturnTab->_yaxis_swap_fld->setChecked(true);
	ValueReturnTab->_yaxis_overwrite_fld->setChecked(true);
	ValueReturnTab->_replot_fld->setChecked(true);

	LayoutTab->_autolayout_fld->setChecked(true);
	LayoutTab->_hgap_fld->setValue(QPUI_PAGE_HGAP);
	LayoutTab->_vgap_fld->setValue(QPUI_PAGE_VGAP);
	LayoutTab->_lmargin_fld->setValue(QPUI_PAGE_LMARGIN);
	LayoutTab->_rmargin_fld->setValue(QPUI_PAGE_RMARGIN);
	LayoutTab->_tmargin_fld->setValue(QPUI_PAGE_TMARGIN);
	LayoutTab->_bmargin_fld->setValue(QPUI_PAGE_BMARGIN);
}

// ----------------------------------

PlotWindow* QpPageItem::newWindow(double width, double height, int* id, int resolution, QPrinter::PageSize pageSize, QPrinter::Orientation orientation)
{
		// find free id
	
	bool found_id=false;
	bool this_one_ok;
	*id=1;
	while(!found_id)
	{
		this_one_ok=true;
#if QT_VERSION < 0x040000
		for(std::list<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end() && this_one_ok; ++i)
#else
			for(QList<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end() && this_one_ok; ++i)
#endif
		{
			this_one_ok=this_one_ok && ((*i)->getId()!=*id);
		}
		if(this_one_ok)
		{
			found_id=true;
		}
		else
		{
			(*id)++;
		}
	}

	PlotWindow* res=new PlotWindow(*id, (width<1000?(int)width: 1000), (height<700?(int)height:700), width, height, resolution, pageSize, orientation, NULL);
#if QT_VERSION < 0x040000
	PlotWindows.push_back(res);
#else
	PlotWindows.append(res);
#endif
	res->setTitle(QString("QSAS Plot: %1 Window #%2").arg(name).arg(*id));
	res->show();
	return res;
}

void QpPageItem::rename(QString newName)
{
	name=newName;
#if QT_VERSION < 0x040000
	for(std::list<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end(); ++i)
#else
		for(QList<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end(); ++i)
#endif
	{
		(*i)->setTitle(QString("QSAS Plot: %1 Window #%2").arg(name).arg((*i)->getId()));

	}

}

// ----------------------------------

PlotWindow* QpPageItem::focusWindow(int id, double width, double height, int resolution, QPrinter::PageSize pageSize, QPrinter::Orientation orientation)
{
	PlotWindow * res;
#if QT_VERSION < 0x040000
	for(std::list<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end(); ++i)
#else
		for(QList<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end(); ++i)
#endif
	{
		if((*i)->getId()==id)
		{
			(*i)->show();
			if(width>0. && height > 0.)
			{
				(*i)->setSize(width, height);
				(*i)->resolution=resolution;
				(*i)->pageSize=pageSize;
				(*i)->orientation=orientation;
			}
			(*i)->setPLFocus();
			(*i)->setActiveWindow();
			(*i)->raise();
			return (*i);
		}
	}
	
	// If reached this point, the window does not exist.
	// Force creation of Window with this id
	
	res=new PlotWindow(id, (width<1000?(int)width: 1000), (height<700?(int)height:700), width, height, resolution, pageSize, orientation, NULL);
#if QT_VERSION < 0x040000
	PlotWindows.push_back(res);
#else
	PlotWindows.append(res);

#endif
	res->setTitle(QString("QSAS Plot: %1 Window #%2").arg(name).arg(id));
	res->show();
	return res;
}

void QpPageItem::clearWindows()
{
#if QT_VERSION < 0x040000
	for(std::list<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end(); ++i)
#else
		for(QList<PlotWindow*>::iterator i=PlotWindows.begin(); i!=PlotWindows.end(); ++i)
#endif
	{
		PlotWindow * temp=*i;
		temp->close();
		delete temp;
	}
	PlotWindows.clear();
}

// ----------------------------------

QpuiReturnStatus QpPageItem::plot(PlotWindow* win)
{
	if(QuSlotDebug())
	{
		printf("QpPageItem::plot\n");
		fflush(stdout);
	}
	
	static PLFLT default_xwidth=0.;
	static PLFLT default_xheight=0.;
	
	plspause(0);
	
	// set background and default colour
	plscol0(0,255,255,255); // background temporarily set to white
	plscol0(1,0,0,0); // default set to black
	
	pladv(0);
	plvpor(0., 1., 0., 1.);
	
	 // To make sure the new width will be validated:
	// plwid only stores changes in width
	// If no change from the previous plot, the default
	// width is not stored in the device buffer.
	// Thus we force a change, to store it.
	plwid(QtWidth2PlWidth(2));
	plwid(QtWidth2PlWidth(1));
	
	// clear colour table 
	
	getPGPColourIndex(QColor(0,0,0), true);
	
	win->setSmooth(true);
	
	if(PageTab->_show_ident_fld->isChecked()) print_iden();
	
	// count number of colour tables used for spectra
	countColourTables();
	
	// now handle each frame within this page
	
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(this->firstChild());
	while(frameItem)
	{
		if(QPUI_FAILURE == frameItem->plot(win))
		{
			QuMsgBar(("Problem plotting frame "+frameItem->text(0)).latin1(),true);
		}
	
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
	
	#if QT_VERSION < 0x040000
	qApp->processEvents(2000);
	#else
	qApp->processEvents(QEventLoop::AllEvents, 2000);
	#endif
	
	return QPUI_SUCCESS; 
}

// ---------------

void QpPageItem::countColourTables()
{
	colourTablesUsed.clear();
   
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(this->firstChild());
	while(frameItem)
	{
		QpPanelItem *panelItem = dynamic_cast<QpPanelItem*>(frameItem->firstChild());
		while(panelItem)
		{
				QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(panelItem->firstChild());
				while(plotItem)
				{
					QpSpectrumItem *spectrumItem = dynamic_cast<QpSpectrumItem*>(plotItem);
					if(spectrumItem)
					{
						int ct = (QpuiColourTable) spectrumItem->ColourscaleTab->_colourscale_table_fld->currentItem();
						bool found = false;
						for(int i=0; i< colourTablesUsed.size(); i++)
						{
							if(colourTablesUsed[i] == ct) 
							{
								found = true;
								break;
							}
						}
						if(!found) colourTablesUsed.push_back(ct);
					}
					plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
				}
				panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
		}
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
}

// -----------------

void QpPageItem::set_colour_table(const QpuiColourTable& colour_table)
{
    PLFLT *h, *s, *l, *i;
    QString str;
    QStringList sl, sl2;
    int npoints;
    
    switch(colour_table)
    {

        case QPUI_RAINBOW_EXT:
            h=new PLFLT[4]; h[0]=300.; h[1]=300.; h[2]=-10.; h[3]=-10.;
            s=new PLFLT[4]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.;
            l=new PLFLT[4]; l[0]=1.; l[1]=.5; l[2]=.5; l[3]=0.;
            
            i=new PLFLT[4]; i[0]=0.; i[1]=0.1; i[2]=0.9; i[3]=1.;
            
            plscmap1l(0, 4, i, h, l, s, NULL);
            
            delete[] h;
            delete[] s;
            delete[] l;
            break;
            
        case QPUI_GRAY_WHITE_HIGH:
            h=new PLFLT[2]; h[0]=0.; h[1]=0.;
            s=new PLFLT[2]; s[0]=0.; s[1]=0.;
            l=new PLFLT[2]; l[0]=0.; l[1]=1.;
            
            i=new PLFLT[2]; i[0]=0.; i[1]=1.;
            
            plscmap1l(0, 2, i, h, l, s, NULL);
            
            delete[] h;
            delete[] s;
            delete[] l;
            delete[] i;
            break;
            
        case QPUI_GRAY_WHITE_LOW:
            h=new PLFLT[2]; h[0]=0.; h[1]=0.;
            s=new PLFLT[2]; s[0]=0.; s[1]=0.;
            l=new PLFLT[2]; l[0]=1.; l[1]=0.;
            
            i=new PLFLT[2]; i[0]=0.; i[1]=1.;
            
            plscmap1l(0, 2, i, h, l, s, NULL);
            
            delete[] h;
            delete[] s;
            delete[] l;
            delete[] i;
            break;
            
        case QPUI_INV_RED_BLUE:
            h=new PLFLT[4]; h[0]=230.; h[1]=230.; h[2]=-10.; h[3]=-10.;
            s=new PLFLT[4]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.;
            l=new PLFLT[4]; l[0]=.9; l[1]=.2; l[2]=.2; l[3]=.9;
            
            i=new PLFLT[4]; i[0]=0.; i[1]=0.499; i[2]=0.5; i[3]=1.;
            
            plscmap1l(0, 4, i, h, l, s, NULL);
            
            delete[] h;
            delete[] s;
            delete[] l;
            delete[] i;

            break;
            
        case QPUI_RED_BLUE:
            h=new PLFLT[4]; h[0]=230.; h[1]=230.; h[2]=-10.; h[3]=-10.;
            s=new PLFLT[4]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.;
            l=new PLFLT[4]; l[0]=.25; l[1]=.9; l[2]=.9; l[3]=.25;
            
            i=new PLFLT[4]; i[0]=0.; i[1]=0.499; i[2]=0.5; i[3]=1.;
            
            plscmap1l(0, 4, i, h, l, s, NULL);
            
            delete[] h;
            delete[] s;
            delete[] l;
            delete[] i;
            
            break;
            
        case QPUI_EARTH:
            h=new PLFLT[2]; h[0]=50.; h[1]=50.;
            s=new PLFLT[2]; s[0]=0.2; s[1]=1.;
            l=new PLFLT[2]; l[0]=0.9; l[1]=0.;
            
            i=new PLFLT[2]; i[0]=0.; i[1]=1.;
            
            plscmap1l(0, 2, i, h, l, s, NULL);
            
            delete[] h;
            delete[] s;
            delete[] l;
            delete[] i;
            break;
	    
	    case QPUI_USER_TABLE:
		    str=QSAS_profiles->current()->ColourTable;
#if QT_VERSION < 0x040000
		    str=str.stripWhiteSpace();
#else
		    str=str.trimmed();
#endif
		    bool isRGB;
		    if(str.isEmpty())
		    {
			    set_colour_table(QPUI_RAINBOW);
			    return;
		    }
#if QT_VERSION < 0x040000
		    sl=QStringList::split(QRegExp("\\(|\\)"), str, false);
#else
		    sl=str.split(QRegExp("\\(|\\)"), QString::SkipEmptyParts);
#endif
		    if(sl.size()<3)
		    {
			    set_colour_table(QPUI_RAINBOW);
			    return;
		    }
#if QT_VERSION < 0x040000
		    if(sl[0].contains("RGB", false))
		    {
			    isRGB=true;
		    }
		    else if(sl[0].contains("HSV", false))
		    {
			    isRGB=false;
		    }
#else
		    if(sl[0].contains("RGB", Qt::CaseInsensitive))
		    {
			    isRGB=true;
		    }
		    else if(sl[0].contains("HSV", Qt::CaseInsensitive))
		    {
			    isRGB=false;
		    }
#endif
		    else
		    {
			    set_colour_table(QPUI_RAINBOW);
			    return;
		    }
		    npoints=sl.size()-1;
		    h=new PLFLT[npoints];
		    s=new PLFLT[npoints];
		    l=new PLFLT[npoints];
		    i=new PLFLT[npoints];
		    for(int index=0; index<npoints; ++index)
		    {
#if QT_VERSION < 0x040000
			    sl2=QStringList::split(",", (sl[index+1]), false);
#else
			    sl2=(sl[index+1]).split(",",  QString::SkipEmptyParts, Qt::CaseInsensitive);
#endif
			    if(sl2.size()!=4)
			    {
				h[index]=0.; s[index]=0.; l[index]=0.; i[index]=0.;
			    }
			    else
			    {
				i[index]=sl2[0].toDouble();
				h[index]=sl2[1].toDouble();
				if(isRGB)
				{
					l[index]=sl2[2].toDouble();
					s[index]=sl2[3].toDouble();
				}
				else
				{
					s[index]=sl2[2].toDouble();
					l[index]=sl2[3].toDouble();
				}
			    }
		    }
		    plscmap1l(isRGB, npoints, i, h, l, s, NULL);
            
		    delete[] h;
		    delete[] s;
		    delete[] l;
		    delete[] i;
		    break;
	    
	case QPUI_RAINBOW:
	    default:
		    h=new PLFLT[2]; h[0]=300.; h[1]=-10.;
		    s=new PLFLT[2]; s[0]=1.; s[1]=1.;
		    l=new PLFLT[2]; l[0]=.5; l[1]=.5;
            
		    i=new PLFLT[2]; i[0]=0.; i[1]=1.;
            
		    plscmap1l(0, 2, i, h, l, s, NULL);
            
		    delete[] h;
		    delete[] s;
		    delete[] l;
		    delete[] i;
            
		    break;
    }
}

// -----------------

void QpPageItem::setLayout()
{
	// does the auto-layout for a page. Ignored for User layout.
	// update layout for page
	if(QuSlotDebug())
	{
		printf("QpPageItem::setLayout %ld\n", this);
		fflush(stdout);
	}

	if( !(LayoutTab->_autolayout_fld->isChecked()) ) return;
	
	double _tmargin = LayoutTab->_tmargin_fld->value();
	double _lmargin = LayoutTab->_lmargin_fld->value();
	double _rmargin = LayoutTab->_rmargin_fld->value();
	double _bmargin = LayoutTab->_bmargin_fld->value();
	double _hgap = LayoutTab->_hgap_fld->value();
	double _vgap = LayoutTab->_vgap_fld->value();
	
	// calculate number of rows and columns
	int num_rows = LayoutTab->_row_num_fld->value();
	int num_cols = LayoutTab->_col_num_fld->value();
	// get widths of panels 
	float frameWidth = 1. - (_lmargin + _rmargin + _hgap*(num_cols - 1));
	frameWidth /= num_cols;
	
	int cnt = 0;
	int col = 0;
	int row = 0;
	float height = 0.;
	float width = 0.;
	
	// calculate left offest and top offset for frames in page
	
	double frameLoffset = _lmargin;
	double frameToffset = _tmargin;

	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(this->firstChild());
	while(frameItem)
	{
	
		// fetch number of panels in this frame
		int numPanels = frameItem->numberInStack();
		numPanels = (numPanels>1)?numPanels:1;
		
		// calculate height of this frame so that all panels are equal height
		float frameHeight = 1. - (_tmargin + _bmargin + _vgap*(num_rows - 1));
		frameHeight /= panelsInColumn(col);
		frameHeight *= numPanels;
		
		// set panel loffset, width, toffset, height
		frameItem->setPanelProperties(frameLoffset, frameWidth, frameToffset, frameHeight);
		
		int rowLast = row;
		int colLast = col;
		cnt++;
		col = (int)cnt  / num_rows;
		row = cnt - col * num_rows;
		
		// add widths of stack panels to loffset
		if(col > colLast) frameLoffset += frameWidth + _hgap;
		else if(col == 0)  frameLoffset = _lmargin; // reset for next row
		
		// add height of stack panel to toffset
		if(row > rowLast) frameToffset += frameHeight + _vgap;
        else if(row ==0) frameToffset = _tmargin;
		
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
		

	}
}

// -----------------------------------
	
int QpPageItem::panelsInColumn(int colno)
{
	// number of panels for all frames in column
	if(QuSlotDebug())
	{
		printf("QpPageItem::panelsInColumn %ld\n", this);
		fflush(stdout);
	}

  	int cnt = 0;
	int row = 0;
	int col = 0;
	int numPanels = 0;
	int num_rows =  LayoutTab->_row_num_fld->value();
	
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(this->firstChild());
	while (frameItem)
	{
		col = (int)cnt  / num_rows;
		row = cnt - col * num_rows;
		cnt++;
		if(colno == col) 
		{	
			// increment by number of panels in this frame
			numPanels += frameItem->numberOfChildren();
		}
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
	return (numPanels>0)?numPanels:1;
}


// -----------------------------------
	
void QpPageItem::setFramesAuto(bool active)
{
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(this->firstChild());
	while(frameItem)
	{
		if(active)
		{
			LayoutTab->updateRowCol(this->numberOfChildren());
			frameItem->PropertiesTab->_autolayout_lbl->setText("Autoset by Page Layout ");
		}
		else
		{
			frameItem->PropertiesTab->_autolayout_lbl->setText(" Page Layout not Autoset");
		}
		frameItem->PropertiesTab->_loffset_fld->setDisabled(active);
		frameItem->PropertiesTab->_toffset_fld->setDisabled(active);
		frameItem->PropertiesTab->_width_fld->setDisabled(active);
		frameItem->PropertiesTab->_height_fld->setDisabled(active);
		frameItem->PropertiesTab->_aspect_ratio_fld->setDisabled(active);
			
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
	
	LayoutTab->_hgap_fld->setEnabled(active);
	LayoutTab->_vgap_fld->setEnabled(active);
	LayoutTab->_lmargin_fld->setEnabled(active);
	LayoutTab->_rmargin_fld->setEnabled(active);
	LayoutTab->_tmargin_fld->setEnabled(active);
	LayoutTab->_bmargin_fld->setEnabled(active);
}
// -------------------------------

bool QpPageItem::containsSpectrum()
{
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(this->firstChild());
	while(frameItem)
	{
		if(frameItem->containsSpectrum()) return true;
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
	return false;
}

// -------------------------------

void QpPageItem::displayIntervalWidget(double x, double y, int pos_x, int pos_y)
{
	PlotWindow* win=(PlotWindow *) sender();
	if(!win) return;
	
	// Search if clicked inside a frame
	
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(firstChild());
	QpFrameItem * selectedFrame=0;
	int nPanel=0;
	while(frameItem && !selectedFrame)
	{
		int numPanels = frameItem->numberOfChildren();
		bool hasSpectrum = containsSpectrum();
		double factor=(hasSpectrum?0.9:1.);
			if(frameItem->PropertiesTab->_loffset_fld->value() <= x &&  
						x <= frameItem->PropertiesTab->_loffset_fld->value() + frameItem->PropertiesTab->_width_fld->value()*factor && 
						frameItem->PropertiesTab->_toffset_fld->value()<= y &&
						y <= frameItem->PropertiesTab->_toffset_fld->value()+ frameItem->PropertiesTab->_height_fld->value())
			{
				selectedFrame=frameItem;
			}
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
	
	if(!selectedFrame)
	{
		// No selected frame, return
		return;
	}
	
	if(selectedFrame->frameType()==QP_TS)
	{
		double tstart = selectedFrame->tAxisTab->_axis_from_time_fld->value();
		double tend = selectedFrame->tAxisTab->_axis_to_time_fld->value();
		
		// look for min and max for all the objects in the frame
		
		double objtmin=tstart;
		double objtmax=tend;
		
		// For all panel
		QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(selectedFrame->firstChild());
		while(panelItem)
		{
			// For all object
			QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(panelItem->firstChild());
			while(plotItem)
			{
				double t1, t2;
				if(plotItem->getXmetaRange(t1, t2))
				{
					if(objtmin>t1) objtmin=t1;
					if(objtmax<t2) objtmax=t2;
				}
				plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
			}
			panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
		}

		// now set up widget

		//disconnect(win, SIGNAL(rescale(double, double)));
		win->disconnect(SIGNAL(rescale(double, double)));
		win->intervalDialog->setAsTimeInterval(objtmin, objtmax, tstart, tend);
		
		connect(win, SIGNAL(rescale(double, double)), selectedFrame->tAxisTab, SLOT(interactiveRescale(double, double)));
		
#if QT_VERSION < 0x040000
		win->intervalDialog->setCaption(selectedFrame->TitleTab->_title_fld->text());
#else
		win->intervalDialog->setWindowTitle(selectedFrame->TitleTab->_title_fld->text());
#endif
		win->intervalDialog->move(win->x()+win->plot->x()+pos_x-win->intervalDialog->width()/2,win->y()+win->plot->y()+pos_y);
#if QT_VERSION < 0x040000
		win->intervalDialog->show();
#else
		win->intervalDialog->setVisible(true);
#endif
	}
	else if(selectedFrame->frameType()==QP_XY)
	{
		double xstart = selectedFrame->xAxisTab->_axis_from_xy_fld->text().toDouble();
		double xend = selectedFrame->xAxisTab->_axis_to_xy_fld->text().toDouble();
		// look for min and max for all the objects in the frame
		
		double objxmin=xstart;
		double objxmax=xend;
		
		// For all panel
		QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(selectedFrame->firstChild());
		while(panelItem)
		{
			// For all object
			QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(panelItem->firstChild());
			while(plotItem)
			{
				double t1, t2;
				if(plotItem->getXmetaRange(t1, t2))
				{
					if(objxmin>t1) objxmin=t1;
					if(objxmax<t2) objxmax=t2;
				}
				plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
			}
			panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
		}

		// now set up widget

		//disconnect(win, SIGNAL(rescale(double, double)));
		win->disconnect(SIGNAL(rescale(double, double)));
		win->intervalDialog->setAsXInterval(objxmin, objxmax, xstart, xend);

		connect(win, SIGNAL(rescale(double, double)), selectedFrame->xAxisTab, SLOT(interactiveRescale(double, double)));
		
#if QT_VERSION < 0x040000
		win->intervalDialog->setCaption(selectedFrame->TitleTab->_title_fld->text());
#else
		win->intervalDialog->setWindowTitle(selectedFrame->TitleTab->_title_fld->text());
#endif
		win->intervalDialog->move(win->x()+win->plot->x()+pos_x-win->intervalDialog->width()/2,win->y()+win->plot->y()+pos_y);
		
#if QT_VERSION < 0x040000
		win->intervalDialog->show();
#else
		win->intervalDialog->setVisible(true);
#endif
	}
}

// -------------------------------

Vistream& operator>>(Vistream& v_is, QpPageItem& page)
{
	if(QuSlotDebug()) cout <<  "QpPageItem restore (>>)" << endl;

	// streams from virtual input stream v_is into item
	QpListItem& plot_item = page;
	v_is >> plot_item;

	// read in page specifics
	
	int _page_size;
	v_is >> _page_size;
	page.PageTab->_page_size_fld->setCurrentItem(_page_size);
	
	double _width;
	v_is >> _width;
	double _height;
	v_is >> _height;
	int _units;
	v_is >> _units;
	page.PageTab->_units_fld->setCurrentItem(_units);
	page.PageTab->oldUnits = _units;
	
	bool readonly = (_page_size != QPUI_CUSTOM);
	page.PageTab->_width_fld->setReadOnly(readonly);
	page.PageTab->_height_fld->setReadOnly(readonly);
	page.PageTab->_width_fld->setEnabled(!readonly);
	page.PageTab->_height_fld->setEnabled(!readonly);
	
	int _dpi;
	v_is >> _dpi;
	page.PageTab->_dpi_fld->setCurrentItem(_dpi);

	switch(_dpi)
	{
		case QPUI_150_DPI:
			page._conversion_factors[QPUI_PIXELS]=150.;
			page._conversion_factors[QPUI_PSPOINTS]=150.;
			break;
			
		case QPUI_300_DPI:
			page._conversion_factors[QPUI_PIXELS]=300.;
			page._conversion_factors[QPUI_PSPOINTS]=300.;
			break;
			
		case QPUI_600_DPI:
			page._conversion_factors[QPUI_PIXELS]=600.;
			page._conversion_factors[QPUI_PSPOINTS]=600.;
			break;
			
		default:
			page._conversion_factors[QPUI_PIXELS]=72.;
			page._conversion_factors[QPUI_PSPOINTS]=72.;
			break;
	}
	
	double conv = page._conversion_factors[_units];
	_width *= conv;
	page.PageTab->_width_fld->setText(double_to_string(_width).c_str());
	_height *= conv;
	page.PageTab->_height_fld->setText(double_to_string(_height).c_str());
	
	int _orientation;
	v_is >> _orientation;
	page.PageTab->_orientation_fld->setCurrentItem(_orientation);

	int _xwin_num;
	v_is >> _xwin_num;
	page.PageTab->_xwin_num_fld->setText(int_to_string(_xwin_num).c_str());
	
	string temp_name;
	v_is >> temp_name;
	page.name=QString(temp_name.c_str());
	
	v_is >> page._filename;
	bool _show_ident;
	v_is >> _show_ident;
	page.PageTab->_show_ident_fld->setChecked(_show_ident);

	int _row_num;
	v_is >> _row_num; // set after frames read in
	
	int _col_num;
	v_is >> _col_num; // set after frames read in
	
	bool _autolayout;
	v_is >> _autolayout;
	page.LayoutTab->_autolayout_fld->setChecked(_autolayout);

	float _hgap;
	v_is >> _hgap;
	page.LayoutTab->_hgap_fld->setValue(_hgap);

	float _vgap;
	v_is >> _vgap;
	page.LayoutTab->_vgap_fld->setValue(_vgap);

	float _lmargin;
	v_is >> _lmargin;
	page.LayoutTab->_lmargin_fld->setValue(_lmargin);
	
	float _rmargin;
	v_is >> _rmargin;
	page.LayoutTab->_rmargin_fld->setValue(_rmargin);
	
	float _tmargin;
	v_is >> _tmargin;
	page.LayoutTab->_tmargin_fld->setValue(_tmargin);
	
	float _bmargin;
	v_is >> _bmargin;
	page.LayoutTab->_bmargin_fld->setValue(_bmargin);


	// read in frames
	int cnt;
	v_is >> cnt;

	for(int i=0; i<cnt; i++)
	{
		QpFrameItem *frameItem = new QpFrameItem(&page, "", (QpFrameItem*)page.lastChild());
		v_is >> *frameItem;
	}
	
	page.LayoutTab->_row_num_fld->setValue(_row_num);
	page.LayoutTab->_col_num_fld->setValue(_col_num);
	page.setFramesAuto(_autolayout);
	
	return v_is;
}

// -----------------------------------------------

Vostream& operator<<(Vostream& v_os, QpPageItem& page)
{
	if(QuSlotDebug()) cout << "QpPageItem save (<<)" << endl;

	// streams item to virtual output stream v_os

	QpListItem& plot_item = page;
	v_os << plot_item;

	// page Details

	int _units = page.PageTab->_units_fld->currentItem();
	double conv = page._conversion_factors[_units];
	
	v_os << (int) page.PageTab->_page_size_fld->currentItem();
	
	double _width = string_to_double( page.PageTab->_width_fld->text().latin1() );
	_width /= conv;
	v_os << _width;
	
	double _height = string_to_double( page.PageTab->_height_fld->text().latin1() );
	_height /= conv;
	v_os << _height;
	v_os << _units;
	
	v_os << (int) page.PageTab->_dpi_fld->currentItem();
	
	v_os << (int) page.PageTab->_orientation_fld->currentItem();
	
	int _xwin_num = string_to_int( page.PageTab->_xwin_num_fld->text().latin1() );
	v_os << _xwin_num;
	v_os << string(page.name.latin1());
	
	v_os << page._filename;
	
	bool _show_ident = page.PageTab->_show_ident_fld->isChecked();
	v_os << _show_ident;
	
	v_os << (int) page.LayoutTab->_row_num_fld->value();
	v_os << (int) page.LayoutTab->_col_num_fld->value();
	bool _autolayout = page.LayoutTab->_autolayout_fld->isChecked();
	v_os << _autolayout;
	v_os << (float) page.LayoutTab->_hgap_fld->value();
	v_os << (float) page.LayoutTab->_vgap_fld->value();

	v_os << (float) page.LayoutTab->_lmargin_fld->value();
	v_os << (float) page.LayoutTab->_rmargin_fld->value();
	v_os << (float) page.LayoutTab->_tmargin_fld->value();
	v_os << (float) page.LayoutTab->_bmargin_fld->value();

	// Frames in page

	int cnt = page.numberOfChildren();
	v_os << cnt;
   
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(page.firstChild());
	while(frameItem)
	{
		v_os << *frameItem;
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}

	return v_os;
}


