// qpui_page_options.cc
//
// Plot Interface Implementation - Page Options 
//
// Janet Barnes
// July 2002
//
//  Tony Allen
// February 2008

#include "qpui.h"
#include "qdutil.h"
#include <qtooltip.h>

//  QpLayoutTab Class
// ------------------

#if QT_VERSION < 0x040000
QpLayoutTab::QpLayoutTab(QTabWidget *tabWidget,  QString name, QpPageItem * _page):
    QpTab(tabWidget, name, QP_LAYOUT_TAB)

{
	// Constructor for the Layout Tab widget shown in the Page Options

	I_Page = _page;
	
	QVBoxLayout *page_layout_top_layout = new QVBoxLayout(this);
	page_layout_top_layout->setSpacing(4);

	QGroupBox *page_panels_groupbox = new QGroupBox(1, Qt::Vertical, this, "page_panels_groupbox");

	page_panels_groupbox->setTitle("Frame Layout");

	QWidget* page_panels_widget = new QWidget(page_panels_groupbox, "page_panels_widget");

	QGridLayout *page_panels_layout = new QGridLayout(page_panels_widget, 3, 5);
	page_panels_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_panels_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* row_num_label = new QLabel("Number of rows: ", page_panels_widget, "row_num_label");
	page_panels_layout->addWidget(row_num_label, 0, 0, Qt::AlignRight);
	_row_num_fld = new QSpinBox(page_panels_widget, "row_num");
	_row_num_fld->setMinValue(1);
	_row_num_fld->setFixedWidth(100);
	page_panels_layout->addWidget(_row_num_fld, 0, 1, Qt::AlignLeft);

	connect(_row_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumRows(int)));

	QLabel* col_num_label = new QLabel("Number of columns: ", page_panels_widget, "col_num_label");
	page_panels_layout->addWidget(col_num_label, 0, 2, Qt::AlignRight);
	_col_num_fld = new QSpinBox(page_panels_widget, "col_num");
	_col_num_fld->setMinValue(1);
	_col_num_fld->setFixedWidth(100);
	page_panels_layout->addWidget(_col_num_fld, 0, 3, Qt::AlignLeft);

	connect(_col_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumCols(int)));

	_autolayout_fld = new QCheckBox("Autolayout Frames", page_panels_widget, "autolayout_fld");
	page_panels_layout->addMultiCellWidget(_autolayout_fld, 1, 1, 0, 1);
	_autolayout_fld->setChecked(true);
	
	connect( _autolayout_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoLayout(bool)) );

	page_panels_layout->addRowSpacing(0,0);
	page_panels_layout->addRowSpacing(1, _row_num_fld->height());
	page_panels_layout->addRowSpacing(2,0);

	page_panels_layout->setRowStretch(0,0);
	page_panels_layout->setRowStretch(1,1);
	page_panels_layout->setRowStretch(2,0);

	page_panels_layout->setColStretch(0,0);
	page_panels_layout->setColStretch(1,0);
	page_panels_layout->setColStretch(2,0);
	page_panels_layout->setColStretch(3,0);
	page_panels_layout->setColStretch(4,1);

	page_layout_top_layout->addWidget(page_panels_groupbox);

	QGroupBox *page_margins_groupbox = new QGroupBox(1, Qt::Vertical, this, "page_margins_groupbox");

	page_margins_groupbox->setTitle("Frame Margins for Autolayout");

	QWidget* page_margins_widget = new QWidget(page_margins_groupbox, "page_margins_widget");

	QGridLayout *page_margins_layout = new QGridLayout(page_margins_widget, 7, 5);
	page_margins_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_margins_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* hgap_label = new QLabel("Horizontal gap: ", page_margins_widget, "hgap_label");
	page_margins_layout->addWidget(hgap_label, 1, 0, Qt::AlignRight);
	_hgap_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_widget, "hgap");
	_hgap_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_hgap_fld, 1, 1, Qt::AlignLeft);
	_hgap_fld->setValue(QPUI_PAGE_HGAP);
	
	connect(_hgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setHGap(double)));

	QLabel* vgap_label = new QLabel("Vertical gap: ", page_margins_widget, "vgap_label");
	page_margins_layout->addWidget(vgap_label, 1, 2, Qt::AlignRight);
	_vgap_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_widget, "vgap");
	_vgap_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_vgap_fld, 1, 3, Qt::AlignLeft);
	_vgap_fld->setValue(QPUI_PAGE_VGAP);

	connect(_vgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setVGap(double)));

	QLabel* lmargin_label = new QLabel("Left margin: ", page_margins_widget, "lmargin_label");
	page_margins_layout->addWidget(lmargin_label, 3, 0, Qt::AlignRight);
	_lmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_widget, "lmargin");
	_lmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_lmargin_fld, 3, 1, Qt::AlignLeft);
	_lmargin_fld->setValue(QPUI_PAGE_LMARGIN);

	connect(_lmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setLMargin(double)));

	QLabel* rmargin_label = new QLabel("Right margin: ", page_margins_widget, "rmargin_label");
	page_margins_layout->addWidget(rmargin_label, 3, 2, Qt::AlignRight);
	_rmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_widget, "rmargin");
	_rmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_rmargin_fld, 3, 3, Qt::AlignLeft);
	_rmargin_fld->setValue(QPUI_PAGE_RMARGIN);

	connect(_rmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setRMargin(double)));

	QLabel* tmargin_label = new QLabel("Top margin: ", page_margins_widget, "tmargin_label");
	page_margins_layout->addWidget(tmargin_label, 5, 0, Qt::AlignRight);
	_tmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_widget, "tmargin");
	_tmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_tmargin_fld, 5, 1, Qt::AlignLeft);
	_tmargin_fld->setValue(QPUI_PAGE_TMARGIN);

	connect(_tmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setTMargin(double)));

	QLabel* bmargin_label = new QLabel("Bottom margin: ", page_margins_widget, "bmargin_label");
	page_margins_layout->addWidget(bmargin_label, 5, 2, Qt::AlignRight);
	_bmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_widget, "bmargin");
	_bmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_bmargin_fld, 5, 3, Qt::AlignLeft);
	_bmargin_fld->setValue(QPUI_PAGE_BMARGIN);

	connect(_bmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setBMargin(double)));

	page_margins_layout->addRowSpacing(0,0);
	page_margins_layout->addRowSpacing(1, _hgap_fld->height());
	page_margins_layout->addRowSpacing(2,0);
	page_margins_layout->addRowSpacing(3, _lmargin_fld->height());
	page_margins_layout->addRowSpacing(4,0);
	page_margins_layout->addRowSpacing(5, _tmargin_fld->height());
	page_margins_layout->addRowSpacing(6,0);

	page_margins_layout->setRowStretch(0,0);
	page_margins_layout->setRowStretch(1,1);
	page_margins_layout->setRowStretch(2,0);
	page_margins_layout->setRowStretch(3,1);
	page_margins_layout->setRowStretch(4,0);
	page_margins_layout->setRowStretch(5,1);
	page_margins_layout->setRowStretch(6,0);

	page_margins_layout->setColStretch(0,0);
	page_margins_layout->setColStretch(1,0);
	page_margins_layout->setColStretch(2,0);
	page_margins_layout->setColStretch(3,0);
	page_margins_layout->setColStretch(4,1);

	page_layout_top_layout->addWidget(page_margins_groupbox);

	page_layout_top_layout->addSpacing(4);
	page_layout_top_layout->addStretch(10);

	page_panels_widget->adjustSize();
}
#else
QpLayoutTab::QpLayoutTab(QTabWidget *tabWidget,  QString name, QpPageItem * _page):
		QpTab(tabWidget, name, QP_LAYOUT_TAB)

{
	// Constructor for the Layout Tab widget shown in the Page Options

	I_Page = _page;
	
	QVBoxLayout *page_layout_top_layout = new QVBoxLayout(this);
	page_layout_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_layout_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *page_panels_groupbox = new QGroupBox(this);

	page_panels_groupbox->setTitle("Frame Layout");

// 	QWidget* page_panels_widget = new QWidget(page_panels_groupbox, "page_panels_widget");

	QGridLayout *page_panels_layout = new QGridLayout;
	page_panels_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_panels_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* row_num_label = new QLabel("Number of rows: ");
	page_panels_layout->addWidget(row_num_label, 0, 0, Qt::AlignRight);
	_row_num_fld = new QSpinBox;
	_row_num_fld->setMinValue(1);
	_row_num_fld->setFixedWidth(100);
	page_panels_layout->addWidget(_row_num_fld, 0, 1, Qt::AlignLeft);

	connect(_row_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumRows(int)));

	QLabel* col_num_label = new QLabel("Number of columns: ");
	page_panels_layout->addWidget(col_num_label, 0, 2, Qt::AlignRight);
	_col_num_fld = new QSpinBox;
	_col_num_fld->setMinValue(1);
	_col_num_fld->setFixedWidth(100);
	page_panels_layout->addWidget(_col_num_fld, 0, 3, Qt::AlignLeft);

	connect(_col_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumCols(int)));

	_autolayout_fld = new QCheckBox("Autolayout Frames");
	page_panels_layout->addMultiCellWidget(_autolayout_fld, 1, 1, 0, 1);
	_autolayout_fld->setChecked(true);
	
	connect( _autolayout_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoLayout(bool)) );

// 	page_panels_layout->addRowSpacing(0,0);
// 	page_panels_layout->addRowSpacing(1, _row_num_fld->height());
// 	page_panels_layout->addRowSpacing(2,0);

	page_panels_layout->setRowStretch(0,0);
	page_panels_layout->setRowStretch(1,1);
	page_panels_layout->setRowStretch(2,0);

	page_panels_layout->setColStretch(0,0);
	page_panels_layout->setColStretch(1,0);
	page_panels_layout->setColStretch(2,0);
	page_panels_layout->setColStretch(3,0);
	page_panels_layout->setColStretch(4,1);

	page_panels_groupbox->setLayout(page_panels_layout);
	
	page_layout_top_layout->addWidget(page_panels_groupbox);

	QGroupBox *page_margins_groupbox = new QGroupBox(this);

	page_margins_groupbox->setTitle("Frame Margins for Autolayout");

// 	QWidget* page_margins_widget = new QWidget(page_margins_groupbox, "page_margins_widget");

	QGridLayout *page_margins_layout = new QGridLayout;
	page_margins_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_margins_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* hgap_label = new QLabel("Horizontal gap: ");
	page_margins_layout->addWidget(hgap_label, 1, 0, Qt::AlignRight);
	_hgap_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "hgap");
	_hgap_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_hgap_fld, 1, 1, Qt::AlignLeft);
	_hgap_fld->setValue(QPUI_PAGE_HGAP);
	
	connect(_hgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setHGap(double)));

	QLabel* vgap_label = new QLabel("Vertical gap: ");
	page_margins_layout->addWidget(vgap_label, 1, 2, Qt::AlignRight);
	_vgap_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "vgap");
	_vgap_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_vgap_fld, 1, 3, Qt::AlignLeft);
	_vgap_fld->setValue(QPUI_PAGE_VGAP);

	connect(_vgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setVGap(double)));

	QLabel* lmargin_label = new QLabel("Left margin: ", page_margins_groupbox, "lmargin_label");
	page_margins_layout->addWidget(lmargin_label, 3, 0, Qt::AlignRight);
	_lmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "lmargin");
	_lmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_lmargin_fld, 3, 1, Qt::AlignLeft);
	_lmargin_fld->setValue(QPUI_PAGE_LMARGIN);

	connect(_lmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setLMargin(double)));

	QLabel* rmargin_label = new QLabel("Right margin: ");
	page_margins_layout->addWidget(rmargin_label, 3, 2, Qt::AlignRight);
	_rmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "rmargin");
	_rmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_rmargin_fld, 3, 3, Qt::AlignLeft);
	_rmargin_fld->setValue(QPUI_PAGE_RMARGIN);

	connect(_rmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setRMargin(double)));

	QLabel* tmargin_label = new QLabel("Top margin: ", page_margins_groupbox, "tmargin_label");
	page_margins_layout->addWidget(tmargin_label, 5, 0, Qt::AlignRight);
	_tmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "tmargin");
	_tmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_tmargin_fld, 5, 1, Qt::AlignLeft);
	_tmargin_fld->setValue(QPUI_PAGE_TMARGIN);

	connect(_tmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setTMargin(double)));

	QLabel* bmargin_label = new QLabel("Bottom margin: ", page_margins_groupbox, "bmargin_label");
	page_margins_layout->addWidget(bmargin_label, 5, 2, Qt::AlignRight);
	_bmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "bmargin");
	_bmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_bmargin_fld, 5, 3, Qt::AlignLeft);
	_bmargin_fld->setValue(QPUI_PAGE_BMARGIN);

	connect(_bmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setBMargin(double)));

// 	page_margins_layout->addRowSpacing(0,0);
// 	page_margins_layout->addRowSpacing(1, _hgap_fld->height());
// 	page_margins_layout->addRowSpacing(2,0);
// 	page_margins_layout->addRowSpacing(3, _lmargin_fld->height());
// 	page_margins_layout->addRowSpacing(4,0);
// 	page_margins_layout->addRowSpacing(5, _tmargin_fld->height());
// 	page_margins_layout->addRowSpacing(6,0);

	page_margins_layout->setRowStretch(0,0);
	page_margins_layout->setRowStretch(1,1);
	page_margins_layout->setRowStretch(2,0);
	page_margins_layout->setRowStretch(3,1);
	page_margins_layout->setRowStretch(4,0);
	page_margins_layout->setRowStretch(5,1);
	page_margins_layout->setRowStretch(6,0);

	page_margins_layout->setColStretch(0,0);
	page_margins_layout->setColStretch(1,0);
	page_margins_layout->setColStretch(2,0);
	page_margins_layout->setColStretch(3,0);
	page_margins_layout->setColStretch(4,1);

	page_margins_groupbox->setLayout(page_margins_layout);
	
	page_layout_top_layout->addWidget(page_margins_groupbox);

	page_layout_top_layout->addSpacing(4);
	page_layout_top_layout->addStretch(10);

// 	page_panels_widget->adjustSize();
}

#endif

// ----------------------------------------------

void QpLayoutTab::setAutoLayout(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   I_Page->setFramesAuto(bChoice);
   
	
	// propogate to other selected pages if this is the one user modified
	if(this->isVisible() ) I_Page->I_ListView->setPageOption(AUTOLAYOUT, 0, 0.0, "", bChoice);

}

// ----------------------------------------------

void QpLayoutTab::updateRowCol(int numFrames)
{
	if(QuSlotDebug())
	{
		printf("QpLayoutTab::updateRowCol\n");
		fflush(stdout);
	}
	
	if( numFrames <= 0) return;

	// called when Frame created or destroyed
	int row_num = (int) ceil((float) numFrames / _col_num_fld->value());
	_row_num_fld->clearFocus();
	_row_num_fld->setValue(row_num); 

    I_Page->setLayout();
	
}

void QpLayoutTab::setNumRows(int row_num)
{
	int col_num = (int) ceil((float) I_Page->numberOfChildren() / row_num);
	_col_num_fld->clearFocus();
	_col_num_fld->setValue(col_num); 

    I_Page->setLayout();
	
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(_row_num_fld->hasFocus() ) I_Page->I_ListView->setPageOption(NUM_ROWS, row_num);
}

void QpLayoutTab::setNumCols(int col_num)
{
	int row_num = (int) ceil((float) I_Page->numberOfChildren()  / col_num);
	_row_num_fld->clearFocus();
	_row_num_fld->setValue(row_num); 

    I_Page->setLayout();
	
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(_col_num_fld->hasFocus() ) I_Page->I_ListView->setPageOption(NUM_COLS, col_num);
}

void QpLayoutTab::setHGap(double gap)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	I_Page->setLayout();

	if(_hgap_fld->hasFocus() ) I_Page->I_ListView->setPageOption(HGAP, 0, gap);
}

void QpLayoutTab::setVGap(double gap)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
      
	I_Page->setLayout();

	if(_vgap_fld->hasFocus() ) I_Page->I_ListView->setPageOption(VGAP, 0, gap);
}

void QpLayoutTab::setLMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	I_Page->setLayout();

	if(_lmargin_fld->hasFocus() ) I_Page->I_ListView->setPageOption(LMARGIN, 0, margin);
}

void QpLayoutTab::setRMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	I_Page->setLayout();

	if(_rmargin_fld->hasFocus() ) I_Page->I_ListView->setPageOption(RMARGIN, 0, margin);
}

void QpLayoutTab::setTMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	I_Page->setLayout();

	if(_tmargin_fld->hasFocus() ) I_Page->I_ListView->setPageOption(TMARGIN, 0, margin);
}

void QpLayoutTab::setBMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	I_Page->setLayout();

	if(_bmargin_fld->hasFocus() ) I_Page->I_ListView->setPageOption(TMARGIN, 0, margin);
}

   

// QpPageTab Class
//----------------

#if QT_VERSION < 0x040000
QpPageTab::QpPageTab(QTabWidget *tabWidget,  QString name, QpPageItem * _page):
    QpTab(tabWidget, name, QP_PAGE_TAB)
{
	// Constructor for the Page Tab widget shown in the Page Options

	I_Page = _page;
  
	QVBoxLayout *page_properties_top_layout=new QVBoxLayout(this);
	page_properties_top_layout->setSpacing(4);

	QGroupBox *page_size_groupbox = new QGroupBox(1, Qt::Vertical, this, "page_size_groupbox");
	page_size_groupbox->setTitle("Size");

	QWidget* page_size_widget = new QWidget(page_size_groupbox, "page_size_widget");
	QGridLayout *page_size_layout = new QGridLayout(page_size_widget, 7, 5);
	page_size_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_size_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* page_size_label = new QLabel("Page size: ", page_size_widget, "page_size_label");
	page_size_layout->addWidget(page_size_label, 1, 0, Qt::AlignRight);
	_page_size_fld = new QComboBox(false, page_size_widget, "page_size");
	_page_size_fld->insertItem("A4", QPUI_A4);
	_page_size_fld->insertItem("Custom", QPUI_CUSTOM);
	_page_size_fld->insertItem("Executive", QPUI_EXECUTIVE);
	_page_size_fld->insertItem("Legal", QPUI_LEGAL);
	_page_size_fld->insertItem("Letter", QPUI_LETTER);
	_page_size_fld->insertItem("Default", QPUI_DEFAULT);

	QFontMetrics page_size_fm(_page_size_fld->font());
	int page_size_w = page_size_fm.width("Executive");
	int page_size_h = page_size_fm.height();
	_page_size_fld->setMinimumSize(page_size_w, page_size_h);
#if QT_VERSION < 0x040000
	_page_size_fld->listBox()->setMinimumSize(page_size_w, page_size_h);
#else
        _page_size_fld->setMinimumSize(page_size_w, page_size_h);
#endif
	QToolTip::add( _page_size_fld, "Select page size");
	page_size_layout->addWidget(_page_size_fld, 1, 1, Qt::AlignLeft);

	connect( _page_size_fld, SIGNAL(activated(int)), this, SLOT(setPageSize(int)) );
	
	if(QSAS_profiles->current()->PageSize=="A4")
		_page_size_fld->setCurrentItem(QPUI_A4);
	else if(QSAS_profiles->current()->PageSize=="Executive")
		_page_size_fld->setCurrentItem(QPUI_EXECUTIVE);
	else if(QSAS_profiles->current()->PageSize=="Legal")
		_page_size_fld->setCurrentItem(QPUI_LEGAL);
	else if(QSAS_profiles->current()->PageSize=="Letter")
		_page_size_fld->setCurrentItem(QPUI_LETTER);
	else if(QSAS_profiles->current()->PageSize=="Default")
		_page_size_fld->setCurrentItem(QPUI_DEFAULT);
	else _page_size_fld->setCurrentItem(QPUI_CUSTOM);
	
	QLabel* units_label = new QLabel("Units: ", page_size_widget, "units_label");
	page_size_layout->addWidget(units_label, 1, 2, Qt::AlignRight);
	_units_fld = new QComboBox(false, page_size_widget, "units");
	_units_fld->insertItem("pixels", QPUI_PIXELS);
	_units_fld->insertItem("inches", QPUI_INCHES);
	_units_fld->insertItem("cm", QPUI_CM);
	_units_fld->insertItem("mm", QPUI_MM);
	_units_fld->insertItem("PS points", QPUI_PSPOINTS);
	QFontMetrics units_fm(_units_fld->font());
	int units_w = units_fm.width("PS points");
	int units_h = page_size_fm.height();
	_units_fld->setMinimumSize(units_w, units_h);
#if QT_VERSION < 0x040000
	_units_fld->listBox()->setMinimumSize(units_w, units_h);
#else
        _units_fld->setMinimumSize(units_w, units_h);
#endif
	QToolTip::add( _units_fld, "Select units for page size");
	page_size_layout->addWidget(_units_fld, 1, 3, Qt::AlignLeft);

	connect( _units_fld, SIGNAL(activated(int)), this, SLOT(setUnits(int)) );
	oldUnits = _units_fld->currentItem();
	
	QLabel* dpi_label = new QLabel("Resolution: ", page_size_widget, "dpi_label");
	page_size_layout->addWidget(dpi_label, 1, 4, Qt::AlignRight);
	_dpi_fld=new QComboBox(false, page_size_widget, "dpi");
	_dpi_fld->insertItem("72 dpi", QPUI_72_DPI);
	_dpi_fld->insertItem("150 dpi", QPUI_150_DPI);
	_dpi_fld->insertItem("300 dpi", QPUI_300_DPI);
	_dpi_fld->insertItem("600 dpi", QPUI_600_DPI);
	switch(QSAS_profiles->current()->PageDPI)
	{
		case 150:
			I_Page->_conversion_factors[QPUI_PIXELS]=150.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=150.;
			_dpi_fld->setCurrentItem(QPUI_150_DPI);
			break;
		case 300:
			I_Page->_conversion_factors[QPUI_PIXELS]=300.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=300.;
			_dpi_fld->setCurrentItem(QPUI_300_DPI);
			break;
		case 600:
			I_Page->_conversion_factors[QPUI_PIXELS]=600.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=600.;
			_dpi_fld->setCurrentItem(QPUI_600_DPI);
			break;
		default:
			I_Page->_conversion_factors[QPUI_PIXELS]=72.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=72.;
			_dpi_fld->setCurrentItem(QPUI_72_DPI);
			break;
	}
	page_size_layout->addWidget(_dpi_fld, 1, 5, Qt::AlignLeft);
	connect(_dpi_fld, SIGNAL(activated(int)), this, SLOT(setDPI(int)));
	
	QLabel* width_label = new QLabel("Width: ", page_size_widget, "width_label");
	page_size_layout->addWidget(width_label, 3, 0, Qt::AlignRight);
	_width_fld = new QLineEdit(page_size_widget, "width");
	_width_fld->setFixedWidth(100);
	_width_fld->setValidator(new QDoubleValidator(_width_fld));

	page_size_layout->addWidget(_width_fld, 3, 1, Qt::AlignLeft);

	connect( _width_fld, SIGNAL(textChanged(const QString&)),this, SLOT(setWidth(const QString&)) );

	QLabel* height_label = new QLabel("Height: ", page_size_widget, "height_label");
	page_size_layout->addWidget(height_label, 3, 2, Qt::AlignRight);
	_height_fld = new QLineEdit(page_size_widget, "height");
	_height_fld->setFixedWidth(100);
	_height_fld->setValidator(new QDoubleValidator(_height_fld));
	page_size_layout->addWidget(_height_fld, 3, 3, Qt::AlignLeft);
  
	connect( _height_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setHeight(const QString&)) );
	
	
	// set appropriate text in width and height fields
	if(QSAS_profiles->current()->PageSize=="Custom")
	{
		double conv = I_Page->_conversion_factors[_units_fld->currentItem()];
		double w=QSAS_profiles->current()->PageWidth;
		double h=QSAS_profiles->current()->PageHeight;
		_width_fld->setText(QString().setNum(w*conv));
		_height_fld->setText(QString().setNum(h*conv));
	}
	setPageSize(_page_size_fld->currentItem());

	QLabel* orientation_label = new QLabel("Orientation: ", page_size_widget, "orientation_label");
	page_size_layout->addWidget(orientation_label, 5, 0, Qt::AlignRight);
	_orientation_fld = new QComboBox(false, page_size_widget, "orientation");
	_orientation_fld->insertItem("Portrait", QPUI_PORTRAIT);
	_orientation_fld->insertItem("Landscape", QPUI_LANDSCAPE);
	QFontMetrics orientation_fm(_orientation_fld->font());
	int orientation_w = orientation_fm.width("Landscape");
	int orientation_h = orientation_fm.height();
	_orientation_fld->setMinimumSize(orientation_w, orientation_h);
#if QT_VERSION < 0x040000
	_orientation_fld->listBox()->setMinimumSize(orientation_w, orientation_h);
#else
        _orientation_fld->setMinimumSize(orientation_w, orientation_h);
#endif
	QToolTip::add( _orientation_fld, "Select portrait or landscape");
	page_size_layout->addWidget(_orientation_fld, 5, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageOrientation=="Portrait")
		_orientation_fld->setCurrentItem(QPUI_PORTRAIT);
	else _orientation_fld->setCurrentItem(QPUI_LANDSCAPE);
	
	connect(_orientation_fld, SIGNAL(activated(int)), this, SLOT(setOrientation(int)) );

	page_size_layout->addRowSpacing(0,0);
	page_size_layout->addRowSpacing(1, _page_size_fld->height());
	page_size_layout->addRowSpacing(2,0);
	page_size_layout->addRowSpacing(3, _width_fld->height());
	page_size_layout->addRowSpacing(4,0);
	page_size_layout->addRowSpacing(5, _orientation_fld->height());
	page_size_layout->addRowSpacing(6,0);

	page_size_layout->setRowStretch(0,0);
	page_size_layout->setRowStretch(1,1);
	page_size_layout->setRowStretch(2,0);
	page_size_layout->setRowStretch(3,1);
	page_size_layout->setRowStretch(4,0);
	page_size_layout->setRowStretch(5,1);
	page_size_layout->setRowStretch(6,0);

	page_size_layout->setColStretch(0,0);
	page_size_layout->setColStretch(1,0);
	page_size_layout->setColStretch(2,0);
	page_size_layout->setColStretch(3,0);
	page_size_layout->setColStretch(4,0);
	page_size_layout->setColStretch(5,0);
	page_size_layout->setColStretch(6,1);

	page_properties_top_layout->addWidget(page_size_groupbox);

	QGroupBox *page_output_groupbox = new QGroupBox(1, Qt::Vertical, this, "page_output_groupbox");

	page_output_groupbox->setTitle("Output");

	QWidget* page_output_widget = new QWidget(page_output_groupbox, "page_output_widget");

	QGridLayout *page_output_layout = new QGridLayout(page_output_widget, 5, 5);
	page_output_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_output_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* xwin_num_label = new QLabel("Plot window id (-1 for auto): ", page_output_widget, "xwin_num_label");
	page_output_layout->addWidget(xwin_num_label,1,0,Qt::AlignRight);
	_xwin_num_fld = new QLineEdit(page_output_widget, "xwin_num");
	_xwin_num_fld->setFixedWidth(50);
	_xwin_num_fld->setValidator(new QIntValidator(_xwin_num_fld));
	_xwin_num_fld->setText("-1");
	page_output_layout->addWidget(_xwin_num_fld, 1, 1, Qt::AlignLeft);

	connect( _xwin_num_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setXwinNum(const QString&)) );

	_show_ident_fld = new QCheckBox("Show user identification", page_output_widget, "show_ident_fld");
	_show_ident_fld->setChecked(true);
	page_output_layout->addMultiCellWidget(_show_ident_fld, 3, 3, 0, 1);
        
	connect( _show_ident_fld, SIGNAL(toggled(bool)), this, SLOT(setIdent(bool)) );

	page_output_layout->addRowSpacing(0,0);
	page_output_layout->addRowSpacing(1, _xwin_num_fld->height());
	page_output_layout->addRowSpacing(2,0);
	page_output_layout->addRowSpacing(3,5);
	page_output_layout->addRowSpacing(4,0);

	page_output_layout->setRowStretch(0,0);
	page_output_layout->setRowStretch(1,1);
	page_output_layout->setRowStretch(2,0);
	page_output_layout->setRowStretch(3,1);
	page_output_layout->setRowStretch(4,0);

	page_output_layout->setColStretch(0,0);
	page_output_layout->setColStretch(1,0);
	page_output_layout->setColStretch(2,0);
	page_output_layout->setColStretch(3,0);
	page_output_layout->setColStretch(4,1);

	page_properties_top_layout->addWidget(page_output_groupbox);

	page_properties_top_layout->addSpacing(4);
	page_properties_top_layout->addStretch(10);

	page_size_widget->adjustSize();
 }
#else
 QpPageTab::QpPageTab(QTabWidget *tabWidget,  QString name, QpPageItem * _page):
QpTab(tabWidget, name, QP_PAGE_TAB)
{
	// Constructor for the Page Tab widget shown in the Page Options

	I_Page = _page;

	QVBoxLayout *page_properties_top_layout=new QVBoxLayout(this);
	page_properties_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_properties_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *page_size_groupbox = new QGroupBox(this);
	page_size_groupbox->setTitle("Size");

// 	QWidget* page_size_widget = new QWidget(page_size_groupbox, "page_size_widget");
	QGridLayout *page_size_layout = new QGridLayout;
	page_size_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_size_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* page_size_label = new QLabel("Page size: ");
	page_size_layout->addWidget(page_size_label, 1, 0, Qt::AlignRight);
	_page_size_fld = new QComboBox(false);
	_page_size_fld->insertItem("A4", QPUI_A4);
	_page_size_fld->insertItem("Custom", QPUI_CUSTOM);
	_page_size_fld->insertItem("Executive", QPUI_EXECUTIVE);
	_page_size_fld->insertItem("Legal", QPUI_LEGAL);
	_page_size_fld->insertItem("Letter", QPUI_LETTER);
	_page_size_fld->insertItem("Default", QPUI_DEFAULT);

	QFontMetrics page_size_fm(_page_size_fld->font());
	int page_size_w = page_size_fm.width("Executive");
	int page_size_h = page_size_fm.height();
	_page_size_fld->setMinimumSize(page_size_w, page_size_h);

	QToolTip::add( _page_size_fld, "Select page size");
	page_size_layout->addWidget(_page_size_fld, 1, 1, Qt::AlignLeft);

	connect( _page_size_fld, SIGNAL(activated(int)), this, SLOT(setPageSize(int)) );
	
	if(QSAS_profiles->current()->PageSize=="A4")
		_page_size_fld->setCurrentItem(QPUI_A4);
	else if(QSAS_profiles->current()->PageSize=="Executive")
		_page_size_fld->setCurrentItem(QPUI_EXECUTIVE);
	else if(QSAS_profiles->current()->PageSize=="Legal")
		_page_size_fld->setCurrentItem(QPUI_LEGAL);
	else if(QSAS_profiles->current()->PageSize=="Letter")
		_page_size_fld->setCurrentItem(QPUI_LETTER);
	else if(QSAS_profiles->current()->PageSize=="Default")
		_page_size_fld->setCurrentItem(QPUI_DEFAULT);
	else _page_size_fld->setCurrentItem(QPUI_CUSTOM);
	
	QLabel* units_label = new QLabel("Units: ");
	page_size_layout->addWidget(units_label, 1, 2, Qt::AlignRight);
	_units_fld = new QComboBox(false);
	_units_fld->insertItem("pixels", QPUI_PIXELS);
	_units_fld->insertItem("inches", QPUI_INCHES);
	_units_fld->insertItem("cm", QPUI_CM);
	_units_fld->insertItem("mm", QPUI_MM);
	_units_fld->insertItem("PS points", QPUI_PSPOINTS);
	QFontMetrics units_fm(_units_fld->font());
	int units_w = units_fm.width("PS points");
	int units_h = page_size_fm.height();
	_units_fld->setMinimumSize(units_w, units_h);

	QToolTip::add( _units_fld, "Select units for page size");
	page_size_layout->addWidget(_units_fld, 1, 3, Qt::AlignLeft);

	connect( _units_fld, SIGNAL(activated(int)), this, SLOT(setUnits(int)) );
	oldUnits = _units_fld->currentItem();
	
	QLabel* dpi_label = new QLabel("Resolution: ");
	page_size_layout->addWidget(dpi_label, 1, 4, Qt::AlignRight);
	_dpi_fld=new QComboBox(false);
	_dpi_fld->insertItem("72 dpi", QPUI_72_DPI);
	_dpi_fld->insertItem("150 dpi", QPUI_150_DPI);
	_dpi_fld->insertItem("300 dpi", QPUI_300_DPI);
	_dpi_fld->insertItem("600 dpi", QPUI_600_DPI);
	switch(QSAS_profiles->current()->PageDPI)
	{
		case 150:
			I_Page->_conversion_factors[QPUI_PIXELS]=150.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=150.;
			_dpi_fld->setCurrentItem(QPUI_150_DPI);
			break;
		case 300:
			I_Page->_conversion_factors[QPUI_PIXELS]=300.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=300.;
			_dpi_fld->setCurrentItem(QPUI_300_DPI);
			break;
		case 600:
			I_Page->_conversion_factors[QPUI_PIXELS]=600.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=600.;
			_dpi_fld->setCurrentItem(QPUI_600_DPI);
			break;
		default:
			I_Page->_conversion_factors[QPUI_PIXELS]=72.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=72.;
			_dpi_fld->setCurrentItem(QPUI_72_DPI);
			break;
	}
	page_size_layout->addWidget(_dpi_fld, 1, 5, Qt::AlignLeft);
	connect(_dpi_fld, SIGNAL(activated(int)), this, SLOT(setDPI(int)));
	
	QLabel* width_label = new QLabel("Width: ");
	page_size_layout->addWidget(width_label, 3, 0, Qt::AlignRight);
	_width_fld = new QLineEdit;
	_width_fld->setFixedWidth(100);
	_width_fld->setValidator(new QDoubleValidator(_width_fld));

	page_size_layout->addWidget(_width_fld, 3, 1, Qt::AlignLeft);

	connect( _width_fld, SIGNAL(textChanged(const QString&)),this, SLOT(setWidth(const QString&)) );

	QLabel* height_label = new QLabel("Height: ");
	page_size_layout->addWidget(height_label, 3, 2, Qt::AlignRight);
	_height_fld = new QLineEdit;
	_height_fld->setFixedWidth(100);
	_height_fld->setValidator(new QDoubleValidator(_height_fld));
	page_size_layout->addWidget(_height_fld, 3, 3, Qt::AlignLeft);
  
	connect( _height_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setHeight(const QString&)) );
	
	
	// set appropriate text in width and height fields
	if(QSAS_profiles->current()->PageSize=="Custom")
	{
		double conv = I_Page->_conversion_factors[_units_fld->currentItem()];
		double w=QSAS_profiles->current()->PageWidth;
		double h=QSAS_profiles->current()->PageHeight;
		_width_fld->setText(QString().setNum(w*conv));
		_height_fld->setText(QString().setNum(h*conv));
	}
	setPageSize(_page_size_fld->currentItem());

	QLabel* orientation_label = new QLabel("Orientation: ");
	page_size_layout->addWidget(orientation_label, 5, 0, Qt::AlignRight);
	_orientation_fld = new QComboBox(false);
	_orientation_fld->insertItem("Portrait", QPUI_PORTRAIT);
	_orientation_fld->insertItem("Landscape", QPUI_LANDSCAPE);
	QFontMetrics orientation_fm(_orientation_fld->font());
	int orientation_w = orientation_fm.width("Landscape");
	int orientation_h = orientation_fm.height();
	_orientation_fld->setMinimumSize(orientation_w, orientation_h);

	QToolTip::add( _orientation_fld, "Select portrait or landscape");
	page_size_layout->addWidget(_orientation_fld, 5, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageOrientation=="Portrait")
		_orientation_fld->setCurrentItem(QPUI_PORTRAIT);
	else _orientation_fld->setCurrentItem(QPUI_LANDSCAPE);
	
	connect(_orientation_fld, SIGNAL(activated(int)), this, SLOT(setOrientation(int)) );

// 	page_size_layout->addRowSpacing(0,0);
// 	page_size_layout->addRowSpacing(1, _page_size_fld->height());
// 	page_size_layout->addRowSpacing(2,0);
// 	page_size_layout->addRowSpacing(3, _width_fld->height());
// 	page_size_layout->addRowSpacing(4,0);
// 	page_size_layout->addRowSpacing(5, _orientation_fld->height());
// 	page_size_layout->addRowSpacing(6,0);

	page_size_layout->setRowStretch(0,0);
	page_size_layout->setRowStretch(1,1);
	page_size_layout->setRowStretch(2,0);
	page_size_layout->setRowStretch(3,1);
	page_size_layout->setRowStretch(4,0);
	page_size_layout->setRowStretch(5,1);
	page_size_layout->setRowStretch(6,0);

	page_size_layout->setColStretch(0,0);
	page_size_layout->setColStretch(1,0);
	page_size_layout->setColStretch(2,0);
	page_size_layout->setColStretch(3,0);
	page_size_layout->setColStretch(4,0);
	page_size_layout->setColStretch(5,0);
	page_size_layout->setColStretch(6,1);
	
	page_size_groupbox->setLayout(page_size_layout);

	page_properties_top_layout->addWidget(page_size_groupbox);

	QGroupBox *page_output_groupbox = new QGroupBox(this);

	page_output_groupbox->setTitle("Output");

// 	QWidget* page_output_widget = new QWidget(page_output_groupbox, "page_output_widget");

	QGridLayout *page_output_layout = new QGridLayout;
	page_output_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_output_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* xwin_num_label = new QLabel("Plot window id (-1 for auto): ");
	page_output_layout->addWidget(xwin_num_label,1,0,Qt::AlignRight);
	_xwin_num_fld = new QLineEdit;
	_xwin_num_fld->setFixedWidth(50);
	_xwin_num_fld->setValidator(new QIntValidator(_xwin_num_fld));
	_xwin_num_fld->setText("-1");
	page_output_layout->addWidget(_xwin_num_fld, 1, 1, Qt::AlignLeft);

	connect( _xwin_num_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setXwinNum(const QString&)) );

	_show_ident_fld = new QCheckBox("Show user identification");
	_show_ident_fld->setChecked(true);
	page_output_layout->addMultiCellWidget(_show_ident_fld, 3, 3, 0, 1);
        
	connect( _show_ident_fld, SIGNAL(toggled(bool)), this, SLOT(setIdent(bool)) );

// 	page_output_layout->addRowSpacing(0,0);
// 	page_output_layout->addRowSpacing(1, _xwin_num_fld->height());
// 	page_output_layout->addRowSpacing(2,0);
// 	page_output_layout->addRowSpacing(3,5);
// 	page_output_layout->addRowSpacing(4,0);

	page_output_layout->setRowStretch(0,0);
	page_output_layout->setRowStretch(1,1);
	page_output_layout->setRowStretch(2,0);
	page_output_layout->setRowStretch(3,1);
	page_output_layout->setRowStretch(4,0);

	page_output_layout->setColStretch(0,0);
	page_output_layout->setColStretch(1,0);
	page_output_layout->setColStretch(2,0);
	page_output_layout->setColStretch(3,0);
	page_output_layout->setColStretch(4,1);
	
	page_output_groupbox->setLayout(page_output_layout);

	page_properties_top_layout->addWidget(page_output_groupbox);

	page_properties_top_layout->addSpacing(4);
	page_properties_top_layout->addStretch(10);

// 	page_size_widget->adjustSize();
}
#endif

void QpPageTab::setPageSize(int page_size)
{
	bool readonly = (page_size != QPUI_CUSTOM);
	double conv=I_Page->_conversion_factors[_units_fld->currentItem()];

	if(readonly)
	{
		double width = I_Page->_width_in_inches[page_size];
		_width_fld->setText(double_to_string(width * conv).c_str());
		double height = I_Page->_height_in_inches[page_size];
		_height_fld->setText(double_to_string(height * conv).c_str());
		
	}
	_width_fld->setReadOnly(readonly);
	_height_fld->setReadOnly(readonly);
	_width_fld->setEnabled(!readonly);
	_height_fld->setEnabled(!readonly);
      
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Page->I_ListView->setPageOption(PAGE_SIZE, page_size);
}

void QpPageTab::setUnits(int units)
{
	double conv = I_Page->_conversion_factors[units];
	double oldConv = I_Page->_conversion_factors[oldUnits];
	conv /= oldConv;
	oldUnits = units;
	
	double width = string_to_double(_width_fld->text().latin1());
	_width_fld->setText(double_to_string(width * conv).c_str());
	double height = atof(_height_fld->text().latin1());
	_height_fld->setText(double_to_string(height * conv).c_str());
     
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Page->I_ListView->setPageOption(PAGE_UNITS, units);
}

void QpPageTab::setDPI(int dpi_index)
{
	double old_dpi=I_Page->_conversion_factors[_units_fld->currentItem()];

	switch(dpi_index)
	{
		case QPUI_150_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=150.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=150.;
			break;
			
		case QPUI_300_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=300.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=300.;
			break;
			
		case QPUI_600_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=600.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=600.;
			break;
			
		default:
			I_Page->_conversion_factors[QPUI_PIXELS]=72.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=72.;
			break;
	}

	double new_dpi=I_Page->_conversion_factors[_units_fld->currentItem()];
	
	double ratio=new_dpi/old_dpi;
	
	if(ratio!=1.)
	{
		double width = string_to_double(_width_fld->text().latin1());
		double height = atof(_height_fld->text().latin1());
		_width_fld->setText(double_to_string(width * ratio).c_str());
		_height_fld->setText(double_to_string(height * ratio).c_str());
	}
	
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Page->I_ListView->setPageOption(PAGE_DPI, dpi_index);
}

void QpPageTab::setWidth(const QString &value)
{   
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
  
	if(_width_fld->hasFocus() ) I_Page->I_ListView->setPageOption(PAGE_WIDTH, 0, 0.0, value);

}

void QpPageTab::setHeight(const QString &value)
{   
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
  
	if(_height_fld->hasFocus() ) I_Page->I_ListView->setPageOption(PAGE_HEIGHT, 0, 0.0, value);

}

void QpPageTab::setOrientation(int iChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Page->I_ListView->setPageOption(PAGE_ORIENTATION, iChoice);
}

void QpPageTab::setXwinNum(const QString &value)
{   
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
  
	if(_xwin_num_fld->hasFocus() ) I_Page->I_ListView->setPageOption(XWIN_NUM, 0, 0.0, value);

}

void QpPageTab::setIdent(bool bChoice)
{   
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
  
	if( this->isVisible() ) I_Page->I_ListView->setPageOption(IDENT, 0, 0.0, "", bChoice);

}


