
// routines to restore QSAS version 2.2.xx plot savesets
#include "qpui.h"
#if QT_VERSION < 0x040000
#include "qgu.h"
#else
#include "qgu_qt4.h"
#endif
#include "Qdos.h"
#include "qwui.h"
#include "TimeUtils.h"

// Globals to old restore only

   int QPUI_release_version_num;
   int QPUI_major_version_num;
   int QPUI_minor_version_num;
   int QPUI_sub_version_num;
   float charScale = 0.7;
   float lineScale = 0.4;
   
typedef enum QpuiLviType_e
{
     QPUI_LVI_PAGE,
     QPUI_LVI_PANEL,
     QPUI_LVI_XYTRACE,
     QPUI_LVI_XYSTACK,
     QPUI_LVI_TSTRACE,
     QPUI_LVI_TSSTACK,
     QPUI_LVI_SPECTROGRAM,
     QPUI_LVI_CONST_TTRACE,
     QPUI_LVI_CONST_YTRACE,
     QPUI_LVI_CONST_XTRACE,
     QPUI_LVI_XYSPECTROGRAM,
     QPUI_LVI_UNKNOWN,
     QPUI_LVI_XYPANEL,
     QPUI_LVI_TSPANEL,
     QPUI_LVI_XYEVENT,
     QPUI_LVI_TSEVENT
} QpuiLviType;

typedef enum QpuiPlotType_e
{
  QPUI_XYTRACE=0,
  QPUI_TSTRACE=1,
  QPUI_SPECTROGRAM=2,
  QPUI_XYSPECTROGRAM=3,
  QPUI_CONST_TTRACE=4,
  QPUI_CONST_YTRACE=5,
  QPUI_CONST_XTRACE=6,
  QPUI_XYSTACK=7,
  QPUI_TSSTACK=8,
  QPUI_UNKNOWN=9,
  QPUI_XYEVENT=10,
  QPUI_TSEVENT=11
} QpuiPlotType;

void QpWindow::restoreV22(Vistream& v_is, bool replace)
{
	// restore

	string guiID;
	v_is >> guiID;
	// read in guiID and version no
	size_t spos=guiID.find("QSAS");
	size_t epos=guiID.find("qpui");

	//if( guiID != "QSASqpui" )
	if(string::npos==spos || string::npos==epos)
	{
		cout << guiID << "Sorry -  restore from QSAS V 2.2 file failed" << endl;
		return;
	}

	QPUI_release_version_num=0;
	QPUI_major_version_num=0;
	QPUI_minor_version_num=0;
	QPUI_sub_version_num=0;

	spos+=4;
	if(epos>spos)
	{
		string version_str=guiID.substr(spos,epos-spos);
		string release_str(version_str);
		size_t pos=version_str.find('.');
		if(pos!=string::npos)
		{
			release_str=version_str.substr(0,pos);
			version_str=version_str.substr(pos+1);
		}
		QPUI_release_version_num=string_to_int(release_str);
		string major_str(version_str);
		pos=version_str.find('.');
		if(pos!=string::npos)
		{
			major_str=version_str.substr(0,pos);
			version_str=version_str.substr(pos+1);
		}
		QPUI_major_version_num=string_to_int(major_str);
		string minor_str(version_str);
		QPUI_minor_version_num=string_to_int(minor_str);
	}
	if(epos!=string::npos)
	{
		string sub_version_str=guiID.substr(epos+4);
		try{
			QPUI_sub_version_num=string_to_int(sub_version_str);
		}
		catch(...){;}
	}

	try{
		QguWindow::_restored_objects->unbind(Name("qpui"));
	}
	catch(NamingException&){;}
	
	QguWindow::_restored_objects->bind_new_context(Name("qpui"));
	
    if(replace==true) _plot_listview->clear();
	
	_plot_listview->blockSignals(true);
	restoreListview(v_is, _plot_listview);
	_plot_listview->blockSignals(false);
	
	_plot_listview->clearSelectedList();
	_plot_listview->setCurrentItem(_plot_listview->firstChild());
	_plot_listview->updateOnSelection();

}

//------------------ 


void restoreListview(Vistream& v_is, QpListView *lv)
{
	// streams from virtual input stream v_is into item

	int cnt;
	v_is >> cnt;

	for(int i=0;i<cnt;i++)
	{
		QString pageName = QString("Page %1").arg(i);
		QpPageItem *page=new QpPageItem(lv, pageName.latin1(), dynamic_cast<QpPageItem*>(lv->lastChild()));

		restorePageItem( v_is, page);
		
		QString name = ((Q_LIST_VIEW_ITEM*)page)->text(NAME_COL);
		while(page->nameAlreadyUsed(name)) name += "A";
		page->setText(NAME_COL, name);
	}
	lv->clearSelectedList();
	return;
}

//------------------ 

void restoreItem(Vistream& v_is, QpListItem* item)
{
	// generic plot item

	QguListItem& qgu_item = *item;
	v_is >> qgu_item;

	string I_name;
	v_is >> I_name;

//	if(I_name != "") item->setText(NAME_COL, I_name.c_str());

   return;
}

//------------------ 

void restorePageItem(Vistream& v_is, QpPageItem* page)
{
	// streams from virtual input stream v_is into item

	restoreItem(v_is, (QpListItem*) page);
	
	int _page_size;
	v_is >> _page_size;
	page->PageTab->_page_size_fld->setCurrentItem(_page_size);
	
	double _width;
	v_is >> _width;
	page->PageTab->_width_fld->setText(double_to_string(_width).c_str());
	
	double _height;
	v_is >> _height;
	page->PageTab->_height_fld->setText(double_to_string(_height).c_str());
	
	int _units;
	v_is >> _units;
	page->PageTab->_units_fld->setCurrentItem(_units);

	int _orientation;
	v_is >> _orientation;
	page->PageTab->_orientation_fld->setCurrentItem(_orientation);

	int _plot_device;
	v_is >> _plot_device;
	
	int _xwin_num;
	v_is >> _xwin_num;
	
	string _filename;
	v_is >> _filename;
	
	bool _show_ident;
	v_is >> _show_ident;
	page->PageTab->_show_ident_fld->setChecked(_show_ident);
	
	int _row_num;
	v_is >> _row_num;
	page->LayoutTab->_row_num_fld->setValue(_row_num);
	
	int _col_num;
	v_is >> _col_num;
	page->LayoutTab->_col_num_fld->setValue(_col_num);
	
	float _hgap;
	v_is >> _hgap;
	page->LayoutTab->_hgap_fld->setValue(_hgap);

	float _vgap;
	v_is >> _vgap;
	page->LayoutTab->_vgap_fld->setValue(_vgap);
	
	float _lmargin;
	v_is >> _lmargin;
	page->LayoutTab->_lmargin_fld->setValue(_lmargin);

	float _rmargin;
	v_is >> _rmargin;
	page->LayoutTab->_rmargin_fld->setValue(_rmargin);

	float _tmargin;
	v_is >> _tmargin;
	page->LayoutTab->_tmargin_fld->setValue(_tmargin);

	float _bmargin;
	v_is >> _bmargin;
	page->LayoutTab->_bmargin_fld->setValue(_bmargin);

	int cnt;
	v_is >> cnt;

	for(int i=0; i<cnt; i++)
	{
		int pt=1;
		if(QPUI_release_version_num>2 ||
			(QPUI_release_version_num==2 && (QPUI_major_version_num>1)) ||
			(QPUI_release_version_num==2 && (QPUI_major_version_num==1) &&
			(QPUI_minor_version_num>=11)))
		{
			v_is >> pt;
		}
		QpuiLviType panelType=(QpuiLviType)pt;
		
		int pos=0;
		switch(panelType)
		{
			case QPUI_LVI_PANEL: default:
			{
				pos=v_is.getpos();
				
				QpFrameItem *frame = new QpFrameItem(page, "Frame ", (QpFrameItem*)page->lastChild(), QP_TS);
				if( restoreTSFrameItem(v_is, frame) == false) 
				{
					frame->_frameType = QP_XY;
					v_is.setpos(pos);
					restoreXYFrameItem( v_is,  frame) ;
				}
				break;
			}
			
			case QPUI_LVI_XYPANEL:
			{
				QpFrameItem *frame = new QpFrameItem(page, "Frame ", (QpFrameItem*)page->lastChild(), QP_XY);
				restoreXYFrameItem( v_is,  frame) ;
				break;
			}
			case QPUI_LVI_TSPANEL:
			{
				QpFrameItem *frame = new QpFrameItem(page, "Frame ", (QpFrameItem*)page->lastChild(), QP_TS);
				restoreTSFrameItem( v_is,  frame) ;
				break;
			}      
		}
	}
	return;
}


// ----------------------

bool restoreTSFrameItem(Vistream& v_is, QpFrameItem *frame)
{
	// streams from virtual input stream v_is into item
   	
	restoreItem(v_is, (QpListItem*) frame);

	// create one panel for now
	QpPanelItem *panel = new QpPanelItem(frame, "Sub-Panel ", (QpPanelItem*)frame->lastChild());
	
	restoreFrameTabs(v_is, frame);

	// this restores axis and other stuff which needs to be distributed
	restorePanelItem(v_is, frame, panel);

	int cnt;
	v_is >> cnt;
	
	QpuiLviType plotType;
	
	QString nullStr="";
	
	int nStack=0;
	for(int i=0; i<cnt; i++)
	{
		int pt=0;
		v_is >> pt;
		plotType=(QpuiLviType)pt;
		
		switch(plotType)
		{
			case QPUI_LVI_TSSTACK:
			{
				QpPanelItem *nextPanel;
				// one panel already exists
				if(nStack == 0) nextPanel = panel;
				else nextPanel = new QpPanelItem(frame, "Sub-Panel ", (QpPanelItem*)frame->lastChild());
				nStack++;
				
				restoreTSstack(v_is, frame, nextPanel);
				break;
			}

			case QPUI_LVI_TSTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "Trace ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_TRACE);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_CONST_TTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "T Trace ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_XT);
				restoreTrace(v_is, trace);
				break;
			}
		
			case QPUI_LVI_CONST_YTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "Y Trace ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_Y);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_SPECTROGRAM:
			{
				QpSpectrumItem *spectrum= new QpSpectrumItem(panel, "Spectrum ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "");
				restoreSpectroGram(v_is, spectrum);
				break;
			}
			
			case QPUI_LVI_TSEVENT:
			{
				QpEventItem* event = new QpEventItem(panel, "Event Label ", dynamic_cast<QpPlotItem*>(panel->lastChild()));
				restoreEvent(v_is, event);
				break;
			}

			default:
				return false;

		}
	}
	return true;
}

// ----------------------

void restoreXYFrameItem(Vistream& v_is, QpFrameItem *frame)
{
	// streams from virtual input stream v_is into item
	
	restoreItem(v_is, (QpListItem*) frame);

	// create one panel for now
	QpPanelItem *panel = new QpPanelItem(frame, "", (QpPanelItem*)frame->lastChild());
	
	
	restoreFrameTabs(v_is, frame);

	// this restores axis and other stuff which needs to be distributed
	restorePanelItem(v_is, frame, panel);

	int cnt;
	v_is >> cnt;
	
	QpuiLviType plotType;
  
	int nStack=0;
	for(int i=0;i<cnt;i++)
	{
		int pt=0;
		v_is >> pt;
		plotType = (QpuiLviType)pt;
		
		switch(plotType)
		{
			case QPUI_LVI_XYSTACK:
			{
				QpPanelItem *nextPanel;
				// one panel already exists
				if(nStack == 0) nextPanel = panel;
				else nextPanel = new QpPanelItem(frame, "Panel ", (QpPanelItem*)frame->lastChild());
				nStack++;
				
				restoreXYstack(v_is, frame, nextPanel);
				break;

			}
			
			case QPUI_LVI_XYTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "Trace ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_TRACE);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_CONST_YTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "constY Trace ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_Y);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_CONST_XTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "constX Trace ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_XT);
				restoreTrace(v_is, trace);
				break;
			}
          
			case QPUI_LVI_XYSPECTROGRAM:
			{
				QpSpectrumItem *spectrum= new QpSpectrumItem(panel, "Spectrum ", dynamic_cast<QpPlotItem*>(panel->lastChild()), "");
				restoreSpectroGram(v_is, spectrum);
				break;
			}

			case QPUI_LVI_XYEVENT:
			{
				QpEventItem* event = new QpEventItem(panel, "Event Label ", dynamic_cast<QpPlotItem*>(panel->lastChild()));
				restoreEvent(v_is, event);
				break;
			}
			
			default:
				return;
		}
	}
	return;
}

// ----------------------

void restorePanelItem(Vistream& v_is, QpFrameItem *frame, QpPanelItem *panel)
{
	// streams from virtual input stream v_is into item

	bool isXY =  (frame->frameType() == QP_XY); 
	
	restoreAxis(v_is, frame->xtAxisTab, false, isXY);
	
	restoreAxis(v_is, panel->yAxisTab, true, isXY);

	restoreAxis(v_is, frame->xtAxisTab, false, isXY);

	if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 && (QPUI_major_version_num>=1)))
	{
       restoreLegend(v_is, panel);
	}

}

// ----------------------

void restoreAxis(Vistream& v_is, QpAxisTab *axis, bool isY, bool isXY)
{
  // streams from virtual input stream v_is into axis object
	bool keep=true;
	if(axis == 0) keep = false;
	bool _defined;
	v_is>>_defined;

	bool autoscale;
	v_is >> autoscale;
	if(keep && autoscale) axis->_axis_autoscale_fld->setCurrentItem(QP_FROM_DATA);
	if(keep && !autoscale) axis->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
	
	bool autolabel;
	v_is >> autolabel;
	if(keep) axis->_axis_autolabel_fld->setChecked(autolabel);

	string _data;
	v_is >> _data;
	
	double from;
	v_is >> from;
	if(keep)
	{
		if(isXY  || isY)
		{

			axis->_axis_from_xy_fld->setText(double_to_string(from).c_str());
		}
		else
		{
			try{
				Time t;
				t.set_time_rep(from);
				axis->_axis_from_time_fld->setTime(t);
			}catch(Exception& e){ std::cerr << "Failed to restore axis T FROM field" << std::endl; }
		}
	}

	double to;
	v_is >> to;
	if(keep)
	{
		if(isXY || isY)
		{
			axis->_axis_to_xy_fld->setText(double_to_string(to).c_str());
		}
		else
		{
			try
			{
				Time t;
				t.set_time_rep(to);
				axis->_axis_to_time_fld->setTime(t);
			} catch(Exception& e){ std::cerr << "Failed to restore axis T TO field" << std::endl; }
		}
	}

	int linlog;
	v_is >> linlog;
	if(keep) axis->_axis_linlog_fld->setCurrentItem(linlog);

	int majorNTicks;
	v_is >> majorNTicks;
	if(keep) axis->_axis_major_ticks_num_fld->setValue(majorNTicks);
	
	int minorNTicks;
	v_is >> minorNTicks;
	if(keep) axis->_axis_minor_ticks_num_fld->setValue(minorNTicks);
	
	int majorTicks;
	v_is >> majorTicks;
	if(keep) axis->_axis_major_tickmarks_fld->setCurrentItem(majorTicks);
	
	int minorTicks;
	v_is >> minorTicks;
	if(keep) axis->_axis_minor_tickmarks_fld->setCurrentItem(minorTicks);
	
	int _tick_labels;
	v_is >> _tick_labels;
	
	string label;
	v_is >> label;
	if(keep) axis->_label_fld->setText(label.c_str());

	// appears to be duplicated
	v_is >> autolabel;
	if(keep) axis->_axis_autolabel_fld->setChecked(autolabel);

	if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 && QPUI_major_version_num>0))
	{
		float charSize;
		v_is >> charSize;
		charSize *= charScale;
		if(keep) axis->_label_char_size_fld->setValue(charSize);
	
		float disp;
		v_is >> disp;
		disp += 1.2*charSize;
		if(keep) axis->_label_disp_fld->setValue(disp);
	
		float rotn;
		v_is >> rotn;
		if(keep) axis->_label_rotation_fld->setValue(rotn);
	
		int font;
		v_is >> font;
		if(keep)
		{
			switch(font)
			{
				case 1: // Roman
					axis->_label_font_fld->setSerifFont();
					axis->_label_font_fld->setItalic(false);
					axis->_label_font_fld->setBold(false);
					break;
					
				case 2: // Italic
					axis->_label_font_fld->setSerifFont();
					axis->_label_font_fld->setItalic(true);
					axis->_label_font_fld->setBold(false);
					break;
					
				default: // Normal
					axis->_label_font_fld->setSansFont();
					axis->_label_font_fld->setItalic(false);
					axis->_label_font_fld->setBold(false);
					break;
			}
		}
	
		int r,g,b;
		v_is>>r;
		v_is>>g;
		v_is>>b;
		if(keep) axis->_label_colour = QColor(r,g,b);
	}
	return;
}


// ----------------------

void restoreFrameTabs(Vistream& v_is, QpFrameItem *frame)
{
	// streams from virtual input stream v_is into panel tab

	float _loffset;
	v_is >> _loffset;
	frame->PropertiesTab->_loffset_fld->setValue(_loffset);
	
	float _width;
	v_is >> _width;
	frame->PropertiesTab->_width_fld->setValue(_width);
	
	float _toffset;
	v_is >> _toffset;
	frame->PropertiesTab->_toffset_fld->setValue(_toffset);
	
	float _height;
	v_is >> _height;
	frame->PropertiesTab->_height_fld->setValue(_height);
	
	bool _aspect_ratio;
	v_is >> _aspect_ratio;
	frame->PropertiesTab->_aspect_ratio_fld->setChecked(_aspect_ratio);
		
	bool _autolayout;
	v_is>>_autolayout;
	if(_autolayout) frame->PropertiesTab->_autolayout_lbl->setText("Autoset by Page Layout ");
	else frame->PropertiesTab->_autolayout_lbl->setText(" Page Layout not Autoset");

	int _line_width;
	v_is >> _line_width;
	_line_width -= 1;
	float f_line_width = float(_line_width);
	f_line_width = (f_line_width * 0.4 ) + 1.0;
	frame->DisplayTab->_line_width_fld->setValue(f_line_width);
	
	float _ticks_length;
	v_is >> _ticks_length;
	frame->DisplayTab->_ticks_length_fld->setValue(_ticks_length);
	
	int r, g, b;
	v_is >> r;
	v_is >> g;
	v_is >> b;
	frame->_line_colour = QColor(r, g, b);
	
	bool _show_top_axis;
	v_is >> _show_top_axis;
	frame->DisplayTab->_show_top_axis_fld->setChecked(_show_top_axis);
		
	bool _show_bottom_axis;
	v_is >> _show_bottom_axis;
	frame->DisplayTab->_show_bottom_axis_fld->setChecked(_show_bottom_axis);
		
	bool _show_left_axis;
	v_is >> _show_left_axis;
	frame->DisplayTab->_show_left_axis_fld->setChecked(_show_left_axis);
		
	bool _show_right_axis;
	v_is >> _show_right_axis;
	frame->DisplayTab->_show_right_axis_fld->setChecked(_show_right_axis);
		
	bool _show_top_ticks_labels;
	v_is >> _show_top_ticks_labels;
	frame->DisplayTab->_show_top_ticks_labels_fld->setChecked(_show_top_ticks_labels);
		
	bool _show_bottom_ticks_labels;
	v_is >> _show_bottom_ticks_labels;
	frame->DisplayTab->_show_bottom_ticks_labels_fld->setChecked(_show_bottom_ticks_labels);
		
	bool _show_left_ticks_labels;
	v_is >> _show_left_ticks_labels;
	frame->DisplayTab->_show_left_ticks_labels_fld->setChecked(_show_left_ticks_labels);
		
	bool _show_right_ticks_labels;
	v_is >> _show_right_ticks_labels;
	frame->DisplayTab->_show_right_ticks_labels_fld->setChecked(_show_right_ticks_labels);
		
	bool _show_top_labels;
	v_is >> _show_top_labels;
	frame->DisplayTab->_show_top_labels_fld->setChecked(_show_top_labels);
		
	bool _show_bottom_labels;
	v_is >> _show_bottom_labels;
	frame->DisplayTab->_show_bottom_labels_fld->setChecked(_show_bottom_labels);
		
	bool _show_left_labels;
	v_is >> _show_left_labels;
	frame->DisplayTab->_show_left_labels_fld->setChecked(_show_left_labels);
		
	bool _show_right_labels;
	v_is >> _show_right_labels;
	frame->DisplayTab->_show_right_labels_fld->setChecked(_show_right_labels);
		
	bool _show_xiszero;
	v_is >> _show_xiszero;
	frame->DisplayTab->_show_xiszero_fld->setChecked(_show_xiszero);
		
	bool _show_yiszero;
	v_is >> _show_yiszero;
	frame->DisplayTab->_show_yiszero_fld->setChecked(_show_yiszero);

	if( QPUI_release_version_num<2 || (QPUI_release_version_num==2 && QPUI_major_version_num<1))
	{
		// old restore files will have labels char size, disp, font & color
		// stored. Need to read these back into dummy variables (label
		// attributes now stored on axis options and not panel options)
		float labels_char_size,labels_disp;
		int labels_font;
		v_is>>labels_char_size;
		v_is>>labels_disp;
		v_is>>labels_font;
		v_is>>r;
		v_is>>g;
		v_is>>b;
	}
	
	bool _autotitle;
	v_is >> _autotitle;
	frame->TitleTab->_autotitle_fld->setChecked(_autotitle);

	string _title;
	v_is >> _title;
	frame->TitleTab->_title_fld->setText(_title.c_str());
	
	float _title_char_size;
	v_is >> _title_char_size;
	_title_char_size *= charScale;
	frame->TitleTab->_title_char_size_fld->setValue(_title_char_size);
	
	float _title_disp;
	v_is >> _title_disp;
	_title_disp += 1.2*_title_char_size;
	frame->TitleTab->_title_disp_fld->setValue(_title_disp);
	
	string _subtitle;
	v_is >> _subtitle;
	frame->TitleTab->_subtitle_fld->setText(_subtitle.c_str());
	
	float _subtitle_char_size;
	v_is >> _subtitle_char_size;
	_subtitle_char_size *= charScale;
	frame->TitleTab->_subtitle_char_size_fld->setValue(_subtitle_char_size);
	
	float _subtitle_disp;
	v_is >> _subtitle_disp;
	_subtitle_disp += 1.2*_subtitle_char_size;
	frame->TitleTab->_subtitle_disp_fld->setValue(_subtitle_disp);

	int _title_just;
	v_is >> _title_just;
	frame->TitleTab->_title_just_fld->setCurrentItem(_title_just);
	
	int _title_font;
	v_is >> _title_font;
	switch(_title_font)
	{
		case 1: // Roman
			frame->TitleTab->_title_font_fld->setSerifFont();
			frame->TitleTab->_title_font_fld->setItalic(false);
			frame->TitleTab->_title_font_fld->setBold(false);
			break;
					
		case 2: // Italic
			frame->TitleTab->_title_font_fld->setSerifFont();
			frame->TitleTab->_title_font_fld->setItalic(true);
			frame->TitleTab->_title_font_fld->setBold(false);
			break;
					
		default: // Normal
			frame->TitleTab->_title_font_fld->setSansFont();
			frame->TitleTab->_title_font_fld->setItalic(false);
			frame->TitleTab->_title_font_fld->setBold(false);
			break;
	}
	
	v_is >> r;
	v_is >> g;
	v_is >> b;
	frame->_title_colour = QColor(r, g, b);
	
	frame->DisplayTab->_show_lr_alternating_axis_fld->setChecked(false);
	frame->DisplayTab->_show_rl_alternating_axis_fld->setChecked(false);
	frame->DisplayTab->_show_lr_alternating_ticks_labels_fld->setChecked(false);
	frame->DisplayTab->_show_rl_alternating_ticks_labels_fld->setChecked(false);
	frame->DisplayTab->_show_lr_alternating_labels_fld->setChecked(false);
	frame->DisplayTab->_show_rl_alternating_labels_fld->setChecked(false);
	frame->DisplayTab->_ticks_char_size_fld->setValue(0.6);

	return;
}

// ----------------------

void restoreLegend(Vistream& v_is, QpPanelItem *panel)
{
	// streams from virtual input stream v_is into legend tab

	bool show_legend;
	v_is >> show_legend;
	panel->LegendTab->_show_legend_fld->setChecked(show_legend);
	
	int legend_border;
	v_is >> legend_border;
	panel->LegendTab->_legend_border_fld->setCurrentItem(legend_border);

	int legend_num_rows;
	v_is >> legend_num_rows;
	panel->LegendTab->_legend_num_rows_fld->setValue(legend_num_rows);
	
	int legend_num_cols;
	v_is >> legend_num_cols;
	panel->LegendTab->_legend_num_cols_fld->setValue(legend_num_cols);
	
	float legend_char_size;
	v_is >> legend_char_size;
	legend_char_size *= charScale;
	panel->LegendTab->_legend_char_size_fld->setValue(legend_char_size);
	
	int legend_max_char_length;
	v_is >> legend_max_char_length;
	panel->LegendTab->_legend_max_char_length_fld->setValue(legend_max_char_length);
	
	float legend_line_length;
	v_is >> legend_line_length;
	panel->LegendTab->_legend_line_length_fld->setValue(legend_line_length);
	
	int legend_just;
	v_is >> legend_just;
	panel->LegendTab->_legend_just_fld->setCurrentItem(legend_just);
	
	int legend_font;
	v_is >> legend_font;
	switch(legend_font)
	{
		case 1: // Roman
			panel->LegendTab->_legend_font_fld->setSerifFont();
			panel->LegendTab->_legend_font_fld->setItalic(false);
			panel->LegendTab->_legend_font_fld->setBold(false);
			break;
					
		case 2: // Italic
			panel->LegendTab->_legend_font_fld->setSerifFont();
			panel->LegendTab->_legend_font_fld->setItalic(true);
			panel->LegendTab->_legend_font_fld->setBold(false);
			break;
					
		default: // Normal
			panel->LegendTab->_legend_font_fld->setSansFont();
			panel->LegendTab->_legend_font_fld->setItalic(false);
			panel->LegendTab->_legend_font_fld->setBold(false);
			break;
	}
	
	int legend_align;
	v_is >> legend_align;
	panel->LegendTab->_legend_align_fld->setCurrentItem(legend_align);
  
	float legend_loffset;
	v_is >> legend_loffset;
	panel->LegendTab->_legend_loffset_fld->setValue(legend_loffset);
	
	float legend_toffset;
	v_is >> legend_toffset;
	panel->LegendTab->_legend_toffset_fld->setValue(legend_toffset);
	
	float legend_cell_width;
	v_is >> legend_cell_width;
	
	float legend_cell_height;
	v_is >> legend_cell_height;
	
	int _legend_box_line_width;
	v_is >> _legend_box_line_width;
	
	float _legend_disp;
	v_is >> _legend_disp;
	
	int legend_background;
	v_is >> legend_background;
	panel->LegendTab->_legend_background_fld->setCurrentItem(legend_background);
	
	float legend_width;
	v_is >> legend_width;
	panel->LegendTab->_legend_width_fld->setValue(legend_width);

	float legend_height;
	v_is >> legend_height;
	panel->LegendTab->_legend_height_fld->setValue(legend_height);
		
	if(QPUI_release_version_num>2 ||
      (QPUI_release_version_num==2 && QPUI_major_version_num>1) ||
      (QPUI_release_version_num==2 && QPUI_major_version_num==1
       && QPUI_minor_version_num>10) ||
      (QPUI_release_version_num==2 && QPUI_major_version_num==1
       && QPUI_minor_version_num==10 && QPUI_sub_version_num>2))
	{
		int _num_traces;
		v_is >> _num_traces;
	}
	return;
}

// ----------------------

void restoreTSstack(Vistream& v_is, QpFrameItem *frame, QpPanelItem *panel)
{
	restoreItem(v_is, (QpListItem*) panel);

	restorePanelItem(v_is, frame, panel);

	int cnt;
	v_is >> cnt;

	QpuiLviType plotType;
	
	for(int i=0;i<cnt;i++)
	{
		int pt= (int)QPUI_LVI_TSTRACE;
		if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 &&
			(QPUI_major_version_num>0 || (QPUI_major_version_num==0 && QPUI_minor_version_num>=3))))
		{
			v_is >> pt;
		}

		plotType = (QpuiLviType) pt;
		switch(plotType)
		{			case QPUI_LVI_TSTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_TRACE);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_CONST_TTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_XT);
				restoreTrace(v_is, trace);
				break;
			}
		
			case QPUI_LVI_CONST_YTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_Y);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_SPECTROGRAM:
			{
				QpSpectrumItem *spectrum= new QpSpectrumItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "");
				restoreSpectroGram(v_is, spectrum);
				break;
			}
			
			case QPUI_LVI_TSEVENT:
			{
				QpEventItem* event = new QpEventItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()));
				restoreEvent(v_is, event);
				break;
			}
		}
	}
	return;
}

//     -----------------------------

void restoreXYstack(Vistream& v_is, QpFrameItem *frame, QpPanelItem *panel)
{
	restoreItem(v_is, (QpListItem*) panel);

	restorePanelItem(v_is, frame, panel);

   int cnt;
   v_is >> cnt;

   QpuiLviType plotType;

   for(int i=0;i<cnt;i++)
   {
		int pt=(int)QPUI_LVI_XYTRACE;
		if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 &&
			(QPUI_major_version_num>0 || (QPUI_major_version_num==0 && QPUI_minor_version_num>=3))))
		{
			v_is >> pt;
		}

		plotType = (QpuiLviType) pt;
		
		switch(plotType)
		{
			case QPUI_LVI_XYTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_TRACE);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_CONST_YTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_Y);
				restoreTrace(v_is, trace);
				break;
			}

			case QPUI_LVI_CONST_XTRACE:
			{
				QpTraceItem *trace = new QpTraceItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "", QP_CONST_XT);
				restoreTrace(v_is, trace);
				break;
			}
          
			case QPUI_LVI_XYSPECTROGRAM:
			{
				QpSpectrumItem *spectrum= new QpSpectrumItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()), "");
				restoreSpectroGram(v_is, spectrum);
				break;
			}

			case QPUI_LVI_XYEVENT:
			{
				QpEventItem* event = new QpEventItem(panel, "", dynamic_cast<QpPlotItem*>(panel->lastChild()));
				restoreEvent(v_is, event);
				break;
			}
      }
   }
   return;
}

//     -----------------------------

void restoreTrace(Vistream& v_is, QpTraceItem *trace)
{
	// streams from virtual input stream v_is into item
	restoreItem(v_is, (QpListItem*) trace);

	QdObject_var ydo ;

	int _plot_type;
	v_is >> _plot_type;

	string _y_data;
	v_is >> _y_data;
	trace->DataTab->_y_data_fld->setText(_y_data.c_str());

	string _x_data;
	v_is >> _x_data;
	trace->DataTab->_x_data_fld->setText(_x_data.c_str());

	v_is >> trace->with;
	if(trace->with)
	{
		DataNamesTable_var qpui_objects;
		try{
			qpui_objects = QguWindow::_restored_objects->resolve_context(Name("qpui"));
		}
		catch(...){;}

		if(qpui_objects.is_nil())
		{
			qpui_objects = new DataNamesTable();
			if(!QguWindow::_restored_objects.is_nil())
			{
				QguWindow::_restored_objects->bind_context(Name("qpui"), qpui_objects);
			}
		}
		bool save_x=(_plot_type == QPUI_XYTRACE || _plot_type == QPUI_XYSTACK);
		
		string y_type;
		v_is >> y_type;
				
		string y_data_base = baseObjectText(_y_data.c_str()).latin1();
		string y_data_new;
		if(y_type != NULL_OBJECT)
		{
			ydo = QdObject::create(y_type);
			v_is >> ydo;
			string list;
			Name y_name(y_data_base);
			if(y_name.num_components())
				list = y_name[0].get_id();
			else if(!QdTimeInterval_var::narrow(ydo).is_nil())
				list= "Time Intervals";
			else if(!QdTime_var::narrow(ydo).is_nil())
				list="Constants";
			else if(!QdRScalar_var::narrow(ydo).is_nil())
				list="Constants";
			else
				list="Data";
				
			y_name.delete_component(0);
			y_data_new = QwuiPutOnWL(ydo, y_name.srep().c_str(), list.c_str()).latin1();
			try{
				qpui_objects->unbind(Name(y_data_base));
			}
			catch(...){;}
			try{
				qpui_objects->bindx(Name(y_data_base), y_data_new);
			}
			catch(...){;}
		}
		else
		{
			try{
				y_data_new = qpui_objects->resolve(Name(y_data_base));
			}
			catch(...){;}
		}
		
		if(save_x)
		{
			string x_type;
			v_is >> x_type;
			string x_data_base = baseObjectText(_x_data.c_str()).latin1();
			string x_data_new;
			if(x_type != NULL_OBJECT)
			{
				QdObject_var xdo;
				xdo = QdObject::create(x_type);
				v_is >> xdo;
				Name x_name = x_data_base;
				x_name.delete_component(0);
				x_data_new = QwuiPutOnWL(xdo, x_name.srep().c_str(),"Data").latin1();
				try{
					qpui_objects->unbind(Name(x_data_base));
				}
				catch(...){;}
				try{
					qpui_objects->bindx(Name(x_data_base),x_data_new);
				}
				catch(...){;}
			}
			else
			{
				try{
					x_data_new = qpui_objects->resolve(Name(x_data_base));
				}
				catch(...){;}
			}
		}
	}
	
	int _y_obj_type;
	
	if(!ydo.is_nil())
	{
		if(_plot_type==QPUI_CONST_TTRACE)
			_y_obj_type=QPUI_TIME_CONSTANT;
		else if(_plot_type==QPUI_CONST_YTRACE)
			_y_obj_type=QPUI_SCALAR_Y_CONSTANT;
		else if(_plot_type==QPUI_CONST_XTRACE)
			_y_obj_type=QPUI_SCALAR_X_CONSTANT;
		else if(_plot_type==QPUI_TSSTACK)
		{
			if(!QdTimeInterval_var::narrow(ydo).is_nil())
			{
				_y_obj_type=QPUI_TIME_CONSTANT;
				_plot_type=QPUI_CONST_TTRACE;
			}
			else if(!QdTime_var::narrow(ydo).is_nil())
			{
				_y_obj_type=QPUI_TIME_CONSTANT;
				_plot_type=QPUI_CONST_TTRACE;
			}
			else if(!QdRScalar_var::narrow(ydo).is_nil())
			{
				_y_obj_type=QPUI_SCALAR_Y_CONSTANT;
				_plot_type=QPUI_CONST_YTRACE;
			}
			else
			{
				_y_obj_type=QPUI_TIME_SERIES;
				_plot_type=QPUI_TSTRACE;
			}
		}
		else if(_plot_type==QPUI_XYSTACK)
		{
			if(!QdRScalar_var::narrow(ydo).is_nil())
			{
				if(_y_obj_type!=QPUI_SCALAR_X_CONSTANT)
				{
					_y_obj_type=QPUI_SCALAR_Y_CONSTANT;
					_plot_type=QPUI_CONST_YTRACE;
				}
				else
					_plot_type=QPUI_CONST_XTRACE;
			}
			else
			{
				_y_obj_type=QPUI_DATA_SERIES;
				_plot_type=QPUI_XYTRACE;
			}
		}
		else if(_plot_type==QPUI_TSTRACE)
			_y_obj_type=QPUI_TIME_SERIES;
   
		if(_y_obj_type==QPUI_SCALAR_X_CONSTANT || _y_obj_type==QPUI_SCALAR_Y_CONSTANT)
		{
			v_is >> _y_obj_type;
		}
		else if(_y_obj_type==QPUI_TIME_CONSTANT)
		{
			if(QPUI_release_version_num<2 ||(QPUI_release_version_num==2 &&
				QPUI_major_version_num<1) || (QPUI_release_version_num==2 &&
				QPUI_major_version_num==1 && QPUI_minor_version_num<10))
			{
				int t_comp;
				v_is >> t_comp;
			}
			else
				v_is >> _y_obj_type;
		}
		else
		{
			if(QPUI_release_version_num<2 ||(QPUI_release_version_num==2 &&
				QPUI_major_version_num<1) || (QPUI_release_version_num==2 &&
				QPUI_major_version_num==1 && QPUI_minor_version_num<10))
			{
				int y_comp;
				v_is >> y_comp;
			}
			else
				v_is >> _y_obj_type;
		}
	}
	else
	{
    if(_plot_type==QPUI_CONST_TTRACE || _plot_type==QPUI_TSSTACK)
    {
      Time time_val;
      if(valid_iso_time_str(_y_data.c_str(),time_val))
        _y_obj_type=QPUI_TIME_CONSTANT;
      _plot_type=QPUI_CONST_TTRACE;
    }
    if(_plot_type==QPUI_CONST_YTRACE ||
     (_plot_type==QPUI_TSSTACK  && _y_obj_type!=QPUI_TIME_CONSTANT)
     || _plot_type==QPUI_XYSTACK || _plot_type==QPUI_CONST_XTRACE)
    {
      bool okay;
      QString(_y_data.c_str()).toDouble(&okay);
      if(_plot_type==QPUI_CONST_YTRACE)
        _y_obj_type=QPUI_SCALAR_Y_CONSTANT;
      else if(_plot_type==QPUI_CONST_XTRACE)
		_y_obj_type=QPUI_SCALAR_X_CONSTANT;
      else
      {
        if(_y_obj_type!=QPUI_SCALAR_X_CONSTANT)
	{
	  _y_obj_type=QPUI_SCALAR_Y_CONSTANT;
	  _plot_type=QPUI_CONST_YTRACE;
	}
	else
	  _plot_type=QPUI_CONST_XTRACE;
      }
    }
    if(_y_obj_type==QPUI_SCALAR_X_CONSTANT ||
      _y_obj_type==QPUI_SCALAR_Y_CONSTANT)
    {
      v_is >> _y_obj_type;
    }
    else if(_y_obj_type==QPUI_TIME_CONSTANT)
    {
      if(QPUI_release_version_num<2 ||(QPUI_release_version_num==2 &&
	QPUI_major_version_num<1) || (QPUI_release_version_num==2 &&
	QPUI_major_version_num==1 && QPUI_minor_version_num<10))
      {
         int t_comp;
         v_is >> t_comp;
       }
       else
         v_is >> _y_obj_type;
    }
    else
    {
       if(QPUI_release_version_num<2 ||(QPUI_release_version_num==2 &&
	QPUI_major_version_num<1) || (QPUI_release_version_num==2 &&
	QPUI_major_version_num==1 && QPUI_minor_version_num<10))
       {
         int y_comp;
         v_is >> y_comp;
       }
       else
         v_is >> _y_obj_type;
    }
  }
  if(QPUI_release_version_num<2 ||
    (QPUI_release_version_num==2 &&  QPUI_major_version_num<1) ||
    (QPUI_release_version_num==2 && QPUI_major_version_num==1
      && QPUI_minor_version_num<10))
  {
     string y_comp_str,x_comp_str;
     int x_comp;
     v_is >> y_comp_str;
     v_is >> x_comp;
     v_is >> x_comp_str;
  }
  if(QPUI_release_version_num>2 ||
    (QPUI_release_version_num==2 && QPUI_major_version_num>1) ||
    (QPUI_release_version_num==2 && QPUI_major_version_num==1
      && QPUI_minor_version_num>10) ||
    (QPUI_release_version_num==2 && QPUI_major_version_num==1
      && QPUI_minor_version_num==10 && QPUI_sub_version_num>1))
  {
	 string _y_comp_str, _x_comp_str;
     int _x_comp, _y_comp;
     v_is >> _y_comp_str;
     v_is >> _y_comp;
     v_is >> _x_comp_str;
     v_is >> _x_comp;
  }
  
	int _connect_style;
	v_is >> _connect_style;
	trace->TraceTab->_connect_style_fld->setCurrentItem(_connect_style);

	int _line_style;
	v_is >> _line_style;
	trace->TraceTab->_line_style_fld->setCurrentItem(_line_style);
	
	int _line_width;
	v_is >> _line_width;
	_line_width -= 1;
	float f_line_width = float(_line_width);
	f_line_width = (f_line_width * lineScale ) + 1.0;
	trace->TraceTab->_line_width_fld->setValue(f_line_width);
	int r,g,b;
	v_is >> r;
	v_is >> g;
	v_is >> b;
	trace->TraceTab->setLineColour( QColor(r,g,b));

	int _symbol_shape;
	v_is >> _symbol_shape;
	trace->TraceTab->_symbol_shape_fld->setCurrentItem(_symbol_shape);
	
	float _symbol_size;
	v_is >> _symbol_size;
	trace->TraceTab->_symbol_size_fld->setValue(_symbol_size);
	
	v_is >> r;
	v_is >> g;
	v_is >> b;
	trace->TraceTab->setSymbolColour(QColor(r,g,b));


	if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 && QPUI_major_version_num>=1))
	{
		string _legend_text;
		v_is >> _legend_text;
		trace->DataTab->_legend_text_fld->setText(_legend_text.c_str());
		
		bool _show_legend;
		v_is >> _show_legend;
		trace->DataTab->_show_legend_fld->setChecked(_show_legend);
	}

	trace->DataTab->_x_data_fld->reValidate();
	trace->DataTab->_y_data_fld->reValidate();
	
	return;
}

//     -----------------------------

void restoreEvent(Vistream& v_is, QpEventItem *event)
{
	restoreItem(v_is, (QpListItem*) event);

	string x_data,y_data;
	v_is >> y_data;
	v_is >> x_data;
	event->eventTab->_y_data_fld->setText(y_data.c_str());
	event->eventTab->_x_data_fld->setText(x_data.c_str());
	
	int _y_obj_type, _x_obj_type;
	v_is >> _y_obj_type;
	v_is >> _x_obj_type;
	
	v_is >> event->with;
	if(event->with)
	{
		DataNamesTable_var qpui_objects;
		try{
			qpui_objects=QguWindow::_restored_objects->resolve_context(Name("qpui"));
		}
		catch(...){;}
		if(qpui_objects.is_nil())
		{
			qpui_objects=new DataNamesTable();
			if(!QguWindow::_restored_objects.is_nil())
			{
				QguWindow::_restored_objects->bind_context(
				Name("qpui"),qpui_objects);
			}
		}
		
		string y_type;
		string y_data_new="";
		v_is >> y_type;
		
		if(y_type!=NULL_OBJECT)
		{
			QdObject_var ydo=QdObject::create(y_type);
			v_is >> ydo;
			string list;
			Name y_name(y_data);
			if(y_name.num_components())
				list=y_name[0].get_id();
			else if(!QdTime_var::narrow(ydo).is_nil())
				list="Constants";
			else if(!QdRScalar_var::narrow(ydo).is_nil())
				list="Constants";
			else
				list="Data";
			
			y_name.delete_component(0);
			y_data_new=QwuiPutOnWL(ydo,y_name.srep().c_str(),list.c_str()).latin1();
			try{
				qpui_objects->unbind(Name(y_data));
			}
			catch(...){;}
			try{
				qpui_objects->bindx(Name(y_data),y_data_new);
			}
			catch(...){;}
		}
		else
		{
			try{
				y_data_new=qpui_objects->resolve(Name(y_data));
			}
			catch(...){;}
		}
		if(y_data_new!="") event->eventTab->_y_data_fld->setText(y_data_new.c_str());
	
		string x_type, x_data_new;
		v_is >> x_type;
		if(x_type!=NULL_OBJECT)
		{
			QdObject_var xdo;
			xdo=QdObject::create(x_type);
			v_is >> xdo;
			string list;
			Name x_name(x_data);
			if(x_name.num_components())
				list=x_name[0].get_id();
			else if(!QdTime_var::narrow(xdo).is_nil())
				list="Constants";
			else if(!QdRScalar_var::narrow(xdo).is_nil())
				list="Constants";
			else
				list="Data";
				
			x_name.delete_component(0);
			x_data_new = QwuiPutOnWL(xdo,x_name.srep().c_str(),list.c_str()).latin1();
			try{
				qpui_objects->unbind(Name(x_data));
			}
			catch(...) {;}
			try{
				qpui_objects->bindx(Name(x_data),x_data_new);
			}
			catch(...){;}
		}
		else
		{
			try{
				x_data_new=qpui_objects->resolve(Name(x_data));
			}
			catch(...){;}
		}
		if(x_data_new!="") event->eventTab->_x_data_fld->setText(x_data_new.c_str());

	}
	
	string _event_label;
	v_is >> _event_label;
	event->eventTab->_event_label_fld->setText(_event_label.c_str());

	float _event_label_char_size;
	v_is >> _event_label_char_size;
	_event_label_char_size *= charScale;
	event->eventTab->_char_size_fld->setValue(_event_label_char_size);

	int font;
	v_is >> font;
	switch(font)
	{
		case 1: // Roman
			event->eventTab->_event_label_font_fld->setSerifFont();
			event->eventTab->_event_label_font_fld->setItalic(false);
			event->eventTab->_event_label_font_fld->setBold(false);
			break;
				
		case 2: // Italic
			event->eventTab->_event_label_font_fld->setSerifFont();
			event->eventTab->_event_label_font_fld->setItalic(true);
			event->eventTab->_event_label_font_fld->setBold(false);
			break;
				
		default: // Normal
			event->eventTab->_event_label_font_fld->setSansFont();
			event->eventTab->_event_label_font_fld->setItalic(false);
			event->eventTab->_event_label_font_fld->setBold(false);
			break;
	}
	
	int r,g,b;
	v_is >> r;
	v_is >> g;
	v_is >> b;
	event->_event_label_colour = QColor(r,g,b);  
	
	return;
}


void restoreSpectroGram(Vistream& v_is, QpSpectrumItem *spectrum)
{
	restoreItem(v_is, (QpListItem*) spectrum);

	bool save_x=false;
	bool save_y=false;
	int _plot_type;
	v_is >> _plot_type;
	
	string z_data_str, _y_start_data, _y_end_data, _x_start_data, _x_end_data; 
	v_is >> z_data_str;
	spectrum->DataTab->_z_data_fld->setText(z_data_str.c_str()); 
	
	if(QPUI_release_version_num<2 || (QPUI_release_version_num==2 && QPUI_major_version_num<1))
	{
		// old restore files will not have plot type of QPUI_XYSPECTROGRAM
		// so all CONST_TRACE types will be stored as 1 less than they currently are
		if((int)_plot_type>4) _plot_type++;
	}
	else
	{	
		v_is >> _y_start_data;
		spectrum->DataTab->_y_start_data_fld->setText(_y_start_data.c_str());
		v_is >> _y_end_data;
		spectrum->DataTab->_y_end_data_fld->setText(_y_end_data.c_str());

		save_y=true;
	}

	if(QPUI_release_version_num<2 || (QPUI_release_version_num==2 && QPUI_major_version_num<1) ||
		(QPUI_release_version_num==2 && QPUI_major_version_num==1 && QPUI_minor_version_num<11))
	{
		// shift QPUI_XYSTACK (2->7) and QPUI_TSSTACK(3->8), and positions 4-8
		// (QPUI_SPECTROGRAM...) up by 2...
		int pt=_plot_type;
		if(pt>=4 && pt<=8)
			_plot_type-=2;
		else if(pt==2 || pt==3)
			_plot_type+=5;
	}

	if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 &&
		QPUI_major_version_num>1) || (QPUI_release_version_num==2 &&
		QPUI_major_version_num==1 && QPUI_minor_version_num>10) ||
		(QPUI_release_version_num==2 && QPUI_major_version_num==1 && 
		QPUI_minor_version_num==10 && QPUI_sub_version_num>0))
	{
		int y_end_comp, y_start_comp, z_comp;
		string _z_comp_str, _y_start_comp_str, _y_end_comp_str;
		v_is >> z_comp;
		v_is >> y_start_comp;
		v_is >> y_end_comp;
		v_is >> _z_comp_str;
		v_is >> _y_start_comp_str;
		v_is >> _y_end_comp_str;
	}

	if(_plot_type==QPUI_XYSPECTROGRAM || _plot_type==QPUI_XYSTACK)
	{
		// only restore xy data if an xy spectrogram
		_plot_type=QPUI_XYSPECTROGRAM;
		save_x=true;
		v_is >> _x_start_data;
		spectrum->DataTab->_x_start_data_fld->setText(_x_start_data.c_str());
		v_is >> _x_end_data;
		spectrum->DataTab->_x_end_data_fld->setText(_x_end_data.c_str());			

		if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 && QPUI_major_version_num>1) ||
			(QPUI_release_version_num==2 && QPUI_major_version_num==1 && QPUI_minor_version_num>10) ||
			(QPUI_release_version_num==2 && QPUI_major_version_num==1 && QPUI_minor_version_num==10 &&
			QPUI_sub_version_num>0))
		{
			int x_end_comp, x_start_comp;
			v_is >> x_start_comp;
			v_is >> x_end_comp;
			string _x_start_comp_str, _x_end_comp_str;
			v_is >> _x_start_comp_str;
			v_is >> _x_end_comp_str;
		}
	}
	else
		_plot_type=QPUI_SPECTROGRAM;
		
	v_is >> spectrum->with;
	if(spectrum->with)
	{
		DataNamesTable_var qpui_objects;
		try{
			qpui_objects=QguWindow::_restored_objects->resolve_context(Name("qpui"));
		}
		catch(...){;}
		if(qpui_objects.is_nil())
		{
			qpui_objects=new DataNamesTable();
			if(!QguWindow::_restored_objects.is_nil())
			{
				QguWindow::_restored_objects->bind_context(
				Name("qpui"),qpui_objects);
			}
		}

		string z_type;
		v_is >> z_type;
		string z_data_base=baseObjectText(z_data_str.c_str()).latin1();
		string z_data_new;
		if(z_type!=NULL_OBJECT)
		{
			QdObject_var zdo=QdObject::create(z_type);
			v_is >> zdo;
			string list;
			Name z_name(z_data_base);
			if(z_name.num_components())
				list=z_name[0].get_id();
			else if(!QdTimeInterval_var::narrow(zdo).is_nil())
				list="Time Intervals";
			else if(!QdTime_var::narrow(zdo).is_nil())
				list="Constants";
			else if(!QdRScalar_var::narrow(zdo).is_nil())
				list="Constants";
			else if(!QdRMatrix_var::narrow(zdo).is_nil())
				list="Constants";
			else
				list="Data";
				
			z_name.delete_component(0);
			z_data_new=QwuiPutOnWL(zdo,z_name.srep().c_str(),list.c_str()).latin1();
			try{
				qpui_objects->unbind(Name(z_data_base));
			}
			catch(...){;}
			try{
				qpui_objects->bindx(Name(z_data_base),z_data_new);
			}
			catch(...){;}
		}
		else
		{
			try{
				z_data_new=qpui_objects->resolve(Name(z_data_base));
			}
			catch(...){;}
		}
		if(z_data_new!="")
		{
			spectrum->DataTab->_z_data_fld->setText(QString(z_data_new.c_str())+QString(compText(z_data_str.c_str()).latin1())); 
		}
		if(save_y)
		{
			string y_start_type;
			v_is >> y_start_type;
			string y_start_data_base=baseObjectText(_y_start_data.c_str()).latin1();
			string y_start_data_new;
			if(y_start_type!=NULL_OBJECT)
			{
				QdObject_var ydo;
				ydo=QdObject::create(y_start_type);
				v_is >> ydo;
				string list;
				Name y_name(y_start_data_base);
				if(y_name.num_components())
					list=y_name[0].get_id();
				else if(!QdTimeInterval_var::narrow(ydo).is_nil())
					list="Time Intervals";
				else if(!QdTime_var::narrow(ydo).is_nil())
					list="Constants";
				else if(!QdRScalar_var::narrow(ydo).is_nil())
					list="Constants";
				else if(!QdRMatrix_var::narrow(ydo).is_nil())
					list="Constnats";
				else
					list="Data";
					
				y_name.delete_component(0);
				y_start_data_new=QwuiPutOnWL(ydo,y_name.srep().c_str(),list.c_str()).latin1();
				try{
					qpui_objects->unbind(Name(y_start_data_base));
				}
				catch(...){;}
				try{
					qpui_objects->bindx(Name(y_start_data_base),y_start_data_new);
				}
				catch(...){;}
			}
			else
			{
				try{
					y_start_data_new=qpui_objects->resolve(Name(y_start_data_base));
				}
				catch(...){;}
			}
			if(y_start_data_new!="")
			{
				spectrum->DataTab->_y_start_data_fld->setText(QString(y_start_data_new.c_str())+QString(compText(_y_start_data.c_str()).latin1()));
			}
			
			string y_end_type;
			v_is >> y_end_type;
			string y_end_data_base=baseObjectText(_y_end_data.c_str()).latin1();
			string y_end_data_new;
			if(y_end_type!=NULL_OBJECT)
			{
				QdObject_var ydo_end;
				ydo_end=QdObject::create(y_end_type);
				v_is >> ydo_end;
				string list;
				Name y_name(y_end_data_base);
				if(y_name.num_components())
					list=y_name[0].get_id();
				else if(!QdTimeInterval_var::narrow(ydo_end).is_nil())
					list="Time Intervals";
				else if(!QdTime_var::narrow(ydo_end).is_nil())
					list="Constants";
				else if(!QdRScalar_var::narrow(ydo_end).is_nil())
					list="Constants";
				else if(!QdRMatrix_var::narrow(ydo_end).is_nil())
					list="Constants";
				else
					list="Data";
        
				y_name.delete_component(0);
				y_end_data_new=QwuiPutOnWL(ydo_end,y_name.srep().c_str(),list.c_str()).latin1();
				try{
					qpui_objects->unbind(Name(y_end_data_base));
				}
				catch(...){;}
				try{
					qpui_objects->bindx(Name(y_end_data_base),y_end_data_new);
				}
				catch(...){;}
			}
			else
			{
				try{
					y_end_data_new=qpui_objects->resolve(Name(y_end_data_base));
				}
				catch(...){;}
			}
			if(y_end_data_new!="")
			{
				spectrum->DataTab->_y_end_data_fld->setText(QString(y_end_data_new.c_str())+QString(compText(_y_end_data.c_str()).latin1()));
			}
		}
		if(save_x)
		{
			string x_start_type;
			v_is >> x_start_type;
			string x_start_data_base= baseObjectText(_x_start_data.c_str()).latin1();
			string x_start_data_new;
			if(x_start_type!=NULL_OBJECT)
			{
				QdObject_var xdo;
				xdo=QdObject::create(x_start_type);
				v_is >> xdo;
				string list;
				Name x_name(x_start_data_base);
				if(x_name.num_components())
					list=x_name[0].get_id();
				else if(!QdTimeInterval_var::narrow(xdo).is_nil())
					list="Time Intervals";
				else if(!QdTime_var::narrow(xdo).is_nil())
					list="Constants";
				else if(!QdRScalar_var::narrow(xdo).is_nil())
					list="Constants";
				else if(!QdRMatrix_var::narrow(xdo).is_nil())
					list="Constants";
				else
					list="Data";
				
				x_name.delete_component(0);
				x_start_data_new=QwuiPutOnWL(xdo,x_name.srep().c_str(),list.c_str()).latin1();
				try{
					qpui_objects->unbind(Name(x_start_data_base));
				}
				catch(...){;}
				try{
					qpui_objects->bindx(Name(x_start_data_base),x_start_data_new);
				}
				catch(...){;}
			}
			else
			{
				try{
					x_start_data_new=qpui_objects->resolve(Name(x_start_data_base));
				}
				catch(...){;}
			}
			if(x_start_data_new!="")
			{
				spectrum->DataTab->_x_start_data_fld->setText(QString(x_start_data_new.c_str())+QString(compText(_x_start_data.c_str()).latin1()));
			}
			string x_end_type;
			v_is >> x_end_type;
			string x_end_data_base=baseObjectText(_x_end_data.c_str()).latin1();
			string x_end_data_new;
			if(x_end_type!=NULL_OBJECT)
			{
				QdObject_var xdo_end;
				xdo_end=QdObject::create(x_end_type);
				v_is >> xdo_end;
				string list;
				Name x_name(x_end_data_base);
				if(x_name.num_components())
					list=x_name[0].get_id();
				else if(!QdTimeInterval_var::narrow(xdo_end).is_nil())
					list="Time Intervals";
				else if(!QdTime_var::narrow(xdo_end).is_nil())
					list="Constants";
				else if(!QdRScalar_var::narrow(xdo_end).is_nil())
					list="Constants";
				else if(!QdRMatrix_var::narrow(xdo_end).is_nil())
					list="Constants";
				else
					list="Data";
       
				x_name.delete_component(0);
				x_end_data_new=QwuiPutOnWL(xdo_end,x_name.srep().c_str(),list.c_str()).latin1();
				try{
					qpui_objects->unbind(Name(x_end_data_base));
				}
				catch(...){;}
				try{
					qpui_objects->bindx(Name(x_end_data_base),x_end_data_new);
				}
				catch(...){;}
			}
			else
			{
				try{
					x_end_data_new=qpui_objects->resolve(Name(x_end_data_base));
				}
				catch(...){;}
			}
			if(x_end_data_new!="")
			{
				spectrum->DataTab->_x_end_data_fld->setText(QString(x_end_data_new.c_str())+QString(compText(_x_end_data.c_str()).latin1()));
			}
		}
	}

	QpColourscaleTab* ct = spectrum->ColourscaleTab;
	int cs_auto;
	v_is >> cs_auto;
	ct->_colourscale_auto_fld->setCurrentItem(cs_auto);

	double _min;
	v_is >> _min;
	ct->_colourscale_min_fld->setDouble(_min);
	double _max;
	v_is >> _max;
	ct->_colourscale_max_fld->setDouble(_max);
	
	v_is >> spectrum->missing_value;

	bool _linlog;
	v_is >> _linlog;
	ct->_colourscale_linlog_fld->setCurrentItem(_linlog);
	int _colour_table;
	v_is >> _colour_table;
	ct->_colourscale_table_fld->setCurrentItem(_colour_table);

	string _label;
	v_is >> _label;
	ct->_colourscale_label_fld->setText(_label.c_str());
	bool _show_colour_bar;
	v_is >> _show_colour_bar;
	ct->_colourscale_show_colour_bar_fld->setChecked(_show_colour_bar);
	float _colour_bar_width;
	v_is >> _colour_bar_width;
	ct->_colourscale_colour_bar_width_fld->setValue(_colour_bar_width);
	float _colour_bar_disp;
	v_is >> _colour_bar_disp;
	ct->_colourscale_colour_bar_disp_fld->setValue(_colour_bar_disp);

	if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 && QPUI_major_version_num>1) ||
		(QPUI_release_version_num==2 && QPUI_major_version_num==1 && QPUI_minor_version_num>10) ||
		(QPUI_release_version_num==2 && QPUI_major_version_num==1 && QPUI_minor_version_num==10 &&
		QPUI_sub_version_num>0))
	{
		bool _autolabel;
		v_is >> _autolabel;
		ct->_colourscale_autolabel_fld->setChecked(_autolabel);
	}
	if(QPUI_release_version_num>2 || (QPUI_release_version_num==2 && QPUI_major_version_num>2) ||
		(QPUI_release_version_num==2 && QPUI_major_version_num==2 && QPUI_minor_version_num>5) )
	{
		float _colour_bar_height;
		v_is >> _colour_bar_height;
		ct->_colourscale_colour_bar_height_fld->setValue(_colour_bar_height);
		float _colour_bar_voffset;
		v_is >> _colour_bar_voffset;
		ct->_colourscale_colour_bar_voffset_fld->setValue(_colour_bar_voffset);
	}
	
	spectrum->DataTab->_z_data_fld->reValidate(); // this will set all axes

	return;

}





