// qpui_spectrogram.cc
//
// Plot Interface Implementation - SpectroGram Object
//
// Janet Barnes
// July 2002
//
// Tony Allen
// Feb 2008

#include "qpui.h"
#include "qar.h"
#include "qdutil.h"
#include "qwui.h"
#include "TimeUtils.h"

extern int qsas_restore_release;

//     QpSpectrumItem CLASS
//     ------------------

QpSpectrumItem::QpSpectrumItem(QpPanelItem *parent, QString name, QpPlotItem *after, QString WL):
    QpPlotItem(parent,name,after,WL)
{
	// Constructor for Spectrum which is a child of a Panel
   
	I_Type = QP_SPECTROGRAM;

	ColourscaleTab = new QpColourscaleTab(tabWidget, "ColourscaleTab", this);
	ColourscaleTab->hide();
	
	DataTab = new QpDataTab(tabWidget, "DataTab", this, QP_SPECTROGRAM);
	DataTab->hide();

	DataTab->_z_data_fld->setText(WL.latin1());	
	DataTab->_z_data_fld->reValidate();
	
	_z_obj_id=0;
	_y_start_obj_id=0;
	_y_end_obj_id=0;
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	
	y_data_start_str = "";
	y_data_end_str = "";
	x_data_start_str = "";
	x_data_end_str = "";

	// starts as not set (used by fill_X_Data and fill_Y_Data to find when toggle changes)
	XYtoggle_in_fill_x = false;
	XYtoggle_in_fill_y = false;
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
 	zdo_id_getZRange = 0;
	
	x_min_data = 0;
	x_max_data = 0;	
	y_min_data = 0;
	y_max_data = 0;	
	z_min_data = 0;
	z_max_data = 0;
	
}

// ---------------------------------------

QpSpectrumItem::~QpSpectrumItem()
{
   // Destructor

	if(DataTab) delete DataTab;
	if(ColourscaleTab) delete ColourscaleTab;
	
}
// ------------------------------------

void QpSpectrumItem::clearObj_id()
{
   // called when we need to force a range update
   _z_obj_id = 0;
   _y_start_obj_id = 0;	
   _x_start_obj_id = 0;	
}

// -------------------------------------------

void QpSpectrumItem::replaceData(QString obj_name)
{
	if(QuSlotDebug()) cout << "QpSpectrumItem::replaceData" << endl;
	if(obj_name.isEmpty()) return;
	
	QString oldCompText = compText( DataTab->_z_data_fld->text() ); 
	if(oldCompText.isEmpty())
	{
		if(this->frameType() == QP_TS)
		{
			QString defComp = QuGetDefaultComp(obj_name, QSAS_profiles->current()->Array3D);
			DataTab->_z_data_fld->setTextKeepComp(defComp); // set data slot
		}
		else
		{
			QString defComp = QuGetDefaultComp(obj_name, QSAS_profiles->current()->Array3DXY); 
			DataTab->_z_data_fld->setTextKeepComp(defComp); // set data slot
		}
	}
	else
	{
		DataTab->_z_data_fld->setTextKeepComp(obj_name); // set data slot
	
	}
	if(!nameLock) this->setText(NAME_COL, DataTab->_z_data_fld->text().latin1()); // set item name
	this->WLcontext = QString(obj_name.latin1());

}

// -------------------------------------------

void QpSpectrumItem::replaceComponent(QString comp)
{
	if(QuSlotDebug()) cout << "QpSpectrumItem::replaceComponent" << endl;
	if(comp.isEmpty()) return;
	
	QString newName = baseObjectText(DataTab->_z_data_fld->text().latin1());
	newName += comp;
	DataTab->_z_data_fld->setText(newName); // set data slot
	DataTab->_z_data_fld->reValidate();
	if(!nameLock) this->setText(NAME_COL, newName);                      // set item name

}

// -------------------------------------------

void QpSpectrumItem::refreshTabs()
{
	if(QuSlotDebug()) cout << "QpSpectrumItem::refreshTabs" << endl;
	DataTab->_z_data_fld->reValidate();

}

// -------------------------------------------

void QpSpectrumItem::showTabs()
{
	if(QuSlotDebug()) cout << "QpSpectrumItem::showTabs" << endl;

	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--) 
	{
		if(tabWidget->page(i))
		{
			tabWidget->page(i)->hide();
			tabWidget->removePage(this->tabWidget->page(i));
		}
	}

	tabWidget->addTab(DataTab, "Data");

	tabWidget->addTab(ColourscaleTab, "Colourscale");
	// value return is always shown
	tabWidget->addTab(I_Panel->I_Frame->I_Page->ValueReturnTab, "Value Return");
	
	if(this->I_ListView->I_LastTab == QP_COLOUR_TAB)  tabWidget->showPage(ColourscaleTab);
	else	tabWidget->showPage(DataTab);
}

// ------------------------------------------

void QpSpectrumItem::changeFrameType(int frameType)
{
	// called when Frame changes type XY <-> TS
	
	if(frameType == QP_TS)
	{
		DataTab->_x_start_label_fld->hide();
		DataTab->_x_start_data_fld->hide();
		DataTab->_x_end_label_fld->hide();
		DataTab->_x_end_data_fld->hide();
		DataTab->_XYtoggle->hide();
		DataTab->gap_toggle->show();
		DataTab->timeAsSec_toggle->hide();
	}
	else
	{
		DataTab->_x_start_label_fld->show();
		DataTab->_x_start_data_fld->show();
		DataTab->_x_end_label_fld->show();
		DataTab->_x_end_data_fld->show();
		DataTab->_XYtoggle->show();
		DataTab->gap_toggle->hide();
		DataTab->timeAsSec_toggle->show();
	}
	
	DataTab->setDataTypeTitle();
	
	// force axis update
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	_z_obj_id = 0;
	DataTab->setZData();
}



// ---------------------------------------

void QpSpectrumItem::get_dobjs()
{
	if( QuSlotDebug() ) cout << "QpSpectrumItem::get_dobjs " <<  endl;

	_zdo = getZDO(); // sets _zdo	
	if(_zdo.is_nil()) return;
  
	_nx_data = fill_X_Data(); // sets _xdo
	_ny_data = fill_Y_Data(); // sets _ydo

}

// ---------------------------------------

void QpSpectrumItem::getLabel(const QpAxes axis, string& label)
{
	if( QuSlotDebug() ) cout << "QpSpectrumItem::getLabel for axis " << (int)axis <<  endl;

	// fetch label for axis
  
	// test on is_nil() done after default label is set below
  
	QdObject_var obj;
	string default_label;
	switch(axis)
	{
		case X_AXIS:
			// if time series plot default label is Universal Time
			if(frameType() == QP_TS)
			{
				label="Universal Time";
				return;
			}
			if(DataTab->timeAsSec_toggle->isChecked())
			{
				label="Seconds";
				return;
			}
			else label="X Axis";

			obj=_xdo;
			break;

		case Y_AXIS:
			label="Y Axis";

			obj=_ydo;
			break;

		case Z_AXIS:
			label="Z Axis";

			obj=_zdo;
			break;
			
		case COL_AXIS:
			label="Colour Bar";

			obj=_zdo;
			break;

		default:
			return;
	}
	if(obj.is_nil()) return;

	if(axis == COL_AXIS)
	{
		string units=QuGetAttrText(UNITS, obj);
		if(units.size() && units!=" ")
		{
			QString sanitised = QString(units.c_str()).replace(" summed", "");
			label = string(sanitised.latin1());
		}
	}
	else
	{
		default_label=QuGetAttrText(LABLAXIS,obj);
		if(default_label.size()==0) default_label=QuGetAttrText(FIELDNAM, obj);
		if(default_label.size()==0) default_label=QuGetAttrText(OBJECT_NAME, obj);
		if(default_label.size()!=0) label=default_label;
	
		string units=QuGetAttrText(UNITS, obj);
		if(units.size() && units!=" ")
		{
			label.append(" (");
			label.append(units);
			label.append(")");
		}
		string scid=QuGetAttrText(SC_ID,obj);
		if(scid.size() && scid!=" ")
		{
			label.append(" SC=");
			label.append(scid);
		}
	}
	
	QString greek = QString(label.c_str());
	greek.replace("theta","#gh");
	greek.replace("phi","#gf");
	label = greek.latin1();
	
   
  return;
}

// -------------------------------------

int QpSpectrumItem::getXData(QdObject_var xdo, QdObject_var xdo_end){
  int nx_data;

  QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
  QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(xdo);
  QdRMatrix_var    rmatrix_x    = QdRMatrix_var::narrow(xdo);
  
  QdTimeSeq_var    timeseq_t_end    = QdTimeSeq_var::narrow(xdo_end);
  QdRScalarSeq_var rscalarseq_x_end = QdRScalarSeq_var::narrow(xdo_end);
  
  // if we don't understand xdo_end object call alternative method
  if(timeseq_t_end.is_nil() && rscalarseq_x_end.is_nil()) return getXData( xdo);
  
	 
  if (!timeseq_t.is_nil() )
  {
  	  nx_data = timeseq_t->sequence_size();
	  _x_start_all.resize(nx_data);
	  _x_end_all.resize(nx_data); 
	  
      // create full array
	  for (int i=0; i< nx_data; i++)
      {		 
		 _x_start_all[i] = timeseq_t[i].get_time_rep();	
	  }
  }
  else if ( !rscalarseq_x.is_nil() )
  {
  	  nx_data = rscalarseq_x->sequence_size();
	  _x_start_all.resize(nx_data);
	  _x_end_all.resize(nx_data); 
	 
	  for (int i=0; i< nx_data; i++)
      {
		 _x_start_all[i] = rscalarseq_x[i]; 
	  }
  }
  else if ( !rmatrix_x.is_nil() )
  {
    if(rmatrix_x->rank() == 1)
	{	
		nx_data = rmatrix_x->dimdata()[0];
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data); 
		  
	     for (int i=0; i< nx_data; i++)
         {
			_x_start_all[i] = (*rmatrix_x)[i];
	     }
	}
	else
	{
      return 0;
	}
  
  }
  else
  {
    return 0;
  }
  
  if (!timeseq_t_end.is_nil() && timeseq_t_end->sequence_size() == nx_data)
  {
	  for (int i=0; i< nx_data; i++)
	  {
	     _x_end_all[i] = timeseq_t_end[i].get_time_rep();	  
	  }
	  
  }
  else if ( !rscalarseq_x_end.is_nil() && rscalarseq_x_end->sequence_size() == nx_data)
  {	  
	  for (int i=0; i< nx_data; i++)
	  {
	    _x_end_all[i] = rscalarseq_x_end[i];	  
	  }

  }
  else  
  {
	 return getXData(xdo);
  }

  return nx_data;
}


// ----------------------------------------------

int QpSpectrumItem::getXData(QdObject_var xdo, QdObject_var xdo_plus, QdObject_var xdo_minus){
  int nx_data;
  
  // remove PEACE white gaps 
  if(frameType() == QP_TS && DataTab->gap_toggle->isChecked() ) 
  {
	return getXData(xdo);
  }
  
  _x_start_all.clear();
  _x_end_all.clear();

    // Allow for data set with single value for delta_plus and delta_minus
  if(xdo_plus.is_nil() ) {
      // if no deltas call basic xdo only method
      if( xdo_minus.is_nil()) {
	     cout << " X Object delta plus and minus are empty" << endl;
	     return getXData(xdo);
      }
	  xdo_plus = new QdRScalar(0.0);
  }
  else if(xdo_minus.is_nil() ) xdo_minus = new QdRScalar(0.0);

  QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
  QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(xdo);
  QdRMatrix_var    rmatrix_x    = QdRMatrix_var::narrow(xdo);
  
  QdTimeSeq_var    timeseq_t_plus    = QdTimeSeq_var::narrow(xdo_plus);
  QdRScalarSeq_var rscalarseq_x_plus = QdRScalarSeq_var::narrow(xdo_plus);
  QdRScalar_var    rscalar_x_plus    = QdRScalar_var::narrow(xdo_plus);
  QdRMatrix_var    rmatrix_x_plus    = QdRMatrix_var::narrow(xdo_plus);
  
  QdTimeSeq_var    timeseq_t_minus    = QdTimeSeq_var::narrow(xdo_minus);
  QdRScalarSeq_var rscalarseq_x_minus = QdRScalarSeq_var::narrow(xdo_minus);
  QdRScalar_var    rscalar_x_minus    = QdRScalar_var::narrow(xdo_minus);
  QdRMatrix_var    rmatrix_x_minus    = QdRMatrix_var::narrow(xdo_minus);
  	
  // if we don't understand xdo_deltas objects call alternative method
  if(timeseq_t_plus.is_nil() && rscalarseq_x_plus.is_nil() && rmatrix_x_plus.is_nil() && rscalar_x_plus.is_nil()) {
	 cout << " X Object delta plus not allowed data type" << endl;
     return getXData(xdo);
  }
  if(timeseq_t_minus.is_nil() && rscalarseq_x_minus.is_nil() && rmatrix_x_minus.is_nil() && rscalar_x_minus.is_nil()) {
	 cout << " X Object delta minus not allowed data type" << endl;
     return getXData(xdo);
  }
  if (!timeseq_t.is_nil() )
  {
  	  nx_data = timeseq_t->sequence_size();

	  QdRScalarSeq_var d_minus;
	  if( !timeseq_t_minus.is_nil() && timeseq_t_minus->sequence_size() == nx_data)
	  {
	     d_minus = new QdRScalarSeq(nx_data);
		 for(int i=0; i<nx_data; i++)
	     {
	        d_minus[i] = timeseq_t_minus[i].get_time_rep();
		 }
	  }
	  else if( !rscalarseq_x_minus.is_nil() && rscalarseq_x_minus->sequence_size() == nx_data)
	  {
	    if(xdo_minus->xref_exists(SI_CONVERSION) )
        {
		   QdObject_var dm = QarConvertUnitsTo(rscalarseq_x_minus, "1>s" ); 
		   if( dm.is_nil() ) d_minus = rscalarseq_x_minus; // assumes seconds
		   else  d_minus = QdRScalarSeq_var::narrow(dm);
	    }
	  	else  d_minus = rscalarseq_x_minus; // assumes seconds
	  }
	  else if( !rscalar_x_minus.is_nil() )
	  {
	  	 d_minus = new QdRScalarSeq(nx_data);
		 QdObject_var  dm;
		 if(xdo_minus->xref_exists(SI_CONVERSION) )
         {
		    dm = QarConvertUnitsTo(rscalar_x_minus, "1>s" );
		 }
		 else dm = rscalar_x_minus; // assumes seconds
		 
		 for(int i=0; i<nx_data; i++)
	     {
	        d_minus[i] = QdRScalar_var::narrow(dm)->scalar();
		 }
	  }
	  else
	  {
	      cout << " X Object delta minus sequence length != time sequence length" << endl;
		  return getXData(xdo);
	  }
	 
	  QdRScalarSeq_var d_plus;
	  if( !timeseq_t_plus.is_nil() && timeseq_t_plus->sequence_size() == nx_data)
	  { 
	     d_plus = new QdRScalarSeq(nx_data);
		 for(int i=0; i<nx_data; i++)
	     {
	        d_plus[i] = timeseq_t_plus[i].get_time_rep();
		 }
	  }
	  else if( !rscalarseq_x_plus.is_nil() && rscalarseq_x_plus->sequence_size() == nx_data)
	  {
		if(xdo_plus->xref_exists(SI_CONVERSION) )
        {
		   QdObject_var dp =  QarConvertUnitsTo(rscalarseq_x_plus, "1>s" ); 
		   if( dp.is_nil() ) d_plus = rscalarseq_x_plus; // assumes seconds
		   else d_plus = QdRScalarSeq_var::narrow(dp);
	    }
	  	else  d_plus = rscalarseq_x_plus; // assumes seconds
	  }
	  else if( !rscalar_x_plus.is_nil() )
	  {
	     d_plus = new QdRScalarSeq(nx_data);
		 QdObject_var  dp;
		 if(xdo_plus->xref_exists(SI_CONVERSION) )
         {
		    dp = QarConvertUnitsTo(rscalar_x_plus, "1.0>s" );
		 }
		 else dp = rscalar_x_plus; // assumes seconds
		 
		 for(int i=0; i<nx_data; i++)
	     {
	        d_plus[i] = QdRScalar_var::narrow(dp)->scalar();
		 }
	  }
	  else
	  {
	      cout << " X Object delta plus sequence length != time sequence length" << endl;
		  return getXData(xdo);
	  }
	 	       	  
      // create  arrays
	    _x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	  for (int i=0; i< nx_data; i++)
	  {
	     double value_ip = timeseq_t[i].get_time_rep();
	     double value_im = value_ip;
		 value_ip += d_plus[i];	
		 value_im -= d_minus[i];	
		_x_end_all[i] = value_ip;
	    _x_start_all[i] = value_im;
	  }

  }
  else if ( !rscalarseq_x.is_nil() )
  {
  	  nx_data = rscalarseq_x->sequence_size();
	  	  
	  QdRScalarSeq_var ss_start;
	  if( !rscalarseq_x_minus.is_nil() && rscalarseq_x_minus->sequence_size() == nx_data)
	  {
	     try{
	        ss_start = QdRScalarSeq_var::narrow(rscalarseq_x - rscalarseq_x_minus);
		 }
		 catch(Exception &e){
		   e.print_msg();
		   return 0;
		 }
	  }
	  else if( !rscalar_x_minus.is_nil() )
	  {
	     try{
	        ss_start = QdRScalarSeq_var::narrow(rscalarseq_x - rscalar_x_minus);
		 }
		 catch(Exception &e){
		   e.print_msg();
		   return 0;
		 }
	  }
	  else
	  {
	      cout << " X Object delta minus sequence length or type error" << endl;
		  return getXData(xdo);
	  }
	 
	  QdRScalarSeq_var ss_end;
	  if( !rscalarseq_x_plus.is_nil() && rscalarseq_x_plus->sequence_size() == nx_data)
	  {
	     try{
	        ss_end = QdRScalarSeq_var::narrow(rscalarseq_x + rscalarseq_x_plus);
		 }
		 catch(Exception &e){
		   e.print_msg();
		   return 0;
		 }
	  }
	  else if( !rscalar_x_plus.is_nil() )
	  {
	     try{
	        ss_end = QdRScalarSeq_var::narrow(rscalarseq_x + rscalar_x_plus);
		 }
		 catch(Exception &e){
		   e.print_msg();
		   return 0;
		 }
	  }
	  else
	  {
	      cout << " X Object delta plus sequence length or type error" << endl;
		  return getXData(xdo);
	  }
	 
	    _x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	  for (int i=0; i< nx_data; i++)
	  {		  
		_x_start_all[i] = ss_start[i]; 
	    _x_end_all[i] = ss_end[i];	  
	  }

  }
  else if ( !rmatrix_x.is_nil() )
  {
    if(rmatrix_x->rank() == 1)
	{	 
		 nx_data = rmatrix_x->dimdata()[0];
		 
		 QdRMatrix_var m_start;
	     if( !rmatrix_x_minus.is_nil() &&  rmatrix_x_minus->dimdata()[0] == nx_data)
	     {
			try{
	           m_start = QdRMatrix_var::narrow(rmatrix_x - rmatrix_x_minus);
		    }
		    catch(Exception &e){
		      e.print_msg();
		      return 0;
		    }
	     }
         else if( !rscalar_x_minus.is_nil() )
	     {
			try{
	           m_start = QdRMatrix_var::narrow(rmatrix_x - rscalar_x_minus);
		    }
		    catch(Exception &e){
		      e.print_msg();
		      return 0;
		    }
	     }
	     else
	     {
	         cout << " X Object delta minus type or dimension error" << endl;
		     return getXData(xdo);
	     }
	 
	     QdRMatrix_var m_end;
	     if( !rmatrix_x_plus.is_nil() &&  rmatrix_x_plus->dimdata()[0] == nx_data)
	     {
			try{
	           m_end = QdRMatrix_var::narrow(rmatrix_x + rmatrix_x_plus);
		    }
		    catch(Exception &e){
		      e.print_msg();
		      return 0;
		    }
	     }
         else if( !rscalar_x_plus.is_nil() )
	     {
			try{
	           m_end = QdRMatrix_var::narrow(rmatrix_x + rscalar_x_plus);
		    }
		    catch(Exception &e){
		      e.print_msg();
		      return 0;
		    }
	     }
	     else
	     {
	         cout << " X Object delta plus type or dimension error" << endl;
		     return getXData(xdo);
	     }

		
	    _x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	     for (int i=0; i< nx_data; i++)
	     {			 
			_x_start_all[i] = (*m_start)[i];
			_x_end_all[i] = (*m_end)[i];	 
		}

	}
	else
	{
	  cout << "X Object matrix must be 1-D" << endl;
      return 0;
	}
  
  }
  else
  {
    cout << " X Object not time or scalar sequence nor 1D array" << endl;
    return 0;
  }
  
  // in the case of monotonic decreasing X the deltas may be applied the wrong way
  // ensure if decreasing that start > end
  
  if (_x_start_all[0] > _x_start_all[nx_data-1])
  {
    // decreasing bins, ensure start > end for each bin
	for(int i=0; i<nx_data; i++)
	{
	   if(_x_start_all[i] < _x_end_all[i] )
	   {
	      double hold = _x_end_all[i];
		  _x_end_all[i] = _x_start_all[i];
		  _x_start_all[i] = hold;
	   }
	}
  }
  return nx_data;
}

// ----------------------------------------------

int QpSpectrumItem::getXData(QdObject_var xdo){
  _x_start_all.clear();
  _x_end_all.clear();

  int nx_data;
  double half_spacing=0.1; // to ensure something plots
 
  QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
  QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(xdo);
  QdRMatrix_var    rmatrix_x    = QdRMatrix_var::narrow(xdo);
  	 
  if (!timeseq_t.is_nil() )
  {
		nx_data = timeseq_t->sequence_size();
	    _x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	 Time spacing_t; 
	  // tolerence is tol * spacing, so half_spacing = tol / 2
        if(0==calc_tol_spacing(spacing_t, *timeseq_t, QDOS::Seconds(0), QSAS_profiles->current()->gapTolerance)) half_spacing = (spacing_t.get_time_rep()) / (QSAS_profiles->current()->gapTolerance*2.);
	  
	  // create  array
	  for (int i=0; i< nx_data; i++)
	  {
	     double value_i = timeseq_t[i].get_time_rep();
	   //  _x_start_all.push_back( value_i - half_spacing);	
	   //  _x_end_all.push_back( value_i +  half_spacing) ;
	     _x_start_all[i] = value_i;	
	  }
	  // find bin widths
	  double diff[nx_data + 1];
		 
	  for(int i=1; i<nx_data; i++)
	  {
			diff[i] = _x_start_all[i] - _x_start_all[i-1];
	  }
	  // handle ends
	  diff[0] = diff[1];
	  diff[nx_data] = diff[nx_data-1];
		 
	  // set start end of bin using widths
	  for(int i=0; i < nx_data; i++)
	  {
		   _x_end_all[i] = _x_start_all[i] + 0.5 * diff[i+1];
		   _x_start_all[i] = _x_start_all[i] - 0.5 * diff[i];
	  
	  }

  }
  else if ( !rscalarseq_x.is_nil() )
  {
		nx_data = rscalarseq_x->sequence_size();
	    _x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);	  
		half_spacing = getSpacingTolerance(rscalarseq_x);
	  
	  for (int i=0; i< nx_data; i++)
      {
	     double value_i =  rscalarseq_x[i];		
		_x_start_all[i] = value_i; 
		_x_end_all[i] = value_i + 2* half_spacing;	  
	  }
  }
  else if ( !rmatrix_x.is_nil() )
  {
    if(rmatrix_x->rank() == 1)
	{	
		nx_data = rmatrix_x->dimdata()[0];
	    _x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		for (int i=0; i< nx_data; i++)
	     {
			_x_start_all[i] = (*rmatrix_x)[i];
	     }
		 // find bin widths
		 double diff[nx_data + 1];
		 
		 for(int i=1; i<nx_data; i++)
		 {
			diff[i] = _x_start_all[i] - _x_start_all[i-1];
		 }
		 // handle ends
		 diff[0] = diff[1];
		 diff[nx_data] = diff[nx_data-1];
		 
		 // set start end of bin using widths
		  for(int i=0; i < nx_data; i++)
		 {
		   _x_end_all[i] = _x_start_all[i] + 0.5 * diff[i+1];
		   _x_start_all[i] = _x_start_all[i] - 0.5 * diff[i];
		 }
	}
	else
	{
	  cout << "X Object matrix must be 1-D" << endl;
      return 0;
	}
  
  }
  else
  {
    cout << " X Object not time or scalar sequence nor 1D array" << endl;
    return 0;
  }
  
  return nx_data;
}

// -------------------------------------------------

int QpSpectrumItem::getYData(QdObject_var ydo, QdObject_var ydo_end)
{
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);
	
	QdRMatrix_var    rmatrix_y    = QdRMatrix_var::narrow(ydo);
    QdRMatrixSeq_var rmatrixseq_y = QdRMatrixSeq_var::narrow(ydo);
	if( !rmatrixseq_y.is_nil() && (rmatrixseq_y->sequence_size() == 1 || rmatrixseq_y->sequence_size() != _nx_data) )
	{
		rmatrix_y = new QdRMatrix(rmatrixseq_y[0]);
		rmatrixseq_y = 0;
	}
	
	QdRMatrix_var rmatrix_y_end = QdRMatrix_var::narrow(ydo_end);
	QdRMatrixSeq_var rmatrixseq_y_end = QdRMatrixSeq_var::narrow(ydo_end);
	if( !rmatrixseq_y_end.is_nil() && (rmatrixseq_y_end->sequence_size() == 1 || rmatrixseq_y_end->sequence_size() != _nx_data ))
	{
		rmatrix_y_end = new QdRMatrix(rmatrixseq_y_end[0]);
		rmatrixseq_y_end  = 0;
	}


  int ny_data=0;
  

  if(!rmatrixseq_y.is_nil() )
  {
	for(int nx=0; nx < _nx_data; ++nx){

      if(rmatrixseq_y->elt(nx).rank() == 1)
	  {
	     ny_data = rmatrixseq_y->elt(nx).dimdata()[0];
		_y_start_all[nx].resize(ny_data);

//          for(int j=0; j<ny_data; j++)
// 	     {
// 	         _y_start_all[nx][j] = rmatrixseq_y->elt(nx)[j];
// 	     }
		_y_start_all[nx] = rmatrixseq_y->elt(nx);
	  }
	  else{
	    cout <<  "Y Object is sequence but not 1-D" << endl;
        return -1;
      }
	}
  }
  else if(!rmatrix_y.is_nil())
  {
  	for(int nx=0; nx < _nx_data; ++nx){
		ny_data = rmatrix_y->dimdata()[0];
       if(rmatrix_y->rank() == 1)
	   {	
			_y_start_all[nx].resize(ny_data);
// 		   for(int j=0; j<ny_data; j++)
// 	       {
// 	          _y_start_all[nx][j] =  (*rmatrix_y)[j] ;
// 	       }
			_y_start_all[nx] =  (*rmatrix_y);
	   }
	   else
	   {
	     cout << "Y Object matrix must be 1-D" << endl;
         return -1;
	   }
	}
  }
  else
  {
	  plmtex("lv",-2.0,0.75,0.0, "Y Object not a matrix or matrix sequence");
      return -1;
  }
  
  // Handle y_end
  
  if( rmatrix_y_end.is_nil() && rmatrixseq_y_end.is_nil()) return getYData( ydo);
  
  if( !rmatrixseq_y_end.is_nil())
  {
	 for(int nx=0; nx< _nx_data; ++nx){

		ny_data = rmatrixseq_y_end->elt(nx).dimdata()[0];
        if(_y_start_all[nx].size() != ny_data)
	    {
    	   cout <<  "Y End Sequence [nx] dimension error" << endl;
           return -1;
        }
		_y_end_all[nx].resize(ny_data);
//         for(int j=0; j<ny_data; j++)
// 	    {
//            _y_end_all[nx][j] =  rmatrixseq_y_end->elt(nx)[j] ;
// 	    }
		_y_end_all[nx] =  rmatrixseq_y_end->elt(nx);
	 }
  }
  else if( !rmatrix_y_end.is_nil())
  {
	 ny_data = rmatrix_y_end->dimdata()[0];
	 for(int nx=0; nx< _nx_data; ++nx){

        if(_y_start_all[nx].size() != ny_data)
	    {
    	   cout << "Y End Object dimension error" << endl;
           return -1;
        }
		_y_end_all[nx].resize(ny_data);
//         for(int j=0; j<ny_data; j++)
// 	    {
//            _y_end_all[nx][j] =  (*rmatrix_y_end)[j] ;
// 	    }
		_y_end_all[nx]=(*rmatrix_y_end);
     }
  }
	  
  return ny_data;
  
}


// -------------------------------------------------

int QpSpectrumItem::getYData(QdObject_var ydo, QdObject_var ydo_plus, QdObject_var ydo_minus)
{
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);
	int ny_data=0;
	double value_i;

	// Allow for data set with single value for delta_plus and delta_minus
	if(ydo_plus.is_nil() ) {
		// if no deltas call basic xdo only method
		if( ydo_minus.is_nil()) {
			cout << " Y Object delta plus and minus are empty" << endl;
			return getYData( ydo);
		}
		ydo_plus = new QdRScalar(0.0);
	}
	else if(ydo_minus.is_nil() ) ydo_minus = new QdRScalar(0.0);

	QdRMatrix_var    rmatrix_y    = QdRMatrix_var::narrow(ydo);
    QdRMatrixSeq_var rmatrixseq_y = QdRMatrixSeq_var::narrow(ydo);

	if(!rmatrixseq_y.is_nil() && ( rmatrixseq_y->sequence_size() == 1 || rmatrixseq_y->sequence_size() != _nx_data) )
	{
		rmatrix_y = new QdRMatrix(rmatrixseq_y[0]);
		rmatrixseq_y = 0;
	}

	QdRScalarSeq_var rscalarseq_y_plus = QdRScalarSeq_var::narrow(ydo_plus);
	QdRScalar_var    rscalar_y_plus    = QdRScalar_var::narrow(ydo_plus);
	QdRMatrix_var    rmatrix_y_plus    = QdRMatrix_var::narrow(ydo_plus);
	QdRMatrixSeq_var    rmatrixseq_y_plus    = QdRMatrixSeq_var::narrow(ydo_plus);

	if(!rmatrixseq_y_plus.is_nil() && ( rmatrixseq_y_plus->sequence_size() == 1 || rmatrixseq_y_plus->sequence_size() != _nx_data) )
	{
		rmatrix_y_plus = new QdRMatrix(rmatrixseq_y_plus[0]);
		rmatrixseq_y_plus = 0;
	}

	QdRScalarSeq_var rscalarseq_y_minus = QdRScalarSeq_var::narrow(ydo_minus);
	QdRScalar_var    rscalar_y_minus    = QdRScalar_var::narrow(ydo_minus);
	QdRMatrix_var    rmatrix_y_minus    = QdRMatrix_var::narrow(ydo_minus);
	QdRMatrixSeq_var    rmatrixseq_y_minus    = QdRMatrixSeq_var::narrow(ydo_minus);

	if(!rmatrixseq_y_minus.is_nil() && (rmatrixseq_y_minus->sequence_size() == 1 || rmatrixseq_y_minus->sequence_size() != _nx_data ) )
	{
		rmatrix_y_minus = new QdRMatrix(rmatrixseq_y_minus[0]);
		rmatrixseq_y_minus = 0;
	}


	if( rscalarseq_y_plus.is_nil() && rmatrix_y_plus.is_nil() && rscalar_y_plus.is_nil() && rmatrixseq_y_plus.is_nil()) {
		cout << " Y Object delta plus not allowed data type" << endl;
		return getYData( ydo);
	}
	if( rscalarseq_y_minus.is_nil() && rmatrix_y_minus.is_nil() && rscalar_y_minus.is_nil() && rmatrixseq_y_minus.is_nil()) {
		cout << " Y Object delta minus not allowed data type" << endl;
		return getYData(  ydo);
	}

	if(!rmatrixseq_y.is_nil())
	{
		if(rmatrixseq_y->elt(0).rank() == 1)
		{
			QdRMatrixSeq_var ms_start;
			QdRMatrixSeq_var ms_end;
	  
			if( !rscalarseq_y_minus.is_nil() )
			{
				try{
					ms_start = QdRMatrixSeq_var::narrow(rmatrixseq_y - rscalarseq_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rscalar_y_minus.is_nil() )
			{
				try{
					ms_start = QdRMatrixSeq_var::narrow(rmatrixseq_y - rscalar_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_minus.is_nil() )
			{
				try{
					ms_start = QdRMatrixSeq_var::narrow(rmatrixseq_y - rmatrix_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrixseq_y_minus.is_nil() )
			{
				if(rmatrixseq_y_minus->sequence_size() != _nx_data )
				{
					cout << "delta plus sequence of wrong length" << endl;
					return -1;
				}
				try{
					ms_start = new QdRMatrixSeq(_nx_data, rmatrixseq_y->elt(0).dimdata());
					for(int i=0; i<_nx_data; i++)
					{
						ms_start[i] = rmatrixseq_y[i] - rmatrixseq_y_minus[i];
					}
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
	  
			if( !rscalarseq_y_plus.is_nil() )
			{
				try{
					ms_end = QdRMatrixSeq_var::narrow(rmatrixseq_y + rscalarseq_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rscalar_y_plus.is_nil() )
			{
				try{
					ms_end = QdRMatrixSeq_var::narrow(rmatrixseq_y + rscalar_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_plus.is_nil() )
			{
				try{
					ms_end = QdRMatrixSeq_var::narrow(rmatrixseq_y + rmatrix_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrixseq_y_plus.is_nil() )
			{
				if(rmatrixseq_y_plus->sequence_size() != _nx_data )
				{
					cout << "delta plus sequence of wrong length" << endl;
					return -1;
				}
				try{
					ms_end = new QdRMatrixSeq(_nx_data, rmatrixseq_y->elt(0).dimdata());
					for(int i=0; i<_nx_data; ++i)
					{
						ms_end[i] = rmatrixseq_y[i] + rmatrixseq_y_plus[i];
					}
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
	  

			if(ms_start->sequence_size() != _nx_data) 
			{
				cout << "delta minus sequence of wrong length" << endl;
				return -1;
			}
			if(ms_end->sequence_size() != _nx_data) 
			{
				cout << "delta plus sequence of wrong length" << endl;
				return -1;
			}
	  
			for(int nx=0; nx< _nx_data; ++nx){

				ny_data = rmatrixseq_y->elt(nx).dimdata()[0];
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);
// 				for(int j=0; j<ny_data; j++)
// 				{
// 					_y_start_all[nx][j] = ms_start->elt(nx)[j];
// 					_y_end_all[nx][j] = ms_end->elt(nx)[j];
// 				}
				_y_start_all[nx]=ms_start->elt(nx);
				_y_end_all[nx]=ms_end->elt(nx);
			}
		}
		else{
			cout << "Y Object is sequence but not 1-D" << endl;
			return -1;
		}
	}
	else if(!rmatrix_y.is_nil())
	{
		if(rmatrix_y->rank() == 1)
		{	
			QdRMatrix_var m_start;
			QdRMatrix_var m_end;
	  
			if(!rscalar_y_minus.is_nil() )
			{
				try{
					m_start = QdRMatrix_var::narrow(rmatrix_y - rscalar_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_minus.is_nil() )
			{	 
				try{
					m_start = QdRMatrix_var::narrow(rmatrix_y - rmatrix_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else
			{
				cout << " Y Object delta plus is sequence " << endl;
				return getYData( ydo);
			}
	  
			if(!rscalar_y_plus.is_nil() )
			{
				try{
					m_end = QdRMatrix_var::narrow(rmatrix_y + rscalar_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_plus.is_nil() )
			{
				try{
					m_end = QdRMatrix_var::narrow(rmatrix_y + rmatrix_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else
			{
				cout << " Y Object delta minus is sequence " << endl;
				return getYData(ydo);
			}

			ny_data = rmatrix_y->dimdata()[0];
			for(int nx=0; nx< _nx_data; ++nx)
			{
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);
	  
// 				for(int j=0; j<ny_data; j++)
// 				{
// 					_y_start_all[nx][j] = (*m_start)[j];
// 					_y_end_all[nx][j] = (*m_end)[j];
// 				}
				_y_start_all[nx]=(*m_start);
				_y_end_all[nx]=(*m_end);
			}

		}
		else
		{
			cout << "Y Object matrix must be 1-D" << endl;
			return -1;
		}
	}
	else
	{
		cout << "Y Object is not a matrix or matrix sequence" << endl;
		return -1;
	}

	// in the case of monotonic decreasing Y the deltas may be applied the wrong way
	// ensure if decreasing that start > end
  
	for(int j=0; j< _nx_data; j++)
	{
		ny_data = _y_start_all[j].size();
		if ( (_y_start_all[j])[0] > (_y_start_all[j])[ny_data-1])
		{
			// decreasing bins, ensure start > end for each bin
			for(int i=0; i<ny_data; i++)
			{
				if((_y_start_all[j])[i] < (_y_end_all[j])[i] )
				{
					double hold = (_y_end_all[j])[i];
					(_y_end_all[j])[i] = (_y_start_all[j])[i];
					(_y_start_all[j])[i] = hold;
				}
			}
		}
	}
	return ny_data;
}

// -------------------------------------------------

int QpSpectrumItem::getYData(QdObject_var ydo) // TODO possible to optimise with gslices?
{
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);

	QdRMatrix_var    rmatrix_y    = QdRMatrix_var::narrow(ydo);
    QdRMatrixSeq_var rmatrixseq_y = QdRMatrixSeq_var::narrow(ydo);
	if( !rmatrixseq_y.is_nil() && ( rmatrixseq_y->sequence_size() == 1 ||  rmatrixseq_y->sequence_size() != _nx_data ))
	{
		rmatrix_y = new QdRMatrix(rmatrixseq_y[0]);
		rmatrixseq_y = 0;
	}

	int ny_data=0;
  
	if(!rmatrixseq_y.is_nil())
	{
		for(int nx=0; nx< _nx_data; nx++){
	  
			if(rmatrixseq_y->elt(nx).rank() == 1)
			{
				ny_data = rmatrixseq_y->elt(nx).dimdata()[0];
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);
				for(int j=0; j<ny_data-1; j++)
				{
					_y_start_all[nx][j] = rmatrixseq_y->elt(nx)[j];
					_y_end_all[nx][j] = rmatrixseq_y->elt(nx)[j+1];
				}
				_y_start_all[nx][ny_data-1] = rmatrixseq_y->elt(nx)[ny_data-1];
				double lastVal = rmatrixseq_y->elt(nx)[ny_data-1];
				double lastValminus = rmatrixseq_y->elt(nx)[ny_data-2];
				_y_end_all[nx][ny_data-1] = ( 2.0*lastVal - lastValminus);
			}
			else{
				cout << "Y Object is sequence but not 1-D" << endl;
				return -1;
			}
		}
	}
	else if(!rmatrix_y.is_nil())
	{
		if(rmatrix_y->rank() != 1)
		{	
			cout << "Y Object matrix must be 1-D" << endl;
			return -1;
		}	

		for(int nx=0; nx< _nx_data; nx++){
	  
			ny_data = rmatrix_y->dimdata()[0];
			_y_start_all[nx].resize(ny_data);
			_y_end_all[nx].resize(ny_data);
			if(ny_data == 1){
				double value = (*rmatrix_y)[0];
				_y_start_all[nx][0] = value;
				_y_end_all[nx][0] = value+1.;
			}
			else
			{
				for(int j=0; j<ny_data-1; j++)
				{
					_y_start_all[nx][j] = (*rmatrix_y)[j];
					_y_end_all[nx][j] = (*rmatrix_y)[j+1];
				}
				double value = (*rmatrix_y)[ny_data-1] ;
				_y_start_all[nx][ny_data-1] = value;
				double value2 = (*rmatrix_y)[ny_data-2];
				value = 2.0*value  - value2 ;
				_y_end_all[nx][ny_data-1] = value;
			}
		}
	}
	else
	{
		cout << "Y Object is not a matrix or matrix sequence" << endl;
		return -1;
	}
  
	return ny_data;
  
}

// -------------------------------------------------

int QpSpectrumItem::isZConformal(int nx_len, int ny_len)
{
   // does something only for 2D matrix or 2D matrix sequence
   if ( QuSlotDebug() )
   {
	printf( "QpSpectrumItem::isZConformal nx=%ld, ny=%ld\n", nx_len , ny_len );
	fflush(stdout);
   }

   QdRMatrixSeq_var rmatrixseq_z = QdRMatrixSeq_var::narrow(_zdo);
   QdRMatrix_var    rmatrix_z    = QdRMatrix_var::narrow(_zdo);
   if( rmatrix_z.is_nil() && rmatrixseq_z.is_nil() ) return 1; // do nothing

   int n0, n1;
   if( !rmatrixseq_z.is_nil() )
   {
      if(  rmatrixseq_z->elt(0).rank() != 2) return 1; // do nothing, not 2D array
      n0 = rmatrixseq_z->elt(0).dimdata()[0];
      n1 = rmatrixseq_z->elt(0).dimdata()[1];
   }
   else
   {
	  if(  rmatrix_z->rank() != 2) return 1; // do nothing, not 2D array
      n0 = rmatrix_z->dimdata()[0];
      n1 = rmatrix_z->dimdata()[1];
   }
   if (n0 == n1)
   {
      // square matrix, so take care
	  if(_zdo->xref_exists(DEPEND_1) && _zdo->xref_exists(DEPEND_2) )
	  {
	     QdObject_var dep1 = _zdo->get_xref(DEPEND_1);
		 int id_1 = dep1->get_id();
	     QdObject_var dep2 = _zdo->get_xref(DEPEND_2);
		 int id_2 = dep2->get_id();
         if(id_1 == _xdo->get_id() && id_2 == _ydo->get_id() ) return 0;
		 else if (id_2 == _xdo->get_id() && id_1 == _ydo->get_id() ) return 1; // default behaviour
		 else 
		 {
	        cout << "Z data is square matrix don't know which axis is which, add DEPEND_1 and DEPEND_2";
	        return -1;
	     }
	  }
	  else return 1; // No idea, do nothing, 1st index is Y, 2nd is X
   }
   else if(n0 == nx_len && n1 == ny_len)
   {
     return 0;
   }
   else if(n0 == ny_len && n1 == nx_len)
   {
     return 1;
   }
   else return -1;
}

//----------------------------

void QpSpectrumItem::getName(string& name)
{
  // fetch name of spectrogram object (for display in listview)

  name = DataTab->_z_data_fld->text().latin1();
  if(name == "")
  {
    if(frameType() == QP_XY)
      name = "XY SpectroGram";
    else
      name = "TS SpectroGram";
  }
}

//----------------------------

string QpSpectrumItem::getTitle()
{
	// fetch title for current spectrogram object

	if(_zdo.is_nil())
	{
		return string("");
	}
	string title = QuGetAttrText(FIELDNAM, _zdo);
	if(title.size() == 0) title += DataTab->_z_data_fld->text().latin1();
  
	return title;
}
// --------------------------------

int QpSpectrumItem::getXmetaIsLog()
{
	 if ( QuSlotDebug() )
	 {
		printf( "QpSpectrumItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	
	int isLog = 0;
	
	// find SCALETYP
		
	if (!_xdo.is_nil() && _xdo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_xdo->get_xref(SCALETYP));
		if(!scaleType.is_nil() )
		{
			if(strcmp(QuToUpper(scaleType->c_str()), "LOG") == 0) isLog = 1;
		}
	}
	return isLog;
}


// -----------------------------------------------------

bool QpSpectrumItem::getXmetaRange(double& xmin, double& xmax)
{
	// gets x min and max values for this spectrogram object
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getXmetaRange" << endl;
    
	bool rangeFound = false;
	
	// reject if no Z object yet
	if(_zdo.is_nil()) return false;

	if ( _nx_data == 0 )   return false;
	
	if(frameType() == QP_TS)
	{
		QdTimeSeq_var tdo = get_timetags(_zdo);
		if(tdo.is_nil() || tdo->sequence_size() == 0) return false;
		
		xmin = Seconds(tdo[0]);
		xmax = Seconds(tdo[tdo->sequence_size()-1]);
		rangeFound = true;
	}
	else
	{
		if( DataTab->timeAsSec_toggle->isChecked() ) 
		{
			QdTimeSeq_var tdo = get_timetags(_zdo);
			if(tdo.is_nil() || tdo->sequence_size() == 0) return false;
		
			xmin = Seconds(tdo[0]);
			xmax = Seconds(tdo[tdo->sequence_size()-1]);
			rangeFound = true;
		}
		else
		{
			if (_xdo->xref_exists(SCALEMIN) && _xdo->xref_exists(SCALEMIN))
			{
				QdRScalar_var smin = QdRScalar_var::narrow(_xdo->get_xref(SCALEMIN));
				QdRScalar_var smax = QdRScalar_var::narrow(_xdo->get_xref(SCALEMAX));
				if(smin.is_nil() || smax.is_nil()) return false;
			
				xmin = *smin;
				xmax = *smax;
				rangeFound = true;
			}
		}
	}
	if(rangeFound) return rangeFound;
	else return getXRange( xmin,  xmax);

}
// -----------------------------------------------------

bool QpSpectrumItem::getXRange(double& xmin, double& xmax)
{
	// gets x min and max values for this spectrogram object
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getXRange" << endl;
    
	// reject if no Z object yet
	if(_zdo.is_nil()) return false;
  	
	xmin = DBL_MAX;
	xmax = -DBL_MAX;
	
	if ( _xdo.is_nil() )   return false;
	
	if(_xdo->get_id() == xdo_id_getXRange)
	{		
		xmin = x_min_data;
		xmax = x_max_data;
		return true;
	}
	xdo_id_getXRange = _xdo->get_id();
  
	for(int i=0; i<_nx_data; i++)
	{
		double mini = _x_start_all[i];
		double maxi = _x_end_all[i];
		// may not be increasing bins
		if(mini < maxi)
		{
			if(mini < xmin) xmin = mini;
			if(maxi > xmax) xmax = maxi;
		}
		else
		{
			if(maxi < xmin) xmin = maxi;
			if(mini > xmax) xmax = mini;
		}
	}
   
   // store for reuse
	x_min_data = xmin;
	x_max_data = xmax;
    
	return true;
}

// --------------------------------

int QpSpectrumItem::getYmetaIsLog()
{
	 if ( QuSlotDebug() )
	 {
		printf( " QpSpectrumItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	 
	int isLog=0;
	// find SCALETYP
		
	if (!_ydo.is_nil() && _ydo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_ydo->get_xref(SCALETYP));
		if(!scaleType.is_nil() )
		{
			if(strcmp(QuToUpper(scaleType->c_str()), "LOG") == 0) isLog = 1;
		}
	}
	return isLog;
}

// ------------------------------------

bool QpSpectrumItem::getYmetaRange(double& ymin, double& ymax)
{
	// gets y min and max values for whole Y axis
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getYmetaRange" << endl;

	ymin=0;
	ymax=0;
	bool rangeFound = false;
  
	if(_zdo.is_nil()) return false;
	
	// reject if no X object yet (also traps no z object)
	if ( _xdo.is_nil() )   return false;
  
	if ( _ydo.is_nil() ) return false;

	if (_ydo->xref_exists(SCALEMIN) && _ydo->xref_exists(SCALEMIN))
	{
		QdRScalar_var smin = QdRScalar_var::narrow(_ydo->get_xref(SCALEMIN));
		ymin = *smin;
		
		QdRScalar_var smax = QdRScalar_var::narrow(_ydo->get_xref(SCALEMAX));
		ymax = *smax;
		rangeFound = true;
	}
	
	if(rangeFound) return rangeFound;
	else	return getYRange(ymin, ymax);
	
}

// ---------------------------------------

bool QpSpectrumItem::getYRange(double& ymin, double& ymax)
{
	// gets y min and max values for all x values in range
	// xmin - xmax for the current spectrogram object
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getYRange" << endl;
	
	double xmin, xmax;
	
	if(frameType()==QP_TS)
	{
		xmin = I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->value();
		xmax = I_Panel->I_Frame->tAxisTab->_axis_to_time_fld->value();
	}
	else
	{
		xmin = I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->value();
		xmax = I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->value();
	}
	
	if(xmin >= xmax) return false;
	
	ymin = DBL_MAX;
	ymax = -DBL_MAX;
	
	bool recalc = false;
	
	// reject if no Z objects yet
	if(_zdo.is_nil()) return false;
	if ( _xdo.is_nil() )   return false;
	if ( _ydo.is_nil() ) return false;
	
	if(x_range_min != xmin || x_range_max != xmax) 
	{
		// keep x range used to calculate y range
		x_range_min = xmin;
		x_range_max = xmax;
		recalc = true;
	}
	
	if( _ydo->get_id() != ydo_id_getYRange) recalc = true;
		
	ydo_id_getYRange = _ydo->get_id();

	if(!recalc)
	{		
		ymin = y_min_data;
		ymax = y_max_data;
		return true;
	}

	for(int nx=0; nx<_x_start_all.size(); nx++)
	{
		double dx1 =  _x_start_all[nx] ;
		double dx2 =  _x_end_all[nx];
      	
		// skip if out of range
		if(dx1 < dx2) 
		{
			if(dx1 < xmin || dx2 > xmax) continue;
		}
		else
		{
			if(dx2 < xmin || dx1 > xmax) continue;
		}

		// find ymin and ymax for each x bin
		for(int j=0; j<_y_start_all[nx].size(); j++)
		{
			double ymini = (_y_start_all[nx])[j];
			double ymaxi = (_y_end_all[nx])[j];
			// may be decreasing bin
			if(ymini < ymaxi) 
			{
				if(ymax < ymaxi) ymax = ymaxi;
				if(ymin > ymini) ymin = ymini;
			}
			else
			{
				// decreasing bin
				if(ymax < ymini) ymax = ymini;
				if(ymin > ymaxi) ymin = ymaxi;
			}
		}
	}
  	
	// store to reuse if nothing changed
	y_min_data = ymin;
	y_max_data = ymax;	

	return true;

}

// -----------------------------------------------------

bool QpSpectrumItem::getZmetaRange(double& zmin,double& zmax)
{
	// gets z min and max values using metadata
  	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZmetaRange" << endl;
	
	bool rangeFound = false;
	
	if(_zdo.is_nil()) return false;

	if (_zdo->xref_exists(SCALEMIN) && _zdo->xref_exists(SCALEMIN))
	{
		QdRScalar_var smin = QdRScalar_var::narrow(_zdo->get_xref(SCALEMIN));
		if(smin.is_nil()) return false;
		zmin = *smin;
		
		QdRScalar_var smax = QdRScalar_var::narrow(_zdo->get_xref(SCALEMAX));
		if(smax.is_nil()) return false;
		zmax = *smax;
		rangeFound = true;
	}
				
	// set lin/log from metadata
	int scaleTypeFlag = QP_LINEAR;
	
	if (_zdo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_zdo->get_xref(SCALETYP));
		if( !scaleType.is_nil() && strcmp(QuToUpper(scaleType->c_str()), "LOG") == 0)
		{
			scaleTypeFlag = QP_LOG;
		}
	}		
	ColourscaleTab->_colourscale_linlog_fld->setCurrentItem(scaleTypeFlag);
	
	return rangeFound;
}

// ----------------------------------

bool QpSpectrumItem::getZRange(double &zmin, double &zmax)
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZRange" << endl;
	
	double xmin, xmax;
	
	if(frameType()==QP_TS)
	{
		xmin = I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->value();
		xmax = I_Panel->I_Frame->tAxisTab->_axis_to_time_fld->value();
	}
	else
	{
		xmin = I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->value();
		xmax = I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->value();
	}
	
	double ymin = I_Panel->yAxisTab->_axis_from_xy_fld->value();
	double ymax = I_Panel->yAxisTab->_axis_to_xy_fld->value();

	// only try if x, y ranges available and valid
	if(xmin >= xmax || ymin >= ymax) return false;

	zmin = DBL_MAX;
    zmax = -DBL_MAX;
	bool recalc = false;
	
	 // fill X will reject if no Z object yet
	if(_zdo.is_nil()) return false;
	if( _zdo->get_id() != zdo_id_getZRange) recalc = true;
		
	zdo_id_getZRange = _zdo->get_id();

    if ( _xdo.is_nil() )   return false;
    if ( _ydo.is_nil() ) return false;
  
	QpAxisTab * axisTab;
	if(I_Panel->frameType() == QP_TS)  axisTab = I_Panel->I_Frame->tAxisTab;
	else  axisTab = I_Panel->I_Frame->xAxisTab;

	if(x_range_min_z != xmin || x_range_max_z != xmax ) 
	{
		x_range_min_z = xmin;
		x_range_max_z = xmax;
		recalc = true;
	}
	
	axisTab = I_Panel->yAxisTab;
	if(y_range_min_z != ymin || y_range_max_z != ymax ) 
	{
		y_range_min_z = ymin;
		y_range_max_z = ymax;
		recalc = true;
	}

	
	if(!recalc)
	{
		zmin = z_min_data;
		zmax = z_max_data;
		return true;
	}
	
    if(_nx_data != _zdo->sequence_size() && _zdo->sequence_size() != 1) return false;
  
    int XtoY = isZConformal(_nx_data, _ny_data);
    if( XtoY == -1)  return false;  

    if( fill_Z_Data( XtoY, false) == -1)  return false;

	double zmin_for_log = DBL_MAX;
	
	for(int nx=0; nx<_x_start_all.size(); nx++)
	{
		double dx1 =  _x_start_all[nx] ;
		double dx2 =  _x_end_all[nx];
		// skip if out of range
		if(dx1 < dx2) 
		{
			if(dx1 < xmin || dx2 > xmax) continue;
		}
		else
		{
			if(dx2 < xmin || dx1 > xmax) continue;
		}

		if (_z_data_all[nx].size() != _y_start_all[nx].size()) return false;
		for(int j=0; j<_y_start_all[nx].size();  j++)
		{
			double dy1 =  (_y_start_all[nx])[j] ;
			double dy2 =  (_y_end_all[nx])[j];
			// skip if out of range
			if(dy1 < dy2) 
			{
				if(dy1 < ymin || dy2 > ymax) continue;
			}
			else
			{
				if(dy2 < ymin || dy1 > ymax) continue;
			}
			double value_j = (_z_data_all[nx])[j];
			if(value_j == missing_value) continue;
			 
			if(value_j < zmin_for_log && value_j != 0 ) zmin_for_log = value_j;
			
			if(value_j < zmin) 	zmin = value_j;
			if(value_j > zmax) zmax = value_j;
			
		}
	 }
	 
	if(ColourscaleTab->_colourscale_linlog_fld->currentItem() == QP_LOG)
	{
		if(zmin_for_log == DBL_MAX) 
		{
			zmin = 1.0; // defensive just in case we found no non-zero values
			zmax = 10.0;
		}
		else zmin = zmin_for_log;
	}
	
	// keep for reuse if nothing changed   
	z_min_data = zmin;
	z_max_data = zmax;
	
     return true;
}



// ------------------------------------

QdObject_var QpSpectrumItem::getZDO()
{
	// get zdo object
  	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZDO" << endl;

	QdObject_var zdo = DataTab->_z_data_fld->getSlotObject();

	if(!zdo.is_nil())
	{
		if(_z_obj_id != zdo->get_id())
		{
			_z_obj_id = zdo->get_id();
			
			// get z object fill value
			QdObject_var fillObj;
			if( zdo->xref_exists(FILL_VALUE)) fillObj = zdo->get_xref(FILL_VALUE);
			QdRScalar_var fill_S = QdRScalar_var::narrow(fillObj);
			if(fill_S.is_nil() ) missing_value = 0; // not handling array type fill
			else missing_value = fill_S->scalar();
		}	
	 
	}
	else _z_obj_id = 0;
	
	return zdo;
}

// ------------------------------------

int QpSpectrumItem::fill_X_Data()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_X_Data" << endl;

	if (_zdo.is_nil() ) return 0; // cannot guarantee to create xdo so don't try till zdo exists

	QdObject_var xdo_end, xdo_delta_plus, xdo_delta_minus;
	
	// find x data object _xdo
	if(frameType() == QP_TS)
	{
		_xdo = get_timetags(_zdo);
		if(_xdo.is_nil())
		{ 
			cout << "No time tags for TS spectrogram";
			return 0;
		}
		
		if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
		if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS); 
		if(_zdo->xref_exists(DEPEND_END_0))  xdo_end = _zdo->get_xref(DEPEND_END_0);
	}
	else
	{
		// X-Y Spectrogram
		
		if(DataTab->timeAsSec_toggle->isChecked())
		{
			QdTimeSeq_var tt = get_timetags(_zdo);
			QdTime_var tOffset = QdTime_var::narrow(DataTab->_x_start_data_fld->getSlotObject());
			if(tt.is_nil() || tOffset.is_nil()) return 0;
			
			double dOffset = Seconds(*tOffset);
			QdRScalarSeq_var ss = new QdRScalarSeq(tt->sequence_size());
			for(int i=0; i<tt->sequence_size(); i++)
			{
				ss[i] = ((double)Seconds(tt[i]) ) - dOffset;
			}
			_xdo = ss;
		}
		else
		{
			// use X data slot object
		
			_xdo = DataTab->_x_start_data_fld->getSlotObject();

			if(!_xdo.is_nil())
			{
				xdo_end = DataTab->_x_end_data_fld->getSlotObject();

				if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
				if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
			} 
			else
			{
				// Nothing in X slot
				if(_zdo->sequence_size() == 1){
					if( DataTab->_XYtoggle->isChecked() )
					{
						// swap X and Y is on
						if(_zdo->xref_exists(DEPEND_1)) _xdo = _zdo->get_xref(DEPEND_1);
						else  _xdo = makeIndexObject(_zdo, 1);

						if(_zdo->xref_exists(DEPEND_END_1)) xdo_end = _zdo->get_xref(DEPEND_END_1);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					else
					{
						if(_zdo->xref_exists(DEPEND_2)) _xdo = _zdo->get_xref(DEPEND_2);
						else  _xdo = makeIndexObject(_zdo, 2);

						if(_zdo->xref_exists(DEPEND_END_2)) xdo_end = _zdo->get_xref(DEPEND_END_2);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					if (XYtoggle_in_fill_x != DataTab->_XYtoggle->isChecked() )
					{
						XYtoggle_in_fill_x = DataTab->_XYtoggle->isChecked();
						_x_start_obj_id = 0; // force recalc as XY toggle changed
					}
				}
				else
				{
					if(DataTab->_XYtoggle->isChecked() )
					{
						// swap X and Y is on
						if(_zdo->xref_exists(DEPEND_1)) _xdo = _zdo->get_xref(DEPEND_1);
						else  _xdo = makeIndexObject(_zdo, 1);

						if(_zdo->xref_exists(DEPEND_END_1)) xdo_end = _zdo->get_xref(DEPEND_END_1);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					else
					{
						// use dep 0
						if(_zdo->xref_exists(DEPEND_0))
							_xdo = _zdo->get_xref(DEPEND_0);
						else
						{
							_xdo = makeIndexObject(_zdo, 0);
						}
					}
					if(_zdo->xref_exists(DEPEND_END_0))
						xdo_end = _zdo->get_xref(DEPEND_END_0);
					else if(!_xdo.is_nil())
					{
						if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
						if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
					}
				}
			} // end else on X slot obj set
		} // end else on use T as Sec toggle
	} // end if-else on spectrum type
  	  
	if(_x_start_obj_id == _xdo->get_id() && (xdo_end.is_nil() || _x_end_obj_id == xdo_end->get_id()) )
	{
		return _nx_data;
	}

	_x_start_obj_id = _xdo->get_id();
	if( !xdo_end.is_nil() ) _x_end_obj_id = xdo_end->get_id();

	int nx_data;
	if( !xdo_end.is_nil() ) 
		nx_data = getXData(_xdo, xdo_end);
	else if( !xdo_delta_plus.is_nil() || !xdo_delta_minus.is_nil()) 
		nx_data = getXData(_xdo, xdo_delta_plus, xdo_delta_minus);
	else  
		nx_data = getXData(_xdo);

	return nx_data;
  
}

void QpSpectrumItem::refreshX()
{
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	fill_X_Data();
}

// -----------------------------------------------------

int QpSpectrumItem::fill_Y_Data()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_Y_Data" << endl;
	if ( _zdo.is_nil() ) return -1; // cannot guarantee to create xdo so don't try till zdo exists
  	
	QdObject_var  ydo = DataTab->_y_start_data_fld->getSlotObject();
	QdObject_var ydo_end;
	QdObject_var ydo_delta_minus;
	QdObject_var ydo_delta_plus;
    
	if(!ydo.is_nil())
	{
		ydo_end = DataTab->_y_end_data_fld->getSlotObject();

	}
	else 
	{
		// use metadata off 
		if( DataTab->_XYtoggle->isChecked() )
		{
			if(_zdo->sequence_size() != 1)
			{
				// Depend 0 is being used, so Y is Dep 1
				if(_zdo->xref_exists(DEPEND_1))
					ydo = _zdo->get_xref(DEPEND_1);
				else
					ydo = makeIndexObject(_zdo, 1);
			
				if(_zdo->xref_exists(DEPEND_END_1))
					ydo_end = _zdo->get_xref(DEPEND_END_1);

			}
			else
			{
				// no Y object but swap XY is on
				if(_zdo->xref_exists(DEPEND_2))
					ydo = _zdo->get_xref(DEPEND_2);
				else
				{
					ydo = makeIndexObject(_zdo, 2);
				}
		
				if(_zdo->xref_exists(DEPEND_END_2))
					ydo_end = _zdo->get_xref(DEPEND_END_2);
			}
		}
		else
		{
			if(_zdo->xref_exists(DEPEND_1))
			{
				QdObject_var d1 = _zdo->get_xref(DEPEND_1);
				QdRMatrix_var mv = QdRMatrix_var::narrow(d1);
				QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(d1);

				if(mv.is_nil() ) ydo = ms;
				else ydo = mv;
			}
			
			if(ydo.is_nil())
				ydo = makeIndexObject(_zdo, 1);
			
			if(_zdo->xref_exists(DEPEND_END_1))
				ydo_end = _zdo->get_xref(DEPEND_END_1);
			
		}
	
		if (XYtoggle_in_fill_y != DataTab->_XYtoggle->isChecked() )
		{
			XYtoggle_in_fill_y = DataTab->_XYtoggle->isChecked();
			_y_start_obj_id = 0; // force update
		}

	}
  
	// ydo must exist by now as we make one if need be
	if(ydo->xref_exists(DELTA_PLUS))
		ydo_delta_plus = ydo->get_xref(DELTA_PLUS);
	if(ydo->xref_exists(DELTA_MINUS))
		ydo_delta_minus = ydo->get_xref(DELTA_MINUS);

	_ydo = ydo;

	
	// ---- ONLY UPDATE IF INPUTS CHANGED ------
	if(_y_start_obj_id == ydo->get_id() && (ydo_end.is_nil() || _y_end_obj_id == ydo_end->get_id())) 
	{
		return _ny_data;
	}

	_y_start_obj_id = ydo->get_id();
	if(!ydo_end.is_nil()) _y_end_obj_id = ydo_end->get_id();
		
	// fill the Y axis bins for each x bin (inhomogeneous arrays OK)
	int ny_data;
	if( !ydo_end.is_nil() )
	{  
		ny_data = getYData( ydo, ydo_end);
	}
	else if( !ydo_delta_plus.is_nil() || !ydo_delta_minus.is_nil()) 
	{
		ny_data = getYData(ydo, ydo_delta_plus, ydo_delta_minus);
	}
	else 
	{ 
		ny_data = getYData(ydo );
	}
	// NOTE: getYData() returns -1 for error, last array dim for sequences, array dim for matrix
	return ny_data;

}

// -------------------------------------

int QpSpectrumItem::fill_Z_Data( int XtoY, bool inPlot)
{
  	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_Z_Data" << endl;
	_z_data_all.clear();
	_z_data_all.resize(_nx_data);

	if(_nx_data != _y_start_all.size()) 
	{
		if ( QuSlotDebug() ) cout << "Array dimensions not correct" << endl;
		return -1;
	}
	
	QdRMatrixSeq_var rmatrixseq_z = QdRMatrixSeq_var::narrow(_zdo);
	QdRMatrix_var    rmatrix_z    = QdRMatrix_var::narrow(_zdo);
  
	if( !rmatrixseq_z.is_nil() ) 
	{	    	 
		if(rmatrixseq_z->elt(0).rank() == 1)
		{
			// Try 1D Mat sequence
			for(int nx=0; nx<_nx_data; ++nx){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);
				_z_data_all[nx]=rmatrixseq_z->elt(nx);
			}
		}
		else if(rmatrixseq_z->elt(0).rank() == 2)
		{
			// Try 2D Mat sequence of 1 record 
			if (rmatrixseq_z->sequence_size() != 1) {
                if(inPlot)
				    plmtex("lv",-2.0,0.5,0.0, "Z data is a 2D matrix sequence with more than one record");
                else
                    std::cout << "Z data is a 2D matrix sequence with more than one record" << std::endl;
				return -1;
			}
	  
			double value_j;	   
			for(int nx=0; nx< _nx_data; ++nx){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);
				for(int j=0; j< ny; ++j)
				{
					if (XtoY == 1) value_j = (rmatrixseq_z->elt(0))[j][nx];
					else           value_j =  (rmatrixseq_z->elt(0))[nx][j];

					_z_data_all[nx][j] = value_j;
				}
			}
		}
		else
		{
            if(inPlot)
			    plmtex("lv",-2.0,0.5,0.0, "Z data is a matrix sequence, but not 1D or 2D with one record");
            else
                std::cout << "Z data is a matrix sequence, but not 1D or 2D with one record" << std::endl;
			return -1;
		}

	}
	else if( !rmatrix_z.is_nil() )
	{
		if(rmatrix_z->rank() == 2)
		{	   
			double value_j;
			for(int nx=0; nx< _nx_data; ++nx){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);
				for(int j=0; j< ny; ++j)
				{
					if (XtoY == 1) value_j =  (*rmatrix_z)[j][nx];
					else      value_j =  (*rmatrix_z)[nx][j];
			 
					_z_data_all[nx][j] = value_j;
				}
			}
		}
		else
		{
            if(inPlot)
			    plmtex("lv",-2.0,0.5,0.0, "Z data is a matrix, but not 2D or 1D sequence");
            else
                std::cout << "Z data is a matrix, but not 2D or 1D sequence" << std::endl;
			return -1;
		}
  
	} 
	else
	{
        if(inPlot)
		    plmtex("lv",-2.0,0.5,0.0, "Z data is not a matrix or matrix sequence");
        else
            std::cout << "Z data is not a matrix or matrix sequence" << std::endl;
		return -1;
	}
	
	return 0;
}
// ----------------------------------

void QpSpectrumItem::setDefaultOptions()
{
	DataTab->timeAsSec_toggle->setChecked(false);
	DataTab->_XYtoggle->setChecked(false);
	DataTab->gap_toggle->setChecked(false);
	
	
	ColourscaleTab->_colourscale_min_fld->setText("");
	ColourscaleTab->_colourscale_max_fld->setText("");
	ColourscaleTab->_colourscale_auto_fld->setCurrentItem(QP_FROM_META);
	ColourscaleTab->_colourscale_linlog_fld->setCurrentItem(QP_LINEAR);
	ColourscaleTab->_colourscale_min_fld->setText("");
	ColourscaleTab->_colourscale_show_colour_bar_fld->setChecked(true);
	ColourscaleTab->_colourscale_colour_bar_width_fld->setValue(QPUI_COLOUR_BAR_WIDTH);
	ColourscaleTab->_colourscale_colour_bar_height_fld->setValue(QPUI_COLOUR_BAR_HEIGHT);
	ColourscaleTab->_colourscale_colour_bar_voffset_fld->setValue(QPUI_COLOUR_BAR_VOFFSET);
	ColourscaleTab->_colourscale_colour_bar_disp_fld->setValue(QPUI_COLOUR_BAR_DISP);
	ColourscaleTab->_colourscale_table_fld->setCurrentItem(QPUI_RAINBOW);
	ColourscaleTab->_colourscale_label_fld->setText("");
	ColourscaleTab->_colourscale_autolabel_fld->setChecked(true);
	
	updateAxes();
}



// --------------------------------------

void QpSpectrumItem::updateAxes()
{
  	if ( QuSlotDebug() ) cout << "QpSpectrumItem::updateAxes" << endl;
	
	I_Panel->I_Frame->updateAxes();
		
	if(ColourscaleTab->_colourscale_auto_fld->currentItem() != QP_FROM_USER) ColourscaleTab->setColourscaleMinMax();
	if(ColourscaleTab->_colourscale_autolabel_fld->isChecked())  ColourscaleTab->setColourScaleAutoLabel();

}

// --------------------------------------
#include <sys/time.h>

QpuiReturnStatus QpSpectrumItem::plot()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::plot" << endl;

	if(I_Panel->yAxisTab->_axis_from_xy_fld->text().isEmpty()) 
	{
		plmtex("lv",-10.0,0.5,0.0,"Y data ?? (check Y-axis range or component selection)");
		return QPUI_FAILURE;
	}
	
	if(frameType() == QP_TS)
	{
		if(I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->text().isEmpty()) 
		{
			plmtex("lv",-10.0,0.5,0.0,"T Data ?? (check T-axis range or time tags)");
			return QPUI_FAILURE;
		}
	}
	else
	{
		if(I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->text().isEmpty()) 
		{
			plmtex("lv",-10.0,0.5,0.0,"X Data ?? (check X-axis range, DEPEND_0 or X object)");
			return QPUI_FAILURE;
		}
	}
	
		// set the colour table
	I_Panel->I_Frame->I_Page->set_colour_table((QpuiColourTable)ColourscaleTab->_colourscale_table_fld->currentItem());
	
	// plots the SpectroGram Object TS or XY
	
	double nominal_spacing = DBL_MAX;
	
	if ( _zdo.is_nil() ){
		plmtex("lv",-2.0,0.5,0.0,"No Z data object");
		return QPUI_FAILURE;
	}
		
	if ( _nx_data == 0 ){
		plmtex("lv",-2.0,0.5,0.0,"Bad X axis object");
		return QPUI_FAILURE;
	}
	
		// populate all y axis bins
	
	if ( _ny_data == -1 ){
		plmtex("lv",-2.0,0.5,0.0,"Bad Y axis object(s)");
		return QPUI_FAILURE;
	}
	
	// --------------------------------------------------
		// Z data object
	if(_nx_data != _zdo->sequence_size() && _zdo->sequence_size() != 1)
	{
		plmtex("lv",-2.0,0.5,0.0,"Z and X lengths differ");
		return QPUI_FAILURE;
	}
	int XtoY = isZConformal(_nx_data, _ny_data);
	if( XtoY == -1)  {
		plmtex("lv",-2.0,0.5,0.0, "Z 2D matrix dimensions don't match axes");
		return QPUI_FAILURE;
	}

	if( fill_Z_Data( XtoY) == -1)  return QPUI_FAILURE;
	
	// set up colour scale
	double zmin = ColourscaleTab->_colourscale_min_fld->value();
	double zmax = ColourscaleTab->_colourscale_max_fld->value();
	if(zmin >= zmax) 
	{
		plmtex("lv",-2.0,0.5,0.0, "Invalid Colourscale Range");
		return QPUI_FAILURE;
	}
	
	if(ColourscaleTab->_colourscale_linlog_fld->currentItem() == QP_LOG)  {
		zmin = SafeLog10(zmin);
		zmax = SafeLog10(zmax);
	}
	
		// find  T axis offset for plot
	
	double x_offset=0.0;
	double x_min, x_max;
	
	if(frameType() == QP_TS)
	{
		x_min = I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->value();
		x_max = I_Panel->I_Frame->tAxisTab->_axis_to_time_fld->value();
	}
	else
	{
		x_min = I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->value();
		x_max = I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->value();
	}
	
	if ( frameType() == QP_TS )
	{
		Time t_plot_start;
		t_plot_start.set_time_rep(x_min);
		Seconds t_midnite(t_plot_start);
		t_midnite -= Seconds(t_plot_start.second_of_day());
		x_offset = (double)t_midnite; 
	}
	
	double zrange = zmax - zmin;
	PLFLT x[4], y[4];
	
	// plot data 
	int plot_id=I_Panel->I_Frame->I_Page->PageTab->_xwin_num_fld->text().toInt();
	PlotWindow* win=I_Panel->I_Frame->I_Page->focusWindow(plot_id);
	
	win->setSmooth(false);

	double y_min = I_Panel->yAxisTab->_axis_from_xy_fld->value();
	double y_max = I_Panel->yAxisTab->_axis_to_xy_fld->value();

	double dx1, dx2, dy1, dy2, zval;
	bool isXLog=(I_Panel->I_Frame->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG);
	bool isYLog=(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG);
	bool isZLog=(ColourscaleTab->_colourscale_linlog_fld->currentItem() == QP_LOG);
	
	for(int nx=0; nx<_x_start_all.size(); ++nx)
	{
		dx1 =  _x_start_all[nx] ;
		dx2 =  _x_end_all[nx];
		
		// skip if both bin edges are out of range
		if((dx1 < x_min || dx1 > x_max) && ((dx2 < x_min || dx2 > x_max))) continue;
		
		
		if(isXLog)
		{
			x[0] = (PLFLT) SafeLog10(dx1 - x_offset);
			x[2] = (PLFLT) SafeLog10(dx2 - x_offset);
		}
		else
		{
			x[0] = (PLFLT) (dx1 - x_offset);
			x[2] = (PLFLT) (dx2 - x_offset);
		}
		x[1] = x[0];
		x[3] = x[2];
		
		for(int j=0; j<_y_start_all[nx].size(); ++j)
		{
			dy1 = (_y_start_all[nx])[j];
			dy2 = (_y_end_all[nx])[j];	
		
					// skip if out of range
			if((dy1 < y_min || dy1 > y_max) && (dy2 < y_min || dy2 > y_max)) continue;
					
			if(isYLog)
			{
				y[0] = SafeLog10(dy1);
				y[1] = SafeLog10(dy2);
			}
			else
			{
				y[0] = (PLFLT) dy1;
				y[1] = (PLFLT) dy2;
			}
			y[2] = y[1];
			y[3] = y[0];
		
			zval=(_z_data_all[nx])[j];
			if( zval != missing_value && finite(zval)) // ALBAN: changed for nothing plotted if data gap
			{
				PLFLT col;
				if(isZLog) 
					col=(SafeLog10(zval) - zmin)/zrange;
				else
					col=(zval-zmin)/zrange;
					
				if(col <0.) col=0.;
				if(col>=0.999) col=0.999;
				plcol1(col);

				plfill(4, x, y);
			}
	
		}
	}

	plcol0(1);

	// set colour bar
	if(ColourscaleTab->_colourscale_show_colour_bar_fld->isChecked())
	{
		PLFLT x1win_save, x2win_save, y1win_save, y2win_save;
		PLFLT x1vp_save, x2vp_save, y1vp_save, y2vp_save;
	
		plgvpw(&x1win_save, &x2win_save, &y1win_save, &y2win_save);
		plgvpd(&x1vp_save, &x2vp_save, &y1vp_save, &y2vp_save);
	
		PLFLT chsize_ndc, dummy;
		
		chsize_ndc=getCharHeight();
	
		x[0] = x2vp_save + ColourscaleTab->_colourscale_colour_bar_disp_fld->value() * chsize_ndc*1.5;
		x[2] = x[0] +  ColourscaleTab->_colourscale_colour_bar_width_fld->value() * chsize_ndc;
		float ht = (y2vp_save - y1vp_save) ;
		y[0] = y1vp_save + ht * ColourscaleTab->_colourscale_colour_bar_voffset_fld->value();
		y[1] = y[0] + ht * ColourscaleTab->_colourscale_colour_bar_height_fld->value();
	
		plvpor(x[0],x[2], y[0], y[1]);
		
		plwind(0., 1., zmin, zmax);
	
		x[0]=0.;
		x[1]=0.;
		x[2]=1.;
		x[3]=1.;

		PLFLT zstep=(zmax-zmin)/200.;
		PLFLT i=0.;
		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./200.)
		{
			y[0] = col;
			y[3] = col;
			y[1] = col+zstep;
			y[2] = y[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}
	
		plcol0(1);
		win->setSmooth(true);
		string yopt = "bcmtsv";
		if(ColourscaleTab->_colourscale_linlog_fld->currentItem() == QP_LOG) yopt += "l";
		plschr(0., I_Panel->I_Frame->DisplayTab->_ticks_char_size_fld->value());
		
		plwid(QtWidth2PlWidth(I_Panel->I_Frame->DisplayTab->_line_width_fld->value()));

		PLFLT _ticks_length = ColourscaleTab->_colourscale_ticklength_fld->value();
		
		plsmaj(0., _ticks_length);
		plsmin(0., _ticks_length*0.6);
		plbox("bc", 0.0, 0, yopt.c_str(), 0.0, 0);
		string label="";
	
		plwid(0);
		
		label = ColourscaleTab->_colourscale_label_fld->text().latin1();
		
		plmtex("l", 1., 0.5, 0.5, SafePGPString(label).c_str());
		plvpor(x1vp_save, x2vp_save, y1vp_save, y2vp_save);
		plwind(x1win_save, x2win_save, y1win_save, y2win_save);
		win->setSmooth(false);
	}
	
	return QPUI_SUCCESS;
}



//     ----------------------

Vistream& operator>>(Vistream& v_is, QpSpectrumItem& spectrogram)
{
  // streams from virtual input stream v_is into spectrogram object

	QpPlotItem& plot_item = spectrogram;
	v_is >> plot_item;
	
	bool save_x = false;
	int type;
	v_is >> type;
	spectrogram.I_Type = (QpItemType) type;

	string z_data_str; 
	v_is >> z_data_str;
	spectrogram.DataTab->_z_data_fld->setText(z_data_str.c_str()); 

	string y_str;
    v_is >> y_str;
	spectrogram.DataTab->_y_start_data_fld->setText(y_str.c_str());
    v_is >> y_str;
	spectrogram.DataTab->_y_end_data_fld->setText(y_str.c_str());
	
	bool _fill_gaps;
	v_is >> _fill_gaps;
	spectrogram.DataTab->gap_toggle->setChecked(_fill_gaps);
	
  
	if(spectrogram.frameType() == QP_XY)
	{
		// only restore xy data if an xy spectrogram
		save_x = true;
		string x_str;
		v_is >> x_str;
		spectrogram.DataTab->_x_start_data_fld->setText(x_str.c_str());
		v_is >> x_str;
		spectrogram.DataTab->_x_end_data_fld->setText(x_str.c_str());			

		bool TasSecToggle;;
		v_is >> TasSecToggle;
		spectrogram.DataTab->timeAsSec_toggle->setChecked(TasSecToggle);
	}

  	string linkText;
	v_is >> linkText;
	spectrogram.setText(LINK_COL, linkText.c_str());
	// do the relinking in listview restore after traces etc are there

	QpColourscaleTab* ct = spectrogram.ColourscaleTab;
	int cs_auto;
	v_is >> cs_auto;
	ct->_colourscale_auto_fld->setCurrentItem(cs_auto);
	double _min;
	v_is >> _min;
	ct->_colourscale_min_fld->setDouble(_min);
	double _max;
	v_is >> _max;
	ct->_colourscale_max_fld->setDouble(_max);
	v_is >> spectrogram.missing_value;
	bool _linlog;
	v_is >> _linlog;
	ct->_colourscale_linlog_fld->setCurrentItem(_linlog);
	int _colour_table;
	v_is >> _colour_table;
	ct->_colourscale_table_fld->setCurrentItem(_colour_table);
	string _label;
	v_is >> _label;
	ct->_colourscale_label_fld->setText(_label.c_str());
	bool _show_colour_bar;
	v_is >> _show_colour_bar;
	ct->_colourscale_show_colour_bar_fld->setChecked(_show_colour_bar);
	float _colour_bar_width;
	v_is >> _colour_bar_width;
	ct->_colourscale_colour_bar_width_fld->setValue(_colour_bar_width);
	float _colour_bar_disp;
	v_is >> _colour_bar_disp;
	ct->_colourscale_colour_bar_disp_fld->setValue(_colour_bar_disp);
	bool _autolabel;
	v_is >> _autolabel;
	ct->_colourscale_autolabel_fld->setChecked(_autolabel);
  
	float _colour_bar_height;
	v_is >> _colour_bar_height;
	ct->_colourscale_colour_bar_height_fld->setValue(_colour_bar_height);
	float _colour_bar_voffset;
	v_is >> _colour_bar_voffset;
	ct->_colourscale_colour_bar_voffset_fld->setValue(_colour_bar_voffset);

	if( qsas_restore_release > 20306)
	{
		float tickLength;
		v_is >> tickLength;
		ct->_colourscale_ticklength_fld->setValue(tickLength);
	}
	
	spectrogram.DataTab->_z_data_fld->reValidate();
	
	return v_is;
}

// -----------------------------------------

Vostream& operator<<(Vostream& v_os, QpSpectrumItem& spectrogram)
{
	// streams spectrogram object to virtual output stream v_os

	QpPlotItem& plot_item = spectrogram;
	v_os << plot_item;

	v_os << spectrogram.I_Type;
	v_os << string(spectrogram.DataTab->_z_data_fld->text().latin1());
	v_os << string(spectrogram.DataTab->_y_start_data_fld->text().latin1());
	v_os << string(spectrogram.DataTab->_y_end_data_fld->text().latin1());
	bool _fill_gaps = spectrogram.DataTab->gap_toggle->isChecked();
	v_os << _fill_gaps;

	bool save_x=false;
	if(spectrogram.frameType() == QP_XY)
	{
		// only save xy data if an XY SpectroGram
		save_x=true;

		v_os << string(spectrogram.DataTab->_x_start_data_fld->text().latin1());
		v_os << string(spectrogram.DataTab->_x_end_data_fld->text().latin1());
		bool TasSecToggle = spectrogram.DataTab->timeAsSec_toggle->isChecked();
		v_os << TasSecToggle;
		
	}

	string linkText;
	if(!spectrogram.text(LINK_COL).isEmpty())
	{	
		linkText = string(spectrogram.text(LINK_COL).latin1());
		linkText += "S"; // temporary to avoid conflict with existing links on restore
	}
	else linkText="";
	v_os << linkText;
	

	QpColourscaleTab* ct = spectrogram.ColourscaleTab;
	int cs_auto = ct->_colourscale_auto_fld->currentItem();
	v_os << cs_auto;
	double _min = ct->_colourscale_min_fld->value();	
	v_os << _min;
	double _max = ct->_colourscale_max_fld->value();
	v_os << _max;
	v_os << spectrogram.missing_value;
	bool _linlog = ct->_colourscale_linlog_fld->currentItem();
	v_os << _linlog;
	int _colour_table = ct->_colourscale_table_fld->currentItem();
	v_os << _colour_table;
	string _label = ct->_colourscale_label_fld->text().latin1();
	v_os << _label;
	bool _show_colour_bar = ct->_colourscale_show_colour_bar_fld->isChecked();
	v_os << _show_colour_bar;
	float _colour_bar_width = ct->_colourscale_colour_bar_width_fld->value();
	v_os << _colour_bar_width;
	float _colour_bar_disp = ct->_colourscale_colour_bar_disp_fld->value();
	v_os << _colour_bar_disp;
	bool _autolabel = ct->_colourscale_autolabel_fld->isChecked();
	v_os << _autolabel; 
	float _colour_bar_height = ct->_colourscale_colour_bar_height_fld->value();
	v_os << _colour_bar_height;
	float _colour_bar_voffset = ct->_colourscale_colour_bar_voffset_fld->value();
	v_os << _colour_bar_voffset;
	float _colour_bar_ticklength = ct->_colourscale_ticklength_fld->value();
	v_os << _colour_bar_ticklength;
	
	return v_os;
}

QdObject_var makeIndexObject(QdObject_var zdo, int indxFlag){
  QdObject_var retObj;
  QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(zdo);
  QdRMatrix_var mm    = QdRMatrix_var::narrow(zdo);
  int indx = indxFlag-1;
  int len=0;
  
  if (indxFlag == 0 && !ms.is_nil() ) 
  {
     // use sequence length
	 len = ms->sequence_size();
  }
  else if( !ms.is_nil() )
  {
        // use index
        if(ms->elt(0).rank() > indx)
	    {
	      len = ms->elt(0).dimdata()[indx];
	    }
	    else 
	    {
	      len = ms->sequence_size();
	    }
  
  }
  else if( !mm.is_nil() )
  { 
        // use index
        if(mm->rank() > indx)
	    {
	      len = mm->dimdata()[indx];
	    }
  
  }
  
  // create a matrix to return
  if (len > 0)
  {
        // create matrix whose values are the index 
        QdRMatrix_var dimIdxObj = new QdRMatrix(Dim(len));
	    for (int i=0; i<len; i++) (*dimIdxObj)[i] = i;
	    retObj = dimIdxObj;
  }
  else
  {
    // This should never be reached, but create something in case
	cout << "CANNOT create axis object " << indxFlag << " (t-axis=0; x-axis=1, yaxis=2)"  << endl;
	QdRMatrix_var dimIdxObj = new QdRMatrix(Dim(1));
	(*dimIdxObj)[0] = 0;
	retObj = dimIdxObj;
  }
  
  return retObj;
}
 
 
