
#include "MatrixEig.h"
//#include <g2c.h>

namespace QDOS {
	
	double house(double * x, int n, double * v)
	{
		int i;
		double beta=0., norm=0., sigma=0.;
	
		for(i=1; i<n; i++)
		{
			v[i]=x[i];
			sigma+=x[i]*x[i];
		}
		v[0]=1.;
	
		if(sigma!=0.)
		{
			double mu=std::sqrt(x[0]*x[0]+sigma);
			if(x[0]<0.)
			{
				v[0]=x[0]-mu;
			}
			else
			{
				v[0]=-sigma/(x[0]+mu);
			}
			beta=2*v[0]*v[0]/(sigma+v[0]*v[0]);
			for(i=1; i<n; i++)
			{
				v[i]=v[i]/v[0];
			}
			v[0]=1.;
		}
		return beta;
	}

	void tridiag(double * a, double * q, int n)
	{
		int i, j, k, sub_size;
		double beta, coeff, norm;
		double * v=new double[n-1]; // Container for Householder vectors
		double * p=new double[n-1]; // Container for Householder matrices
		double * w=new double[n-1];
		double * beta_arr=new double[n-1];
		double * q_temp=new double [n*n];
	
		for(k=0; k<n; k++)
		{
			q[k*n+k]=1.;
		}
	
		for(k=0; k<n-2; k++)
		{
			double * x=&a[k*n+k+1];
			sub_size=n-k-1;
		
			beta=house(x, sub_size, v);
			beta_arr[k]=beta;
		//p=b*a(k+1:n, k+1:n)*v;
		
			for(i=0; i<sub_size; i++)
			{
				p[i]=0.;
				for(j=0; j<sub_size; j++)
				{
					p[i]+=a[(i+k+1)*n+j+k+1]*v[j];
				}
				p[i]*=beta;
			}
		
		//w=p-(b*p'*v/2)*v;
			coeff=0.;
			norm=0.;
			for(i=0; i<sub_size; i++)
			{
				coeff+=p[i]*v[i];
				norm+=a[(i+k+1)*n+k]*a[(i+k+1)*n+k];
			}
			coeff=coeff*beta/2.;
			for(i=0; i<sub_size; i++)
			{
				w[i]=p[i]-coeff*v[i];
			}
		
		//a(k+1, k)=norm(a(k+1:n, k)); a(k, k+1)=a(k+1, k);
			norm=std::sqrt(norm);
			a[(k+1)*n+k]=a[k*n+k+1]=norm;
		
			for(i=0; i<sub_size; i++)
			{
				for(j=0; j<=i; j++)
				{
					a[(i+k+1)*n+j+k+1]=a[(j+k+1)*n+i+k+1]=a[(i+k+1)*n+j+k+1]-v[i]*w[j]-w[i]*v[j];
				}
			// v storage for Q computation
				if(i!=0)
				{
					a[(i+k+1)*n+k]=v[i];
				}
			}
		}
	
		for(k=n-3; k>=0; k--)
		{
			sub_size=n-k-1;
			for(i=0; i<sub_size; i++)
			{
				if(i==0)
					v[i]= 1;
				else
				{
					v[i]= a[(i+k+1)*n+k];
					a[(i+k+1)*n+k]=a[k*n+i+k+1]=0.;
				}
			}
		// Copy of q in q_temp
			for(i=0; i<sub_size; i++)
			{
				for(j=0; j<sub_size; j++)
				{
					q_temp[(i+k+1)*n+j+k+1]=q[(i+k+1)*n+j+k+1];
				}
			}
			for(i=0; i<sub_size; i++)
			{
				for(j=0; j<sub_size; j++)
				{
					coeff=0.;
					for(int l=0; l<sub_size; l++)
					{
						coeff+=v[l]*q_temp[(l+k+1)*n+j+k+1];
					}
					q[(i+k+1)*n+j+k+1]-=beta_arr[k]*coeff*v[i];
				}
			}
		}

		delete[] p;
		delete[] v;
		delete[] w;
		delete[] beta_arr;
		delete[] q_temp;
	}

	void givens(double a, double b, double * c, double * s)
	{
		if(b==0)
		{
			*c=1.;
			*s=0.;
		}
		else
		{
			double tau;
			if(fabs(b)>fabs(a))
			{
				tau=-a/b;
				*s=1./std::sqrt(1+tau*tau);
				*c=(*s)*tau;
			}
			else
			{
				tau=-b/a;
				*c=1./std::sqrt(1+tau*tau);
				*s=(*c)*tau;
			}
		}
	}

	void apply_givens(double * a, int size, double c, double s, int i, int k, int p, int q)
	{
		int j;
		double t1, t2;
	
		for(j=p; j<q; j++)
		{
			t1=a[i*size+j];
			t2=a[k*size+j];
			a[i*size+j]=c*t1-s*t2;
			a[k*size+j]=s*t1+c*t2;
		}
	
		for(j=p; j<q; j++)
		{
			t1=a[j*size+i];
			t2=a[j*size+k];
			a[j*size+i]=c*t1-s*t2;
			a[j*size+k]=s*t1+c*t2;
		}
	}

	void symmqr_step(double * t, double * Q, int size, int p, int q)
	{
		double d=(t[(q-2)*size+q-2]-t[(q-1)*size+q-1])/2.;
		double tq2=t[(q-1)*size+q-2]*t[(q-1)*size+q-2];
		double sg=(d>0. ? 1. : (d<0. ? -1. : 0.));
		double mu=t[(q-1)*size+q-1]-tq2/(d+sg*std::sqrt(d*d+tq2));
	
		double x=t[p*size+p]-mu;
		double z=t[(p+1)*size+p];
		int k, j;
		double c, s;
		double t1, t2;
		for(k=p; k<q-1; k++)
		{
			givens(x, z, &c, &s);
			apply_givens(t, size, c, s, k, k+1, p, q);
		
			for(j=0; j<size; j++)
			{
				t1=Q[j*size+k];
				t2=Q[j*size+k+1];
				Q[j*size+k]=c*t1-s*t2;
				Q[j*size+k+1]=s*t1+c*t2;
			}
		
			if(k<q-2)
			{
				x=t[(k+1)*size+k];
				z=t[(k+2)*size+k];
			}
		}
	}

	void symmqr(double* t, double * Q, int size)
	{
		int p, q, i, end_loop, iter;
		p=0;
		q=size-1;
		end_loop=0;
	
	
		while(q>p)
		{
			for(i=0; i<size-1; i++)
			{
				if(fabs(t[i*size+i+1])<=1e-10)
				{
					t[i*size+i+1]=t[(i+1)*size+i]=0.;
				}
			}
		
			end_loop=0;
			iter=q-1;
			while(!end_loop && iter>0)
			{
				if(t[(iter+1)*size+iter]==0.)
				{
					q=iter;
					iter--;
				}
				else
				{
					end_loop=1;
				}
			}
		
			end_loop=0;
			iter=p;
			while(!end_loop && iter<size-1)
			{
				if(t[(iter+1)*size+iter]==0.)
				{
					p=++iter;
				}
				else
				{
					end_loop=1;
				}
			}
		
			if(q>p)
			{
				symmqr_step(t, Q, size, p, q+1);
			//display_mat(t, size, size);
			}
		
		}
	}

	
	
	void sort(double * a, double * q, int size)
	{
		double temp;
		for(int i=0; i<size; i++)
		{
			for(int j=0; j<size-i-1; j++)
			{
				if(a[j*size+j]>a[(j+1)*size+j+1])
				{
					temp=a[(j+1)*size+j+1];
					a[(j+1)*size+j+1]=a[j*size+j];
					a[j*size+j]=temp;
					
					for(int k=0; k<size; k++)
					{
						temp=q[k*size+j+1];
						q[k*size+j+1]=q[k*size+j];
						q[k*size+j]=temp;
					}
				
				}
			}
		}
	}
	
	void eigen(double * a, double * v, int n)
	{
		double * q=new double[n*n];
		memset(q, 0x0, n*n*sizeof(double));
		tridiag(a, q, n);
		
		symmqr(a, q, n);
	
		sort(a, q, n);
		
		for(int i=0; i<n; i++) v[i]=a[i*n+i];
		memcpy(a, q, n*n*sizeof(double));
		//double * q_temp=q;
		delete[] q;
	}


}  // end namespace QDOS        

