#include "qprofile.h"
#include <iostream>

using namespace std;

QProfilesList * QSAS_profiles;

QString addTokens(const QString& in)
{
#ifndef WINDOWS
	QString home=QString(getenv("HOME"));
#else
	QString home=QString(getenv("USERPROFILE"));
#endif
	QString qsas_home=QString(getenv("QSAS_HOME"));
	
	QString res=in;
	res.replace(qsas_home, QString("${QSAS_HOME}"));
	res.replace(home, QString("${HOME}"));
	return res;
}

QString removeTokens(const QString& in)
{
#ifndef WINDOWS
	QString home=QString(getenv("HOME"));
#else
	QString home=QString(getenv("USERPROFILE"));
#endif
	QString qsas_home=QString(getenv("QSAS_HOME"));
	
	QString res=in;
	res.replace(QString("${QSAS_HOME}"), qsas_home);
	res.replace(QString("${HOME}"), home);
	return res;
}

QProfile::QProfile(QString name)
{
	this->name=name;
	
#ifndef WINDOWS
	char * home=getenv("HOME");
	char * qsas_home=getenv("QSAS_HOME");
	
	SaveRestorePath=QString(qsas_home).append("/tmp");
	UserLibPath=QString(home).append("/MyQsas/lib");
	ConfigPath=QString(qsas_home).append("/examples");
	FilterBankPath=QString(qsas_home).append("/examples/FilterBank");
#else
	char * windir=getenv("WINDIR");
	char * home=getenv("USERPROFILE");
	char * qsas_home=getenv("QSAS_HOME");
	
	SaveRestorePath=QString(qsas_home).append("\\tmp");
	UserLibPath=QString(home).append("\\MyQsas\\lib");
	ConfigPath=QString(qsas_home).append("\\examples");
	FilterBankPath=QString(qsas_home).append("\\examples\\FilterBank");
#endif
	
	Vector3D="R_XYZ";
	Array3D="iSS";
	Array3DtoScalar="SSS";
	Array3DXY="iiSt";
	Array3DtoScalarXY="iSSt";
	PageSize="A4";
	PageOrientation="Landscape";
	PageDPI=150;
	PageWidth=8.; //inches
	PageHeight=11.694;
	TimeRep="HMS";
	UseKPrinter=0;
	SansFont="";
	SerifFont="";
	MonotypeFont="";
	DecorativeFont="";
	VectoriseSVGFonts=false;
	ColourTable="HSV(0,260,1,1)(.1,260,1,.5)(.9,0,1,.5)(1,0,1,0)";
	LineWidth=1.;
	BaseFontSize=0.5;
    gapTolerance=1.5;
	
	Height=40;
	QpuiHeight=700;
	QpuiWidth=900;
	fontSize=DEFAULT_FONT_SIZE;
	GUImargins=4; // Value for spacing and margins
	timeIntervalButton=0;
	
	Debug=0;
	QrtnDebug=0;
	
}

QProfile::QProfile(QProfile * p)
{
	this->name=p->name;
	
	SaveRestorePath=p->SaveRestorePath;
	UserLibPath=p->UserLibPath;
	ConfigPath=p->ConfigPath;
	FilterBankPath=p->FilterBankPath;
	
	Vector3D=p->Vector3D;
	Array3D=p->Array3D;
	Vector3D=p->Vector3D;
	Array3DXY=p->Array3DXY;
	Array3DtoScalarXY=p->Array3DtoScalarXY;
	PageSize=p->PageSize;
	PageDPI=p->PageDPI;
	PageOrientation=p->PageOrientation;
	PageWidth=p->PageWidth;
	PageHeight=p->PageHeight;
	TimeRep=p->TimeRep;
	UseKPrinter=p->UseKPrinter;
	SansFont=p->SansFont;
	SerifFont=p->SerifFont;
	MonotypeFont=p->MonotypeFont;
	DecorativeFont=p->DecorativeFont;
	VectoriseSVGFonts=p->VectoriseSVGFonts;
	ColourTable=p->ColourTable;
	LineWidth=p->LineWidth;
	BaseFontSize=p->BaseFontSize;
    gapTolerance=p->gapTolerance;
	
	Height=p->Height;
	QpuiHeight=p->QpuiHeight;
	QpuiWidth=p->QpuiWidth;
	fontSize=p->fontSize;
	GUImargins=p->GUImargins;
	timeIntervalButton=p->timeIntervalButton;
	
	Debug=p->Debug;
	QrtnDebug=p->QrtnDebug;
	
}

QProfilesList::QProfilesList()
{
	defaultProfile=new QProfile;
	customProfile=NULL;
	currentName=defaultProfile->getName();
}

QProfilesList::~QProfilesList()
{
	for(int i=0; i<presets.size(); i++)
		delete presets[i];
	presets.clear();
	
	delete defaultProfile;
	delete customProfile;
}

bool QProfilesList::load()
{
#ifndef WINDOWS
	char * home=getenv("HOME");
	string file_name=string(home).append("/.qsasrc");
#else
	char * home=getenv("USERPROFILE");
	string file_name=string(home).append("\\qsas.ini");
#endif
	ifstream file(file_name.c_str());
	
	QProfile * currentProfile=NULL;
	
	char tag[50];
	char myStr[500];
	QString myQStr;
	int myInt;
	float myFloat;
	double myDouble;
	bool myBool;
	while (file.good())
	{
		file >> tag;
		if(strcmp(tag, "CURRENT_PROFILE")==0)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentName=myQStr;
		}
		else if(strcmp(tag, "PROFILE")==0)
		{
			file.getline(myStr, 500);
			if(currentProfile!=NULL)
			{
				add(currentProfile);
			}
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile=fromName(myQStr);
			if(currentProfile==NULL)
				currentProfile=new QProfile(myQStr);
		}
		else if(strcmp(tag, "3D_VECTOR")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->Vector3D=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->Array3D=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY_TO_SCALAR")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->Array3DtoScalar=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY_XY")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->Array3DXY=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY_TO_SCALAR_XY")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->Array3DtoScalarXY=myQStr;
		}
		else if(strcmp(tag, "PAGE_SIZE")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->PageSize=myQStr;
		}
		else if(strcmp(tag, "PAGE_ORIENTATION")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->PageOrientation=myQStr;
		}
		else if(strcmp(tag, "PAGE_WIDTH")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->PageWidth=myDouble;
		}
		else if(strcmp(tag, "PAGE_HEIGHT")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->PageHeight=myDouble;
		}
		else if(strcmp(tag, "PAGE_DPI")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->PageDPI=myInt;
		}
		else if(strcmp(tag, "TIME_REPRESENTATION")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->TimeRep=myQStr;
		}
		else if(strcmp(tag, "USE_KPRINTER")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->UseKPrinter=myBool;
		}
		else if(strcmp(tag, "SANS_FONT")==0)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->SansFont=myQStr;
		}
		else if(strcmp(tag, "SERIF_FONT")==0)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->SerifFont=myQStr;
		}
		else if(strcmp(tag, "MONOTYPE_FONT")==0)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->MonotypeFont=myQStr;
		}
		else if(strcmp(tag, "DECORATIVE_FONT")==0)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->DecorativeFont=myQStr;
		}
		else if(strcmp(tag, "VECTORISE_SVG_FONTS")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->VectoriseSVGFonts=myBool;
		}
		else if(strcmp(tag, "COLOUR_TABLE")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->ColourTable=myQStr;
		}
		else if(strcmp(tag, "LINE_WIDTH")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->LineWidth=myDouble;
		}
		else if(strcmp(tag, "BASE_FONT_SIZE")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->BaseFontSize=myDouble;
		}
        else if(strcmp(tag, "GAP_TOLERANCE")==0 && currentProfile!=NULL)
        {
            file >> myDouble;
            currentProfile->gapTolerance=myDouble;
        }
		else if(strcmp(tag, "SAVE_RESTORE")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->SaveRestorePath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "USER_LIB")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->UserLibPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "CONFIG")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->ConfigPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "FILTER_BANK")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
#if QT_VERSION < 0x040000
			myQStr=QString(myStr).stripWhiteSpace();
#else
			myQStr=QString(myStr).trimmed();
#endif
			currentProfile->FilterBankPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "HEIGHT")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->Height=myInt;
		}
		else if(strcmp(tag, "PLOT_UI_HEIGHT")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->QpuiHeight=myInt;
		}
		else if(strcmp(tag, "PLOT_UI_WIDTH")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->QpuiWidth=myInt;
		}
		else if(strcmp(tag, "FONT_SIZE")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->fontSize=myInt;
		}
		else if(strcmp(tag, "GUI_MARGINS")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->GUImargins=myInt;
		}
		else if(strcmp(tag, "TIME_INTERVAL_BUTTON")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->timeIntervalButton=myInt;
		}
		else if(strcmp(tag, "DEBUG")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->Debug=myBool;
		}
		else if(strcmp(tag, "QRTN_DEBUG")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->QrtnDebug=myBool;
		}
		else
		{
			cout << "unknown tag " << tag << " - ignoring" << endl;
			file.getline(myStr, 500);
		}
	}
	
	if(currentProfile!=NULL) add(currentProfile);

	file.close();
	QProfile* test=current();
	if(test==NULL) currentName=defaultProfile->getName();

	return true;
}

bool QProfilesList::save()
{
#ifndef WINDOWS
	char * home=getenv("HOME");
	string file_name=string(home).append("/.qsasrc");
#else
	char * home=getenv("USERPROFILE");
	string file_name=string(home).append("\\qsas.ini");
#endif
	ofstream file(file_name.c_str());

	if(!file.good()) return false;
	
	file << "CURRENT_PROFILE " << currentName.latin1() << endl;
	
	file << endl;

	for(int i=0; i<presets.size()+1; ++i) // we do not save default
	{
		QProfile * prof;
		if(i<presets.size())
		{
			prof=presets[i];
		}
		else
		{
			prof=customProfile;
		}
		
		if(prof!=NULL)
		{
			file << "PROFILE " << prof->getName().latin1() << endl;
			file << "3D_VECTOR " << prof->Vector3D.latin1() << endl;
			file << "3D_ARRAY " << prof->Array3D.latin1() << endl;
			file << "3D_ARRAY_TO_SCALAR " << prof->Array3DtoScalar.latin1() << endl;
			file << "3D_ARRAY_XY " << prof->Array3DXY.latin1() << endl;
			file << "3D_ARRAY_TO_SCALAR_XY " << prof->Array3DtoScalarXY.latin1() << endl;
			file << "PAGE_SIZE " << prof->PageSize.latin1() << endl;
			file << "PAGE_DPI " << prof->PageDPI << endl;
			file << "PAGE_ORIENTATION " << prof->PageOrientation.latin1() << endl;
			file << "PAGE_WIDTH " << prof->PageWidth << endl;
			file << "PAGE_HEIGHT " << prof->PageHeight << endl;
			file << "TIME_REPRESENTATION " << prof->TimeRep.latin1() << endl;
			file << "USE_KPRINTER " << prof->UseKPrinter << endl;
			if(!prof->SansFont.isEmpty())
			{
#if QT_VERSION >= 0x040000
				file << "SANS_FONT "<< prof->SansFont.toLatin1().data() << endl;
#else
				file << "SANS_FONT "<< prof->SansFont.latin1() << endl;
#endif
			}
			else
			{
				file << "SANS_FONT _NONE_" << endl;
			}
			if(!prof->SerifFont.isEmpty())
			{
#if QT_VERSION >= 0x040000
				file << "SERIF_FONT "<< prof->SerifFont.toLatin1().data() << endl;
#else
				file << "SERIF_FONT "<< prof->SerifFont.latin1() << endl;
#endif
			}
			else
			{
				file << "SERIF_FONT _NONE_" << endl;
			}
			if(!prof->MonotypeFont.isEmpty())
			{
#if QT_VERSION >= 0x040000
				file << "MONOTYPE_FONT "<< prof->MonotypeFont.toLatin1().data() << endl;
#else
				file << "MONOTYPE_FONT "<< prof->MonotypeFont.latin1() << endl;
#endif
			}
			else
			{
				file << "MONOTYPE_FONT _NONE_" << endl;
			}
			if(!prof->DecorativeFont.isEmpty())
			{
#if QT_VERSION >= 0x040000
				file << "DECORATIVE_FONT "<< prof->DecorativeFont.toLatin1().data() << endl;
#else
				file << "DECORATIVE_FONT "<< prof->DecorativeFont.latin1() << endl;
#endif
			}
			else
			{
				file << "DECORATIVE_FONT _NONE_" << endl;
			}
			file << "VECTORISE_SVG_FONTS " << prof->VectoriseSVGFonts << endl;
			if(!prof->ColourTable.isEmpty())
			{
#if QT_VERSION >= 0x040000
				file << "COLOUR_TABLE "<< prof->ColourTable.toLatin1().data() << endl;
#else
				file << "COLOUR_TABLE "<< prof->ColourTable.latin1() << endl;
#endif
			}
			else
			{
				file << "COLOUR_TABLE _NONE_" << endl;
			}
			file << "LINE_WIDTH " << prof->LineWidth << endl;
			file << "BASE_FONT_SIZE " << prof->BaseFontSize << endl;
            file << "GAP_TOLERANCE " << prof->gapTolerance << endl;
			file << "SAVE_RESTORE " << addTokens(prof->SaveRestorePath).latin1() << endl;
			file << "USER_LIB " << addTokens(prof->UserLibPath).latin1() << endl;
			file << "CONFIG " << addTokens(prof->ConfigPath).latin1() << endl;
			file << "FILTER_BANK " << addTokens(prof->FilterBankPath).latin1() << endl;
			file << "HEIGHT " << prof->Height << endl;
			file << "PLOT_UI_HEIGHT " << prof->QpuiHeight << endl;
			file << "PLOT_UI_WIDTH " << prof->QpuiWidth << endl;
			file << "FONT_SIZE " << prof->fontSize << endl;
			file << "GUI_MARGINS " << prof->GUImargins << endl;
			file << "TIME_INTERVAL_BUTTON " << prof->timeIntervalButton << endl;
			file << "DEBUG " << prof->Debug << endl;
			file << "QRTN_DEBUG " << prof->QrtnDebug << endl;
			file << endl;
		}
	}
	file.close();

	
	return true;
}

QProfile * QProfilesList::current()
{
	QProfile* current=fromName(currentName);
	if(current==NULL)
	{
		return defaultProfile;
	}
	return current;
}

QProfile * QProfilesList::fromName(const QString& name)
{
	if(name==defaultProfile->getName())
	{
		return defaultProfile;
	}
	if(customProfile!=NULL)
	{
		if(name==customProfile->getName())
		{
			return customProfile;
		}
	}
	
	QProfile * res=NULL;
	for(int i=0; i<presets.size() && res==NULL; ++i)
	{
		if(presets[i]->getName()==name) res=presets[i];
	}
	return res;
}

bool QProfilesList::setCurrent(const QString& name)
{
	if(name==defaultProfile->getName())
	{
		currentName=name;
		return true;
	}
	if(customProfile!=NULL)
	{
		if(name==customProfile->getName())
		{
			currentName=name;
			return true;
		}
	}
	
	for(int i=0; i<presets.size(); ++i)
	{
		if(presets[i]->getName()==name)
		{
			currentName=name;
			return true;
		}
	}
	return false;
}

void QProfilesList::names(vector<QString>& names)
{
	names.clear();
	names.push_back(defaultProfile->getName());
	for(int i=0; i<presets.size(); ++i)
	{
		names.push_back(presets[i]->getName());
	}
	if(customProfile!=NULL) names.push_back(customProfile->getName());
}

bool QProfilesList::add(QProfile* prof)
{
	QProfile* res=fromName(prof->getName());
	
	if(res)
	{
		return false;
	}
	
	if(prof->getName()=="Custom")
	{
		customProfile=prof;
	}
	else
	{
		presets.push_back(prof);
	}
	
	return true;
}


bool QProfilesList::remove(const QString& name)
{
	bool found=false;
	int i, index;
	
	if(customProfile!=NULL && name=="Custom")
	{
		delete customProfile;
		customProfile=NULL;
		return true;
	}
	
	for(i=0; i<presets.size() && !found; ++i)
	{
		found=(presets[i]->getName()==name);
		index=i;
	}


	if(found)
	{
		vector<QProfile*>::iterator iter=presets.begin()+index;
		delete presets[index];
		presets.erase(iter);
		if(name==currentName) currentName=defaultProfile->getName();
	}
	
	return found;
}

void QProfilesList::newCustom(QProfile * p)
{
	if(customProfile!=NULL)
	{
		delete customProfile;
	}
	
	customProfile=p;
	customProfile->setName("Custom");
}


